@@ -25,6 +25,8 @@
 #include "game_constants.h"
 #include <wx/stdpaths.h>
 #include <platform_util.h>
+//#include "interpolation.h"
+
 #ifndef WIN32
 #include <errno.h>
 #endif
@@ -62,6 +64,7 @@ const string g3dviewerVersionString= "v1.3.6";
 string fileFormat = "png";
 
 namespace Glest { namespace Game {
+
 string getGameReadWritePath(string lookupKey) {
 	string path = "";
     if(path == "" && getenv("GLESTHOME") != NULL) {
@@ -179,12 +182,12 @@ void printParameterHelp(const char *argv0, bool foundInvalidArgs) {
 	printf("\n                     \t\tWhere x is a g3d filename to load separated with a");
 	printf("\n                     \t\tcomma and one or more skill names to try loading:");
 	printf("\n                     \t\texample:");
-	printf("\n %s %s=techs/megapack/factions/tech/units/battle_machine,attack_skill,stop_skill",extractFileFromDirectoryPath(argv0).c_str(),(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_UNIT]));
+	printf("\n %s\n %s=techs/megapack/factions/tech/units/battle_machine,attack_skill,stop_skill",extractFileFromDirectoryPath(argv0).c_str(),(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_UNIT]));
 
 	printf("\n%s=x\t\t\tAuto load the model specified in path/filename x",(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_MODEL]));
 	printf("\n                     \t\tWhere x is a g3d filename to load:");
 	printf("\n                     \t\texample:");
-	printf("\n %s %s=techs/megapack/factions/tech/units/battle_machine/models/battle_machine_dying.g3d",extractFileFromDirectoryPath(argv0).c_str(),(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_MODEL]));
+	printf("\n %s\n %s=techs/megapack/factions/tech/units/battle_machine/models/battle_machine_dying.g3d",extractFileFromDirectoryPath(argv0).c_str(),(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_MODEL]));
 
 	printf("\n%s=x\tAnimation value when loading a model",(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_MODEL_ANIMATION_VALUE]));
 	printf("\n                     \t\tWhere x is a decimal value from -1.0 to 1.0:");
@@ -201,14 +204,14 @@ void printParameterHelp(const char *argv0, bool foundInvalidArgs) {
 	printf("\n                     \t\tenable_normals, disable_grid, disable_wireframe,");
 	printf("\n                     \t\tdisable_normals, saveas-<filename>, resize-wxh");
 	printf("\n                     \t\texample:");
-	printf("\n %s %s=transparent,disable_grid,saveas-test.png,resize-800x600 %s=techs/megapack/factions/tech/units/battle_machine/models/battle_machine_dying.g3d",extractFileFromDirectoryPath(argv0).c_str(),(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_AUTO_SCREENSHOT]),(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_MODEL]));
+	printf("\n %s\n %s=transparent,disable_grid,saveas-test.png,resize-800x600\n %s=techs/megapack/factions/tech/units/battle_machine/models/battle_machine_dying.g3d",extractFileFromDirectoryPath(argv0).c_str(),(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_AUTO_SCREENSHOT]),(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_MODEL]));
 
 	//     "================================================================================"
 	printf("\n%s=x",(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_PARTICLE]));
 	printf("\n                     \t\tAuto load the particle specified in path/filename x");
 	printf("\n                     \t\tWhere x is a Particle XML filename to load:");
 	printf("\n                     \t\texample:");
-	printf("\n %s %s=techs/megapack/factions/persian/units/genie/glow_particles.xml",extractFileFromDirectoryPath(argv0).c_str(),(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_PARTICLE]));
+	printf("\n %s\n %s=techs/megapack/factions/persian/units/genie/glow_particles.xml",extractFileFromDirectoryPath(argv0).c_str(),(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_PARTICLE]));
 
 	printf("\n%s=x",(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_PARTICLE_PROJECTILE]));
 	printf("\n                     \t\tAuto load the projectile particle specified in");
@@ -216,7 +219,7 @@ void printParameterHelp(const char *argv0, bool foundInvalidArgs) {
 	printf("\n                     \t\tWhere x is a Projectile Particle Definition XML");
 	printf("\n                     \t\t        filename to load:");
 	printf("\n                     \t\texample:");
-	printf("\n %s %s=techs/megapack/factions/persian/units/genie/particle_proj.xml",extractFileFromDirectoryPath(argv0).c_str(),(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_PARTICLE_PROJECTILE]));
+	printf("\n %s\n %s=techs/megapack/factions/persian/units/genie/particle_proj.xml",extractFileFromDirectoryPath(argv0).c_str(),(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_PARTICLE_PROJECTILE]));
 
 	printf("\n%s=x",(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_PARTICLE_SPLASH]));
 	printf("\n                     \t\tAuto load the splash particle specified in");
@@ -224,7 +227,7 @@ void printParameterHelp(const char *argv0, bool foundInvalidArgs) {
 	printf("\n                     \t\tWhere x is a Splash Particle Definition XML");
 	printf("\n                     \t\t        filename to load:");
 	printf("\n                     \t\texample:");
-	printf("\n %s %s=techs/megapack/factions/persian/units/genie/particle_splash.xml",extractFileFromDirectoryPath(argv0).c_str(),(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_PARTICLE_SPLASH]));
+	printf("\n %s\n %s=techs/megapack/factions/persian/units/genie/particle_splash.xml",extractFileFromDirectoryPath(argv0).c_str(),(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_PARTICLE_SPLASH]));
 
 	printf("\n%s=x",(const char *)wxConvCurrent->cWX2MB(GAME_ARGS[GAME_ARG_LOAD_PARTICLE_LOOP_VALUE]));
 	printf("\n                     \t\tParticle loop value when loading one or more");
@@ -284,7 +287,8 @@ MainWindow::MainWindow(	std::pair<string,vector<string> > unitToLoad,
 		        wxSize(Renderer::windowW, Renderer::windowH)),
 		        model(NULL), glCanvas(NULL), renderer(NULL),
 		        initTextureManager(true), timer(NULL),
-		        startupSettingsInited(false)
+		        startupSettingsInited(false),
+		        fileDialog(NULL),colorPicker(NULL)
 {
 	this->appPath = appPath;
 	Properties::setApplicationPath(executable_path(appPath));
@@ -293,6 +297,7 @@ MainWindow::MainWindow(	std::pair<string,vector<string> > unitToLoad,
 	string iniFilePath = extractDirectoryPathFromFile(config.getFileName(false));
     //getGlPlatformExtensions();
 
+	isControlKeyPressed = false;
 	int args[] = { WX_GL_RGBA, WX_GL_DOUBLEBUFFER,  WX_GL_MIN_ALPHA,  8  }; // to prevent flicker
 	//int args[] = { WX_GL_RGBA, WX_GL_MIN_ALPHA,  0  }; // to prevent flicker
 	glCanvas = new GlCanvas(this, args);
@@ -309,7 +314,7 @@ MainWindow::MainWindow(	std::pair<string,vector<string> > unitToLoad,
 
 	if(modelPath != "") {
 		this->modelPathList.push_back(modelPath);
-		printf("Startup Adding model [%s] list size %lu\n",modelPath.c_str(),this->modelPathList.size());
+		printf("Startup Adding model [%s] list size " MG_SIZE_T_SPECIFIER "\n",modelPath.c_str(),this->modelPathList.size());
 	}
 	if(particlePath != "") {
 		this->particlePathList.push_back(particlePath);
@@ -448,7 +453,7 @@ MainWindow::MainWindow(	std::pair<string,vector<string> > unitToLoad,
 	std::auto_ptr<wchar_t> wstr(Ansi2WideString(appPath.c_str()));
 	
 	wstring launchApp = wstring(wstr.get()) + L" \"%1\"";
-	DWORD len = launchApp.length() + 1;
+	DWORD len = (DWORD)launchApp.length() + 1;
 	RegSetValueEx(keyHandle, NULL, 0, REG_SZ, (PBYTE)launchApp.c_str(), len);
 	RegCloseKey(keyHandle);
 
@@ -456,7 +461,7 @@ MainWindow::MainWindow(	std::pair<string,vector<string> > unitToLoad,
 	RegCreateKeyEx(HKEY_CURRENT_USER,subKey.c_str(),0, NULL, 0, KEY_ALL_ACCESS, NULL, &keyHandle, &dwDisposition);
 	//Set the value.
 	launchApp = L"megaglest.g3d";
-	len = launchApp.length() + 1;
+	len = (DWORD)launchApp.length() + 1;
 	RegSetValueEx(keyHandle, NULL, 0, REG_SZ, (PBYTE)launchApp.c_str(), len);
 	RegCloseKey(keyHandle);
 
@@ -528,19 +533,29 @@ void MainWindow::setupStartupSettings() {
 }
 
 MainWindow::~MainWindow(){
-	delete renderer;
-	renderer = NULL;
-	delete model;
-	model = NULL;
 	delete timer;
 	timer = NULL;
-	delete glCanvas;
+
+	delete fileDialog;
+	fileDialog = NULL;
+
+	delete model;
+	model = NULL;
+
+	delete renderer;
+	renderer = NULL;
+
+	//delete glCanvas;
+	if(glCanvas) glCanvas->Destroy();
 	glCanvas = NULL;
 
 }
 
 void MainWindow::init() {
-#if wxCHECK_VERSION(2, 9, 1)
+
+#if wxCHECK_VERSION(2, 9, 3)
+	glCanvas->setCurrentGLContext();
+#elif wxCHECK_VERSION(2, 9, 1)
 
 #else
 	glCanvas->SetCurrent();
@@ -700,7 +715,21 @@ void MainWindow::onClose(wxCloseEvent &event){
 	//printf("OnClose about to END\n");
 	//fflush(stdout);
 
-	delete this;
+	delete timer;
+	timer = NULL;
+
+	delete model;
+	model = NULL;
+
+	delete renderer;
+	renderer = NULL;
+
+	//delete glCanvas;
+	if(glCanvas) glCanvas->Destroy();
+	glCanvas = NULL;
+
+	//delete this;
+	this->Destroy();
 }
 
 // for the mousewheel
@@ -775,13 +804,13 @@ void MainWindow::onMouseMove(wxMouseEvent &event){
 
 void MainWindow::onMenuFileLoad(wxCommandEvent &event){
 	try {
-		string fileName;
+		//string fileName;
 		fileDialog->SetWildcard(wxT("G3D files (*.g3d)|*.g3d;*.G3D"));
 		fileDialog->SetMessage(wxT("Selecting Glest Model for current view."));
 
 		if(fileDialog->ShowModal()==wxID_OK){
 			modelPathList.clear();
-			string file = "";
+			string file;
 #ifdef WIN32
 			const wxWX2MBbuf tmp_buf = wxConvCurrent->cWX2MB(fileDialog->GetPath());
 			file = tmp_buf;
@@ -805,7 +834,7 @@ void MainWindow::onMenuFileLoad(wxCommandEvent &event){
 
 void MainWindow::onMenuFileLoadParticleXML(wxCommandEvent &event){
 	try {
-		string fileName;
+		//string fileName;
 		fileDialog->SetWildcard(wxT("XML files (*.xml)|*.xml"));
 
 		if(isControlKeyPressed == true) {
@@ -817,7 +846,7 @@ void MainWindow::onMenuFileLoadParticleXML(wxCommandEvent &event){
 
 		if(fileDialog->ShowModal()==wxID_OK){
 			//string path = (const char*)wxFNCONV(fileDialog->GetPath().c_str());
-			string file = "";
+			string file;
 #ifdef WIN32
 			const wxWX2MBbuf tmp_buf = wxConvCurrent->cWX2MB(fileDialog->GetPath());
 			file = tmp_buf;
@@ -839,7 +868,7 @@ void MainWindow::onMenuFileLoadParticleXML(wxCommandEvent &event){
 
 void MainWindow::onMenuFileLoadProjectileParticleXML(wxCommandEvent &event){
 	try {
-		string fileName;
+		//string fileName;
 		fileDialog->SetWildcard(wxT("XML files (*.xml)|*.xml"));
 
 		if(isControlKeyPressed == true) {
@@ -851,7 +880,7 @@ void MainWindow::onMenuFileLoadProjectileParticleXML(wxCommandEvent &event){
 
 		if(fileDialog->ShowModal()==wxID_OK){
 			//string path = (const char*)wxFNCONV(fileDialog->GetPath().c_str());
-			string file = "";
+			string file;
 #ifdef WIN32
 			const wxWX2MBbuf tmp_buf = wxConvCurrent->cWX2MB(fileDialog->GetPath());
 			file = tmp_buf;
@@ -873,7 +902,7 @@ void MainWindow::onMenuFileLoadProjectileParticleXML(wxCommandEvent &event){
 
 void MainWindow::onMenuFileLoadSplashParticleXML(wxCommandEvent &event){
 	try {
-		string fileName;
+		//string fileName;
 		fileDialog->SetWildcard(wxT("XML files (*.xml)|*.xml"));
 
 		if(isControlKeyPressed == true) {
@@ -885,7 +914,7 @@ void MainWindow::onMenuFileLoadSplashParticleXML(wxCommandEvent &event){
 
 		if(fileDialog->ShowModal()==wxID_OK){
 			//string path = (const char*)wxFNCONV(fileDialog->GetPath().c_str());
-			string file = "";
+			string file;
 #ifdef WIN32
 			const wxWX2MBbuf tmp_buf = wxConvCurrent->cWX2MB(fileDialog->GetPath());
 			file = tmp_buf;
@@ -1203,14 +1232,14 @@ void MainWindow::loadModel(string path) {
     try {
         if(path != "" && fileExists(path) == true) {
             this->modelPathList.push_back(path);
-            printf("Adding model [%s] list size %lu\n",path.c_str(),this->modelPathList.size());
+            printf("Adding model [%s] list size " MG_SIZE_T_SPECIFIER "\n",path.c_str(),this->modelPathList.size());
         }
 
         string titlestring=winHeader;
         for(unsigned int idx =0; idx < this->modelPathList.size(); idx++) {
             string modelPath = this->modelPathList[idx];
 
-            //printf("Loading model [%s] %u of %lu\n",modelPath.c_str(),idx, this->modelPathList.size());
+            //printf("Loading model [%s] %u of " MG_SIZE_T_SPECIFIER "\n",modelPath.c_str(),idx, this->modelPathList.size());
 
             if(timer) timer->Stop();
             delete model;
@@ -1234,6 +1263,8 @@ void MainWindow::loadModel(string path) {
 
 void MainWindow::loadParticle(string path) {
 	if(timer) timer->Stop();
+
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] about to load [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str());
 	if(path != "" && fileExists(path) == true) {
 		renderer->end();
 		unitParticleSystems.clear();
@@ -1248,9 +1279,11 @@ void MainWindow::loadParticle(string path) {
 			this->particlePathList.clear();
 			this->particlePathList.push_back(path);
 		}
+
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] added file [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str());
 	}
 
-	try{
+	try {
 	if(this->particlePathList.empty() == false) {
         string titlestring=winHeader;
 		for(unsigned int idx = 0; idx < this->particlePathList.size(); idx++) {
@@ -1267,13 +1300,14 @@ void MainWindow::loadParticle(string path) {
 
 			std::string unitXML = dir + folderDelimiter + extractFileFromDirectoryPath(dir) + ".xml";
 
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] looking for unit XML [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,unitXML.c_str());
+
 			//int size   = -1;
 			//int height = -1;
+			int size  = 0;
+			int height= 0;
 
 			if(fileExists(unitXML) == true) {
-
-				int size  = 0;
-				int height= 0;
 				{
 				XmlTree xmlTree;
 				xmlTree.load(unitXML,Properties::getTagReplacementValues());
@@ -1286,36 +1320,43 @@ void MainWindow::loadParticle(string path) {
 				}
 
                 // std::cout << "About to load [" << particlePath << "] from [" << dir << "] unit [" << unitXML << "]" << std::endl;
-
-				std::map<string,vector<pair<string, string> > > loadedFileList;
-                UnitParticleSystemType *unitParticleSystemType = new UnitParticleSystemType();
-                unitParticleSystemType->load(NULL, dir, dir + folderDelimiter + particlePath, //### if we knew which particle it was, we could be more accurate
-                		renderer,loadedFileList,"g3dviewer","");
-                unitParticleSystemTypes.push_back(unitParticleSystemType);
-
-                for(std::vector<UnitParticleSystemType *>::const_iterator it= unitParticleSystemTypes.begin(); it != unitParticleSystemTypes.end(); ++it) {
-                    UnitParticleSystem *ups= new UnitParticleSystem(200);
-                    (*it)->setValues(ups);
-                    if(size > 0) {
-                        //getCurrVectorFlat() + Vec3f(0.f, type->getHeight()/2.f, 0.f);
-                        Vec3f vec = Vec3f(0.f, height / 2.f, 0.f);
-                        ups->setPos(vec);
-                    }
-                    //ups->setFactionColor(getFaction()->getTexture()->getPixmap()->getPixel3f(0,0));
-                    ups->setFactionColor(renderer->getPlayerColorTexture(playerColor)->getPixmap()->getPixel3f(0,0));
-                    unitParticleSystems.push_back(ups);
-                    renderer->manageParticleSystem(ups);
-
-                    ups->setVisible(true);
-                }
-
-                if(path != "" && fileExists(path) == true) {
-                    renderer->initModelManager();
-                    if(initTextureManager) {
-                    	renderer->initTextureManager();
-                    }
-                }
 			}
+			else {
+				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] unit XML NOT FOUND [%s] using default position values\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,unitXML.c_str());
+
+				size  = 1;
+				height= 1;
+			}
+
+			std::map<string,vector<pair<string, string> > > loadedFileList;
+            UnitParticleSystemType *unitParticleSystemType = new UnitParticleSystemType();
+            unitParticleSystemType->load(NULL, dir, dir + folderDelimiter + particlePath, //### if we knew which particle it was, we could be more accurate
+            		renderer,loadedFileList,"g3dviewer","");
+            unitParticleSystemTypes.push_back(unitParticleSystemType);
+
+            for(std::vector<UnitParticleSystemType *>::const_iterator it= unitParticleSystemTypes.begin(); it != unitParticleSystemTypes.end(); ++it) {
+                UnitParticleSystem *ups= new UnitParticleSystem(200);
+                (*it)->setValues(ups);
+                if(size > 0) {
+                    //getCurrVectorFlat() + Vec3f(0.f, type->getHeight()/2.f, 0.f);
+                    Vec3f vec = Vec3f(0.f, height / 2.f, 0.f);
+                    ups->setPos(vec);
+                }
+                //ups->setFactionColor(getFaction()->getTexture()->getPixmap()->getPixel3f(0,0));
+                ups->setFactionColor(renderer->getPlayerColorTexture(playerColor)->getPixmap()->getPixel3f(0,0));
+                unitParticleSystems.push_back(ups);
+                renderer->manageParticleSystem(ups);
+
+                ups->setVisible(true);
+            }
+
+            if(path != "" && fileExists(path) == true) {
+                renderer->initModelManager();
+                if(initTextureManager) {
+                	renderer->initTextureManager();
+                }
+            }
+
 		}
 		SetTitle(ToUnicode(titlestring));
 	}
@@ -1328,7 +1369,7 @@ void MainWindow::loadParticle(string path) {
 }
 
 void MainWindow::loadProjectileParticle(string path) {
-	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] about to load [%s] particleProjectilePathList.size() = %lu\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),this->particleProjectilePathList.size());
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] about to load [%s] particleProjectilePathList.size() = " MG_SIZE_T_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),this->particleProjectilePathList.size());
 
 	if(timer) timer->Stop();
 	if(path != "" && fileExists(path) == true) {
@@ -1349,7 +1390,7 @@ void MainWindow::loadProjectileParticle(string path) {
 
 	try {
 	if(this->particleProjectilePathList.empty() == false) {
-		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("this->particleProjectilePathList.size() = %lu\n",this->particleProjectilePathList.size());
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("this->particleProjectilePathList.size() = " MG_SIZE_T_SPECIFIER "\n",this->particleProjectilePathList.size());
 
         string titlestring=winHeader;
 		for(unsigned int idx = 0; idx < this->particleProjectilePathList.size(); idx++) {
@@ -1366,11 +1407,11 @@ void MainWindow::loadProjectileParticle(string path) {
 
 			std::string unitXML = dir + folderDelimiter + extractFileFromDirectoryPath(dir) + ".xml";
 
-			int size   = -1;
-			int height = -1;
+			int size   = 1;
+			int height = 1;
 
 			if(fileExists(unitXML) == true) {
-				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] loading [%s] idx = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,unitXML.c_str(),idx);
+				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] loading [%s] idx = %u\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,unitXML.c_str(),idx);
 
 				XmlTree xmlTree;
 				xmlTree.load(unitXML,Properties::getTagReplacementValues());
@@ -1386,7 +1427,7 @@ void MainWindow::loadProjectileParticle(string path) {
 
 			string particleFile = dir + folderDelimiter + particlePath;
 			{
-				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] loading [%s] idx = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,particleFile.c_str(),idx);
+				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] loading [%s] idx = %u\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,particleFile.c_str(),idx);
 				XmlTree xmlTree;
 				xmlTree.load(particleFile,Properties::getTagReplacementValues());
 				//const XmlNode *particleSystemNode= xmlTree.getRootNode();
@@ -1394,7 +1435,7 @@ void MainWindow::loadProjectileParticle(string path) {
 				// std::cout << "Loaded successfully, loading values..." << std::endl;
 			}
 
-			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] loading [%s] idx = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,particleFile.c_str(),idx);
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] loading [%s] idx = %u\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,particleFile.c_str(),idx);
 			std::map<string,vector<pair<string, string> > > loadedFileList;
 			ParticleSystemTypeProjectile *projectileParticleSystemType= new ParticleSystemTypeProjectile();
 			projectileParticleSystemType->load(NULL, dir, //### we don't know if there are overrides in the unit XML
@@ -1425,7 +1466,7 @@ void MainWindow::loadProjectileParticle(string path) {
 				renderer->manageParticleSystem(ps);
 			}
 
-			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] loaded [%s] idx = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,particleFile.c_str(),idx);
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] loaded [%s] idx = %u\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,particleFile.c_str(),idx);
 		}
 		SetTitle(ToUnicode(titlestring));
 
@@ -1446,7 +1487,7 @@ void MainWindow::loadProjectileParticle(string path) {
 }
 
 void MainWindow::loadSplashParticle(string path) {  // uses ParticleSystemTypeSplash::load  (and own list...)
-	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] about to load [%s] particleSplashPathList.size() = %lu\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),this->particleSplashPathList.size());
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] about to load [%s] particleSplashPathList.size() = " MG_SIZE_T_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),this->particleSplashPathList.size());
 
 	if(timer) timer->Stop();
 	if(path != "" && fileExists(path) == true) {
@@ -1482,8 +1523,8 @@ void MainWindow::loadSplashParticle(string path) {  // uses ParticleSystemTypeSp
 
 			std::string unitXML = dir + folderDelimiter + extractFileFromDirectoryPath(dir) + ".xml";
 
-			int size   = -1;
-			int height = -1;
+			int size   = 1;
+			int height = 1;
 
 			if(fileExists(unitXML) == true) {
 				XmlTree xmlTree;
@@ -1549,7 +1590,7 @@ void MainWindow::loadSplashParticle(string path) {  // uses ParticleSystemTypeSp
 		wxMessageDialog(NULL, ToUnicode(e.what()), ToUnicode("Not a Mega-Glest projectile particle XML file, or broken"), wxOK | wxICON_ERROR).ShowModal();
 	}
 	if(timer) timer->Start(100);
-	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] after load [%s] particleSplashPathList.size() = %lu\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),this->particleSplashPathList.size());
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] after load [%s] particleSplashPathList.size() = " MG_SIZE_T_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),this->particleSplashPathList.size());
 }
 
 void MainWindow::onMenuModeNormals(wxCommandEvent &event){
@@ -1998,6 +2039,8 @@ END_EVENT_TABLE()
 // ===============================================
 
 bool App::OnInit() {
+	//InterpolationData::setEnableCache(false);
+
 	SystemFlags::VERBOSE_MODE_ENABLED  = false;
 	//Renderer::windowW = 1920;
 	//Renderer::windowH = 1440;
@@ -2466,8 +2509,9 @@ int App::MainLoop(){
 	}
 	catch(const exception &e){
 		wxMessageDialog(NULL, ToUnicode(e.what()), ToUnicode("Exception"), wxOK | wxICON_ERROR).ShowModal();
-		return 0;
+		return 1;
 	}
+	return 0;
 }
 
 int App::OnExit(){
