@@ -32,7 +32,7 @@ using namespace Shared::Graphics;
 
 namespace Glest{ namespace Game{
 
-AiInterface::AiInterface(Game &game, int factionIndex, int teamIndex, int useStartLocation) {
+AiInterface::AiInterface(Game &game, int factionIndex, int teamIndex, int useStartLocation) : fp(NULL) {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
 	this->world= game.getWorld();
@@ -51,18 +51,29 @@ AiInterface::AiInterface(Game &game, int factionIndex, int teamIndex, int useSta
 	logLevel= Config::getInstance().getInt("AiLog");
 	redir= Config::getInstance().getBool("AiRedir");
 
+	aiLogFile = getLogFilename();
+	if(getGameReadWritePath(GameConstants::path_logs_CacheLookupKey) != "") {
+		aiLogFile = getGameReadWritePath(GameConstants::path_logs_CacheLookupKey) + aiLogFile;
+	}
+	else {
+        string userData = Config::getInstance().getString("UserData_Root","");
+        if(userData != "") {
+        	endPathWithSlash(userData);
+        }
+        aiLogFile = userData + aiLogFile;
+	}
+
 	//clear log file
-	if(logLevel>0){
+	if(logLevel > 0) {
 #ifdef WIN32
-		FILE *f= _wfopen(Shared::Platform::utf8_decode(getLogFilename()).c_str(), L"wt");
+		fp = _wfopen(Shared::Platform::utf8_decode(aiLogFile).c_str(), L"wt");
 #else
-		FILE *f= fopen(getLogFilename().c_str(), "wt");
+		fp = fopen(aiLogFile.c_str(), "wt");
 #endif
-		if(f==NULL){
-			throw runtime_error("Can't open file: "+getLogFilename());
+		if(fp == NULL) {
+			throw megaglest_runtime_error("Can't open file: [" + aiLogFile + "]");
 		}
-		fprintf(f, "%s", "MegaGlest AI log file\n\n");
-		fclose(f);
+		fprintf(fp, "MegaGlest AI log file for Tech [%s] Faction [%s] #%d\n\n",this->gameSettings->getTech().c_str(),this->world->getFaction(this->factionIndex)->getType()->getName().c_str(),this->factionIndex);
 	}
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 }
@@ -70,6 +81,11 @@ AiInterface::AiInterface(Game &game, int factionIndex, int teamIndex, int useSta
 AiInterface::~AiInterface() {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] deleting AI factionIndex = %d, teamIndex = %d\n",__FILE__,__FUNCTION__,__LINE__,this->factionIndex,this->teamIndex);
     cacheUnitHarvestResourceLookup.clear();
+
+    if(fp) {
+    	fclose(fp);
+    	fp = NULL;
+    }
 }
 // ==================== main ====================
 
@@ -81,20 +97,13 @@ void AiInterface::update() {
 // ==================== misc ====================
 
 void AiInterface::printLog(int logLevel, const string &s){
-    if(this->logLevel>=logLevel){
+    if(this->logLevel >= logLevel) {
 		string logString= "(" + intToStr(factionIndex) + ") " + s;
 
 		//print log to file
-#ifdef WIN32
-		FILE *f= _wfopen(utf8_decode(getLogFilename()).c_str(), L"at");
-#else
-		FILE *f= fopen(getLogFilename().c_str(), "at");
-#endif
-		if(f==NULL){
-			throw runtime_error("Can't open file: "+getLogFilename());
+		if(fp != NULL) {
+			fprintf(fp, "%s\n", logString.c_str());
 		}
-		fprintf(f, "%s\n", logString.c_str());
-		fclose(f);
 
 		//redirect to console
 		if(redir) {
@@ -117,56 +126,57 @@ bool AiInterface::executeCommandOverNetwork() {
 	return faction->getCpuControl(enableServerControlledAI,isNetworkGame,role);
 }
 
-CommandResult AiInterface::giveCommandSwitchTeamVote(const Faction* faction, SwitchTeamVote *vote) {
+std::pair<CommandResult,string> AiInterface::giveCommandSwitchTeamVote(const Faction* faction, SwitchTeamVote *vote) {
 	assert(this->gameSettings != NULL);
 
 	commander->trySwitchTeamVote(faction,vote);
-	return crSuccess;
+	return std::pair<CommandResult,string>(crSuccess,"");
 }
 
-CommandResult AiInterface::giveCommand(int unitIndex, CommandClass commandClass, const Vec2i &pos){
+std::pair<CommandResult,string> AiInterface::giveCommand(int unitIndex, CommandClass commandClass, const Vec2i &pos){
 	assert(this->gameSettings != NULL);
 
+	std::pair<CommandResult,string> result(crFailUndefined,"");
 	if(executeCommandOverNetwork() == true) {
 		const Unit *unit = getMyUnit(unitIndex);
-		CommandResult result = commander->tryGiveCommand(unit, unit->getType()->getFirstCtOfClass(commandClass), pos, unit->getType(),CardinalDir::NORTH);
-
+		result = commander->tryGiveCommand(unit, unit->getType()->getFirstCtOfClass(commandClass), pos, unit->getType(),CardinalDir::NORTH);
 		return result;
 	}
 	else {
 		Command *c= new Command (world->getFaction(factionIndex)->getUnit(unitIndex)->getType()->getFirstCtOfClass(commandClass), pos);
 
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
-		CommandResult result = world->getFaction(factionIndex)->getUnit(unitIndex)->giveCommand(c);
+		result = world->getFaction(factionIndex)->getUnit(unitIndex)->giveCommand(c);
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
 		return result;
 	}
 }
 
-CommandResult AiInterface::giveCommand(const Unit *unit, const CommandType *commandType, const Vec2i &pos, int unitGroupCommandId) {
+std::pair<CommandResult,string> AiInterface::giveCommand(const Unit *unit, const CommandType *commandType, const Vec2i &pos, int unitGroupCommandId) {
 	assert(this->gameSettings != NULL);
 
+	std::pair<CommandResult,string> result(crFailUndefined,"");
 	if(unit == NULL) {
-	    char szBuf[1024]="";
-	    sprintf(szBuf,"In [%s::%s Line: %d] Can not find AI unit in AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,factionIndex);
-		throw runtime_error(szBuf);
+	    char szBuf[8096]="";
+	    snprintf(szBuf,8096,"In [%s::%s Line: %d] Can not find AI unit in AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,factionIndex);
+		throw megaglest_runtime_error(szBuf);
 	}
     const UnitType* unitType= unit->getType();
 	if(unitType == NULL) {
-	    char szBuf[1024]="";
-	    sprintf(szBuf,"In [%s::%s Line: %d] Can not find AI unittype with unit id: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unit->getId(),factionIndex);
-		throw runtime_error(szBuf);
+	    char szBuf[8096]="";
+	    snprintf(szBuf,8096,"In [%s::%s Line: %d] Can not find AI unittype with unit id: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unit->getId(),factionIndex);
+		throw megaglest_runtime_error(szBuf);
 	}
 	if(commandType == NULL) {
-	    char szBuf[1024]="";
-	    sprintf(szBuf,"In [%s::%s Line: %d] commandType == NULL, unit id: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unit->getId(),factionIndex);
-		throw runtime_error(szBuf);
+	    char szBuf[8096]="";
+	    snprintf(szBuf,8096,"In [%s::%s Line: %d] commandType == NULL, unit id: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unit->getId(),factionIndex);
+		throw megaglest_runtime_error(szBuf);
 	}
     const CommandType* ct= unit->getType()->findCommandTypeById(commandType->getId());
 	if(ct == NULL) {
-	    char szBuf[4096]="";
-	    sprintf(szBuf,"In [%s::%s Line: %d]\nCan not find AI command type for:\nunit = %d\n[%s]\n[%s]\nactual local factionIndex = %d.\nGame out of synch.",
+	    char szBuf[8096]="";
+	    snprintf(szBuf,8096,"In [%s::%s Line: %d]\nCan not find AI command type for:\nunit = %d\n[%s]\n[%s]\nactual local factionIndex = %d.\nGame out of synch.",
             __FILE__,__FUNCTION__,__LINE__,
             unit->getId(), unit->getFullName().c_str(),unit->getDesc().c_str(),
             unit->getFaction()->getIndex());
@@ -175,11 +185,11 @@ CommandResult AiInterface::giveCommand(const Unit *unit, const CommandType *comm
 
 	    std::string worldLog = world->DumpWorldToLog();
 	    std::string sError = "worldLog = " + worldLog + " " + string(szBuf);
-		throw runtime_error(sError);
+		throw megaglest_runtime_error(sError);
 	}
 
 	if(executeCommandOverNetwork() == true) {
-		CommandResult result = commander->tryGiveCommand(unit, commandType, pos,
+		result = commander->tryGiveCommand(unit, commandType, pos,
 				unit->getType(),CardinalDir::NORTH, false, NULL,unitGroupCommandId);
 		return result;
 	}
@@ -190,32 +200,33 @@ CommandResult AiInterface::giveCommand(const Unit *unit, const CommandType *comm
 		Unit *unitToCommand = faction->findUnit(unit->getId());
 		Command *cmd = new Command(commandType, pos);
 		cmd->setUnitCommandGroupId(unitGroupCommandId);
-		CommandResult result = unitToCommand->giveCommand(cmd);
+		result = unitToCommand->giveCommand(cmd);
 
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 		return result;
 	}
 }
 
-CommandResult AiInterface::giveCommand(int unitIndex, const CommandType *commandType, const Vec2i &pos, int unitGroupCommandId) {
+std::pair<CommandResult,string> AiInterface::giveCommand(int unitIndex, const CommandType *commandType, const Vec2i &pos, int unitGroupCommandId) {
 	assert(this->gameSettings != NULL);
 
+	std::pair<CommandResult,string> result(crFailUndefined,"");
 	const Unit *unit = getMyUnit(unitIndex);
 	if(unit == NULL) {
-	    char szBuf[1024]="";
-	    sprintf(szBuf,"In [%s::%s Line: %d] Can not find AI unit with index: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unitIndex,factionIndex);
-		throw runtime_error(szBuf);
+	    char szBuf[8096]="";
+	    snprintf(szBuf,8096,"In [%s::%s Line: %d] Can not find AI unit with index: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unitIndex,factionIndex);
+		throw megaglest_runtime_error(szBuf);
 	}
     const UnitType* unitType= unit->getType();
 	if(unitType == NULL) {
-	    char szBuf[1024]="";
-	    sprintf(szBuf,"In [%s::%s Line: %d] Can not find AI unittype with unit index: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unitIndex,factionIndex);
-		throw runtime_error(szBuf);
+	    char szBuf[8096]="";
+	    snprintf(szBuf,8096,"In [%s::%s Line: %d] Can not find AI unittype with unit index: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unitIndex,factionIndex);
+		throw megaglest_runtime_error(szBuf);
 	}
     const CommandType* ct= unit->getType()->findCommandTypeById(commandType->getId());
 	if(ct == NULL) {
-	    char szBuf[4096]="";
-	    sprintf(szBuf,"In [%s::%s Line: %d]\nCan not find AI command type for:\nunit = %d\n[%s]\n[%s]\nactual local factionIndex = %d.\nGame out of synch.",
+	    char szBuf[8096]="";
+	    snprintf(szBuf,8096,"In [%s::%s Line: %d]\nCan not find AI command type for:\nunit = %d\n[%s]\n[%s]\nactual local factionIndex = %d.\nGame out of synch.",
             __FILE__,__FUNCTION__,__LINE__,
             unit->getId(), unit->getFullName().c_str(),unit->getDesc().c_str(),
             unit->getFaction()->getIndex());
@@ -224,12 +235,12 @@ CommandResult AiInterface::giveCommand(int unitIndex, const CommandType *command
 
 	    std::string worldLog = world->DumpWorldToLog();
 	    std::string sError = "worldLog = " + worldLog + " " + string(szBuf);
-		throw runtime_error(sError);
+		throw megaglest_runtime_error(sError);
 	}
 
 	if(executeCommandOverNetwork() == true) {
 		const Unit *unit = getMyUnit(unitIndex);
-		CommandResult result = commander->tryGiveCommand(unit, commandType, pos, unit->getType(),CardinalDir::NORTH);
+		result = commander->tryGiveCommand(unit, commandType, pos, unit->getType(),CardinalDir::NORTH);
 		return result;
 	}
 	else {
@@ -237,32 +248,33 @@ CommandResult AiInterface::giveCommand(int unitIndex, const CommandType *command
 
 		Command *cmd = new Command(commandType, pos);
 		cmd->setUnitCommandGroupId(unitGroupCommandId);
-		CommandResult result = world->getFaction(factionIndex)->getUnit(unitIndex)->giveCommand(cmd);
+		result = world->getFaction(factionIndex)->getUnit(unitIndex)->giveCommand(cmd);
 
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 		return result;
 	}
 }
 
-CommandResult AiInterface::giveCommand(int unitIndex, const CommandType *commandType, const Vec2i &pos, const UnitType *ut) {
+std::pair<CommandResult,string> AiInterface::giveCommand(int unitIndex, const CommandType *commandType, const Vec2i &pos, const UnitType *ut) {
 	assert(this->gameSettings != NULL);
 
+	std::pair<CommandResult,string> result(crFailUndefined,"");
 	const Unit *unit = getMyUnit(unitIndex);
 	if(unit == NULL) {
-	    char szBuf[1024]="";
-	    sprintf(szBuf,"In [%s::%s Line: %d] Can not find AI unit with index: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unitIndex,factionIndex);
-		throw runtime_error(szBuf);
+	    char szBuf[8096]="";
+	    snprintf(szBuf,8096,"In [%s::%s Line: %d] Can not find AI unit with index: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unitIndex,factionIndex);
+		throw megaglest_runtime_error(szBuf);
 	}
     const UnitType* unitType= unit->getType();
 	if(unitType == NULL) {
-	    char szBuf[1024]="";
-	    sprintf(szBuf,"In [%s::%s Line: %d] Can not find AI unittype with unit index: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unitIndex,factionIndex);
-		throw runtime_error(szBuf);
+	    char szBuf[8096]="";
+	    snprintf(szBuf,8096,"In [%s::%s Line: %d] Can not find AI unittype with unit index: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unitIndex,factionIndex);
+		throw megaglest_runtime_error(szBuf);
 	}
     const CommandType* ct= unit->getType()->findCommandTypeById(commandType->getId());
 	if(ct == NULL) {
-	    char szBuf[4096]="";
-	    sprintf(szBuf,"In [%s::%s Line: %d]\nCan not find AI command type for:\nunit = %d\n[%s]\n[%s]\nactual local factionIndex = %d.\nGame out of synch.",
+	    char szBuf[8096]="";
+	    snprintf(szBuf,8096,"In [%s::%s Line: %d]\nCan not find AI command type for:\nunit = %d\n[%s]\n[%s]\nactual local factionIndex = %d.\nGame out of synch.",
             __FILE__,__FUNCTION__,__LINE__,
             unit->getId(), unit->getFullName().c_str(),unit->getDesc().c_str(),
             unit->getFaction()->getIndex());
@@ -271,18 +283,18 @@ CommandResult AiInterface::giveCommand(int unitIndex, const CommandType *command
 
 	    std::string worldLog = world->DumpWorldToLog();
 	    std::string sError = "worldLog = " + worldLog + " " + string(szBuf);
-		throw runtime_error(sError);
+		throw megaglest_runtime_error(sError);
 	}
 
 	if(executeCommandOverNetwork() == true) {
 		const Unit *unit = getMyUnit(unitIndex);
-		CommandResult result = commander->tryGiveCommand(unit, commandType, pos, ut,CardinalDir::NORTH);
+		result = commander->tryGiveCommand(unit, commandType, pos, ut,CardinalDir::NORTH);
 		return result;
 	}
 	else {
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
-		CommandResult result = world->getFaction(factionIndex)->getUnit(unitIndex)->giveCommand(new Command(commandType, pos, ut, CardinalDir::NORTH));
+		result = world->getFaction(factionIndex)->getUnit(unitIndex)->giveCommand(new Command(commandType, pos, ut, CardinalDir::NORTH));
 
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
@@ -290,26 +302,27 @@ CommandResult AiInterface::giveCommand(int unitIndex, const CommandType *command
 	}
 }
 
-CommandResult AiInterface::giveCommand(int unitIndex, const CommandType *commandType, Unit *u){
+std::pair<CommandResult,string> AiInterface::giveCommand(int unitIndex, const CommandType *commandType, Unit *u){
 	assert(this->gameSettings != NULL);
 	assert(this->commander != NULL);
 
+	std::pair<CommandResult,string> result(crFailUndefined,"");
 	const Unit *unit = getMyUnit(unitIndex);
 	if(unit == NULL) {
-	    char szBuf[1024]="";
-	    sprintf(szBuf,"In [%s::%s Line: %d] Can not find AI unit with index: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unitIndex,factionIndex);
-		throw runtime_error(szBuf);
+	    char szBuf[8096]="";
+	    snprintf(szBuf,8096,"In [%s::%s Line: %d] Can not find AI unit with index: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unitIndex,factionIndex);
+		throw megaglest_runtime_error(szBuf);
 	}
     const UnitType* unitType= unit->getType();
 	if(unitType == NULL) {
-	    char szBuf[1024]="";
-	    sprintf(szBuf,"In [%s::%s Line: %d] Can not find AI unittype with unit index: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unitIndex,factionIndex);
-		throw runtime_error(szBuf);
+	    char szBuf[8096]="";
+	    snprintf(szBuf,8096,"In [%s::%s Line: %d] Can not find AI unittype with unit index: %d, AI factionIndex = %d. Game out of synch.",__FILE__,__FUNCTION__,__LINE__,unitIndex,factionIndex);
+		throw megaglest_runtime_error(szBuf);
 	}
     const CommandType* ct= (commandType != NULL ? unit->getType()->findCommandTypeById(commandType->getId()) : NULL);
 	if(ct == NULL) {
-	    char szBuf[4096]="";
-	    sprintf(szBuf,"In [%s::%s Line: %d]\nCan not find AI command type for:\nunit = %d\n[%s]\n[%s]\nactual local factionIndex = %d.\nGame out of synch.",
+	    char szBuf[8096]="";
+	    snprintf(szBuf,8096,"In [%s::%s Line: %d]\nCan not find AI command type for:\nunit = %d\n[%s]\n[%s]\nactual local factionIndex = %d.\nGame out of synch.",
             __FILE__,__FUNCTION__,__LINE__,
             unit->getId(), unit->getFullName().c_str(),unit->getDesc().c_str(),
             unit->getFaction()->getIndex());
@@ -318,21 +331,21 @@ CommandResult AiInterface::giveCommand(int unitIndex, const CommandType *command
 
 	    std::string worldLog = world->DumpWorldToLog();
 	    std::string sError = "worldLog = " + worldLog + " " + string(szBuf);
-		throw runtime_error(sError);
+		throw megaglest_runtime_error(sError);
 	}
 
 	if(executeCommandOverNetwork() == true) {
 		Unit *targetUnit = u;
 		const Unit *unit = getMyUnit(unitIndex);
 
-		CommandResult result = commander->tryGiveCommand(unit, commandType, Vec2i(0), unit->getType(),CardinalDir::NORTH,false,targetUnit);
+		result = commander->tryGiveCommand(unit, commandType, Vec2i(0), unit->getType(),CardinalDir::NORTH,false,targetUnit);
 
 		return result;
 	}
 	else {
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
-		CommandResult result = world->getFaction(factionIndex)->getUnit(unitIndex)->giveCommand(new Command(commandType, u));
+		result = world->getFaction(factionIndex)->getUnit(unitIndex)->giveCommand(new Command(commandType, u));
 
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
@@ -390,9 +403,9 @@ const Resource *AiInterface::getResource(const ResourceType *rt){
 
 Unit *AiInterface::getMyUnitPtr(int unitIndex) {
 	if(unitIndex >= world->getFaction(factionIndex)->getUnitCount()) {
-		char szBuf[1024]="";
-		sprintf(szBuf,"In [%s::%s Line: %d] unitIndex >= world->getFaction(factionIndex)->getUnitCount(), unitIndex = %d, world->getFaction(factionIndex)->getUnitCount() = %d",__FILE__,__FUNCTION__,__LINE__,unitIndex,world->getFaction(factionIndex)->getUnitCount());
-		throw runtime_error(szBuf);
+		char szBuf[8096]="";
+		snprintf(szBuf,8096,"In [%s::%s Line: %d] unitIndex >= world->getFaction(factionIndex)->getUnitCount(), unitIndex = %d, world->getFaction(factionIndex)->getUnitCount() = %d",__FILE__,__FUNCTION__,__LINE__,unitIndex,world->getFaction(factionIndex)->getUnitCount());
+		throw megaglest_runtime_error(szBuf);
 	}
 
 	return world->getFaction(factionIndex)->getUnit(unitIndex);
@@ -484,15 +497,6 @@ bool AiInterface::isResourceNear(const Vec2i &pos, const ResourceType *rt, Vec2i
 					if(r->getType() == rt) {
 						resourcePos= pos + Vec2i(i,j);
 
-						//if(faction != NULL) {
-						//	char szBuf[4096]="";
-						//				sprintf(szBuf,"[%s::%s Line: %d] [isresourcenear] pos [%s] resourcePos [%s] fallbackToPeersHarvestingSameResource [%d] rt [%s]",
-						//						__FILE__,__FUNCTION__,__LINE__,pos.getString().c_str(),resourcePos.getString().c_str(),fallbackToPeersHarvestingSameResource,rt->getName().c_str());
-
-						//	SystemFlags::OutputDebug(SystemFlags::debugWorldSynch,"----------------------------------- START [%d] ------------------------------------------------\n",faction->getFrameCount());
-						//	SystemFlags::OutputDebug(SystemFlags::debugWorldSynch,"%s",szBuf);
-						//	SystemFlags::OutputDebug(SystemFlags::debugWorldSynch,"------------------------------------ END [%d] -------------------------------------------------\n",faction->getFrameCount());
-						//}
 						return true;
 					}
 				}
@@ -623,8 +627,8 @@ bool AiInterface::reqsOk(const CommandType *ct){
     return world->getFaction(factionIndex)->reqsOk(ct);
 }
 
-bool AiInterface::checkCosts(const ProducibleType *pt){
-	return world->getFaction(factionIndex)->checkCosts(pt);
+bool AiInterface::checkCosts(const ProducibleType *pt, const CommandType *ct) {
+	return world->getFaction(factionIndex)->checkCosts(pt,ct);
 }
 
 bool AiInterface::isFreeCells(const Vec2i &pos, int size, Field field){
@@ -667,4 +671,77 @@ bool AiInterface::factionUsesResourceType(const FactionType *factionType, const
 	return factionUsesResourceType;
 }
 
+void AiInterface::saveGame(XmlNode *rootNode) const {
+	std::map<string,string> mapTagReplacements;
+	XmlNode *aiInterfaceNode = rootNode->addChild("AiInterface");
+
+//    World *world;
+//    Commander *commander;
+//    Console *console;
+//    GameSettings *gameSettings;
+//
+//    Ai ai;
+	ai.saveGame(aiInterfaceNode);
+//    int timer;
+	aiInterfaceNode->addAttribute("timer",intToStr(timer), mapTagReplacements);
+//    int factionIndex;
+	aiInterfaceNode->addAttribute("factionIndex",intToStr(factionIndex), mapTagReplacements);
+//    int teamIndex;
+	aiInterfaceNode->addAttribute("teamIndex",intToStr(teamIndex), mapTagReplacements);
+//	//config
+//	bool redir;
+	aiInterfaceNode->addAttribute("redir",intToStr(redir), mapTagReplacements);
+//    int logLevel;
+	aiInterfaceNode->addAttribute("logLevel",intToStr(logLevel), mapTagReplacements);
+//    std::map<const ResourceType *,int> cacheUnitHarvestResourceLookup;
+	for(std::map<const ResourceType *,int>::const_iterator iterMap = cacheUnitHarvestResourceLookup.begin();
+			iterMap != cacheUnitHarvestResourceLookup.end(); ++iterMap) {
+		XmlNode *cacheUnitHarvestResourceLookupNode = aiInterfaceNode->addChild("cacheUnitHarvestResourceLookup");
+
+		cacheUnitHarvestResourceLookupNode->addAttribute("key",iterMap->first->getName(), mapTagReplacements);
+		cacheUnitHarvestResourceLookupNode->addAttribute("value",intToStr(iterMap->second), mapTagReplacements);
+	}
+}
+
+// AiInterface::AiInterface(Game &game, int factionIndex, int teamIndex, int useStartLocation) {
+void AiInterface::loadGame(const XmlNode *rootNode, Faction *faction) {
+	XmlNode *aiInterfaceNode = NULL;
+	vector<XmlNode *> aiInterfaceNodeList = rootNode->getChildList("AiInterface");
+	for(unsigned int i = 0; i < aiInterfaceNodeList.size(); ++i) {
+		XmlNode *node = aiInterfaceNodeList[i];
+		if(node->getAttribute("factionIndex")->getIntValue() == faction->getIndex()) {
+			aiInterfaceNode = node;
+			break;
+		}
+	}
+
+	if(aiInterfaceNode != NULL) {
+		factionIndex = aiInterfaceNode->getAttribute("factionIndex")->getIntValue();
+		teamIndex = aiInterfaceNode->getAttribute("teamIndex")->getIntValue();
+
+		ai.loadGame(aiInterfaceNode, faction);
+		//firstTime = timeflowNode->getAttribute("firstTime")->getFloatValue();
+
+		timer = aiInterfaceNode->getAttribute("timer")->getIntValue();
+	//    int factionIndex;
+		//factionIndex = aiInterfaceNode->getAttribute("factionIndex")->getIntValue();
+	//    int teamIndex;
+		//teamIndex = aiInterfaceNode->getAttribute("teamIndex")->getIntValue();
+	//	//config
+	//	bool redir;
+		redir = aiInterfaceNode->getAttribute("redir")->getIntValue() != 0;
+	//    int logLevel;
+		logLevel = aiInterfaceNode->getAttribute("logLevel")->getIntValue();
+
+	//    std::map<const ResourceType *,int> cacheUnitHarvestResourceLookup;
+	//	for(std::map<const ResourceType *,int>::const_iterator iterMap = cacheUnitHarvestResourceLookup.begin();
+	//			iterMap != cacheUnitHarvestResourceLookup.end(); ++iterMap) {
+	//		XmlNode *cacheUnitHarvestResourceLookupNode = aiInterfaceNode->addChild("cacheUnitHarvestResourceLookup");
+	//
+	//		cacheUnitHarvestResourceLookupNode->addAttribute("key",iterMap->first->getName(), mapTagReplacements);
+	//		cacheUnitHarvestResourceLookupNode->addAttribute("value",intToStr(iterMap->second), mapTagReplacements);
+	//	}
+	}
+}
+
 }}//end namespace
