@@ -357,6 +357,7 @@ AiRuleMassiveAttack::AiRuleMassiveAttack(Ai *ai):
 	AiRule(ai)
 {
 	ultraAttack=false;
+	field = fLand;
 }
 
 bool AiRuleMassiveAttack::test(){
@@ -400,10 +401,13 @@ void AiRuleAddTasks::execute(){
 
 	//standard tasks
 	if(ai->outputAIBehaviourToConsole()) printf("Add a TASK - AiRuleAddTasks adding ProduceTask(ucWorker) workerCount = %d, RULE Name[%s]\n",workerCount,this->getName().c_str());
+	//ai->getAiInterface()->printLog(4, "Add a TASK - AiRuleAddTasks adding ProduceTask(ucWorker) workerCount = %d, RULE Name[%s]",workerCount,this->getName().c_str());
 
 	//emergency workers
 	if(workerCount < 4){
 		if(ai->outputAIBehaviourToConsole()) printf("AAA AiRuleAddTasks adding ProduceTask(ucWorker) workerCount = %d, RULE Name[%s]\n",workerCount,this->getName().c_str());
+		//ai->getAiInterface()->printLog(4, "AAA AiRuleAddTasks adding ProduceTask(ucWorker) workerCount = %d, RULE Name[%s]",workerCount,this->getName().c_str());
+
 		ai->addPriorityTask(new ProduceTask(ucWorker));
 	}
 	else{
@@ -412,6 +416,8 @@ void AiRuleAddTasks::execute(){
 		{
 			if(ai->outputAIBehaviourToConsole()) printf("AAA AiRuleAddTasks adding #1 workerCount = %d[%.2f], buildingCount = %d[%.2f] warriorCount = %d[%.2f] upgradeCount = %d RULE Name[%s]\n",
 					workerCount,workerRatio,buildingCount,buildingRatio,warriorCount,warriorRatio,upgradeCount,this->getName().c_str());
+			//ai->getAiInterface()->printLog(4, "AAA AiRuleAddTasks adding #1 workerCount = %d[%.2f], buildingCount = %d[%.2f] warriorCount = %d[%.2f] upgradeCount = %d RULE Name[%s]",
+			//		workerCount,workerRatio,buildingCount,buildingRatio,warriorCount,warriorRatio,upgradeCount,this->getName().c_str());
 
 			//workers
 			if(workerCount<5) ai->addTask(new ProduceTask(ucWorker));
@@ -425,7 +431,7 @@ void AiRuleAddTasks::execute(){
 			if(warriorRatio<0.30) ai->addTask(new ProduceTask(ucWarrior));
 			if(workerCount>=10) ai->addTask(new ProduceTask(ucWarrior));
 			if(workerCount>=15) ai->addTask(new ProduceTask(ucWarrior));
-			if(warriorCount<ai->minWarriors+2)
+			if(warriorCount < ai->getMinWarriors() + 2)
 			{
 				ai->addTask(new ProduceTask(ucWarrior));
 				if( buildingCount>9 )
@@ -441,13 +447,13 @@ void AiRuleAddTasks::execute(){
 			}
 
 			//buildings
-			if(buildingCount<6 || buildingRatio<0.20) ai->addTask(new BuildTask());
-			if(buildingCount<10 && workerCount>12) ai->addTask(new BuildTask());
+			if(buildingCount<6 || buildingRatio<0.20) ai->addTask(new BuildTask((UnitType *)NULL));
+			if(buildingCount<10 && workerCount>12) ai->addTask(new BuildTask((UnitType *)NULL));
 			//upgrades
-			if(upgradeCount==0 && workerCount>5) ai->addTask(new UpgradeTask());
-			if(upgradeCount==1 && workerCount>10) ai->addTask(new UpgradeTask());
-			if(upgradeCount==2 && workerCount>15) ai->addTask(new UpgradeTask());
-			if(ai->isStableBase()) ai->addTask(new UpgradeTask());
+			if(upgradeCount==0 && workerCount>5) ai->addTask(new UpgradeTask((const UpgradeType *)NULL));
+			if(upgradeCount==1 && workerCount>10) ai->addTask(new UpgradeTask((const UpgradeType *)NULL));
+			if(upgradeCount==2 && workerCount>15) ai->addTask(new UpgradeTask((const UpgradeType *)NULL));
+			if(ai->isStableBase()) ai->addTask(new UpgradeTask((const UpgradeType *)NULL));
 		}
 		else if(ai->getAiInterface()->getControlType() == ctCpuEasy ||
 				ai->getAiInterface()->getControlType() == ctNetworkCpuEasy)
@@ -455,6 +461,8 @@ void AiRuleAddTasks::execute(){
 
 			if(ai->outputAIBehaviourToConsole()) printf("AAA AiRuleAddTasks adding #2 workerCount = %d[%.2f], buildingCount = %d[%.2f] warriorCount = %d[%.2f] upgradeCount = %d RULE Name[%s]\n",
 					workerCount,workerRatio,buildingCount,buildingRatio,warriorCount,warriorRatio,upgradeCount,this->getName().c_str());
+			//ai->getAiInterface()->printLog(4, "AAA AiRuleAddTasks adding #2 workerCount = %d[%.2f], buildingCount = %d[%.2f] warriorCount = %d[%.2f] upgradeCount = %d RULE Name[%s]\n",
+			//		workerCount,workerRatio,buildingCount,buildingRatio,warriorCount,warriorRatio,upgradeCount,this->getName().c_str());
 
 			//workers
 			if(workerCount<buildingCount+2) ai->addTask(new ProduceTask(ucWorker));
@@ -468,19 +476,21 @@ void AiRuleAddTasks::execute(){
 			if(workerCount>=15) ai->addTask(new ProduceTask(ucWarrior));
 
 			//buildings
-			if(buildingCount<6 || buildingRatio<0.20) ai->addTask(new BuildTask());
-			if(buildingCount<10 && ai->isStableBase()) ai->addTask(new BuildTask());
+			if(buildingCount<6 || buildingRatio<0.20) ai->addTask(new BuildTask((UnitType *)NULL));
+			if(buildingCount<10 && ai->isStableBase()) ai->addTask(new BuildTask((UnitType *)NULL));
 
 			//upgrades
-			if(upgradeCount==0 && workerCount>6) ai->addTask(new UpgradeTask());
-			if(upgradeCount==1 && workerCount>7) ai->addTask(new UpgradeTask());
-			if(upgradeCount==2 && workerCount>9) ai->addTask(new UpgradeTask());
+			if(upgradeCount==0 && workerCount>6) ai->addTask(new UpgradeTask((const UpgradeType *)NULL));
+			if(upgradeCount==1 && workerCount>7) ai->addTask(new UpgradeTask((const UpgradeType *)NULL));
+			if(upgradeCount==2 && workerCount>9) ai->addTask(new UpgradeTask((const UpgradeType *)NULL));
 			//if(ai->isStableBase()) ai->addTask(new UpgradeTask());
 		}
 		else
 		{// normal CPU / UltraCPU ...
 			if(ai->outputAIBehaviourToConsole()) printf("AAA AiRuleAddTasks adding #3 workerCount = %d[%.2f], buildingCount = %d[%.2f] warriorCount = %d[%.2f] upgradeCount = %d RULE Name[%s]\n",
 					workerCount,workerRatio,buildingCount,buildingRatio,warriorCount,warriorRatio,upgradeCount,this->getName().c_str());
+			//ai->getAiInterface()->printLog(4, "AAA AiRuleAddTasks adding #3 workerCount = %d[%.2f], buildingCount = %d[%.2f] warriorCount = %d[%.2f] upgradeCount = %d RULE Name[%s]\n",
+			//		workerCount,workerRatio,buildingCount,buildingRatio,warriorCount,warriorRatio,upgradeCount,this->getName().c_str());
 
 			//workers
 			if(workerCount<5) ai->addTask(new ProduceTask(ucWorker));
@@ -496,14 +506,14 @@ void AiRuleAddTasks::execute(){
 			if(workerCount>=15) ai->addTask(new ProduceTask(ucWarrior));
 
 			//buildings
-			if(buildingCount<6 || buildingRatio<0.20) ai->addTask(new BuildTask());
-			if(buildingCount<10 && workerCount>12) ai->addTask(new BuildTask());
+			if(buildingCount<6 || buildingRatio<0.20) ai->addTask(new BuildTask((UnitType *)NULL));
+			if(buildingCount<10 && workerCount>12) ai->addTask(new BuildTask((UnitType *)NULL));
 
 			//upgrades
-			if(upgradeCount==0 && workerCount>5) ai->addTask(new UpgradeTask());
-			if(upgradeCount==1 && workerCount>10) ai->addTask(new UpgradeTask());
-			if(upgradeCount==2 && workerCount>15) ai->addTask(new UpgradeTask());
-			if(ai->isStableBase()) ai->addTask(new UpgradeTask());
+			if(upgradeCount==0 && workerCount>5) ai->addTask(new UpgradeTask((const UpgradeType *)NULL));
+			if(upgradeCount==1 && workerCount>10) ai->addTask(new UpgradeTask((const UpgradeType *)NULL));
+			if(upgradeCount==2 && workerCount>15) ai->addTask(new UpgradeTask((const UpgradeType *)NULL));
+			if(ai->isStableBase()) ai->addTask(new UpgradeTask((const UpgradeType *)NULL));
 		}
 	}
 }
@@ -542,6 +552,8 @@ bool AiRuleBuildOneFarm::test(){
 							//printf("AiRuleBuildOneFarm returning true, RULE Name[%s] ut [%s] producedType [%s]\n",this->getName().c_str(),ut->getName().c_str(),producedType->getName().c_str());
 
 							if(ai->outputAIBehaviourToConsole()) printf("AiRuleBuildOneFarm returning true, RULE Name[%s]\n",this->getName().c_str());
+							//aiInterface->printLog(4, "AiRuleBuildOneFarm returning true, RULE Name[%s]\n",this->getName().c_str());
+
 							return true;
 						}
 					}
@@ -577,6 +589,10 @@ bool AiRuleProduceResourceProducer::test(){
 		const Resource *r= aiInterface->getResource(rt);
 
 		if(ai->outputAIBehaviourToConsole()) printf("CONSUMABLE [%s][%d] Testing AI RULE Name[%s]\n",rt->getName().c_str(), r->getBalance(), this->getName().c_str());
+		//aiInterface->printLog(4, "CONSUMABLE [%s][%d] Testing AI RULE Name[%s]",rt->getName().c_str(), r->getBalance(), this->getName().c_str());
+		char szBuf[8096]="";
+		snprintf(szBuf,8096,"CONSUMABLE [%s][%d] Testing AI RULE Name[%s]",rt->getName().c_str(), r->getBalance(), this->getName().c_str());
+		aiInterface->printLog(4, szBuf);
 
 		bool factionUsesResourceType = aiInterface->factionUsesResourceType(aiInterface->getMyFactionType(), rt);
 		//if(rt->getClass()==rcConsumable && r->getBalance()<0){
@@ -587,14 +603,22 @@ bool AiRuleProduceResourceProducer::test(){
         }
     }
 
+	int targetStaticResourceCount = minStaticResources;
+	if(aiInterface->getMyFactionType()->getAIBehaviorStaticOverideValue(aibsvcMinStaticResourceCount) != INT_MAX) {
+		targetStaticResourceCount = aiInterface->getMyFactionType()->getAIBehaviorStaticOverideValue(aibsvcMinStaticResourceCount);
+	}
+
 	//statics second
 	for(int i=0; i < aiInterface->getTechTree()->getResourceTypeCount(); ++i) {
         rt= aiInterface->getTechTree()->getResourceType(i);
 		const Resource *r= aiInterface->getResource(rt);
 
-		if(ai->outputAIBehaviourToConsole()) printf("STATIC [%s][%d] [min %d] Testing AI RULE Name[%s]\n",rt->getName().c_str(), r->getAmount(), minStaticResources, this->getName().c_str());
+		if(ai->outputAIBehaviourToConsole()) printf("STATIC [%s][%d] [min %d] Testing AI RULE Name[%s]\n",rt->getName().c_str(), r->getAmount(), targetStaticResourceCount, this->getName().c_str());
+		char szBuf[8096]="";
+		snprintf(szBuf,8096,"STATIC resource check [%s][%d] [min %d] Testing AI RULE Name[%s]",rt->getName().c_str(), r->getAmount(), targetStaticResourceCount, this->getName().c_str());
+		aiInterface->printLog(4, szBuf);
 
-		if(rt->getClass() == rcStatic && r->getAmount() < minStaticResources) {
+		if(rt->getClass() == rcStatic && r->getAmount() < targetStaticResourceCount) {
 			bool factionUsesResourceType = aiInterface->factionUsesResourceType(aiInterface->getMyFactionType(), rt);
 			if(factionUsesResourceType == true) {
 				interval= longInterval;
@@ -604,6 +628,7 @@ bool AiRuleProduceResourceProducer::test(){
     }
 
 	if(ai->outputAIBehaviourToConsole()) printf("STATIC returning FALSE\n");
+	aiInterface->printLog(4, "Static Resource check returning FALSE");
 
 	interval= shortInterval;
 	return false;
@@ -635,18 +660,23 @@ bool AiRuleProduce::test(){
 	return true;
 }
 
-void AiRuleProduce::execute(){
-	if(produceTask!=NULL){
+void AiRuleProduce::execute() {
+	AiInterface *aiInterface= ai->getAiInterface();
+	if(produceTask!=NULL) {
 
 		if(ai->outputAIBehaviourToConsole()) printf("AiRuleProduce producing [%s]\n",(produceTask->getUnitType() != NULL ? produceTask->getUnitType()->getName().c_str() : "null"));
+		//aiInterface->printLog(4, "AiRuleProduce producing [%s]",(produceTask->getUnitType() != NULL ? produceTask->getUnitType()->getName().c_str() : "null"));
+		char szBuf[8096]="";
+		snprintf(szBuf,8096,"AiRuleProduce producing [%s]",(produceTask->getUnitType() != NULL ? produceTask->getUnitType()->getName().c_str() : "null"));
+		aiInterface->printLog(4, szBuf);
 
 		//generic produce task, produce random unit that has the skill or produces the resource
-		if(produceTask->getUnitType()==NULL){
+		if(produceTask->getUnitType() == NULL) {
 			produceGeneric(produceTask);
 		}
 
 		//specific produce task, produce if possible, retry if not enough resources
-		else{
+		else {
 			produceSpecific(produceTask);
 		}
 
@@ -721,6 +751,7 @@ void AiRuleProduce::produceGeneric(const ProduceTask *pt) {
 				bool produceIt= false;
 
 				if(ai->outputAIBehaviourToConsole()) printf("produceGeneric [%p] Testing AI RULE Name[%s]\n",pt->getResourceType(), this->getName().c_str());
+				//aiInterface->printLog(4, "produceGeneric [%p] Testing AI RULE Name[%s]",pt->getResourceType(), this->getName().c_str());
 
 				//if the unit produces the resource
 				if(pt->getResourceType() != NULL) {
@@ -728,6 +759,7 @@ void AiRuleProduce::produceGeneric(const ProduceTask *pt) {
 
 					if(r != NULL) {
 						if(ai->outputAIBehaviourToConsole()) printf("produceGeneric r = [%s][%d] Testing AI RULE Name[%s]\n",r->getDescription().c_str(),r->getAmount(), this->getName().c_str());
+						//aiInterface->printLog(4, "produceGeneric r = [%s][%d] Testing AI RULE Name[%s]",r->getDescription().c_str(),r->getAmount(), this->getName().c_str());
 					}
 
 					if(r != NULL && r->getAmount() < 0) {
@@ -738,6 +770,7 @@ void AiRuleProduce::produceGeneric(const ProduceTask *pt) {
 				else {
 					//if the unit is from the right class
 					if(ai->outputAIBehaviourToConsole()) printf("produceGeneric right class = [%d] Testing AI RULE Name[%s]\n",producedUnit->isOfClass(pt->getUnitClass()), this->getName().c_str());
+					//aiInterface->printLog(4, "produceGeneric right class = [%d] Testing AI RULE Name[%s]",producedUnit->isOfClass(pt->getUnitClass()), this->getName().c_str());
 
 					if(producedUnit->isOfClass(pt->getUnitClass())){
 						if(aiInterface->reqsOk(ct) && aiInterface->reqsOk(producedUnit)){
@@ -760,6 +793,7 @@ void AiRuleProduce::produceGeneric(const ProduceTask *pt) {
 	if(ableUnits.empty() == false) {
 
 		if(ai->outputAIBehaviourToConsole()) printf("produceGeneric !ableUnits.empty(), ableUnits.size() = [%d] Testing AI RULE Name[%s]\n",(int)ableUnits.size(), this->getName().c_str());
+		//aiInterface->printLog(4, "produceGeneric !ableUnits.empty(), ableUnits.size() = [%d] Testing AI RULE Name[%s]",(int)ableUnits.size(), this->getName().c_str());
 
 		//priority for non produced units
 		for(unsigned int i=0; i < ableUnits.size(); ++i) {
@@ -780,15 +814,60 @@ void AiRuleProduce::produceSpecific(const ProduceTask *pt){
 
 	AiInterface *aiInterface= ai->getAiInterface();
 
-	if(ai->outputAIBehaviourToConsole()) printf("produceSpecific aiInterface->reqsOk(pt->getUnitType()) = [%d] Testing AI RULE Name[%s]\n",aiInterface->reqsOk(pt->getUnitType()), this->getName().c_str());
+	if(ai->outputAIBehaviourToConsole()) printf("produceSpecific aiInterface->reqsOk(pt->getUnitType()) = [%s][%d] Testing AI RULE Name[%s]\n",pt->getUnitType()->getName().c_str(),aiInterface->reqsOk(pt->getUnitType()), this->getName().c_str());
+	char szBuf[8096]="";
+	snprintf(szBuf,8096,"produceSpecific aiInterface->reqsOk(pt->getUnitType()) = [%s][%d] Testing AI RULE Name[%s]",pt->getUnitType()->getName().c_str(),aiInterface->reqsOk(pt->getUnitType()), this->getName().c_str());
+	aiInterface->printLog(4, szBuf);
 
 	//if unit meets requirements
-	if(aiInterface->reqsOk(pt->getUnitType())){
+	if(aiInterface->reqsOk(pt->getUnitType())) {
 
-		if(ai->outputAIBehaviourToConsole()) printf("produceSpecific aiInterface->checkCosts(pt->getUnitType()) = [%d] Testing AI RULE Name[%s]\n",aiInterface->checkCosts(pt->getUnitType()), this->getName().c_str());
+		const CommandType *ctypeForCostCheck = NULL;
+		//for each unit
+		for(int i=0; i<aiInterface->getMyUnitCount(); ++i){
+
+			//for each command
+			const UnitType *ut= aiInterface->getMyUnit(i)->getType();
+			for(int j=0; j<ut->getCommandTypeCount(); ++j){
+				const CommandType *ct= ut->getCommandType(j);
+
+				//if the command is produce
+				if(ct->getClass()==ccProduce || ct->getClass()==ccMorph){
+					const UnitType *producedUnit= static_cast<const UnitType*>(ct->getProduced());
+
+					//if units match
+					if(producedUnit == pt->getUnitType()){
+
+						if(ai->outputAIBehaviourToConsole()) printf("produceSpecific aiInterface->reqsOk(ct) = [%d] Testing AI RULE Name[%s]\n",aiInterface->reqsOk(ct), this->getName().c_str());
+						//aiInterface->printLog(4, "produceSpecific aiInterface->reqsOk(ct) = [%d] Testing AI RULE Name[%s]",aiInterface->reqsOk(ct), this->getName().c_str());
+
+						if(aiInterface->reqsOk(ct)){
+							if(ctypeForCostCheck == NULL || ct->getClass() == ccMorph) {
+								if(ctypeForCostCheck != NULL && ct->getClass() == ccMorph) {
+									const MorphCommandType *mct = dynamic_cast<const MorphCommandType *>(ct);
+									if(mct->getIgnoreResourceRequirements() == true) {
+										ctypeForCostCheck= ct;
+									}
+								}
+								else {
+									ctypeForCostCheck= ct;
+								}
+							}
+						}
+					}
+				}
+			}
+		}
+
+		if(ai->outputAIBehaviourToConsole()) printf("produceSpecific aiInterface->checkCosts(pt->getUnitType()) = [%d] Testing AI RULE Name[%s]\n",aiInterface->checkCosts(pt->getUnitType(),ctypeForCostCheck), this->getName().c_str());
+		snprintf(szBuf,8096,"produceSpecific aiInterface->checkCosts(pt->getUnitType()) = [%d] Testing AI RULE Name[%s]",aiInterface->checkCosts(pt->getUnitType(),ctypeForCostCheck), this->getName().c_str());
+		aiInterface->printLog(4, szBuf);
 
 		//if unit doesnt meet resources retry
-		if(!aiInterface->checkCosts(pt->getUnitType())){
+		if(aiInterface->checkCosts(pt->getUnitType(),ctypeForCostCheck) == false) {
+			snprintf(szBuf,8096,"Check costs FAILED.");
+			aiInterface->printLog(4, szBuf);
+
 			ai->retryTask(pt);
 			return;
 		}
@@ -816,6 +895,7 @@ void AiRuleProduce::produceSpecific(const ProduceTask *pt){
 					if(producedUnit == pt->getUnitType()){
 
 						if(ai->outputAIBehaviourToConsole()) printf("produceSpecific aiInterface->reqsOk(ct) = [%d] Testing AI RULE Name[%s]\n",aiInterface->reqsOk(ct), this->getName().c_str());
+						//aiInterface->printLog(4, "produceSpecific aiInterface->reqsOk(ct) = [%d] Testing AI RULE Name[%s]",aiInterface->reqsOk(ct), this->getName().c_str());
 
 						if(aiInterface->reqsOk(ct)){
 							defCt= ct;
@@ -828,9 +908,32 @@ void AiRuleProduce::produceSpecific(const ProduceTask *pt){
 		}
 
 		//produce from random producer
-		if(!producers.empty()){
+		if(producers.empty() == false) {
 
 			if(ai->outputAIBehaviourToConsole()) printf("produceSpecific producers.empty() = [%d] Testing AI RULE Name[%s]\n",producers.empty(), this->getName().c_str());
+			//aiInterface->printLog(4, "produceSpecific producers.empty() = [%d] Testing AI RULE Name[%s]",producers.empty(), this->getName().c_str());
+			snprintf(szBuf,8096,"produceSpecific producers.empty() = [%d] Testing AI RULE Name[%s]",producers.empty(), this->getName().c_str());
+			aiInterface->printLog(4, szBuf);
+
+			// Narrow down producers list to those who are not busy if possible
+			vector<int> idle_producers;
+			for(unsigned int i = 0; i < producers.size(); ++i) {
+				int currentProducerIndex = producers[i];
+				if(currentProducerIndex >= aiInterface->getMyUnitCount()) {
+					char szBuf[8096]="";
+					printf("In [%s::%s Line: %d] currentProducerIndex >= aiInterface->getMyUnitCount(), currentProducerIndex = %d, aiInterface->getMyUnitCount() = %d, i = %u,producers.size() = " MG_SIZE_T_SPECIFIER "\n",__FILE__,__FUNCTION__,__LINE__,currentProducerIndex,aiInterface->getMyUnitCount(),i,producers.size());
+					snprintf(szBuf,8096,"In [%s::%s Line: %d] currentProducerIndex >= aiInterface->getMyUnitCount(), currentProducerIndex = %d, aiInterface->getMyUnitCount() = %u, i = %u,producers.size() = " MG_SIZE_T_SPECIFIER "",__FILE__,__FUNCTION__,__LINE__,currentProducerIndex,aiInterface->getMyUnitCount(),i,producers.size());
+					throw megaglest_runtime_error(szBuf);
+				}
+
+				const Unit *unit = aiInterface->getMyUnit(currentProducerIndex);
+				if(unit->anyCommand() == false) {
+					idle_producers.push_back(currentProducerIndex);
+				}
+			}
+			if(idle_producers.empty() == false) {
+				producers = idle_producers;
+			}
 
 			if(	aiInterface->getControlType() == ctCpuMega ||
 				aiInterface->getControlType() == ctNetworkCpuMega)
@@ -849,16 +952,16 @@ void AiRuleProduce::produceSpecific(const ProduceTask *pt){
 					currentProducerIndex=producers[prIndex];
 
 					if(currentProducerIndex >= aiInterface->getMyUnitCount()) {
-						char szBuf[1024]="";
-						printf("In [%s::%s Line: %d] currentProducerIndex >= aiInterface->getMyUnitCount(), currentProducerIndex = %d, aiInterface->getMyUnitCount() = %d, i = %d,producers.size() = %lu\n",__FILE__,__FUNCTION__,__LINE__,currentProducerIndex,aiInterface->getMyUnitCount(),i,(unsigned long)producers.size());
-						sprintf(szBuf,"In [%s::%s Line: %d] currentProducerIndex >= aiInterface->getMyUnitCount(), currentProducerIndex = %d, aiInterface->getMyUnitCount() = %d, i = %d,producers.size() = %lu",__FILE__,__FUNCTION__,__LINE__,currentProducerIndex,aiInterface->getMyUnitCount(),i,(unsigned long)producers.size());
-						throw runtime_error(szBuf);
+						char szBuf[8096]="";
+						printf("In [%s::%s Line: %d] currentProducerIndex >= aiInterface->getMyUnitCount(), currentProducerIndex = %d, aiInterface->getMyUnitCount() = %d, i = %u,producers.size() = " MG_SIZE_T_SPECIFIER "\n",__FILE__,__FUNCTION__,__LINE__,currentProducerIndex,aiInterface->getMyUnitCount(),i,producers.size());
+						snprintf(szBuf,8096,"In [%s::%s Line: %d] currentProducerIndex >= aiInterface->getMyUnitCount(), currentProducerIndex = %d, aiInterface->getMyUnitCount() = %u, i = %u,producers.size() = " MG_SIZE_T_SPECIFIER "",__FILE__,__FUNCTION__,__LINE__,currentProducerIndex,aiInterface->getMyUnitCount(),i,producers.size());
+						throw megaglest_runtime_error(szBuf);
 					}
 					if(prIndex >= producers.size()) {
-						char szBuf[1024]="";
-						printf("In [%s::%s Line: %d] prIndex >= producers.size(), currentProducerIndex = %d, i = %d,producers.size() = %lu \n",__FILE__,__FUNCTION__,__LINE__,prIndex,i,(unsigned long)producers.size());
-						sprintf(szBuf,"In [%s::%s Line: %d] currentProducerIndex >= producers.size(), currentProducerIndex = %d, i = %d,producers.size() = %lu",__FILE__,__FUNCTION__,__LINE__,currentProducerIndex,i,(unsigned long)producers.size());
-						throw runtime_error(szBuf);
+						char szBuf[8096]="";
+						printf("In [%s::%s Line: %d] prIndex >= producers.size(), currentProducerIndex = %d, i = %u,producers.size() = " MG_SIZE_T_SPECIFIER " \n",__FILE__,__FUNCTION__,__LINE__,prIndex,i,producers.size());
+						snprintf(szBuf,8096,"In [%s::%s Line: %d] currentProducerIndex >= producers.size(), currentProducerIndex = %d, i = %u,producers.size() = " MG_SIZE_T_SPECIFIER "",__FILE__,__FUNCTION__,__LINE__,currentProducerIndex,i,producers.size());
+						throw megaglest_runtime_error(szBuf);
 					}
 
 					currentCommandCount=aiInterface->getMyUnit(currentProducerIndex)->getCommandSize();
@@ -877,8 +980,9 @@ void AiRuleProduce::produceSpecific(const ProduceTask *pt){
 				if(	aiInterface->getMyUnit(bestIndex)->getCommandSize() > 2) {
 					// maybe we need another producer of this kind if possible!
 					if(aiInterface->reqsOk(aiInterface->getMyUnit(bestIndex)->getType())) {
-						if(ai->getCountOfClass(ucBuilding) > 5)
+						if(ai->getCountOfClass(ucBuilding) > 5) {
 							ai->addTask(new BuildTask(aiInterface->getMyUnit(bestIndex)->getType()));
+						}
 					}
 					// need to calculate another producer, maybe its better to produce another warrior with another producer
 					vector<int> backupProducers;
@@ -914,16 +1018,16 @@ void AiRuleProduce::produceSpecific(const ProduceTask *pt){
 							currentProducerIndex=backupProducers[prIndex];
 
 							if(currentProducerIndex >= aiInterface->getMyUnitCount()) {
-								char szBuf[1024]="";
-								printf("In [%s::%s Line: %d] currentProducerIndex >= aiInterface->getMyUnitCount(), currentProducerIndex = %d, aiInterface->getMyUnitCount() = %d, i = %d,backupProducers.size() = %lu\n",__FILE__,__FUNCTION__,__LINE__,currentProducerIndex,aiInterface->getMyUnitCount(),i,(unsigned long)backupProducers.size());
-								sprintf(szBuf,"In [%s::%s Line: %d] currentProducerIndex >= aiInterface->getMyUnitCount(), currentProducerIndex = %d, aiInterface->getMyUnitCount() = %d, i = %d,backupProducers.size() = %lu",__FILE__,__FUNCTION__,__LINE__,currentProducerIndex,aiInterface->getMyUnitCount(),i,(unsigned long)backupProducers.size());
-								throw runtime_error(szBuf);
+								char szBuf[8096]="";
+								printf("In [%s::%s Line: %d] currentProducerIndex >= aiInterface->getMyUnitCount(), currentProducerIndex = %d, aiInterface->getMyUnitCount() = %d, i = %u,backupProducers.size() = " MG_SIZE_T_SPECIFIER "\n",__FILE__,__FUNCTION__,__LINE__,currentProducerIndex,aiInterface->getMyUnitCount(),i,backupProducers.size());
+								snprintf(szBuf,8096,"In [%s::%s Line: %d] currentProducerIndex >= aiInterface->getMyUnitCount(), currentProducerIndex = %d, aiInterface->getMyUnitCount() = %d, i = %u,backupProducers.size() = " MG_SIZE_T_SPECIFIER "",__FILE__,__FUNCTION__,__LINE__,currentProducerIndex,aiInterface->getMyUnitCount(),i,backupProducers.size());
+								throw megaglest_runtime_error(szBuf);
 							}
 							if(prIndex >= backupProducers.size()) {
-								char szBuf[1024]="";
-								printf("In [%s::%s Line: %d] prIndex >= backupProducers.size(), currentProducerIndex = %d, i = %d,backupProducers.size() = %lu \n",__FILE__,__FUNCTION__,__LINE__,prIndex,i,(unsigned long)backupProducers.size());
-								sprintf(szBuf,"In [%s::%s Line: %d] currentProducerIndex >= backupProducers.size(), currentProducerIndex = %d, i = %d,backupProducers.size() = %lu",__FILE__,__FUNCTION__,__LINE__,currentProducerIndex,i,(unsigned long)backupProducers.size());
-								throw runtime_error(szBuf);
+								char szBuf[8096]="";
+								printf("In [%s::%s Line: %d] prIndex >= backupProducers.size(), currentProducerIndex = %d, i = %u,backupProducers.size() = " MG_SIZE_T_SPECIFIER " \n",__FILE__,__FUNCTION__,__LINE__,prIndex,i,backupProducers.size());
+								snprintf(szBuf,8096,"In [%s::%s Line: %d] currentProducerIndex >= backupProducers.size(), currentProducerIndex = %d, i = %u,backupProducers.size() = " MG_SIZE_T_SPECIFIER "",__FILE__,__FUNCTION__,__LINE__,currentProducerIndex,i,backupProducers.size());
+								throw megaglest_runtime_error(szBuf);
 							}
 
 							currentCommandCount=aiInterface->getMyUnit(currentProducerIndex)->getCommandSize();
@@ -957,6 +1061,10 @@ void AiRuleProduce::produceSpecific(const ProduceTask *pt){
 						if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
 						if(ai->outputAIBehaviourToConsole()) printf("mega #1 produceSpecific giveCommand to unit [%s] commandType [%s]\n",aiInterface->getMyUnit(bestIndex)->getType()->getName().c_str(),ut->getCommandType(commandIndex)->getName().c_str());
+						//aiInterface->printLog(4, "mega #1 produceSpecific giveCommand to unit [%s] commandType [%s]",aiInterface->getMyUnit(bestIndex)->getType()->getName().c_str(),ut->getCommandType(commandIndex)->getName().c_str());
+						snprintf(szBuf,8096,"mega #1 produceSpecific giveCommand to unit [%s] commandType [%s]",aiInterface->getMyUnit(bestIndex)->getType()->getName().c_str(),ut->getCommandType(commandIndex)->getName().c_str());
+						aiInterface->printLog(4, szBuf);
+
 						aiInterface->giveCommand(bestIndex, ut->getCommandType(commandIndex));
 					}
 					else
@@ -973,6 +1081,10 @@ void AiRuleProduce::produceSpecific(const ProduceTask *pt){
 						}
 
 						if(ai->outputAIBehaviourToConsole()) printf("mega #2 produceSpecific giveCommand to unit [%s] commandType [%s]\n",aiInterface->getMyUnit(bestIndex)->getType()->getName().c_str(),defCt->getName().c_str());
+						//aiInterface->printLog(4, "mega #2 produceSpecific giveCommand to unit [%s] commandType [%s]",aiInterface->getMyUnit(bestIndex)->getType()->getName().c_str(),defCt->getName().c_str());
+						snprintf(szBuf,8096,"mega #2 produceSpecific giveCommand to unit [%s] commandType [%s]",aiInterface->getMyUnit(bestIndex)->getType()->getName().c_str(),defCt->getName().c_str());
+						aiInterface->printLog(4, szBuf);
+
 						aiInterface->giveCommand(bestIndex, defCt);
 					}
 				}
@@ -992,6 +1104,10 @@ void AiRuleProduce::produceSpecific(const ProduceTask *pt){
 						}
 
 						if(ai->outputAIBehaviourToConsole()) printf("mega #3 produceSpecific giveCommand to unit [%s] commandType [%s]\n",aiInterface->getMyUnit(bestIndex)->getType()->getName().c_str(),defCt->getName().c_str());
+						//aiInterface->printLog(4, "mega #3 produceSpecific giveCommand to unit [%s] commandType [%s]",aiInterface->getMyUnit(bestIndex)->getType()->getName().c_str(),defCt->getName().c_str());
+						snprintf(szBuf,8096,"mega #3 produceSpecific giveCommand to unit [%s] commandType [%s]",aiInterface->getMyUnit(bestIndex)->getType()->getName().c_str(),defCt->getName().c_str());
+						aiInterface->printLog(4, szBuf);
+
 						aiInterface->giveCommand(bestIndex, defCt);
 					}
 					if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
@@ -1006,6 +1122,10 @@ void AiRuleProduce::produceSpecific(const ProduceTask *pt){
 						defCt = producersDefaultCommandType[bestIndex][bestCommandTypeIndex];
 					}
 					if(ai->outputAIBehaviourToConsole()) printf("mega #4 produceSpecific giveCommand to unit [%s] commandType [%s]\n",aiInterface->getMyUnit(bestIndex)->getType()->getName().c_str(),defCt->getName().c_str());
+					//aiInterface->printLog(4, "mega #4 produceSpecific giveCommand to unit [%s] commandType [%s]",aiInterface->getMyUnit(bestIndex)->getType()->getName().c_str(),defCt->getName().c_str());
+					snprintf(szBuf,8096,"mega #4 produceSpecific giveCommand to unit [%s] commandType [%s]",aiInterface->getMyUnit(bestIndex)->getType()->getName().c_str(),defCt->getName().c_str());
+					aiInterface->printLog(4, szBuf);
+
 					aiInterface->giveCommand(bestIndex, defCt);
 				}
 			}
@@ -1026,6 +1146,10 @@ void AiRuleProduce::produceSpecific(const ProduceTask *pt){
 				if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] producers.size() = %d, producerIndex = %d, pIndex = %d, producersDefaultCommandType.size() = %d\n",__FILE__,__FUNCTION__,__LINE__,producers.size(),producerIndex,pIndex,producersDefaultCommandType.size());
 
 				if(ai->outputAIBehaviourToConsole()) printf("produceSpecific giveCommand to unit [%s] commandType [%s]\n",aiInterface->getMyUnit(producerIndex)->getType()->getName().c_str(),defCt->getName().c_str());
+				//aiInterface->printLog(4, "produceSpecific giveCommand to unit [%s] commandType [%s]",aiInterface->getMyUnit(producerIndex)->getType()->getName().c_str(),defCt->getName().c_str());
+				snprintf(szBuf,8096,"produceSpecific giveCommand to unit [%s] commandType [%s]",aiInterface->getMyUnit(producerIndex)->getType()->getName().c_str(),defCt->getName().c_str());
+				aiInterface->printLog(4, szBuf);
+
 				aiInterface->giveCommand(producerIndex, defCt);
 			}
 		}
@@ -1057,6 +1181,7 @@ bool AiRuleBuild::test(){
 void AiRuleBuild::execute() {
 	if(buildTask!=NULL) {
 		if(ai->outputAIBehaviourToConsole()) printf("BUILD AiRuleBuild Unit Name[%s]\n",(buildTask->getUnitType() != NULL ? buildTask->getUnitType()->getName().c_str() : "null"));
+		//aiInterface->printLog(4, "BUILD AiRuleBuild Unit Name[%s]",(buildTask->getUnitType() != NULL ? buildTask->getUnitType()->getName().c_str() : "null"));
 
 		//generic build task, build random building that can be built
 		if(buildTask->getUnitType() == NULL) {
@@ -1203,7 +1328,7 @@ void AiRuleBuild::buildSpecific(const BuildTask *bt) {
 	if(aiInterface->reqsOk(bt->getUnitType())) {
 
 		//retry if not enough resources
-		if(aiInterface->checkCosts(bt->getUnitType()) == false) {
+		if(aiInterface->checkCosts(bt->getUnitType(),NULL) == false) {
 			ai->retryTask(bt);
 			return;
 		}
@@ -1267,7 +1392,7 @@ void AiRuleBuild::buildSpecific(const BuildTask *bt) {
 							if(ai->getAiInterface()->isFreeCells(tryPos - Vec2i(spacing), bt->getUnitType()->getSize() + spacing * 2, fLand)) {
 								enemies.clear();
 								ai->getAiInterface()->getWorld()->getUnitUpdater()->findEnemiesForCell(tryPos,bt->getUnitType()->getSize(),enemySightDistanceToAvoid,ai->getAiInterface()->getMyFaction(),enemies,true);
-								if(enemies.size() <= 0) {
+								if(enemies.empty() == true) {
 									searchPos = tryPos;
 								}
 							}
@@ -1283,7 +1408,7 @@ void AiRuleBuild::buildSpecific(const BuildTask *bt) {
 							if(ai->getAiInterface()->isFreeCells(tryPos - Vec2i(spacing), bt->getUnitType()->getSize() + spacing * 2, fLand)) {
 								enemies.clear();
 								ai->getAiInterface()->getWorld()->getUnitUpdater()->findEnemiesForCell(tryPos,bt->getUnitType()->getSize(),enemySightDistanceToAvoid,ai->getAiInterface()->getMyFaction(),enemies,true);
-								if(enemies.size() <= 0) {
+								if(enemies.empty() == true) {
 									searchPos = tryPos;
 								}
 							}
@@ -1318,6 +1443,7 @@ void AiRuleBuild::buildSpecific(const BuildTask *bt) {
 
 bool AiRuleBuild::isDefensive(const UnitType *building){
 	if(ai->outputAIBehaviourToConsole()) printf("BUILD isDefensive check for Unit Name[%s] result = %d\n",building->getName().c_str(),building->hasSkillClass(scAttack));
+	//aiInterface->printLog(4, "BUILD isDefensive check for Unit Name[%s] result = %d",building->getName().c_str(),building->hasSkillClass(scAttack));
 
 	return building->hasSkillClass(scAttack);
 }
@@ -1326,10 +1452,13 @@ bool AiRuleBuild::isResourceProducer(const UnitType *building){
 	for(int i= 0; i<building->getCostCount(); i++){
 		if(building->getCost(i)->getAmount()<0){
 			if(ai->outputAIBehaviourToConsole()) printf("BUILD isResourceProducer check for Unit Name[%s] result = true\n",building->getName().c_str());
+			//aiInterface->printLog(4, "BUILD isResourceProducer check for Unit Name[%s] result = true",building->getName().c_str());
 			return true;
 		}
 	}
 	if(ai->outputAIBehaviourToConsole()) printf("BUILD isResourceProducer check for Unit Name[%s] result = false\n",building->getName().c_str());
+	//aiInterface->printLog(4, "BUILD isResourceProducer check for Unit Name[%s] result = false",building->getName().c_str());
+
 	return false;
 }
 
@@ -1341,11 +1470,14 @@ bool AiRuleBuild::isWarriorProducer(const UnitType *building){
 
 			if(ut->isOfClass(ucWarrior)){
 				if(ai->outputAIBehaviourToConsole()) printf("BUILD isWarriorProducer check for Unit Name[%s] result = true\n",building->getName().c_str());
+				//aiInterface->printLog(4, "BUILD isWarriorProducer check for Unit Name[%s] result = true",building->getName().c_str());
 				return true;
 			}
 		}
 	}
 	if(ai->outputAIBehaviourToConsole()) printf("BUILD isWarriorProducer check for Unit Name[%s] result = false\n",building->getName().c_str());
+	//aiInterface->printLog(4, "BUILD isWarriorProducer check for Unit Name[%s] result = false",building->getName().c_str());
+
 	return false;
 }
 
@@ -1460,7 +1592,7 @@ void AiRuleUpgrade::upgradeSpecific(const UpgradeTask *upgt){
 	if(aiInterface->reqsOk(upgt->getUpgradeType())){
 
 		//if resources dont meet retry
-		if(!aiInterface->checkCosts(upgt->getUpgradeType())){
+		if(!aiInterface->checkCosts(upgt->getUpgradeType(), NULL)) {
 			ai->retryTask(upgt);
 			return;
 		}
