@@ -27,14 +27,14 @@ using namespace Shared::Platform;
 namespace Glest { namespace Game {
 
 const char *mailString				= " http://bugs.megaglest.org";
-const string glestVersionString 	= "v3.6.0.3";
+const string glestVersionString 	= "v3.7.1";
 #if defined(SVNVERSION)
 const string SVN_Rev 			= string("Rev: ") + string(SVNVERSION);
 #elif defined(SVNVERSIONHEADER)
 #include "svnversion.h"
 const string SVN_Rev 			= string("Rev: ") + string(SVNVERSION);
 #else
-const string SVN_Rev 			= "$Rev: 3070 $";
+const string SVN_Rev 			= "$Rev: 3944 $";
 #endif
 
 string getCrashDumpFileName(){
@@ -42,30 +42,63 @@ string getCrashDumpFileName(){
 }
 
 string getPlatformNameString() {
-	static string platform = "";
+	static string platform;
 	if(platform == "") {
 #if defined(WIN32)
-	platform = "Windows";
+
 	#if defined(__MINGW32__)
-	platform = "W-MINGW";
+	platform = "W-Ming32";
+	#else
+	platform = "Windows";
 	#endif
 
 #elif defined(__FreeBSD__)
 	platform = "FreeBSD";
+#elif defined(__NetBSD__)
+	platform = "NetBSD";
+#elif defined(__OpenBSD__)
+	platform = "OpenBSD";
+
 #elif defined(__APPLE__)
 	platform = "MacOSX";
+#elif defined(_AIX)
+	platform = "AIX";
+#elif defined(__ANDROID__)
+	platform = "Android";
+#elif defined(__BEOS__)
+	platform = "BEOS";
+#elif defined(__gnu_linux__)
+	platform = "Linux";
+#elif defined(__sun)
+	platform = "Solaris";
+
 #elif defined(__GNUC__)
-	platform = "GNU";
+
 	#if defined(__MINGW32__)
-	platform = "L-MINGW";
+	platform = "L-Ming32";
+	#else
+	platform = "GNU";
 	#endif
 
 #else
 	platform = "???";
 #endif
 
-#if defined(_M_X64) || defined(_M_IA64) || defined(_M_AMD64) || defined(__x86_64__) || defined(_WIN64)
-	platform += "_64bit";
+#if defined(_M_X64) || defined(_M_AMD64) || defined(__x86_64__) || defined(_WIN64)
+	platform += "-X64";
+#elif defined(_M_ALPHA) || defined(__alpha__)
+	platform += "-ALPHA";
+#elif defined(_M_IA64) || defined(__ia64__)
+	platform += "-IA64";
+#elif defined(_M_MRX000) || defined(__mips__)
+	platform += "-MIPS";
+#elif defined(_M_PPC) || defined(__powerpc__)
+	platform += "-POWERPC";
+#elif defined(__sparc__)
+	platform += "-SPARC";
+#elif defined(_M_ARM_FP) || defined(__arm__) || defined(_M_ARM)
+	platform += "-ARM";
+
 #endif
 	}
 	return platform;
@@ -147,16 +180,16 @@ string getAboutString1(int i) {
 	switch(i) {
 	case 0: return "MegaGlest " + glestVersionString + " (" + "Shared Library " + sharedLibVersionString + ")";
 	case 1: return "Built: " + string(__DATE__) + " " + SVN_Rev;
-	case 2: return "Copyright 2001-2011 The MegaGlest Team";
+	case 2: return "Copyright 2001-2012 The MegaGlest Team";
 	}
 	return "";
 }
 
 string getAboutString2(int i) {
 	switch(i) {
-	case 0: return "Web: http://www.megaglest.org  http://glest.org";
+	case 0: return "Web: http://megaglest.org";
 	case 1: return "Bug reports: " + string(mailString);
-	case 2: return "Irc: irc://irc.freenode.net/glest";
+	case 2: return "IRC: irc://irc.freenode.net/megaglest";
 	}
 	return "";
 }
