@@ -21,6 +21,8 @@
 #include "game_util.h"
 #include "lang.h"
 #include "game_settings.h"
+#include "video_player.h"
+#include "byte_order.h"
 #include "leak_dumper.h"
 
 using namespace Shared::Sound;
@@ -76,6 +78,16 @@ CoreData::CoreData() {
 	menuFontBig3D=NULL;
 	menuFontVeryBig3D=NULL;
 	consoleFont3D=NULL;
+
+	introVideoFilename="";
+	mainMenuVideoFilename="";
+
+	battleEndWinVideoFilename="";
+	battleEndWinVideoFilenameFallback="";
+	battleEndWinMusicFilename="";
+	battleEndLoseVideoFilename="";
+	battleEndLoseVideoFilenameFallback="";
+	battleEndLoseMusicFilename="";
 }
 
 CoreData::~CoreData() {
@@ -87,6 +99,70 @@ void CoreData::cleanup() {
 	waterSounds.getSoundsPtr()->clear();
 }
 
+Texture2D *CoreData::getTextureBySystemId(TextureSystemType type) const {
+	Texture2D *result = NULL;
+	switch(type) {
+	case tsyst_logoTexture:
+		result = logoTexture;
+		break;
+	//std::vector<Texture2D *> logoTextureList;
+	case tsyst_backgroundTexture:
+		result = backgroundTexture;
+		break;
+	case tsyst_fireTexture:
+		result = fireTexture;
+		break;
+	case tsyst_teamColorTexture:
+		result = teamColorTexture;
+		break;
+	case tsyst_snowTexture:
+		result = snowTexture;
+		break;
+	case tsyst_waterSplashTexture:
+		result = waterSplashTexture;
+		break;
+	case tsyst_customTexture:
+		result = customTexture;
+		break;
+	case tsyst_buttonSmallTexture:
+		result = buttonSmallTexture;
+		break;
+	case tsyst_buttonBigTexture:
+		result = buttonBigTexture;
+		break;
+	case tsyst_horizontalLineTexture:
+		result = horizontalLineTexture;
+		break;
+	case tsyst_verticalLineTexture:
+		result = verticalLineTexture;
+		break;
+	case tsyst_checkBoxTexture:
+		result = checkBoxTexture;
+		break;
+	case tsyst_checkedCheckBoxTexture:
+		result = checkedCheckBoxTexture;
+		break;
+	case tsyst_gameWinnerTexture:
+		result = gameWinnerTexture;
+		break;
+	case tsyst_notOnServerTexture:
+		result = notOnServerTexture;
+		break;
+	case tsyst_onServerDifferentTexture:
+		result = onServerDifferentTexture;
+		break;
+	case tsyst_onServerTexture:
+		result = onServerTexture;
+		break;
+	case tsyst_onServerInstalledTexture:
+		result = onServerInstalledTexture;
+		break;
+
+    //std::vector<Texture2D *> miscTextureList;
+	}
+	return result;
+}
+
 void CoreData::load() {
 	string data_path = getGameReadWritePath(GameConstants::path_data_CacheLookupKey);
 	if(data_path != "") {
@@ -103,6 +179,7 @@ void CoreData::load() {
 	if(backgroundTexture) {
 		backgroundTexture->setMipmap(false);
 		backgroundTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/menu/textures/back.tga"));
+		backgroundTexture->setTextureSystemId(tsyst_backgroundTexture);
 	}
 
 	fireTexture= renderer.newTexture2D(rsGlobal);
@@ -110,6 +187,7 @@ void CoreData::load() {
 		fireTexture->setFormat(Texture::fAlpha);
 		fireTexture->getPixmap()->init(1);
 		fireTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/misc_textures/fire_particle.tga"));
+		fireTexture->setTextureSystemId(tsyst_fireTexture);
 	}
 
 	teamColorTexture= renderer.newTexture2D(rsGlobal);
@@ -117,6 +195,7 @@ void CoreData::load() {
 		teamColorTexture->setFormat(Texture::fAlpha);
 		teamColorTexture->getPixmap()->init(1);
 		teamColorTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/misc_textures/team_color_texture.tga"));
+		teamColorTexture->setTextureSystemId(tsyst_teamColorTexture);
 	}
 
 	snowTexture= renderer.newTexture2D(rsGlobal);
@@ -125,33 +204,40 @@ void CoreData::load() {
 		snowTexture->setFormat(Texture::fAlpha);
 		snowTexture->getPixmap()->init(1);
 		snowTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/misc_textures/snow_particle.tga"));
+		snowTexture->setTextureSystemId(tsyst_snowTexture);
 	}
 
 	customTexture= renderer.newTexture2D(rsGlobal);
 	if(customTexture) {
 		customTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/menu/textures/custom_texture.tga"));
+		customTexture->setTextureSystemId(tsyst_customTexture);
 	}
 
 	notOnServerTexture= renderer.newTexture2D(rsGlobal);
 	if(notOnServerTexture) {
 		notOnServerTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/menu/textures/not_on_server.tga"));
+		notOnServerTexture->setTextureSystemId(tsyst_notOnServerTexture);
 	}
 
 	onServerDifferentTexture= renderer.newTexture2D(rsGlobal);
 	if(onServerDifferentTexture) {
 		onServerDifferentTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/menu/textures/on_server_different.tga"));
+		onServerDifferentTexture->setTextureSystemId(tsyst_onServerDifferentTexture);
 
 		onServerTexture= renderer.newTexture2D(rsGlobal);
 		onServerTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/menu/textures/on_server.tga"));
+		onServerTexture->setTextureSystemId(tsyst_onServerTexture);
 
 		onServerInstalledTexture= renderer.newTexture2D(rsGlobal);
 		onServerInstalledTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/menu/textures/on_server_installed.tga"));
+		onServerInstalledTexture->setTextureSystemId(tsyst_onServerInstalledTexture);
 	}
 
 	logoTexture= renderer.newTexture2D(rsGlobal);
 	if(logoTexture) {
 		logoTexture->setMipmap(false);
 		logoTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/menu/textures/logo.tga"));
+		logoTexture->setTextureSystemId(tsyst_logoTexture);
 	}
 
 	logoTextureList.clear();
@@ -227,48 +313,56 @@ void CoreData::load() {
 		waterSplashTexture->setFormat(Texture::fAlpha);
 		waterSplashTexture->getPixmap()->init(1);
 		waterSplashTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/misc_textures/water_splash.tga"));
+		waterSplashTexture->setTextureSystemId(tsyst_waterSplashTexture);
 	}
 
 	buttonSmallTexture= renderer.newTexture2D(rsGlobal);
 	if(buttonSmallTexture) {
 		buttonSmallTexture->setForceCompressionDisabled(true);
 		buttonSmallTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/menu/textures/button_small.tga"));
+		buttonSmallTexture->setTextureSystemId(tsyst_buttonSmallTexture);
 	}
 
 	buttonBigTexture= renderer.newTexture2D(rsGlobal);
 	if(buttonBigTexture) {
 		buttonBigTexture->setForceCompressionDisabled(true);
 		buttonBigTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/menu/textures/button_big.tga"));
+		buttonBigTexture->setTextureSystemId(tsyst_buttonBigTexture);
 	}
 
 	horizontalLineTexture= renderer.newTexture2D(rsGlobal);
 	if(horizontalLineTexture) {
 		horizontalLineTexture->setForceCompressionDisabled(true);
 		horizontalLineTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/menu/textures/line_horizontal.tga"));
+		horizontalLineTexture->setTextureSystemId(tsyst_horizontalLineTexture);
 	}
 
 	verticalLineTexture= renderer.newTexture2D(rsGlobal);
 	if(verticalLineTexture) {
 		verticalLineTexture->setForceCompressionDisabled(true);
 		verticalLineTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/menu/textures/line_vertical.tga"));
+		verticalLineTexture->setTextureSystemId(tsyst_verticalLineTexture);
 	}
 
 	checkBoxTexture= renderer.newTexture2D(rsGlobal);
 	if(checkBoxTexture) {
 		checkBoxTexture->setForceCompressionDisabled(true);
 		checkBoxTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/menu/textures/checkbox.tga"));
+		checkBoxTexture->setTextureSystemId(tsyst_checkBoxTexture);
 	}
 
 	checkedCheckBoxTexture= renderer.newTexture2D(rsGlobal);
 	if(checkedCheckBoxTexture) {
 		checkedCheckBoxTexture->setForceCompressionDisabled(true);
 		checkedCheckBoxTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/menu/textures/checkbox_checked.tga"));
+		checkedCheckBoxTexture->setTextureSystemId(tsyst_checkedCheckBoxTexture);
 	}
 
 	gameWinnerTexture= renderer.newTexture2D(rsGlobal);
 	if(gameWinnerTexture) {
 		gameWinnerTexture->setForceCompressionDisabled(true);
 		gameWinnerTexture->getPixmap()->load(getGameCustomCoreDataPath(data_path, "data/core/misc_textures/game_winner.png"));
+		gameWinnerTexture->setTextureSystemId(tsyst_gameWinnerTexture);
 	}
 
 	loadFonts();
@@ -279,6 +373,7 @@ void CoreData::load() {
 	clickSoundC.load(getGameCustomCoreDataPath(data_path, "data/core/menu/sound/click_c.wav"));
 	attentionSound.load(getGameCustomCoreDataPath(data_path, "data/core/menu/sound/attention.wav"));
 	highlightSound.load(getGameCustomCoreDataPath(data_path, "data/core/menu/sound/highlight.wav"));
+	markerSound.load(getGameCustomCoreDataPath(data_path, "data/core/menu/sound/sonar.wav"));
 
 	XmlTree xmlTree;
 	//string data_path = getGameReadWritePath(GameConstants::path_data_CacheLookupKey);
@@ -305,6 +400,7 @@ void CoreData::load() {
 	menuMusic.open(getGameCustomCoreDataPath(data_path, "data/core/" + menuMusicPath + menuMusicFile));
 	menuMusic.setNext(&menuMusic);
 
+	cleanup();
 	waterSounds.resize(6);
 
 	for(int i=0; i<6; ++i){
@@ -314,6 +410,240 @@ void CoreData::load() {
 		}
 	}
 
+
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false &&
+			Shared::Graphics::VideoPlayer::hasBackEndVideoPlayer() == true) {
+		string data_path = getGameReadWritePath(GameConstants::path_data_CacheLookupKey);
+		Config &config= Config::getInstance();
+
+		introVideoFilename = config.getString("IntroVideoURL","");
+		introVideoFilenameFallback = config.getString("IntroVideoURLFallback","");
+		if(introVideoFilename == "") {
+			string introVideoPath = getGameCustomCoreDataPath(data_path, "") + "data/core/menu/videos/intro.*";
+			vector<string> introVideos;
+			findAll(introVideoPath, introVideos, false, false);
+			for(int i = 0; i < introVideos.size(); ++i) {
+				string video = getGameCustomCoreDataPath(data_path, "") + "data/core/menu/videos/" + introVideos[i];
+				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Checking if intro video [%s] exists\n",video.c_str());
+
+				if(fileExists(video)) {
+					introVideoFilename = video;
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("FOUND intro video [%s] will use this file\n",video.c_str());
+					break;
+				}
+			}
+
+			if(introVideoFilename == "") {
+				introVideoPath = data_path + "data/core/menu/videos/intro.*";
+				introVideos.clear();
+				findAll(introVideoPath, introVideos, false, false);
+				for(int i = 0; i < introVideos.size(); ++i) {
+					string video = data_path + "data/core/menu/videos/" + introVideos[i];
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Checking if intro video [%s] exists\n",video.c_str());
+
+					if(fileExists(video)) {
+						introVideoFilename = video;
+						if(SystemFlags::VERBOSE_MODE_ENABLED) printf("FOUND intro video [%s] will use this file\n",video.c_str());
+						break;
+					}
+				}
+			}
+		}
+
+		mainMenuVideoFilename = config.getString("MainMenuVideoURL","");
+		mainMenuVideoFilenameFallback = config.getString("MainMenuVideoURLFallback","");
+		if(mainMenuVideoFilename == "") {
+			string mainVideoPath = getGameCustomCoreDataPath(data_path, "") + "data/core/menu/videos/main.*";
+			vector<string> mainVideos;
+			findAll(mainVideoPath, mainVideos, false, false);
+			for(int i = 0; i < mainVideos.size(); ++i) {
+				string video = getGameCustomCoreDataPath(data_path, "") + "data/core/menu/videos/" + mainVideos[i];
+				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Checking if mainmenu video [%s] exists\n",video.c_str());
+
+				if(fileExists(video)) {
+					mainMenuVideoFilename = video;
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("FOUND mainmenu video [%s] will use this file\n",video.c_str());
+					break;
+				}
+			}
+
+			if(mainMenuVideoFilename == "") {
+				mainVideoPath = data_path + "data/core/menu/videos/main.*";
+				mainVideos.clear();
+				findAll(mainVideoPath, mainVideos, false, false);
+				for(int i = 0; i < mainVideos.size(); ++i) {
+					string video = data_path + "data/core/menu/videos/" + mainVideos[i];
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Checking if mainmenu video [%s] exists\n",video.c_str());
+
+					if(fileExists(video)) {
+						mainMenuVideoFilename = video;
+						if(SystemFlags::VERBOSE_MODE_ENABLED) printf("FOUND mainmenu video [%s] will use this file\n",video.c_str());
+						break;
+					}
+				}
+			}
+		}
+
+		battleEndWinVideoFilename = config.getString("BattleEndWinVideoURL","");
+		battleEndWinVideoFilenameFallback = config.getString("BattleEndWinVideoURLFallback","");
+
+		if(battleEndWinVideoFilename == "") {
+			string battleEndWinVideoPath = getGameCustomCoreDataPath(data_path, "") + "data/core/menu/videos/battle_end_win.*";
+			vector<string> battleEndWinVideos;
+			findAll(battleEndWinVideoPath, battleEndWinVideos, false, false);
+			for(int i = 0; i < battleEndWinVideos.size(); ++i) {
+				string video = getGameCustomCoreDataPath(data_path, "") + "data/core/menu/videos/" + battleEndWinVideos[i];
+				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Checking if battle end win video [%s] exists\n",video.c_str());
+
+				if(fileExists(video)) {
+					battleEndWinVideoFilename = video;
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("FOUND battle end win video [%s] will use this file\n",video.c_str());
+					break;
+				}
+			}
+
+			if(battleEndWinVideoFilename == "") {
+				battleEndWinVideoPath = data_path + "data/core/menu/videos/battle_end_win.*";
+				battleEndWinVideos.clear();
+				findAll(battleEndWinVideoPath, battleEndWinVideos, false, false);
+				for(int i = 0; i < battleEndWinVideos.size(); ++i) {
+					string video = data_path + "data/core/menu/videos/" + battleEndWinVideos[i];
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Checking if battle end win video [%s] exists\n",video.c_str());
+
+					if(fileExists(video)) {
+						battleEndWinVideoFilename = video;
+						if(SystemFlags::VERBOSE_MODE_ENABLED) printf("FOUND battle end video win [%s] will use this file\n",video.c_str());
+						break;
+					}
+				}
+			}
+		}
+
+		battleEndWinMusicFilename = config.getString("BattleEndWinMusicFilename","");
+		if(battleEndWinMusicFilename == "") {
+			string battleEndWinPath = getGameCustomCoreDataPath(data_path, "") + "data/core/menu/music/battle_end_win.*";
+			vector<string> battleEndWinMusic;
+			findAll(battleEndWinPath, battleEndWinMusic, false, false);
+			for(int i = 0; i < battleEndWinMusic.size(); ++i) {
+				string music = getGameCustomCoreDataPath(data_path, "") + "data/core/menu/music/" + battleEndWinMusic[i];
+				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Checking if battle end win music [%s] exists\n",music.c_str());
+
+				if(fileExists(music)) {
+					battleEndWinMusicFilename = music;
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("FOUND battle end win music [%s] will use this file\n",music.c_str());
+					break;
+				}
+			}
+
+			if(battleEndWinMusicFilename == "") {
+				battleEndWinPath = data_path + "data/core/menu/music/battle_end_win.*";
+				battleEndWinMusic.clear();
+				findAll(battleEndWinPath, battleEndWinMusic, false, false);
+				for(int i = 0; i < battleEndWinMusic.size(); ++i) {
+					string music = data_path + "data/core/menu/music/" + battleEndWinMusic[i];
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Checking if battle end win music [%s] exists\n",music.c_str());
+
+					if(fileExists(music)) {
+						battleEndWinMusicFilename = music;
+						if(SystemFlags::VERBOSE_MODE_ENABLED) printf("FOUND battle end music win [%s] will use this file\n",music.c_str());
+						break;
+					}
+				}
+			}
+		}
+
+		battleEndLoseVideoFilename = config.getString("BattleEndLoseVideoURL","");
+		battleEndLoseVideoFilenameFallback = config.getString("BattleEndLoseVideoURLFallback","");
+
+		if(battleEndLoseVideoFilename == "") {
+			string battleEndLoseVideoPath = getGameCustomCoreDataPath(data_path, "") + "data/core/menu/videos/battle_end_lose.*";
+			vector<string> battleEndLoseVideos;
+			findAll(battleEndLoseVideoPath, battleEndLoseVideos, false, false);
+			for(int i = 0; i < battleEndLoseVideos.size(); ++i) {
+				string video = getGameCustomCoreDataPath(data_path, "") + "data/core/menu/videos/" + battleEndLoseVideos[i];
+				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Checking if battle end lose video [%s] exists\n",video.c_str());
+
+				if(fileExists(video)) {
+					battleEndLoseVideoFilename = video;
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("FOUND battle end lose video [%s] will use this file\n",video.c_str());
+					break;
+				}
+			}
+
+			if(battleEndLoseVideoFilename == "") {
+				battleEndLoseVideoPath = data_path + "data/core/menu/videos/battle_end_lose.*";
+				battleEndLoseVideos.clear();
+				findAll(battleEndLoseVideoPath, battleEndLoseVideos, false, false);
+				for(int i = 0; i < battleEndLoseVideos.size(); ++i) {
+					string video = data_path + "data/core/menu/videos/" + battleEndLoseVideos[i];
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Checking if battle end lose video [%s] exists\n",video.c_str());
+
+					if(fileExists(video)) {
+						battleEndLoseVideoFilename = video;
+						if(SystemFlags::VERBOSE_MODE_ENABLED) printf("FOUND battle end video lose [%s] will use this file\n",video.c_str());
+						break;
+					}
+				}
+			}
+		}
+
+		battleEndLoseMusicFilename = config.getString("BattleEndLoseMusicFilename","");
+		if(battleEndLoseMusicFilename == "") {
+			string battleEndLosePath = getGameCustomCoreDataPath(data_path, "") + "data/core/menu/music/battle_end_lose.*";
+			vector<string> battleEndLoseMusic;
+			findAll(battleEndLosePath, battleEndLoseMusic, false, false);
+			for(int i = 0; i < battleEndLoseMusic.size(); ++i) {
+				string music = getGameCustomCoreDataPath(data_path, "") + "data/core/menu/music/" + battleEndLoseMusic[i];
+				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Checking if battle end lose music [%s] exists\n",music.c_str());
+
+				if(fileExists(music)) {
+					battleEndLoseMusicFilename = music;
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("FOUND battle end lose music [%s] will use this file\n",music.c_str());
+					break;
+				}
+			}
+
+			if(battleEndLoseMusicFilename == "") {
+				battleEndLosePath = data_path + "data/core/menu/music/battle_end_lose.*";
+				battleEndLoseMusic.clear();
+				findAll(battleEndLosePath, battleEndLoseMusic, false, false);
+				for(int i = 0; i < battleEndLoseMusic.size(); ++i) {
+					string music = data_path + "data/core/menu/music/" + battleEndLoseMusic[i];
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Checking if battle end lose music [%s] exists\n",music.c_str());
+
+					if(fileExists(music)) {
+						battleEndLoseMusicFilename = music;
+						if(SystemFlags::VERBOSE_MODE_ENABLED) printf("FOUND battle end music lose [%s] will use this file\n",music.c_str());
+						break;
+					}
+				}
+			}
+		}
+	}
+}
+
+bool CoreData::hasIntroVideoFilename() const {
+	//bool result = (introVideoFilename != "" && fileExists(introVideoFilename) == true);
+	bool result = (introVideoFilename != "");
+	return result;
+}
+
+bool CoreData::hasMainMenuVideoFilename() const {
+	//bool result = (mainMenuVideoFilename != "" && fileExists(mainMenuVideoFilename) == true);
+	bool result = (mainMenuVideoFilename != "");
+	return result;
+}
+
+bool CoreData::hasBattleEndVideoFilename(bool won) const {
+	//bool result = (mainMenuVideoFilename != "" && fileExists(mainMenuVideoFilename) == true);
+	bool result = false;
+	if(won == true) {
+		result =(battleEndWinVideoFilename != "");
+	}
+	else {
+		result =(battleEndLoseVideoFilename != "");
+	}
+	return result;
 }
 
 void CoreData::loadFonts() {
@@ -336,7 +666,7 @@ void CoreData::loadFonts() {
 		displayFontNamePostfix = lang.get("FontDisplayPostfix");
 	}
 	if(lang.hasString("FontDisplayBaseSize") == true) {
-		displayFontSize = strToInt(lang.get("FontDisplayBaseSize"));
+		displayFontSize = computeFontSize(strToInt(lang.get("FontDisplayBaseSize")));
 	}
 
 	//printf("displayFontNamePostfix [%s]\n",displayFontNamePostfix.c_str());
@@ -383,7 +713,7 @@ void CoreData::loadFonts() {
 		displayFontNameSmallPostfix = lang.get("FontDisplayPostfix");
 	}
 	if(lang.hasString("FontDisplaySmallBaseSize") == true) {
-		displayFontNameSmallSize = strToInt(lang.get("FontDisplaySmallBaseSize"));
+		displayFontNameSmallSize = computeFontSize(strToInt(lang.get("FontDisplaySmallBaseSize")));
 	}
 
 	string displayFontNameSmall = displayFontNameSmallPrefix + intToStr(displayFontNameSmallSize) + displayFontNameSmallPostfix;
@@ -419,6 +749,7 @@ void CoreData::loadFonts() {
 	string menuFontNameNormalPrefix		= config.getString("FontMenuNormalPrefix");
 	string menuFontNameNormalPostfix	= config.getString("FontMenuNormalPostfix");
 	int menuFontNameNormalSize			= computeFontSize(config.getInt("FontMenuNormalBaseSize"));
+	//printf("#1 menuFontNameNormalSize = %d\n",menuFontNameNormalSize);
 
 	if(lang.hasString("FontMenuNormalPrefix") == true) {
 		menuFontNameNormalPrefix = lang.get("FontMenuNormalPrefix");
@@ -427,7 +758,8 @@ void CoreData::loadFonts() {
 		menuFontNameNormalPostfix = lang.get("FontMenuNormalPostfix");
 	}
 	if(lang.hasString("FontMenuNormalBaseSize") == true) {
-		menuFontNameNormalSize = strToInt(lang.get("FontMenuNormalBaseSize"));
+		menuFontNameNormalSize = computeFontSize(strToInt(lang.get("FontMenuNormalBaseSize")));
+		//printf("#2 menuFontNameNormalSize = %d\n",menuFontNameNormalSize);
 	}
 
 	string menuFontNameNormal= menuFontNameNormalPrefix + intToStr(menuFontNameNormalSize) + menuFontNameNormalPostfix;
@@ -473,7 +805,7 @@ void CoreData::loadFonts() {
 		menuFontNameBigPostfix = lang.get("FontMenuBigPostfix");
 	}
 	if(lang.hasString("FontMenuBigBaseSize") == true) {
-		menuFontNameBigSize = strToInt(lang.get("FontMenuBigBaseSize"));
+		menuFontNameBigSize = computeFontSize(strToInt(lang.get("FontMenuBigBaseSize")));
 	}
 
 	string menuFontNameBig= menuFontNameBigPrefix+intToStr(menuFontNameBigSize)+menuFontNameBigPostfix;
@@ -517,7 +849,7 @@ void CoreData::loadFonts() {
 		menuFontNameVeryBigPostfix = lang.get("FontMenuBigPostfix");
 	}
 	if(lang.hasString("FontMenuVeryBigBaseSize") == true) {
-		menuFontNameVeryBigSize = strToInt(lang.get("FontMenuVeryBigBaseSize"));
+		menuFontNameVeryBigSize = computeFontSize(strToInt(lang.get("FontMenuVeryBigBaseSize")));
 	}
 
 	string menuFontNameVeryBig= menuFontNameVeryBigPrefix + intToStr(menuFontNameVeryBigSize) + menuFontNameVeryBigPostfix;
@@ -564,7 +896,7 @@ void CoreData::loadFonts() {
 		consoleFontNamePostfix = lang.get("FontConsolePostfix");
 	}
 	if(lang.hasString("FontConsoleBaseSize") == true) {
-		consoleFontNameSize = strToInt(lang.get("FontConsoleBaseSize"));
+		consoleFontNameSize = computeFontSize(strToInt(lang.get("FontConsoleBaseSize")));
 	}
 
 	string consoleFontName= consoleFontNamePrefix + intToStr(consoleFontNameSize) + consoleFontNamePostfix;
@@ -649,36 +981,37 @@ void CoreData::saveGameSettingsToFile(std::string fileName, GameSettings *gameSe
 #endif
 
 	saveGameFile << "Description=" << gameSettings->getDescription() << std::endl;
-	saveGameFile << "MapFilterIndex=" << gameSettings->getMapFilterIndex() << std::endl;
+	saveGameFile << "MapFilterIndex=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getMapFilterIndex()) << std::endl;
 	saveGameFile << "Map=" << gameSettings->getMap() << std::endl;
 	saveGameFile << "Tileset=" << gameSettings->getTileset() << std::endl;
 	saveGameFile << "TechTree=" << gameSettings->getTech() << std::endl;
-	saveGameFile << "DefaultUnits=" << gameSettings->getDefaultUnits() << std::endl;
-	saveGameFile << "DefaultResources=" << gameSettings->getDefaultResources() << std::endl;
-	saveGameFile << "DefaultVictoryConditions=" << gameSettings->getDefaultVictoryConditions() << std::endl;
-	saveGameFile << "FogOfWar=" << gameSettings->getFogOfWar() << std::endl;
-	saveGameFile << "AdvancedIndex=" << advancedIndex << std::endl;
-	saveGameFile << "AllowObservers=" << gameSettings->getAllowObservers() << std::endl;
-	saveGameFile << "FlagTypes1=" << gameSettings->getFlagTypes1() << std::endl;
-	saveGameFile << "EnableObserverModeAtEndGame=" << gameSettings->getEnableObserverModeAtEndGame() << std::endl;
-	saveGameFile << "AiAcceptSwitchTeamPercentChance=" << gameSettings->getAiAcceptSwitchTeamPercentChance() << std::endl;
-	saveGameFile << "PathFinderType=" << gameSettings->getPathFinderType() << std::endl;
-	saveGameFile << "EnableServerControlledAI=" << gameSettings->getEnableServerControlledAI() << std::endl;
-	saveGameFile << "NetworkFramePeriod=" << gameSettings->getNetworkFramePeriod() << std::endl;
-	saveGameFile << "NetworkPauseGameForLaggedClients=" << gameSettings->getNetworkPauseGameForLaggedClients() << std::endl;
+	saveGameFile << "DefaultUnits=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getDefaultUnits()) << std::endl;
+	saveGameFile << "DefaultResources=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getDefaultResources()) << std::endl;
+	saveGameFile << "DefaultVictoryConditions=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getDefaultVictoryConditions()) << std::endl;
+	saveGameFile << "FogOfWar=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getFogOfWar()) << std::endl;
+	saveGameFile << "AdvancedIndex=" << Shared::PlatformByteOrder::toCommonEndian(advancedIndex) << std::endl;
+	saveGameFile << "AllowObservers=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getAllowObservers()) << std::endl;
+	saveGameFile << "FlagTypes1=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getFlagTypes1()) << std::endl;
+	saveGameFile << "EnableObserverModeAtEndGame=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getEnableObserverModeAtEndGame()) << std::endl;
+	saveGameFile << "AiAcceptSwitchTeamPercentChance=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getAiAcceptSwitchTeamPercentChance()) << std::endl;
+	saveGameFile << "FallbackCpuMultiplier=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getFallbackCpuMultiplier()) << std::endl;
+	saveGameFile << "PathFinderType=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getPathFinderType()) << std::endl;
+	saveGameFile << "EnableServerControlledAI=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getEnableServerControlledAI()) << std::endl;
+	saveGameFile << "NetworkFramePeriod=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getNetworkFramePeriod()) << std::endl;
+	saveGameFile << "NetworkPauseGameForLaggedClients=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getNetworkPauseGameForLaggedClients()) << std::endl;
 
-	saveGameFile << "FactionThisFactionIndex=" << gameSettings->getThisFactionIndex() << std::endl;
-	saveGameFile << "FactionCount=" << gameSettings->getFactionCount() << std::endl;
+	saveGameFile << "FactionThisFactionIndex=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getThisFactionIndex()) << std::endl;
+	saveGameFile << "FactionCount=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getFactionCount()) << std::endl;
 
 	for(int i = 0; i < GameConstants::maxPlayers; ++i) {
 		int slotIndex = gameSettings->getStartLocationIndex(i);
 
-		saveGameFile << "FactionControlForIndex" 		<< slotIndex << "=" << gameSettings->getFactionControl(i) << std::endl;
-		saveGameFile << "ResourceMultiplierIndex" 		<< slotIndex << "=" << gameSettings->getResourceMultiplierIndex(i) << std::endl;
-		saveGameFile << "FactionTeamForIndex" 			<< slotIndex << "=" << gameSettings->getTeam(i) << std::endl;
-		saveGameFile << "FactionStartLocationForIndex" 	<< slotIndex << "=" << gameSettings->getStartLocationIndex(i) << std::endl;
-		saveGameFile << "FactionTypeNameForIndex" 		<< slotIndex << "=" << gameSettings->getFactionTypeName(i) << std::endl;
-		saveGameFile << "FactionPlayerNameForIndex" 	<< slotIndex << "=" << gameSettings->getNetworkPlayerName(i) << std::endl;
+		saveGameFile << "FactionControlForIndex" 		<< Shared::PlatformByteOrder::toCommonEndian(slotIndex) << "=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getFactionControl(i)) << std::endl;
+		saveGameFile << "ResourceMultiplierIndex" 		<< Shared::PlatformByteOrder::toCommonEndian(slotIndex) << "=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getResourceMultiplierIndex(i)) << std::endl;
+		saveGameFile << "FactionTeamForIndex" 			<< Shared::PlatformByteOrder::toCommonEndian(slotIndex) << "=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getTeam(i)) << std::endl;
+		saveGameFile << "FactionStartLocationForIndex" 	<< Shared::PlatformByteOrder::toCommonEndian(slotIndex) << "=" << Shared::PlatformByteOrder::toCommonEndian(gameSettings->getStartLocationIndex(i)) << std::endl;
+		saveGameFile << "FactionTypeNameForIndex" 		<< Shared::PlatformByteOrder::toCommonEndian(slotIndex) << "=" << gameSettings->getFactionTypeName(i) << std::endl;
+		saveGameFile << "FactionPlayerNameForIndex" 	<< Shared::PlatformByteOrder::toCommonEndian(slotIndex) << "=" << gameSettings->getNetworkPlayerName(i) << std::endl;
     }
 
 #if defined(WIN32) && !defined(__MINGW32__)
@@ -687,22 +1020,27 @@ void CoreData::saveGameSettingsToFile(std::string fileName, GameSettings *gameSe
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s] Line: %d\n",__FILE__,__FUNCTION__,__LINE__);
 }
 
-void CoreData::loadGameSettingsFromFile(std::string fileName, GameSettings *gameSettings) {
+bool CoreData::loadGameSettingsFromFile(std::string fileName, GameSettings *gameSettings) {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s] Line: %d\n",__FILE__,__FUNCTION__,__LINE__);
 
+	bool fileWasFound = false;
     Config &config = Config::getInstance();
     string userData = config.getString("UserData_Root","");
     if(userData != "") {
     	endPathWithSlash(userData);
     }
-    fileName = userData + fileName;
+    if(fileExists(userData + fileName) == true) {
+    	fileName = userData + fileName;
+    	fileWasFound = true;
+    }
 
     if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] fileName = [%s]\n",__FILE__,__FUNCTION__,__LINE__,fileName.c_str());
 
     if(fileExists(fileName) == false) {
-    	return;
+    	return false;
     }
 
+    fileWasFound = true;
     if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] fileName = [%s]\n",__FILE__,__FUNCTION__,__LINE__,fileName.c_str());
 
 	Properties properties;
@@ -725,6 +1063,7 @@ void CoreData::loadGameSettingsFromFile(std::string fileName, GameSettings *game
 	gameSettings->setFlagTypes1(properties.getInt("FlagTypes1","0"));
 	gameSettings->setEnableObserverModeAtEndGame(properties.getBool("EnableObserverModeAtEndGame"));
 	gameSettings->setAiAcceptSwitchTeamPercentChance(properties.getInt("AiAcceptSwitchTeamPercentChance","30"));
+	gameSettings->setFallbackCpuMultiplier(properties.getInt("FallbackCpuMultiplier","1"));
 
 	gameSettings->setPathFinderType(static_cast<PathFinderType>(properties.getInt("PathFinderType",intToStr(pfBasic).c_str())));
 	gameSettings->setEnableServerControlledAI(properties.getBool("EnableServerControlledAI","true"));
@@ -757,6 +1096,8 @@ void CoreData::loadGameSettingsFromFile(std::string fileName, GameSettings *game
 	}
 
     if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s] Line: %d\n",__FILE__,__FUNCTION__,__LINE__);
+
+    return fileWasFound;
 }
 
 // ================== PRIVATE ========================
