@@ -24,6 +24,7 @@
 #include "core_data.h"
 #include "renderer.h"
 #include <algorithm>
+#include "config.h"
 #include "leak_dumper.h"
 
 using namespace std;
@@ -54,7 +55,7 @@ string Lang::getDefaultLanguage() const {
 
 void Lang::loadStrings(string uselanguage, bool loadFonts,
 		bool fallbackToDefault) {
-	if(uselanguage.length() == 2) {
+	if(uselanguage.length() == 2 || (uselanguage.length() == 5 && uselanguage[2] == '-')) {
 		uselanguage = getLanguageFile(uselanguage);
 	}
 	bool languageChanged = (uselanguage != this->language);
@@ -192,7 +193,7 @@ void Lang::loadStrings(string uselanguage, Properties &properties, bool fileMust
 	}
 	else if(fileExists(languageFile) == false && fallbackToDefault == true) {
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] path = [%s]\n",__FILE__,__FUNCTION__,__LINE__,languageFile.c_str());
-		//throw runtime_error("File NOT FOUND, can't open file: [" + languageFile + "]");
+		//throw megaglest_runtime_error("File NOT FOUND, can't open file: [" + languageFile + "]");
 		printf("Language file NOT FOUND, can't open file: [%s] switching to default language: %s\n",languageFile.c_str(),DEFAULT_LANGUAGE);
 
 		languageFile = getGameCustomCoreDataPath(data_path, "data/lang/" + string(DEFAULT_LANGUAGE) + ".lng");
@@ -208,6 +209,24 @@ bool Lang::isUTF8Language() const {
 void Lang::loadScenarioStrings(string scenarioDir, string scenarioName){
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scenarioDir = [%s] scenarioName = [%s]\n",__FILE__,__FUNCTION__,__LINE__,scenarioDir.c_str(),scenarioName.c_str());
 
+	// First try to find scenario lng file in userdata
+	Config &config = Config::getInstance();
+    vector<string> scenarioPaths = config.getPathListForType(ptScenarios);
+    if(scenarioPaths.size() > 1) {
+        string &scenarioPath = scenarioPaths[1];
+		endPathWithSlash(scenarioPath);
+
+		string currentPath = scenarioPath;
+		endPathWithSlash(currentPath);
+		string scenarioFolder = currentPath + scenarioName + "/";
+		string path = scenarioFolder + scenarioName + "_" + language + ".lng";
+
+		//try to load the current language first
+		if(fileExists(path)) {
+			scenarioDir = scenarioPath;
+		}
+    }
+
 	string currentPath = scenarioDir;
 	endPathWithSlash(currentPath);
 	string scenarioFolder = currentPath + scenarioName + "/";
@@ -238,6 +257,102 @@ void Lang::loadScenarioStrings(string scenarioDir, string scenarioName){
 	}
 }
 
+void Lang::loadTechTreeStrings(string techTree) {
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] techTree = [%s]\n",__FILE__,__FUNCTION__,__LINE__,techTree.c_str());
+
+	string currentPath = "";
+	Config &config = Config::getInstance();
+    vector<string> techPaths = config.getPathListForType(ptTechs);
+    for(int idx = 0; idx < techPaths.size(); idx++) {
+        string &techPath = techPaths[idx];
+		endPathWithSlash(techPath);
+
+        //printf("techPath [%s]\n",techPath.c_str());
+
+		if(folderExists(techPath + techTree) == true) {
+			currentPath = techPath;
+			endPathWithSlash(currentPath);
+			break;
+		}
+    }
+
+	string techTreeFolder = currentPath + techTree + "/";
+	string path = techTreeFolder + "lang/" + techTree + "_" + language + ".lng";
+	string pathDefault = techTreeFolder + "lang/" + techTree + "_default.lng";
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] path = [%s] pathDefault = [%s]\n",__FILE__,__FUNCTION__,__LINE__,path.c_str(),pathDefault.c_str());
+
+	techTreeStrings.clear();
+	techTreeStringsDefault.clear();
+
+	//try to load the current language first
+	if(fileExists(path)) {
+		techTreeStrings.load(path);
+	}
+	else {
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] path not found [%s]\n",__FILE__,__FUNCTION__,__LINE__,path.c_str());
+
+		//try english otherwise
+		path = techTreeFolder + "lang/" + techTree + "_english.lng";
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] path = [%s]\n",__FILE__,__FUNCTION__,__LINE__,path.c_str());
+
+		if(fileExists(path)) {
+			techTreeStrings.load(path);
+		}
+	}
+
+	if(fileExists(pathDefault)) {
+		techTreeStringsDefault.load(pathDefault);
+	}
+}
+
+void Lang::loadTilesetStrings(string tileset) {
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] tileset = [%s]\n",__FILE__,__FUNCTION__,__LINE__,tileset.c_str());
+
+	string currentPath = "";
+	Config &config = Config::getInstance();
+    vector<string> tilesetPaths = config.getPathListForType(ptTilesets);
+    for(int idx = 0; idx < tilesetPaths.size(); idx++) {
+        string &tilesetPath = tilesetPaths[idx];
+		endPathWithSlash(tilesetPath);
+
+        //printf("tilesetPath [%s]\n",tilesetPath.c_str());
+
+		if(folderExists(tilesetPath + tileset) == true) {
+			currentPath = tilesetPath;
+			endPathWithSlash(currentPath);
+			break;
+		}
+    }
+
+	string tilesetFolder = currentPath + tileset + "/";
+	string path = tilesetFolder + "lang/" + tileset + "_" + language + ".lng";
+	string pathDefault = tilesetFolder + "lang/" + tileset + "_default.lng";
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] path = [%s] pathDefault = [%s]\n",__FILE__,__FUNCTION__,__LINE__,path.c_str(),pathDefault.c_str());
+
+	tilesetStrings.clear();
+	tilesetStringsDefault.clear();
+
+	//try to load the current language first
+	if(fileExists(path)) {
+		tilesetStrings.load(path);
+	}
+	else {
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] path not found [%s]\n",__FILE__,__FUNCTION__,__LINE__,path.c_str());
+
+		//try english otherwise
+		path = tilesetFolder + "lang/" + tileset + "_english.lng";
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] path = [%s]\n",__FILE__,__FUNCTION__,__LINE__,path.c_str());
+
+		if(fileExists(path)) {
+			tilesetStrings.load(path);
+		}
+	}
+
+	if(fileExists(pathDefault)) {
+		tilesetStringsDefault.load(pathDefault);
+	}
+}
+
 bool Lang::hasString(const string &s, string uselanguage, bool fallbackToDefault) {
 	bool result = false;
 	try {
@@ -246,13 +361,15 @@ bool Lang::hasString(const string &s, string uselanguage, bool fallbackToDefault
 			if(otherLanguageStrings.find(uselanguage) == otherLanguageStrings.end()) {
 				loadStrings(uselanguage, otherLanguageStrings[uselanguage], false);
 			}
-			string result2 = otherLanguageStrings[uselanguage].getString(s);
+			//string result2 = otherLanguageStrings[uselanguage].getString(s);
+			otherLanguageStrings[uselanguage].getString(s);
 			//printf("#b result2 [%s]\n",result2.c_str());
 
 			result = true;
 		}
 		else {
-			string result2 = strings.getString(s);
+			//string result2 = strings.getString(s);
+			strings.getString(s);
 			result = true;
 		}
 	}
@@ -305,18 +422,21 @@ string Lang::get(const string &s, string uselanguage, bool fallbackToDefault) {
 		if(strings.getpath() != "") {
 			if(fallbackToDefault == false || SystemFlags::VERBOSE_MODE_ENABLED) {
 				if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false) {
-					SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s] uselanguage [%s] text [%s]\n",__FILE__,__FUNCTION__,__LINE__,ex.what(),uselanguage.c_str(),s.c_str());
+					if(SystemFlags::VERBOSE_MODE_ENABLED) SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s] uselanguage [%s] text [%s]\n",__FILE__,__FUNCTION__,__LINE__,ex.what(),uselanguage.c_str(),s.c_str());
 				}
 			}
 		}
 
 		//printf("#2 fallbackToDefault = %d [%s] uselanguage [%s] DEFAULT_LANGUAGE  [%s] this->language [%s]\n",fallbackToDefault,s.c_str(),uselanguage.c_str(),DEFAULT_LANGUAGE,this->language.c_str());
-		if(fallbackToDefault == true && uselanguage != DEFAULT_LANGUAGE && this->language != DEFAULT_LANGUAGE) {
+
+		//if(fallbackToDefault == true && uselanguage != DEFAULT_LANGUAGE && this->language != DEFAULT_LANGUAGE) {
+		if( uselanguage != DEFAULT_LANGUAGE && this->language != DEFAULT_LANGUAGE) {
 			return get(s, DEFAULT_LANGUAGE, false);
 		}
 
 		return "???" + s + "???";
 	}
+	return "???" + s + "???";
 }
 
 string Lang::getScenarioString(const string &s) {
@@ -331,6 +451,67 @@ string Lang::getScenarioString(const string &s) {
 		}
 		return "???" + s + "???";
 	}
+	return "???" + s + "???";
+}
+
+string Lang::getTechTreeString(const string &s,const char *defaultValue) {
+	try{
+		string result = "";
+
+		if(techTreeStrings.hasString(s) == true || defaultValue == NULL) {
+			if(techTreeStrings.hasString(s) == false && techTreeStringsDefault.hasString(s) == true) {
+				result = techTreeStringsDefault.getString(s);
+			}
+			else {
+				result = techTreeStrings.getString(s);
+			}
+		}
+		else if(techTreeStringsDefault.hasString(s) == true) {
+			result = techTreeStringsDefault.getString(s);
+		}
+		else if(defaultValue != NULL) {
+			result = defaultValue;
+		}
+		replaceAll(result, "\\n", "\n");
+		return result;
+	}
+	catch(exception &ex) {
+		if(techTreeStrings.getpath() != "") {
+			SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",__FILE__,__FUNCTION__,__LINE__,ex.what());
+		}
+		return "???" + s + "???";
+	}
+	return "???" + s + "???";
+}
+
+string Lang::getTilesetString(const string &s,const char *defaultValue) {
+	try{
+		string result = "";
+
+		if(tilesetStrings.hasString(s) == true || defaultValue == NULL) {
+			if(tilesetStrings.hasString(s) == false && tilesetStringsDefault.hasString(s) == true) {
+				result = tilesetStringsDefault.getString(s);
+			}
+			else {
+				result = tilesetStrings.getString(s);
+			}
+		}
+		else if(tilesetStringsDefault.hasString(s) == true) {
+			result = tilesetStringsDefault.getString(s);
+		}
+		else if(defaultValue != NULL) {
+			result = defaultValue;
+		}
+		replaceAll(result, "\\n", "\n");
+		return result;
+	}
+	catch(exception &ex) {
+		if(tilesetStrings.getpath() != "") {
+			SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",__FILE__,__FUNCTION__,__LINE__,ex.what());
+		}
+		return "???" + s + "???";
+	}
+	return "???" + s + "???";
 }
 
 bool Lang::fileMatchesISO630Code(string uselanguage, string testLanguageFile) {
@@ -344,7 +525,8 @@ bool Lang::fileMatchesISO630Code(string uselanguage, string testLanguageFile) {
 			result = true;
 		}
 	}
-	catch(exception &ex) {
+	//catch(const exception &ex) {
+	catch(...) {
 	}
 
 	return result;
@@ -388,7 +570,8 @@ string Lang::getNativeLanguageName(string uselanguage, string testLanguageFile)
 	try {
 		result = stringsTest.getString("NativeLanguageName");
 	}
-	catch(exception &ex) {
+	//catch(const exception &ex) {
+	catch(...) {
 	}
 
 	return result;
@@ -430,7 +613,7 @@ map<string,string> Lang::getDiscoveredLanguageList(bool searchKeyIsLangName) {
 	vector<string> langResults2;
 	findAll(data_path + "data/lang/*.lng", langResults2, true);
 	if(langResults2.empty() && langResults.empty()) {
-        throw runtime_error("There are no lang files");
+        throw megaglest_runtime_error("There are no lang files");
 	}
 	for(unsigned int i = 0; i < langResults2.size(); ++i) {
 		string testLanguage = langResults2[i];
