@@ -20,6 +20,7 @@
 #include "game_constants.h"
 #include "util.h"
 #include "platform_common.h"
+#include "conversion.h"
 #include "leak_dumper.h"
 
 using namespace Shared::Xml;
@@ -166,7 +167,7 @@ void ParticleSystemType::load(const XmlNode *particleSystemNode, const string &d
 			if(modelNode->hasChild("cycles")) {
 				modelCycle = modelNode->getChild("cycles")->getAttribute("value")->getFloatValue();
 				if(modelCycle < 0.0)
-					throw runtime_error("negative model cycle value is bad");
+					throw megaglest_runtime_error("negative model cycle value is bad");
 			}
 		}
 	}
@@ -259,7 +260,7 @@ void ParticleSystemType::load(const XmlNode *particleSystemNode, const string &d
 	if(particleSystemNode->hasChild("child-particles")) {
     		const XmlNode *childrenNode= particleSystemNode->getChild("child-particles");
     		if(childrenNode->getAttribute("value")->getBoolValue()) {
-			for(int i = 0; i < childrenNode->getChildCount(); ++i) {
+			for(unsigned int i = 0; i < childrenNode->getChildCount(); ++i) {
 				const XmlNode *particleFileNode= childrenNode->getChild("particle-file",i);
 				string path= particleFileNode->getAttribute("path")->getRestrictedValue();
 				UnitParticleSystemType *unitParticleSystemType= new UnitParticleSystemType();
@@ -304,6 +305,63 @@ void ParticleSystemType::setValues(AttackParticleSystem *ats){
 	ats->setBlendMode(ParticleSystem::strToBlendMode(mode));
 }
 
+void ParticleSystemType::saveGame(XmlNode *rootNode) {
+	std::map<string,string> mapTagReplacements;
+	XmlNode *particleSystemTypeNode = rootNode->addChild("ParticleSystemType");
+
+//	string type;
+	particleSystemTypeNode->addAttribute("type",type, mapTagReplacements);
+//	Texture2D *texture;
+//	Model *model;
+//	float modelCycle;
+	particleSystemTypeNode->addAttribute("modelCycle",floatToStr(modelCycle), mapTagReplacements);
+//	string primitive;
+	particleSystemTypeNode->addAttribute("primitive",primitive, mapTagReplacements);
+//	Vec3f offset;
+	particleSystemTypeNode->addAttribute("offset",offset.getString(), mapTagReplacements);
+//	Vec4f color;
+	particleSystemTypeNode->addAttribute("color",color.getString(), mapTagReplacements);
+//	Vec4f colorNoEnergy;
+	particleSystemTypeNode->addAttribute("colorNoEnergy",colorNoEnergy.getString(), mapTagReplacements);
+//	float size;
+	particleSystemTypeNode->addAttribute("size",floatToStr(size), mapTagReplacements);
+//	float sizeNoEnergy;
+	particleSystemTypeNode->addAttribute("sizeNoEnergy",floatToStr(sizeNoEnergy), mapTagReplacements);
+//	float speed;
+	particleSystemTypeNode->addAttribute("speed",floatToStr(speed), mapTagReplacements);
+//	float gravity;
+	particleSystemTypeNode->addAttribute("gravity",floatToStr(gravity), mapTagReplacements);
+//	float emissionRate;
+	particleSystemTypeNode->addAttribute("emissionRate",floatToStr(emissionRate), mapTagReplacements);
+//	int energyMax;
+	particleSystemTypeNode->addAttribute("energyMax",intToStr(energyMax), mapTagReplacements);
+//	int energyVar;
+	particleSystemTypeNode->addAttribute("energyVar",intToStr(energyVar), mapTagReplacements);
+//	string mode;
+	particleSystemTypeNode->addAttribute("mode",mode, mapTagReplacements);
+//	bool teamcolorNoEnergy;
+	particleSystemTypeNode->addAttribute("teamcolorNoEnergy",intToStr(teamcolorNoEnergy), mapTagReplacements);
+//    bool teamcolorEnergy;
+	particleSystemTypeNode->addAttribute("teamcolorEnergy",intToStr(teamcolorEnergy), mapTagReplacements);
+//    int alternations;
+	particleSystemTypeNode->addAttribute("alternations",intToStr(alternations), mapTagReplacements);
+//    int particleSystemStartDelay;
+	particleSystemTypeNode->addAttribute("particleSystemStartDelay",intToStr(particleSystemStartDelay), mapTagReplacements);
+//	typedef std::list<UnitParticleSystemType*> Children;
+//	Children children;
+	for(Children::iterator it = children.begin(); it != children.end(); ++it) {
+		(*it)->saveGame(particleSystemTypeNode);
+	}
+//    bool minmaxEnabled;
+	particleSystemTypeNode->addAttribute("minmaxEnabled",intToStr(minmaxEnabled), mapTagReplacements);
+//    int minHp;
+	particleSystemTypeNode->addAttribute("minHp",intToStr(minHp), mapTagReplacements);
+//    int maxHp;
+	particleSystemTypeNode->addAttribute("maxHp",intToStr(maxHp), mapTagReplacements);
+//    bool minmaxIsPercent;
+	particleSystemTypeNode->addAttribute("minmaxIsPercent",intToStr(minmaxIsPercent), mapTagReplacements);
+}
+
 // ===========================================================
 //	class ParticleSystemTypeProjectile
 // ===========================================================
@@ -342,7 +400,8 @@ void ParticleSystemTypeProjectile::load(const XmlNode* particleFileNode, const s
 
 		//trajectory speed
 		const XmlNode *tajectorySpeedNode= tajectoryNode->getChild("speed");
-		trajectorySpeed= tajectorySpeedNode->getAttribute("value")->getFloatValue()/GameConstants::updateFps;
+		trajectorySpeed= tajectorySpeedNode->getAttribute("value")->getFloatValue() / (float)GameConstants::updateFps;
+		//printf("[%s] trajectorySpeed = %f\n",path.c_str(),trajectorySpeed);
 
 		if(trajectory=="parabolic" || trajectory=="spiral"){
 			//trajectory scale
@@ -364,7 +423,7 @@ void ParticleSystemTypeProjectile::load(const XmlNode* particleFileNode, const s
 	}
 	catch(const exception &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",__FILE__,__FUNCTION__,__LINE__,e.what());
-		throw runtime_error("Error loading ParticleSystem: "+ path + "\n" +e.what());
+		throw megaglest_runtime_error("Error loading ParticleSystem: "+ path + "\n" +e.what());
 	}
 }
 
@@ -374,6 +433,8 @@ ProjectileParticleSystem *ParticleSystemTypeProjectile::create() {
 	ParticleSystemType::setValues(ps);
 
 	ps->setTrajectory(ProjectileParticleSystem::strToTrajectory(trajectory));
+
+	//printf("Setting trajectorySpeed = %f\n",trajectorySpeed);
 	ps->setTrajectorySpeed(trajectorySpeed);
 	ps->setTrajectoryScale(trajectoryScale);
 	ps->setTrajectoryFrequency(trajectoryFrequency);
@@ -383,6 +444,22 @@ ProjectileParticleSystem *ParticleSystemTypeProjectile::create() {
 	return ps;
 }
 
+void ParticleSystemTypeProjectile::saveGame(XmlNode *rootNode) {
+	ParticleSystemType::saveGame(rootNode);
+
+	std::map<string,string> mapTagReplacements;
+	XmlNode *particleSystemTypeProjectileNode = rootNode->addChild("ParticleSystemTypeProjectile");
+
+//	string trajectory;
+	particleSystemTypeProjectileNode->addAttribute("trajectory",trajectory, mapTagReplacements);
+//	float trajectorySpeed;
+	particleSystemTypeProjectileNode->addAttribute("trajectorySpeed",floatToStr(trajectorySpeed), mapTagReplacements);
+//	float trajectoryScale;
+	particleSystemTypeProjectileNode->addAttribute("trajectoryScale",floatToStr(trajectoryScale), mapTagReplacements);
+//	float trajectoryFrequency;
+	particleSystemTypeProjectileNode->addAttribute("trajectoryFrequency",floatToStr(trajectoryFrequency), mapTagReplacements);
+}
+
 // ===========================================================
 //	class ParticleSystemTypeSplash
 // ===========================================================
@@ -432,7 +509,7 @@ void ParticleSystemTypeSplash::load(const XmlNode* particleFileNode, const strin
 	}
 	catch(const exception &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",__FILE__,__FUNCTION__,__LINE__,e.what());
-		throw runtime_error("Error loading ParticleSystem: "+ path + "\n" +e.what());
+		throw megaglest_runtime_error("Error loading ParticleSystem: "+ path + "\n" +e.what());
 	}
 }
 
@@ -452,4 +529,22 @@ SplashParticleSystem *ParticleSystemTypeSplash::create(){
 	return ps;
 }
 
+void ParticleSystemTypeSplash::saveGame(XmlNode *rootNode) {
+	ParticleSystemType::saveGame(rootNode);
+
+	std::map<string,string> mapTagReplacements;
+	XmlNode *particleSystemTypeSplashNode = rootNode->addChild("ParticleSystemTypeSplash");
+
+//	float emissionRateFade;
+	particleSystemTypeSplashNode->addAttribute("emissionRateFade",floatToStr(emissionRateFade), mapTagReplacements);
+//	float verticalSpreadA;
+	particleSystemTypeSplashNode->addAttribute("verticalSpreadA",floatToStr(verticalSpreadA), mapTagReplacements);
+//	float verticalSpreadB;
+	particleSystemTypeSplashNode->addAttribute("verticalSpreadB",floatToStr(verticalSpreadB), mapTagReplacements);
+//	float horizontalSpreadA;
+	particleSystemTypeSplashNode->addAttribute("horizontalSpreadA",floatToStr(horizontalSpreadA), mapTagReplacements);
+//	float horizontalSpreadB;
+	particleSystemTypeSplashNode->addAttribute("horizontalSpreadB",floatToStr(horizontalSpreadB), mapTagReplacements);
+}
+
 }}//end mamespace
