@@ -12,9 +12,13 @@
 #ifndef _GLEST_GAME_PARTICLETYPE_H_
 #define _GLEST_GAME_PARTICLETYPE_H_
 
+#ifdef WIN32
+    #include <winsock2.h>
+    #include <winsock.h>
+#endif
+
 #include <string>
 #include <list>
-
 #include "particle.h"
 #include "factory.h"
 #include "texture.h"
@@ -106,6 +110,8 @@ public:
 
     string getType() const { return type; };
 
+    virtual void saveGame(XmlNode *rootNode);
+
 protected:
 
 };
@@ -128,6 +134,7 @@ public:
 			string parentLoader, string techtreePath);
 	ProjectileParticleSystem *create();
 
+	virtual void saveGame(XmlNode *rootNode);
 };
 
 // ===========================================================
@@ -143,6 +150,8 @@ public:
 			string parentLoader, string techtreePath);
 	SplashParticleSystem *create();
 
+	virtual void saveGame(XmlNode *rootNode);
+
 private:
 	float emissionRateFade;
 	float verticalSpreadA;
