@@ -17,7 +17,7 @@
 #include "config.h"
 #include "game_constants.h"
 #include "platform_common.h"
-
+#include "conversion.h"
 #include "leak_dumper.h"
 
 using namespace Shared::Xml;
@@ -75,7 +75,7 @@ void UnitParticleSystemType::load(const XmlNode *particleSystemNode, const strin
 		direction.y= directionNode->getAttribute("y")->getFloatValue();
 		direction.z= directionNode->getAttribute("z")->getFloatValue();
 		if((shape == UnitParticleSystem::sConical) && (0.0 == direction.length()))
-			throw runtime_error("direction cannot be zero");
+			throw megaglest_runtime_error("direction cannot be zero");
 		// ought to warn about 0 directions generally
 	}
 
@@ -96,12 +96,12 @@ void UnitParticleSystemType::load(const XmlNode *particleSystemNode, const strin
 		const XmlNode *minRadiusNode= particleSystemNode->getChild("min-radius");
 		minRadius= minRadiusNode->getAttribute("value")->getFloatValue();
 		if(minRadius > radius)
-			throw runtime_error("min-radius cannot be bigger than radius");
+			throw megaglest_runtime_error("min-radius cannot be bigger than radius");
 	} else {
 		minRadius = 0;
 	}
 	if((minRadius == 0) && (shape == UnitParticleSystem::sConical)) {
-		minRadius = 0.001; // fudge it so we aren't generating particles that are exactly centred
+		minRadius = 0.001f; // fudge it so we aren't generating particles that are exactly centred
 		if(minRadius > radius)
 			radius = minRadius;
 	}
@@ -174,8 +174,8 @@ void UnitParticleSystemType::load(const XmlNode *particleSystemNode, const strin
 		const XmlNode* delayNode = particleSystemNode->getChild("delay");
 		const float delay_secs = delayNode->getAttribute("value")->getFloatValue();
 		if(delay_secs < 0)
-			throw runtime_error("particle effect delay cannot be negative");
-		delay = delay_secs * GameConstants::updateFps;
+			throw megaglest_runtime_error("particle effect delay cannot be negative");
+		delay = (int)delay_secs * GameConstants::updateFps;
 	} else{
 		delay= 0;
 	}
@@ -185,13 +185,23 @@ void UnitParticleSystemType::load(const XmlNode *particleSystemNode, const strin
 		const XmlNode* lifetimeNode = particleSystemNode->getChild("lifetime");
 		const float lifetime_secs = lifetimeNode->getAttribute("value")->getFloatValue();
 		if(lifetime_secs < 0 && lifetime_secs != -1)
-			throw runtime_error("particle effect lifetime cannot be negative (-1 means inherited from parent particle)");
-		lifetime = lifetime_secs * GameConstants::updateFps;
+			throw megaglest_runtime_error("particle effect lifetime cannot be negative (-1 means inherited from parent particle)");
+		lifetime = (int)lifetime_secs * GameConstants::updateFps;
 	} else{
 		lifetime= -1; //default
 	}
 }
 
+ObjectParticleSystemType::ObjectParticleSystemType() : UnitParticleSystemType() {
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s Line: %d] NEW [%p]\n",__FUNCTION__,__LINE__,this);
+}
+ObjectParticleSystemType::~ObjectParticleSystemType() {
+	if(SystemFlags::VERBOSE_MODE_ENABLED) {
+		printf("In [%s Line: %d] NEW [%p]\n",__FUNCTION__,__LINE__,this);
+		//printf("%s\n",PlatformExceptionHandler::getStackTrace().c_str());
+	}
+}
+
 const void UnitParticleSystemType::setValues(UnitParticleSystem *ups){
 	// whilst we extend ParticleSystemType we don't use ParticleSystemType::setValues()
 	// add instances of all children; some settings will cascade to all children
@@ -276,8 +286,52 @@ void UnitParticleSystemType::load(const XmlNode *particleFileNode, const string
 	}
 	catch(const exception &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",__FILE__,__FUNCTION__,__LINE__,e.what());
-		throw runtime_error("Error loading ParticleSystem: "+ path + "\n" +e.what());
+		throw megaglest_runtime_error("Error loading ParticleSystem: "+ path + "\n" +e.what());
 	}
 }
 
+void UnitParticleSystemType::saveGame(XmlNode *rootNode) {
+	ParticleSystemType::saveGame(rootNode);
+
+	std::map<string,string> mapTagReplacements;
+	XmlNode *unitParticleSystemTypeNode = rootNode->addChild("UnitParticleSystemType");
+
+//	UnitParticleSystem::Shape shape;
+	unitParticleSystemTypeNode->addAttribute("shape",intToStr(shape), mapTagReplacements);
+//	float angle;
+	unitParticleSystemTypeNode->addAttribute("angle",floatToStr(angle), mapTagReplacements);
+//	float radius;
+	unitParticleSystemTypeNode->addAttribute("radius",floatToStr(radius), mapTagReplacements);
+//	float minRadius;
+	unitParticleSystemTypeNode->addAttribute("minRadius",floatToStr(minRadius), mapTagReplacements);
+//	float emissionRateFade;
+	unitParticleSystemTypeNode->addAttribute("emissionRateFade",floatToStr(emissionRateFade), mapTagReplacements);
+//	Vec3f direction;
+	unitParticleSystemTypeNode->addAttribute("direction",direction.getString(), mapTagReplacements);
+//    bool relative;
+	unitParticleSystemTypeNode->addAttribute("relative",intToStr(relative), mapTagReplacements);
+//    bool relativeDirection;
+	unitParticleSystemTypeNode->addAttribute("relativeDirection",intToStr(relativeDirection), mapTagReplacements);
+//    bool fixed;
+	unitParticleSystemTypeNode->addAttribute("fixed",intToStr(fixed), mapTagReplacements);
+//    int staticParticleCount;
+	unitParticleSystemTypeNode->addAttribute("staticParticleCount",intToStr(staticParticleCount), mapTagReplacements);
+//	bool isVisibleAtNight;
+	unitParticleSystemTypeNode->addAttribute("isVisibleAtNight",intToStr(isVisibleAtNight), mapTagReplacements);
+//	bool isVisibleAtDay;
+	unitParticleSystemTypeNode->addAttribute("isVisibleAtDay",intToStr(isVisibleAtDay), mapTagReplacements);
+//	bool isDaylightAffected;
+	unitParticleSystemTypeNode->addAttribute("isDaylightAffected",intToStr(isDaylightAffected), mapTagReplacements);
+//	bool radiusBasedStartenergy;
+	unitParticleSystemTypeNode->addAttribute("radiusBasedStartenergy",intToStr(radiusBasedStartenergy), mapTagReplacements);
+//	int delay;
+	unitParticleSystemTypeNode->addAttribute("delay",intToStr(delay), mapTagReplacements);
+//	int lifetime;
+	unitParticleSystemTypeNode->addAttribute("lifetime",intToStr(lifetime), mapTagReplacements);
+//	float startTime;
+	unitParticleSystemTypeNode->addAttribute("startTime",floatToStr(startTime), mapTagReplacements);
+//	float endTime;
+	unitParticleSystemTypeNode->addAttribute("endTime",floatToStr(endTime), mapTagReplacements);
+}
+
 }}//end mamespace
