@@ -12,6 +12,11 @@
 #ifndef _GLEST_GAME_UNITPARTICLETYPE_H_
 #define _GLEST_GAME_UNITPARTICLETYPE_H_
 
+#ifdef WIN32
+    #include <winsock2.h>
+    #include <winsock.h>
+#endif
+
 #include <string>
 #include <list>
 
@@ -67,6 +72,7 @@ protected:
 
 public:
 	UnitParticleSystemType();
+	virtual ~UnitParticleSystemType() {};
 
 	void load(const XmlNode *particleSystemNode, const string &dir,
 			RendererInterface *newTexture, std::map<string,vector<pair<string, string> > > &loadedFileList,
@@ -82,9 +88,13 @@ public:
 
 	const void setValues (UnitParticleSystem *uts);
 	bool hasTexture() const { return(texture != NULL); }
+	virtual void saveGame(XmlNode *rootNode);
 };
 
-class ObjectParticleSystemType: public UnitParticleSystemType{
+class ObjectParticleSystemType: public UnitParticleSystemType {
+public:
+	ObjectParticleSystemType();
+	virtual ~ObjectParticleSystemType();
 };
 
 }}//end namespace
