@@ -110,7 +110,9 @@ Gui::Gui(){
 	lastQuadCalcFrame=0;
 	selectionCalculationFrameSkip=10;
 	minQuadSize=20;
-	selectedResourceObject=NULL;
+	selectedResourceObjectPos=Vec2i(-1,-1);
+	highlightedResourceObjectPos=Vec2i(-1,-1);
+	highlightedUnitId=-1;
 	hudTexture=NULL;
 	commander=NULL;
 	world=NULL;
@@ -145,6 +147,31 @@ const UnitType *Gui::getBuilding() const{
     assert(selectingBuilding);
     return choosenBuildingType;
 }
+const Object *Gui::getSelectedResourceObject()	const{
+	if(selectedResourceObjectPos.x==-1){
+		return NULL;
+	}
+	else {
+		return world->getMap()->getSurfaceCell(selectedResourceObjectPos)->getObject();
+	}
+}
+Object *Gui::getHighlightedResourceObject()	const{
+	if(highlightedResourceObjectPos.x==-1){
+		return NULL;
+	}
+	else {
+		return world->getMap()->getSurfaceCell(highlightedResourceObjectPos)->getObject();
+	}
+}
+
+Unit* Gui::getHighlightedUnit()	const{
+	if(highlightedUnitId==-1){
+		return NULL;
+	}
+	else {
+		return world->findUnitById(highlightedUnitId);
+	}
+}
 
 // ==================== is ====================
 
@@ -396,7 +423,7 @@ void Gui::onSelectionChanged(){
 void Gui::giveOneClickOrders(){
 	//printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
-	CommandResult result;
+	std::pair<CommandResult,string> result(crFailUndefined,"");
 	bool queueKeyDown = isKeyDown(queueCommandKey);
 	if(selection.isUniform()){
 		result= commander->tryGiveCommand(&selection, activeCommandType, Vec2i(0), (Unit*)NULL,  queueKeyDown);
@@ -431,11 +458,11 @@ void Gui::giveDefaultOrders(int x, int y,const Unit *targetUnit, bool paintMouse
 	bool queueKeyDown = isKeyDown(queueCommandKey);
 	Vec2i targetPos=Vec2i(x, y);
 	//give order
-	CommandResult result= commander->tryGiveCommand(&selection, targetPos, targetUnit, queueKeyDown);
+	std::pair<CommandResult,string> result= commander->tryGiveCommand(&selection, targetPos, targetUnit, queueKeyDown);
 
 	//graphical result
 	addOrdersResultToConsole(activeCommandClass, result);
-	if(result == crSuccess || result == crSomeFailed) {
+	if(result.first == crSuccess || result.first == crSomeFailed) {
 		if(paintMouse3d)
 			mouse3d.enable();
 
@@ -453,7 +480,7 @@ void Gui::giveDefaultOrders(int x, int y,const Unit *targetUnit, bool paintMouse
 
 void Gui::giveTwoClickOrders(int x, int y , bool prepared) {
 	//printf("In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
-	CommandResult result;
+	std::pair<CommandResult,string> result(crFailUndefined,"");
 
 	//compute target
 	const Unit *targetUnit= NULL;
@@ -489,7 +516,7 @@ void Gui::giveTwoClickOrders(int x, int y , bool prepared) {
 
 	//graphical result
 	addOrdersResultToConsole(activeCommandClass, result);
-	if(result == crSuccess || result == crSomeFailed) {
+	if(result.first == crSuccess || result.first == crSomeFailed) {
 		if(prepared == false) {
 			mouse3d.enable();
 		}
@@ -566,6 +593,7 @@ void Gui::clickCommonCommand(CommandClass commandClass){
 			break;
 		}
 	}
+	computeDisplay();
 }
 
 void Gui::mouseDownDisplayUnitSkills(int posDisplay) {
@@ -738,10 +766,10 @@ void Gui::computeDisplay(){
 	display.clear();
 
 	// ================ PART 1 ================
-
-	if(selection.isEmpty() && selectedResourceObject != NULL) {
+	const Object *selectedResourceObject =getSelectedResourceObject();
+	if(selection.isEmpty() && selectedResourceObject != NULL && selectedResourceObject->getResource() != NULL) {
 		Resource *r = selectedResourceObject->getResource();
-		display.setTitle(r->getType()->getName());
+		display.setTitle(r->getType()->getName(true));
 		display.setText(lang.get("Amount")+ ": "+intToStr(r->getAmount())+" / "+intToStr(r->getType()->getDefResPerPatch()));
 		//display.setProgressBar(r->);
 		display.setUpImage(0, r->getType()->getImage());
@@ -922,24 +950,24 @@ int Gui::computePosDisplay(int x, int y){
 	return posDisplay;
 }
 
-void Gui::addOrdersResultToConsole(CommandClass cc, CommandResult result){
+void Gui::addOrdersResultToConsole(CommandClass cc, std::pair<CommandResult,string> result) {
 
-    switch(result){
+    switch(result.first) {
 	case crSuccess:
 		break;
 	case crFailReqs:
         switch(cc){
         case ccBuild:
-            console->addStdMessage("BuildingNoReqs");
+            console->addStdMessage("BuildingNoReqs",result.second);
 		    break;
         case ccProduce:
-            console->addStdMessage("UnitNoReqs");
+            console->addStdMessage("UnitNoReqs",result.second);
             break;
         case ccMorph:
-            console->addStdMessage("MorphNoReqs");
+            console->addStdMessage("MorphNoReqs",result.second);
             break;
         case ccUpgrade:
-            console->addStdMessage("UpgradeNoReqs");
+            console->addStdMessage("UpgradeNoReqs",result.second);
             break;
         default:
             break;
@@ -948,16 +976,16 @@ void Gui::addOrdersResultToConsole(CommandClass cc, CommandResult result){
 	case crFailRes:
         switch(cc){
         case ccBuild:
-            console->addStdMessage("BuildingNoRes");
+            console->addStdMessage("BuildingNoRes",result.second);
 		    break;
         case ccProduce:
-            console->addStdMessage("UnitNoRes");
+            console->addStdMessage("UnitNoRes",result.second);
             break;
         case ccMorph:
-            console->addStdMessage("MorphNoRes");
+            console->addStdMessage("MorphNoRes",result.second);
             break;
         case ccUpgrade:
-            console->addStdMessage("UpgradeNoRes");
+            console->addStdMessage("UpgradeNoRes",result.second);
             break;
 		default:
 			break;
@@ -965,11 +993,11 @@ void Gui::addOrdersResultToConsole(CommandClass cc, CommandResult result){
 		break;
 
     case crFailUndefined:
-        console->addStdMessage("InvalidOrder");
+        console->addStdMessage("InvalidOrder",result.second);
         break;
 
     case crSomeFailed:
-        console->addStdMessage("SomeOrdersFailed");
+        console->addStdMessage("SomeOrdersFailed",result.second);
         break;
     }
 }
@@ -989,9 +1017,13 @@ void Gui::computeSelected(bool doubleClick, bool force){
 
 	if( force || ( lastQuadCalcFrame+selectionCalculationFrameSkip < game->getTotalRenderFps() ) ){
 		lastQuadCalcFrame=game->getTotalRenderFps();
-		selectedResourceObject=NULL;
+		const Object *selectedResourceObject=NULL;
+		selectedResourceObjectPos=Vec2i(-1,-1);
 		if(selectionQuad.isEnabled() && selectionQuad.getPosUp().dist(selectionQuad.getPosDown())<minQuadSize){
 			Renderer::getInstance().computeSelected(units, selectedResourceObject, true, selectionQuad.getPosDown(), selectionQuad.getPosDown());
+			if(selectedResourceObject!=NULL){
+				selectedResourceObjectPos=Map::toSurfCoords(selectedResourceObject->getMapPos());
+			}
 		}
 		else{
 			Renderer::getInstance().computeSelected(units, selectedResourceObject, false, selectionQuad.getPosDown(), selectionQuad.getPosUp());
@@ -1045,11 +1077,14 @@ bool Gui::computeTarget(const Vec2i &screenPos, Vec2i &targetPos, const Unit *&t
 	if(uc.empty() == false){
 		targetUnit= getRelevantObjectFromSelection(&uc);
 		targetPos= targetUnit->getPos();
+		highlightedUnitId=targetUnit->getId();
+		getHighlightedUnit()->resetHighlight();
 		return true;
 	}
 	else if(obj != NULL){
 		targetUnit= NULL;
-
+		highlightedResourceObjectPos=Map::toSurfCoords(obj->getMapPos());
+		getHighlightedResourceObject()->resetHighlight();
 		// get real click pos
 		renderer.computePosition(screenPos, targetPos);
 
@@ -1143,6 +1178,13 @@ bool Gui::computeTarget(const Vec2i &screenPos, Vec2i &targetPos, const Unit *&t
 		if(renderer.computePosition(screenPos, targetPos)){
 			validPosObjWorld= true;
 			posObjWorld= targetPos;
+
+			if(world->getMap()->getSurfaceCell(Map::toSurfCoords(targetPos))->getResource()!=NULL)
+			{
+				highlightedResourceObjectPos=Map::toSurfCoords(targetPos);
+				getHighlightedResourceObject()->resetHighlight();
+			}
+
 			return true;
 		}
 		else{
@@ -1162,10 +1204,142 @@ Unit* Gui::getRelevantObjectFromSelection(Selection::UnitContainer *uc){
 	return resultUnit;
 }
 
-void  Gui::removingObjectEvent(Object* o){
-	if(getSelectedResourceObject()==o){
-		selectedResourceObject=NULL;
+void Gui::saveGame(XmlNode *rootNode) const {
+	std::map<string,string> mapTagReplacements;
+	XmlNode *guiNode = rootNode->addChild("Gui");
+
+	//External objects
+//	RandomGen random;
+	guiNode->addAttribute("random",intToStr(random.getLastNumber()), mapTagReplacements);
+//	const Commander *commander;
+//	const World *world;
+//	const Game *game;
+//	GameCamera *gameCamera;
+//	Console *console;
+//
+//	//Positions
+//	Vec2i posObjWorld;		//world coords
+	guiNode->addAttribute("posObjWorld",posObjWorld.getString(), mapTagReplacements);
+//	bool validPosObjWorld;
+	guiNode->addAttribute("validPosObjWorld",intToStr(validPosObjWorld), mapTagReplacements);
+//	//display
+//	const UnitType *choosenBuildingType;
+	if(choosenBuildingType != NULL) {
+		const Faction* thisFaction= world->getThisFaction();
+		guiNode->addAttribute("choosenBuildingType",choosenBuildingType->getName(), mapTagReplacements);
+		guiNode->addAttribute("choosenBuildingTypeFactionIndex",intToStr(thisFaction->getIndex()), mapTagReplacements);
 	}
+//	const CommandType *activeCommandType;
+	if(activeCommandType != NULL) {
+		guiNode->addAttribute("activeCommandType",activeCommandType->getName(), mapTagReplacements);
+	}
+
+//	CommandClass activeCommandClass;
+	guiNode->addAttribute("activeCommandClass",intToStr(activeCommandClass), mapTagReplacements);
+//	int activePos;
+	guiNode->addAttribute("activePos",intToStr(activePos), mapTagReplacements);
+//	int lastPosDisplay;
+	guiNode->addAttribute("lastPosDisplay",intToStr(lastPosDisplay), mapTagReplacements);
+//	//composite
+//	Display display;
+	display.saveGame(guiNode);
+//	Mouse3d mouse3d;
+//	Selection selection;
+	selection.saveGame(guiNode);
+//	SelectionQuad selectionQuad;
+//	int lastQuadCalcFrame;
+	guiNode->addAttribute("lastQuadCalcFrame",intToStr(lastQuadCalcFrame), mapTagReplacements);
+//	int selectionCalculationFrameSkip;
+	guiNode->addAttribute("selectionCalculationFrameSkip",intToStr(selectionCalculationFrameSkip), mapTagReplacements);
+//	int minQuadSize;
+	guiNode->addAttribute("minQuadSize",intToStr(minQuadSize), mapTagReplacements);
+//	Chrono lastGroupRecallTime;
+	//guiNode->addAttribute("lastGroupRecallTime",intToStr(lastGroupRecallTime.getMillis()), mapTagReplacements);
+//	int lastGroupRecall;
+	guiNode->addAttribute("lastGroupRecall",intToStr(lastGroupRecall), mapTagReplacements);
+//	//states
+//	bool selectingBuilding;
+	guiNode->addAttribute("selectingBuilding",intToStr(selectingBuilding), mapTagReplacements);
+//	bool selectingPos;
+	guiNode->addAttribute("selectingPos",intToStr(selectingPos), mapTagReplacements);
+//	bool selectingMeetingPoint;
+	guiNode->addAttribute("selectingMeetingPoint",intToStr(selectingMeetingPoint), mapTagReplacements);
+//	CardinalDir selectedBuildingFacing;
+	guiNode->addAttribute("selectedBuildingFacing",intToStr(selectedBuildingFacing), mapTagReplacements);
+//	const Object *selectedResourceObject;
+//
+//	Texture2D* hudTexture;
+}
+
+void Gui::loadGame(const XmlNode *rootNode, World *world) {
+	const XmlNode *guiNode = rootNode->getChild("Gui");
+
+	//External objects
+//	RandomGen random;
+	random.setLastNumber(guiNode->getAttribute("random")->getIntValue());
+//	const Commander *commander;
+//	const World *world;
+//	const Game *game;
+//	GameCamera *gameCamera;
+//	Console *console;
+//
+//	//Positions
+//	Vec2i posObjWorld;		//world coords
+	//guiNode->addAttribute("posObjWorld",posObjWorld.getString(), mapTagReplacements);
+	posObjWorld = Vec2i::strToVec2(guiNode->getAttribute("posObjWorld")->getValue());
+//	bool validPosObjWorld;
+	validPosObjWorld = guiNode->getAttribute("validPosObjWorld")->getIntValue() != 0;
+//	//display
+//	const UnitType *choosenBuildingType;
+//	if(choosenBuildingType != NULL) {
+//		guiNode->addAttribute("choosenBuildingType",choosenBuildingType->getName(), mapTagReplacements);
+//	}
+	if(guiNode->hasAttribute("choosenBuildingType") == true) {
+		string unitType = guiNode->getAttribute("choosenBuildingType")->getValue();
+		int factionIndex = guiNode->getAttribute("choosenBuildingTypeFactionIndex")->getIntValue();
+		choosenBuildingType = world->getFaction(factionIndex)->getType()->getUnitType(unitType);
+	}
+//	const CommandType *activeCommandType;
+	//if(activeCommandType != NULL) {
+	//	guiNode->addAttribute("activeCommandType",activeCommandType->getName(), mapTagReplacements);
+	//}
+
+//	CommandClass activeCommandClass;
+	//guiNode->addAttribute("activeCommandClass",intToStr(activeCommandClass), mapTagReplacements);
+//	int activePos;
+	activePos = guiNode->getAttribute("activePos")->getIntValue();
+//	int lastPosDisplay;
+	lastPosDisplay = guiNode->getAttribute("lastPosDisplay")->getIntValue();
+//	//composite
+//	Display display;
+	display.loadGame(guiNode);
+//	Mouse3d mouse3d;
+//	Selection selection;
+	selection.loadGame(guiNode,world);
+//	SelectionQuad selectionQuad;
+//	int lastQuadCalcFrame;
+	// don't load this! lastQuadCalcFrame = guiNode->getAttribute("lastQuadCalcFrame")->getIntValue();
+	lastQuadCalcFrame = game->getTotalRenderFps();
+//	int selectionCalculationFrameSkip;
+	selectionCalculationFrameSkip = guiNode->getAttribute("selectionCalculationFrameSkip")->getIntValue();
+//	int minQuadSize;
+	minQuadSize = guiNode->getAttribute("minQuadSize")->getIntValue();
+//	Chrono lastGroupRecallTime;
+	//guiNode->addAttribute("lastGroupRecallTime",intToStr(lastGroupRecallTime.getMillis()), mapTagReplacements);
+//	int lastGroupRecall;
+	lastGroupRecall = guiNode->getAttribute("lastGroupRecall")->getIntValue();
+//	//states
+//	bool selectingBuilding;
+	//selectingBuilding = guiNode->getAttribute("selectingBuilding")->getIntValue();
+//	bool selectingPos;
+	//guiNode->addAttribute("selectingPos",intToStr(selectingPos), mapTagReplacements);
+//	bool selectingMeetingPoint;
+	//guiNode->addAttribute("selectingMeetingPoint",intToStr(selectingMeetingPoint), mapTagReplacements);
+//	CardinalDir selectedBuildingFacing;
+	//guiNode->addAttribute("selectedBuildingFacing",intToStr(selectedBuildingFacing), mapTagReplacements);
+//	const Object *selectedResourceObject;
+//
+//	Texture2D* hudTexture;
 }
 
 }}//end namespace
