@@ -24,6 +24,9 @@
 //#include "glm.h"
 //#include "md5util.h"
 //#include "Mathlib.h"
+
+#include "video_player.h"
+
 #include "leak_dumper.h"
 
 using namespace Shared::Util;
@@ -90,6 +93,14 @@ Intro::Intro(Program *program):
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
 	CoreData &coreData= CoreData::getInstance();
+	SoundRenderer &soundRenderer= SoundRenderer::getInstance();
+
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false &&
+		(Shared::Graphics::VideoPlayer::hasBackEndVideoPlayer() == false ||
+		CoreData::getInstance().hasIntroVideoFilename() == false)) {
+		soundRenderer.playMusic(CoreData::getInstance().getIntroMusic());
+	}
+
 	const Metrics &metrics= Metrics::getInstance();
 	int w= metrics.getVirtualW();
 	int h= metrics.getVirtualH();
@@ -97,6 +108,7 @@ Intro::Intro(Program *program):
 	mouseX = 0;
 	mouseY = 0;
 	mouse2d = 0;
+	exitAfterIntroVideo = false;
 
 	Renderer &renderer= Renderer::getInstance();
 	//renderer.init3dListMenu(NULL);
@@ -204,7 +216,7 @@ Intro::Intro(Program *program):
 			}
 		}
 
-		if(models.size() == 0) {
+		if(models.empty() == true) {
 			introPath = data_path + "data/core/menu/main_model/intro*.g3d";
 			//vector<string> introModels;
 			findAll(introPath, introModels, false, false);
@@ -222,23 +234,22 @@ Intro::Intro(Program *program):
 		if(showIntroModelsRandom == true) {
 			std::vector<Model *> modelList;
 
-			unsigned int seed = time(NULL);
-			srand(seed);
+			//unsigned int seed = time(NULL);
+			Chrono seed(true);
+			srand((unsigned int)seed.getCurTicks());
 			int failedLookups=0;
 			std::map<int,bool> usedIndex;
 			for(;modelList.size() < models.size();) {
 				int index = rand() % models.size();
 				if(usedIndex.find(index) != usedIndex.end()) {
 					failedLookups++;
-					seed = time(NULL) / failedLookups;
-					srand(seed);
+					srand((unsigned int)seed.getCurTicks() / failedLookups);
 					continue;
 				}
 				//printf("picIndex = %d list count = %d\n",picIndex,coreData.getMiscTextureList().size());
 				modelList.push_back(models[index]);
 				usedIndex[index]=true;
-				seed = time(NULL) / modelList.size();
-				srand(seed);
+				srand((unsigned int)seed.getCurTicks() / modelList.size());
 			}
 			models = modelList;
 		}
@@ -413,23 +424,34 @@ Intro::Intro(Program *program):
 
 		std::vector<Texture2D *> intoTexList;
 		if(showIntroPicsRandom == true) {
-			unsigned int seed = time(NULL);
-			srand(seed);
+			//unsigned int seed = time(NULL);
+			Chrono seed(true);
+			srand((unsigned int)seed.getCurTicks());
 			int failedLookups=0;
 			std::map<int,bool> usedIndex;
 			for(;intoTexList.size() < showIntroPics;) {
 				int picIndex = rand() % coreData.getMiscTextureList().size();
 				if(usedIndex.find(picIndex) != usedIndex.end()) {
 					failedLookups++;
-					seed = time(NULL) / failedLookups;
-					srand(seed);
-					continue;
+					srand((unsigned int)seed.getCurTicks() / failedLookups);
+
+					if(failedLookups > 10000) {
+						for(unsigned int i = 0;
+								i < coreData.getMiscTextureList().size(); ++i) {
+							if(usedIndex.find(i) == usedIndex.end()) {
+								picIndex = i;
+								break;
+							}
+						}
+					}
+					else {
+						continue;
+					}
 				}
 				//printf("picIndex = %d list count = %d\n",picIndex,coreData.getMiscTextureList().size());
 				intoTexList.push_back(coreData.getMiscTextureList()[picIndex]);
 				usedIndex[picIndex]=true;
-				seed = time(NULL) / intoTexList.size();
-				srand(seed);
+				srand((unsigned int)seed.getCurTicks() / intoTexList.size());
 			}
 		}
 		else {
@@ -480,15 +502,41 @@ Intro::Intro(Program *program):
 		}
 	}
 
-	//test = NULL;
-	//Shared::Graphics::md5::initMD5OpenGL(data_path + "data/core/shaders/");
-	//md5Test = Shared::Graphics::md5::getMD5ObjectFromLoaderScript("/home/softcoder/Code/megaglest/trunk/mk/linux/mydata/test/mv1/mv1.loader");
-
-	SoundRenderer &soundRenderer= SoundRenderer::getInstance();
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false) {
+		renderer.swapBuffers();
+	}
 
 	SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
-	soundRenderer.playMusic(CoreData::getInstance().getIntroMusic());
+	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false &&
+		Shared::Graphics::VideoPlayer::hasBackEndVideoPlayer() == true &&
+		CoreData::getInstance().hasIntroVideoFilename() == true) {
+		string introVideoFile = CoreData::getInstance().getIntroVideoFilename();
+		string introVideoFileFallback = CoreData::getInstance().getIntroVideoFilenameFallback();
+
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Intro Video [%s] [%s]\n",introVideoFile.c_str(),introVideoFileFallback.c_str());
+
+		Context *c= GraphicsInterface::getInstance().getCurrentContext();
+		SDL_Surface *screen = static_cast<ContextGl*>(c)->getPlatformContextGlPtr()->getScreen();
+
+		string vlcPluginsPath = Config::getInstance().getString("VideoPlayerPluginsPath","");
+		//printf("screen->w = %d screen->h = %d screen->format->BitsPerPixel = %d\n",screen->w,screen->h,screen->format->BitsPerPixel);
+		Shared::Graphics::VideoPlayer player(
+				&Renderer::getInstance(),
+				introVideoFile,
+				introVideoFileFallback,
+				screen,
+				0,0,
+				screen->w,
+				screen->h,
+				screen->format->BitsPerPixel,
+				false,
+				vlcPluginsPath,
+				SystemFlags::VERBOSE_MODE_ENABLED);
+		player.PlayVideo();
+		exitAfterIntroVideo = true;
+		return;
+	}
 
 	SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 }
@@ -505,6 +553,11 @@ Intro::~Intro() {
 }
 
 void Intro::update() {
+	if(exitAfterIntroVideo == true) {
+		mouseUpLeft(0, 0);
+		//cleanup();
+		return;
+	}
 	timer++;
 	if(timer > introTime * GameConstants::updateFps / 1000){
 	    SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
@@ -517,6 +570,7 @@ void Intro::update() {
 	    SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
 		AutoTest::getInstance().updateIntro(program);
+		return;
 
 		SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 	}
@@ -614,6 +668,7 @@ void Intro::render() {
 	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == true) {
 		return;
 	}
+
 	int difTime=0;
 
 	canRender();
@@ -625,60 +680,9 @@ void Intro::render() {
 	renderer.clearZBuffer();
 	renderer.loadCameraMatrix(&camera);
 
-//	const Vec3f &position= camera.getConstPosition();
-//	Quaternion orientation= camera.getOrientation().conjugate();
-//	Shared::Graphics::md5::Matrix4x4f modelViewMatrix;
-//	float *mtx = orientation.toMatrix4().ptr();
-//	for(unsigned int i = 0; i < 16; ++i) {
-//		modelViewMatrix._m[i] = mtx[i];
-//	}
-
 	renderModelBackground();
 	renderer.renderParticleManager(rsMenu);
 
-	//printf("animTimer.deltaTime () = %f anim = %f animTimer.deltaTime() / 25.0 = %f\n",animTimer.deltaTime (),anim,animTimer.deltaTime() / 25.0);
-	//double anim = animTimer.deltaTime();
-	//Shared::Graphics::md5::renderMD5Object(md5Test, animTimer.deltaTime() / 30.0, &modelViewMatrix);
-
-//	if(test == NULL) {
-//		glClearColor (0.0, 0.0, 0.0, 0.0);
-//		glEnable(GL_DEPTH_TEST);
-//		glShadeModel (GL_SMOOTH);
-//
-//		test = glmReadOBJ("/home/softcoder/Code/megaglest/trunk/mk/linux/r_stack_fall.obj");
-//		glmUnitize(test);
-//		glmFacetNormals(test);
-//		glmVertexNormals(test, 90.0);
-//
-//		int h = 900;
-//		int w = 1680;
-//		glViewport (0, 0, (GLsizei) w, (GLsizei) h);
-//		glMatrixMode (GL_PROJECTION);
-//		glLoadIdentity ();
-//		gluPerspective(60.0, (GLfloat) w/(GLfloat) h, 1.0, 20.0);
-//		glMatrixMode (GL_MODELVIEW);
-//	}
-//	if(test != NULL) {
-//
-//		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-//		glLoadIdentity();
-//		glTranslatef(0,0,-5);
-//
-//		glPushMatrix();
-//		// I added these to be able to rotate the whole scene so you can see the box and textures
-//		glRotatef(90,1,0,0);
-//		glRotatef(0,0,1,0);
-//		glRotatef(0,0,0,1);
-//		glmDraw(test, GLM_SMOOTH| GLM_TEXTURE);
-//		//glmDraw(test, GLM_SMOOTH| GLM_TEXTURE|GLM_COLOR);
-//		//glmDraw(test, GLM_FLAT);
-//		glPopMatrix();
-//
-//		renderer.swapBuffers();
-//		return;
-//		//printf("Rendering test");
-//	}
-
 	renderer.reset2d();
 
 	for(int i = 0; i < texts.size(); ++i) {
@@ -804,7 +808,9 @@ void Intro::mouseUpLeft(int x, int y) {
 
 	SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
-	soundRenderer.playMusic(CoreData::getInstance().getMenuMusic());
+	if(CoreData::getInstance().hasMainMenuVideoFilename() == false) {
+		soundRenderer.playMusic(CoreData::getInstance().getMenuMusic());
+	}
 
 	SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
