@@ -106,7 +106,7 @@ MenuStateKeysetup::MenuStateKeysetup(Program *program, MainMenu *mainMenu,
 
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
-		//throw runtime_error("Test!");
+		//throw megaglest_runtime_error("Test!");
 
 		for(int i = 0; i < mergedProperties.size(); ++i) {
 
@@ -149,8 +149,8 @@ MenuStateKeysetup::MenuStateKeysetup(Program *program, MainMenu *mainMenu,
 		keyScrollBar.setVisibleStart(0);
 	}
 	catch(const std::exception &ex) {
-		char szBuf[4096]="";
-		sprintf(szBuf,"In [%s::%s %d] Error detected:\n%s\n",__FILE__,__FUNCTION__,__LINE__,ex.what());
+		char szBuf[8096]="";
+		snprintf(szBuf,8096,"In [%s::%s %d] Error detected:\n%s\n",__FILE__,__FUNCTION__,__LINE__,ex.what());
 		SystemFlags::OutputDebug(SystemFlags::debugError,szBuf);
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"%s",szBuf);
 
@@ -217,11 +217,11 @@ void MenuStateKeysetup::mouseClick(int x, int y, MouseButton mouseButton){
 	SoundRenderer &soundRenderer= SoundRenderer::getInstance();
 
 	if(mainMessageBox.getEnabled()){
-		int button= 1;
+		int button= 0;
 		if(mainMessageBox.mouseClick(x, y, button))
 		{
 			soundRenderer.playFx(coreData.getClickSoundA());
-			if(button==1)
+			if(button==0)
 			{
 				mainMessageBox.setEnabled(false);
 			}
@@ -274,8 +274,8 @@ void MenuStateKeysetup::mouseClick(int x, int y, MouseButton mouseButton){
         if(userProperties.empty() == false) {
 			Config &configKeys = Config::getInstance(std::pair<ConfigType,ConfigType>(cfgMainKeys,cfgUserKeys));
 			string userKeysFile = configKeys.getFileName(true);
-	        if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] save file [%s] userProperties.size() = %lu\n",__FILE__,__FUNCTION__,__LINE__,userKeysFile.c_str(),userProperties.size());
-	        if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] save file [%s] userProperties.size() = %lu\n",__FILE__,__FUNCTION__,__LINE__,userKeysFile.c_str(),userProperties.size());
+	        if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] save file [%s] userProperties.size() = " MG_SIZE_T_SPECIFIER "\n",__FILE__,__FUNCTION__,__LINE__,userKeysFile.c_str(),userProperties.size());
+	        if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] save file [%s] userProperties.size() = " MG_SIZE_T_SPECIFIER "\n",__FILE__,__FUNCTION__,__LINE__,userKeysFile.c_str(),userProperties.size());
 
 			configKeys.setUserProperties(userProperties);
 			configKeys.save();
@@ -304,6 +304,12 @@ void MenuStateKeysetup::mouseClick(int x, int y, MouseButton mouseButton){
     }
 }
 
+void MenuStateKeysetup::mouseUp(int x, int y, const MouseButton mouseButton){
+	if (mouseButton == mbLeft) {
+		keyScrollBar.mouseUp(x, y);
+	}
+}
+
 void MenuStateKeysetup::mouseMove(int x, int y, const MouseState *ms){
     buttonReturn.mouseMove(x, y);
     buttonOk.mouseMove(x, y);
@@ -422,12 +428,11 @@ void MenuStateKeysetup::keyDown(SDL_KeyboardEvent key) {
 //	}
 
 	char szCharText[20]="";
-	sprintf(szCharText,"%c",hotkeyChar);
-	char *utfStr = String::ConvertToUTF8(&szCharText[0]);
+	snprintf(szCharText,20,"%c",hotkeyChar);
+	char *utfStr = ConvertToUTF8(&szCharText[0]);
 
-	char szBuf[1024] = "";
-	//sprintf(szBuf,"%s [%d][%d]",keyName.c_str(),key.keysym.sym,keysym);
-	sprintf(szBuf,"%s [%s][%d][%d][%d]",keyName.c_str(),utfStr,key.keysym.sym,hotkeyChar,key.keysym.unicode);
+	char szBuf[8096] = "";
+	snprintf(szBuf,8096,"%s [%s][%d][%d][%d][%d]",keyName.c_str(),utfStr,key.keysym.sym,hotkeyChar,key.keysym.unicode,key.keysym.mod);
 	labelTestValue.setText(szBuf);
 
 	delete [] utfStr;
