@@ -230,7 +230,7 @@ void ConnectionSlotThread::execute() {
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",__FILE__,__FUNCTION__,__LINE__,ex.what());
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
-		throw runtime_error(ex.what());
+		throw megaglest_runtime_error(ex.what());
 	}
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s] Line: %d\n",__FILE__,__FUNCTION__,__LINE__);
 }
@@ -243,6 +243,7 @@ ConnectionSlot::ConnectionSlot(ServerInterface* serverInterface, int playerIndex
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s] Line: %d\n",__FILE__,__FUNCTION__,__LINE__);
 
 	this->mutexSocket = new Mutex();
+	this->socket = NULL;
 	this->mutexCloseConnection = new Mutex();
 	this->mutexPendingNetworkCommandList = new Mutex();
 	this->socketSynchAccessor = new Mutex();
@@ -250,13 +251,14 @@ ConnectionSlot::ConnectionSlot(ServerInterface* serverInterface, int playerIndex
 	this->sessionKey 		= 0;
 	this->serverInterface	= serverInterface;
 	this->playerIndex		= playerIndex;
-	this->playerStatus		= 0;
+	this->playerStatus		= npst_None;
 	this->playerLanguage	= "";
 	this->currentFrameCount = 0;
 	this->currentLagCount	= 0;
 	this->gotLagCountWarning = false;
 	this->lastReceiveCommandListTime	= 0;
 	this->receivedNetworkGameStatus = false;
+	this->canAcceptConnections = true;
 
 	this->setSocket(NULL);
 	this->slotThreadWorker 	= NULL;
@@ -366,7 +368,7 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 			this->setReceivedDataSynchCheck(false);
 
 			// Is the listener socket ready to be read?
-			if(checkForNewClients == true) {
+			if(checkForNewClients == true && this->canAcceptConnections == true) {
 
 				//if(chrono.getMillis() > 1) printf("In [%s::%s Line: %d] action running for msecs: %lld\n",__FILE__,__FUNCTION__,__LINE__,(long long int)chrono.getMillis());
 
@@ -451,7 +453,9 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 					if(this->isConnected() == true) {
 					//RandomGen random;
 					//sessionKey = random.randRange(-100000, 100000);
-					srand(time(NULL) / (this->playerIndex + 1));
+					Chrono seed(true);
+					srand((unsigned int)seed.getCurTicks() / (this->playerIndex + 1));
+
 					sessionKey = rand() % 1000000;
 
 					if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] accepted new client connection, serverInterface->getOpenSlotCount() = %d, sessionKey = %d\n",__FILE__,__FUNCTION__,__LINE__,serverInterface->getOpenSlotCount(),sessionKey);
@@ -460,8 +464,16 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 						if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] !!!!!!!!WARNING - no open slots, disconnecting client\n",__FILE__,__FUNCTION__,__LINE__);
 
 						//if(this->getSocket() != NULL) {
-							NetworkMessageIntro networkMessageIntro(sessionKey,getNetworkVersionSVNString(), getHostName(), playerIndex, nmgstNoSlots, 0, ServerSocket::getFTPServerPort(),"");
-							sendMessage(&networkMessageIntro);
+						NetworkMessageIntro networkMessageIntro(
+								sessionKey,
+								getNetworkVersionSVNString(),
+								getHostName(),
+								playerIndex,
+								nmgstNoSlots,
+								0,
+								ServerSocket::getFTPServerPort(),
+								"");
+						sendMessage(&networkMessageIntro);
 						//}
 
 						//if(chrono.getMillis() > 1) printf("In [%s::%s Line: %d] action running for msecs: %lld\n",__FILE__,__FUNCTION__,__LINE__,(long long int)chrono.getMillis());
@@ -472,8 +484,16 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 						if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] client will be assigned to the next open slot\n",__FILE__,__FUNCTION__,__LINE__);
 
 						//if(this->getSocket() != NULL) {
-							NetworkMessageIntro networkMessageIntro(sessionKey,getNetworkVersionSVNString(), getHostName(), playerIndex, nmgstOk, 0, ServerSocket::getFTPServerPort(),"");
-							sendMessage(&networkMessageIntro);
+						NetworkMessageIntro networkMessageIntro(
+								sessionKey,
+								getNetworkVersionSVNString(),
+								getHostName(),
+								playerIndex,
+								nmgstOk,
+								0,
+								ServerSocket::getFTPServerPort(),
+								"");
+						sendMessage(&networkMessageIntro);
 
 							//if(chrono.getMillis() > 1) printf("In [%s::%s Line: %d] action running for msecs: %lld\n",__FILE__,__FUNCTION__,__LINE__,(long long int)chrono.getMillis());
 						//}
@@ -495,7 +515,14 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 				this->clearChatInfo();
 
 				bool gotTextMsg = true;
-				for(;this->hasDataToRead() == true && gotTextMsg == true;) {
+				bool gotCellMarkerMsg = true;
+				bool waitForLaggingClient = false;
+				bool waitedForLaggingClient = false;
+
+				for(;waitForLaggingClient == true ||
+						(this->hasDataToRead() == true &&
+						 (gotTextMsg == true || gotCellMarkerMsg == true));) {
+					waitForLaggingClient = false;
 					if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] polling for networkMessageType...\n",__FILE__,__FUNCTION__,__LINE__);
 
 					NetworkMessageType networkMessageType= getNextMessageType();
@@ -503,6 +530,7 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 					if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] networkMessageType = %d\n",__FILE__,__FUNCTION__,__LINE__,networkMessageType);
 
 					gotTextMsg = false;
+					gotCellMarkerMsg = false;
 					//process incoming commands
 					switch(networkMessageType) {
 
@@ -559,12 +587,101 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 						}
 						break;
 
+						case nmtMarkCell:
+						{
+							if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] got nmtMarkCell gotIntro = %d\n",__FILE__,__FUNCTION__,__LINE__,gotIntro);
+
+							if(gotIntro == true) {
+								NetworkMessageMarkCell networkMessageMarkCell;
+								if(receiveMessage(&networkMessageMarkCell)) {
+					            	MarkedCell msg(networkMessageMarkCell.getTarget(),
+					            			       networkMessageMarkCell.getFactionIndex(),
+					            			       networkMessageMarkCell.getText().c_str(),
+					            			       networkMessageMarkCell.getPlayerIndex());
+
+					            	this->addMarkedCell(msg);
+					            	gotCellMarkerMsg = true;
+								}
+								else {
+									if(SystemFlags::getSystemSettingType(SystemFlags::debugError).enabled) SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d]\nInvalid message type before intro handshake [%d]\nDisconnecting socket for slot: %d [%s].\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,networkMessageType,this->playerIndex,this->getIpAddress().c_str());
+									this->serverInterface->notifyBadClientConnectAttempt(this->getIpAddress());
+									close();
+									return;
+								}
+							}
+							else {
+								if(SystemFlags::getSystemSettingType(SystemFlags::debugError).enabled) SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d]\nInvalid message type before intro handshake [%d]\nDisconnecting socket for slot: %d [%s].\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,networkMessageType,this->playerIndex,this->getIpAddress().c_str());
+								this->serverInterface->notifyBadClientConnectAttempt(this->getIpAddress());
+								close();
+								return;
+							}
+						}
+						break;
+
+						case nmtUnMarkCell:
+						{
+							if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] got nmtUnMarkCell gotIntro = %d\n",__FILE__,__FUNCTION__,__LINE__,gotIntro);
+
+							if(gotIntro == true) {
+								NetworkMessageUnMarkCell networkMessageMarkCell;
+								if(receiveMessage(&networkMessageMarkCell)) {
+					            	UnMarkedCell msg(networkMessageMarkCell.getTarget(),
+					            			       networkMessageMarkCell.getFactionIndex());
+
+					            	this->addUnMarkedCell(msg);
+					            	gotCellMarkerMsg = true;
+								}
+								else {
+									if(SystemFlags::getSystemSettingType(SystemFlags::debugError).enabled) SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d]\nInvalid message type before intro handshake [%d]\nDisconnecting socket for slot: %d [%s].\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,networkMessageType,this->playerIndex,this->getIpAddress().c_str());
+									this->serverInterface->notifyBadClientConnectAttempt(this->getIpAddress());
+									close();
+									return;
+								}
+							}
+							else {
+								if(SystemFlags::getSystemSettingType(SystemFlags::debugError).enabled) SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d]\nInvalid message type before intro handshake [%d]\nDisconnecting socket for slot: %d [%s].\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,networkMessageType,this->playerIndex,this->getIpAddress().c_str());
+								this->serverInterface->notifyBadClientConnectAttempt(this->getIpAddress());
+								close();
+								return;
+							}
+						}
+						break;
+
+						case nmtHighlightCell:
+						{
+							if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] got nmtMarkCell gotIntro = %d\n",__FILE__,__FUNCTION__,__LINE__,gotIntro);
+
+							if(gotIntro == true) {
+								NetworkMessageHighlightCell networkMessageHighlightCell;
+								if(receiveMessage(&networkMessageHighlightCell)) {
+					            	MarkedCell msg(networkMessageHighlightCell.getTarget(),
+					            			networkMessageHighlightCell.getFactionIndex(),"none",-1);
+
+					            	this->setHighlightedCell(msg);
+					            	gotCellMarkerMsg = true;
+								}
+								else {
+									if(SystemFlags::getSystemSettingType(SystemFlags::debugError).enabled) SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d]\nInvalid message type before intro handshake [%d]\nDisconnecting socket for slot: %d [%s].\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,networkMessageType,this->playerIndex,this->getIpAddress().c_str());
+									this->serverInterface->notifyBadClientConnectAttempt(this->getIpAddress());
+									close();
+									return;
+								}
+							}
+							else {
+								if(SystemFlags::getSystemSettingType(SystemFlags::debugError).enabled) SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d]\nInvalid message type before intro handshake [%d]\nDisconnecting socket for slot: %d [%s].\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,networkMessageType,this->playerIndex,this->getIpAddress().c_str());
+								this->serverInterface->notifyBadClientConnectAttempt(this->getIpAddress());
+								close();
+								return;
+							}
+						}
+						break;
+
 						//command list
 						case nmtCommandList: {
 
 							if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] got nmtCommandList gotIntro = %d\n",__FILE__,__FUNCTION__,__LINE__,gotIntro);
 
-							//throw runtime_error("test");
+							//throw megaglest_runtime_error("test");
 
 							if(gotIntro == true) {
 								NetworkMessageCommandList networkMessageCommandList;
@@ -602,7 +719,7 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 
 							NetworkMessageIntro networkMessageIntro;
 							if(receiveMessage(&networkMessageIntro)) {
-								int msgSessionId = networkMessageIntro.getSessionId();
+								int32 msgSessionId = networkMessageIntro.getSessionId();
 								this->name= networkMessageIntro.getName();
 								this->versionString = networkMessageIntro.getVersionString();
 								this->connectedRemoteIPAddress = networkMessageIntro.getExternalIp();
@@ -622,7 +739,12 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 								}
 								else {
 									//check consistency
+									if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+
 									bool compatible = checkVersionComptability(getNetworkVersionSVNString(), networkMessageIntro.getVersionString());
+
+									if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+
 									if(compatible == false) {
 										if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
@@ -710,20 +832,47 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 
 										char szBuf[1024]="";
 										snprintf(szBuf,1023,"In [%s::%s Line: %d] Invalid networkMessageLaunch.getMessageType() = %d",__FILE__,__FUNCTION__,__LINE__,networkMessageLaunch.getMessageType());
-										throw runtime_error(szBuf);
+										throw megaglest_runtime_error(szBuf);
 									}
 
-									GameSettings gameSettingsBuffer;
-									networkMessageLaunch.buildGameSettings(&gameSettingsBuffer);
+									int minHeadLessPlayersRequired = Config::getInstance().getInt("MinHeadlessPlayersRequired","2");
+									if(networkMessageLaunch.getMessageType() == nmtLaunch &&
+											this->serverInterface->getConnectedSlotCount() < minHeadLessPlayersRequired) {
+								    	Lang &lang= Lang::getInstance();
+								    	const vector<string> languageList = this->serverInterface->getGameSettings()->getUniqueNetworkPlayerLanguages();
+								    	for(unsigned int i = 0; i < languageList.size(); ++i) {
+								    		char szBuf[4096]="";
+											string msgTemplate = "You must have have at least %d player(s) connected to start this game!";
+											if(lang.hasString("HeadlessAdminRequiresMorePlayers",languageList[i]) == true) {
+												msgTemplate = lang.get("HeadlessAdminRequiresMorePlayers",languageList[i]);
+											}
+					#ifdef WIN32
+											_snprintf(szBuf,4095,msgTemplate.c_str(),minHeadLessPlayersRequired);
+					#else
+											snprintf(szBuf,4095,msgTemplate.c_str(),minHeadLessPlayersRequired);
+					#endif
+											if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] %s\n",__FILE__,__FUNCTION__,__LINE__,szBuf);
 
-									//printf("Connection slot got networkMessageLaunch.getMessageType() = %d, got map [%s]\n",networkMessageLaunch.getMessageType(),gameSettings.getMap().c_str());
-									//printf("\n\n\n\n=====Connection slot got settings:\n%s\n",gameSettings.toString().c_str());
+											string sMsg = szBuf;
+											bool echoLocal = lang.isLanguageLocal(languageList[i]);
+											this->serverInterface->sendTextMessage(sMsg,-1, echoLocal, languageList[i], this->getPlayerIndex());
 
-									//this->serverInterface->setGameSettings(&gameSettingsBuffer,false);
-									this->serverInterface->broadcastGameSetup(&gameSettingsBuffer, true);
+											//printf("Lang [%s] msgTemplate [%s] echoLocal = %d\n",languageList[i].c_str(),msgTemplate.c_str(),echoLocal);
+								    	}
+									}
+									else {
+										GameSettings gameSettingsBuffer;
+										networkMessageLaunch.buildGameSettings(&gameSettingsBuffer);
 
-									if(networkMessageLaunch.getMessageType() == nmtLaunch) {
-										this->serverInterface->setMasterserverAdminRequestLaunch(true);
+										//printf("Connection slot got networkMessageLaunch.getMessageType() = %d, got map [%s]\n",networkMessageLaunch.getMessageType(),gameSettings.getMap().c_str());
+										//printf("\n\n\n\n=====Connection slot got settings:\n%s\n",gameSettings.toString().c_str());
+
+										//this->serverInterface->setGameSettings(&gameSettingsBuffer,false);
+										this->serverInterface->broadcastGameSetup(&gameSettingsBuffer, true);
+
+										if(networkMessageLaunch.getMessageType() == nmtLaunch) {
+											this->serverInterface->setMasterserverAdminRequestLaunch(true);
+										}
 									}
 								}
 								else {
@@ -768,12 +917,12 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 									}
 
 									//tileset
-									int32 tilesetCRC = getFolderTreeContentsCheckSumRecursively(config.getPathListForType(ptTilesets,scenarioDir), string("/") + serverInterface->getGameSettings()->getTileset() + string("/*"), ".xml", NULL);
-									int32 techCRC    = getFolderTreeContentsCheckSumRecursively(config.getPathListForType(ptTechs,scenarioDir), "/" + serverInterface->getGameSettings()->getTech() + "/*", ".xml", NULL);
+									uint32 tilesetCRC = getFolderTreeContentsCheckSumRecursively(config.getPathListForType(ptTilesets,scenarioDir), string("/") + serverInterface->getGameSettings()->getTileset() + string("/*"), ".xml", NULL);
+									uint32 techCRC    = getFolderTreeContentsCheckSumRecursively(config.getPathListForType(ptTechs,scenarioDir), "/" + serverInterface->getGameSettings()->getTech() + "/*", ".xml", NULL);
 									Checksum checksum;
 									string file = Map::getMapPath(serverInterface->getGameSettings()->getMap(),scenarioDir,false);
 									checksum.addFile(file);
-									int32 mapCRC = checksum.getSum();
+									uint32 mapCRC = checksum.getSum();
 
 									networkGameDataSynchCheckOkMap      = (networkMessageSynchNetworkGameDataStatus.getMapCRC() == mapCRC);
 									networkGameDataSynchCheckOkTile     = (networkMessageSynchNetworkGameDataStatus.getTilesetCRC() == tilesetCRC);
@@ -828,7 +977,7 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 												this->setNetworkGameDataSynchCheckTechMismatchReport(report);
 											}
 											if(networkGameDataSynchCheckOkMap == false) {
-												vctFileList.push_back(std::pair<string,int32>(Map::getMapPath(serverInterface->getGameSettings()->getMap(),scenarioDir,false),mapCRC));
+												vctFileList.push_back(std::pair<string,uint32>(Map::getMapPath(serverInterface->getGameSettings()->getMap(),scenarioDir,false),mapCRC));
 											}
 
 											//for(int i = 0; i < vctFileList.size(); i++)
@@ -907,7 +1056,7 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 									FileTransferInfo fileInfo;
 									fileInfo.hostType   = eServer;
 									//fileInfo.serverIP   = this->ip.getString();
-									fileInfo.serverPort = Config::getInstance().getInt("ServerPort",intToStr(GameConstants::serverPort).c_str());
+									fileInfo.serverPort = Config::getInstance().getInt("PortServer",intToStr(GameConstants::serverPort).c_str());
 									fileInfo.fileName   = networkMessageSynchNetworkGameDataFileGet.getFileName();
 
 									FileTransferSocketThread *fileXferThread = new FileTransferSocketThread(fileInfo);
@@ -988,7 +1137,7 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 								if(SystemFlags::getSystemSettingType(SystemFlags::debugNetwork).enabled) SystemFlags::OutputDebug(SystemFlags::debugNetwork,"In [%s::%s Line: %d] networkMessageType = %d\n",__FILE__,__FUNCTION__,__LINE__,networkMessageType);
 
 								if(gotIntro == true) {
-									//throw runtime_error("Unexpected message in connection slot: " + intToStr(networkMessageType));
+									//throw megaglest_runtime_error("Unexpected message in connection slot: " + intToStr(networkMessageType));
 									string sErr = "Unexpected message in connection slot: " + intToStr(networkMessageType);
 									//sendTextMessage(sErr,-1);
 									//DisplayErrorMessage(sErr);
@@ -1005,6 +1154,39 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 								}
 							}
 					}
+
+					// This may end up continuously lagging and not disconnecting players who have
+					// just the 'wrong' amount of lag (but not enough to be horrible for a disconnect)
+					if(Config::getInstance().getBool("AutoClientLagCorrection","true") == true) {
+						double LAG_CHECK_GRACE_PERIOD 		= 15;
+
+						if(this->serverInterface->getGameStartTime() > 0 &&
+								difftime((long int)time(NULL),this->serverInterface->getGameStartTime()) >= LAG_CHECK_GRACE_PERIOD) {
+							if(this->isConnected() == true) {
+								double clientLag = this->serverInterface->getCurrentFrameCount() - this->getCurrentFrameCount();
+								double clientLagCount = (gameSettings.getNetworkFramePeriod() > 0 ? (clientLag / gameSettings.getNetworkFramePeriod()) : 0);
+								double clientLagTime = difftime((long int)time(NULL),this->getLastReceiveCommandListTime());
+
+								double maxFrameCountLagAllowed 		= 10;
+								double maxClientLagTimeAllowed 		= 8;
+
+								// New lag check
+								if((maxFrameCountLagAllowed > 0 && clientLagCount > maxFrameCountLagAllowed) ||
+									(maxClientLagTimeAllowed > 0 && clientLagTime > maxClientLagTimeAllowed)) {
+
+									waitForLaggingClient = true;
+									if(waitedForLaggingClient == false) {
+										waitedForLaggingClient = true;
+										printf("*TESTING*: START Waiting for lagging client playerIndex = %d [%s] clientLagCount = %f [%f]\n",playerIndex,name.c_str(),clientLagCount,clientLagTime);
+									}
+								}
+							}
+						}
+					}
+				}
+
+				if(waitedForLaggingClient == true) {
+					printf("*TESTING*: FINISHED Waiting for lagging client playerIndex = %d [%s]\n",playerIndex,name.c_str());
 				}
 
 				//if(chrono.getMillis() > 1) printf("In [%s::%s Line: %d] action running for msecs: %lld\n",__FILE__,__FUNCTION__,__LINE__,(long long int)chrono.getMillis());
@@ -1039,7 +1221,7 @@ void ConnectionSlot::update(bool checkForNewClients,int lockedSlotIndex) {
 void ConnectionSlot::validateConnection() {
 	if(this->isConnected() == true && 
 		gotIntro == false && connectedTime > 0 &&
-		difftime(time(NULL),connectedTime) > GameConstants::maxClientConnectHandshakeSecs) {
+		difftime((long int)time(NULL),connectedTime) > GameConstants::maxClientConnectHandshakeSecs) {
 		close();
 	}
 }
@@ -1095,11 +1277,11 @@ bool ConnectionSlot::updateCompleted(ConnectionSlotEvent *event) {
 	return (waitingForThread == false);
 }
 
-void ConnectionSlot::sendMessage(const NetworkMessage* networkMessage) {
+void ConnectionSlot::sendMessage(NetworkMessage* networkMessage) {
 	MutexSafeWrapper safeMutex(socketSynchAccessor,CODE_AT_LINE);
 
 	// Skip text messages not intended for the players preferred language
-	const NetworkMessageText *textMsg = dynamic_cast<const NetworkMessageText *>(networkMessage);
+	NetworkMessageText *textMsg = dynamic_cast<NetworkMessageText *>(networkMessage);
 	if(textMsg != NULL) {
 		//printf("\n\n\n~~~ SERVER HAS NetworkMessageText target [%s] player [%s] msg[%s]\n\n\n",textMsg->getTargetLanguage().c_str(),this->getNetworkPlayerLanguage().c_str(), textMsg->getText().c_str());
 
