@@ -15,6 +15,7 @@
 #include "socket.h"
 #include "game_constants.h"
 #include "network_types.h"
+#include "byte_order.h"
 #include "leak_dumper.h"
 
 using Shared::Platform::Socket;
@@ -43,6 +44,9 @@ enum NetworkMessageType {
 	nmtSwitchSetupRequest,
 	nmtPlayerIndexMessage,
 	nmtLoadingStatusMessage,
+	nmtMarkCell,
+	nmtUnMarkCell,
+	nmtHighlightCell,
 
 	nmtCount
 };
@@ -63,14 +67,23 @@ static const int maxLanguageStringSize= 60;
 
 class NetworkMessage {
 public:
+	static bool useOldProtocol;
 	virtual ~NetworkMessage(){}
 	virtual bool receive(Socket* socket)= 0;
-	virtual void send(Socket* socket) const = 0;
+	virtual void send(Socket* socket) = 0;
+	virtual size_t getDataSize() const = 0;
+
+	void dump_packet(string label, const void* data, int dataSize);
 
 protected:
 	//bool peek(Socket* socket, void* data, int dataSize);
 	bool receive(Socket* socket, void* data, int dataSize,bool tryReceiveUntilDataSizeMet);
-	void send(Socket* socket, const void* data, int dataSize) const;
+	void send(Socket* socket, const void* data, int dataSize);
+
+	virtual const char * getPackedMessageFormat() const = 0;
+	virtual unsigned int getPackedSize() = 0;
+	virtual void unpackMessage(unsigned char *buf) = 0;
+	virtual unsigned char * packMessage() = 0;
 };
 
 // =====================================================
@@ -98,16 +111,29 @@ private:
 		uint32 ftpPort;
 		NetworkString<maxLanguageStringSize> language;
 	};
+	void toEndian();
+	void fromEndian();
 
 private:
 	Data data;
 
+protected:
+
 public:
 	NetworkMessageIntro();
 	NetworkMessageIntro(int32 sessionId, const string &versionString,
 			const string &name, int playerIndex, NetworkGameStateType gameState,
 			uint32 externalIp, uint32 ftpPort, const string &playerLanguage);
 
+
+	virtual const char * getPackedMessageFormat() const;
+	virtual unsigned int getPackedSize();
+	virtual void unpackMessage(unsigned char *buf);
+	virtual unsigned char * packMessage();
+
+
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
 	int32 getSessionId() const 					{ return data.sessionId;}
 	string getVersionString() const				{ return data.versionString.getString(); }
 	string getName() const						{ return data.name.getString(); }
@@ -118,7 +144,9 @@ public:
 	string getPlayerLanguage() const			{ return data.language.getString(); }
 
 	virtual bool receive(Socket* socket);
-	virtual void send(Socket* socket) const;
+	virtual void send(Socket* socket);
+
+	string toString() const;
 };
 #pragma pack(pop)
 
@@ -136,21 +164,31 @@ private:
 		int32 pingFrequency;
 		int64 pingTime;
 	};
+	void toEndian();
+	void fromEndian();
 
 private:
 	Data data;
 	int64 pingReceivedLocalTime;
 
+protected:
+	virtual const char * getPackedMessageFormat() const;
+	virtual unsigned int getPackedSize();
+	virtual void unpackMessage(unsigned char *buf);
+	virtual unsigned char * packMessage();
+
 public:
 	NetworkMessagePing();
 	NetworkMessagePing(int32 pingFrequency, int64 pingTime);
 
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
 	int32 getPingFrequency() const	{return data.pingFrequency;}
 	int64 getPingTime() const	{return data.pingTime;}
 	int64 getPingReceivedLocalTime() const { return pingReceivedLocalTime; }
 
 	virtual bool receive(Socket* socket);
-	virtual void send(Socket* socket) const;
+	virtual void send(Socket* socket);
 };
 #pragma pack(pop)
 
@@ -165,20 +203,30 @@ class NetworkMessageReady: public NetworkMessage{
 private:
 	struct Data{
 		int8 messageType;
-		int32 checksum;
+		uint32 checksum;
 	};
+	void toEndian();
+	void fromEndian();
 
 private:
 	Data data;
 
+protected:
+	virtual const char * getPackedMessageFormat() const;
+	virtual unsigned int getPackedSize();
+	virtual void unpackMessage(unsigned char *buf);
+	virtual unsigned char * packMessage();
+
 public:
 	NetworkMessageReady();
-	NetworkMessageReady(int32 checksum);
+	NetworkMessageReady(uint32 checksum);
 
-	int32 getChecksum() const	{return data.checksum;}
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
+	uint32 getChecksum() const	{return data.checksum;}
 
 	virtual bool receive(Socket* socket);
-	virtual void send(Socket* socket) const;
+	virtual void send(Socket* socket);
 };
 #pragma pack(pop)
 
@@ -208,11 +256,11 @@ private:
 		int32 networkPlayerStatuses[GameConstants::maxPlayers]; //networkPlayerStatuses
 		NetworkString<maxSmallStringSize> networkPlayerLanguages[GameConstants::maxPlayers];
 
-		int32 mapCRC;
-		int32 tilesetCRC;
-		int32 techCRC;
+		uint32 mapCRC;
+		uint32 tilesetCRC;
+		uint32 techCRC;
 		NetworkString<maxSmallStringSize> factionNameList[maxFactionCRCCount];
-		int32 factionCRCList[maxFactionCRCCount];
+		uint32 factionCRCList[maxFactionCRCCount];
 
 		int8 factionControls[GameConstants::maxPlayers];
 		int8 resourceMultiplierIndex[GameConstants::maxPlayers];
@@ -234,26 +282,40 @@ private:
 		uint32 flagTypes1;
 
 		int8 aiAcceptSwitchTeamPercentChance;
+		int8 cpuReplacementMultiplier;
 		int32 masterserver_admin;
+		int32 masterserver_admin_factionIndex;
+
+		NetworkString<maxStringSize> scenario;
 	};
+	void toEndian();
+	void fromEndian();
 
 private:
 	Data data;
 
+protected:
+	virtual const char * getPackedMessageFormat() const;
+	virtual unsigned int getPackedSize();
+	virtual void unpackMessage(unsigned char *buf);
+	virtual unsigned char * packMessage();
+
 public:
 	NetworkMessageLaunch();
 	NetworkMessageLaunch(const GameSettings *gameSettings,int8 messageType);
 
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
 	void buildGameSettings(GameSettings *gameSettings) const;
 	int getMessageType() const { return data.messageType; }
 
 	int getMapCRC() const { return data.mapCRC; }
 	int getTilesetCRC() const { return data.tilesetCRC; }
 	int getTechCRC() const { return data.techCRC; }
-	vector<pair<string,int32> > getFactionCRCList() const;
+	vector<pair<string,uint32> > getFactionCRCList() const;
 
 	virtual bool receive(Socket* socket);
-	virtual void send(Socket* socket) const;
+	virtual void send(Socket* socket);
 };
 #pragma pack(pop)
 
@@ -263,10 +325,9 @@ public:
 //	Message to order a commands to several units
 // =====================================================
 
-//const int32 commandListHeaderSize = 6;
-
 #pragma pack(push, 1)
 class NetworkMessageCommandList: public NetworkMessage {
+
 private:
 	//static const int maxCommandCount = 2496; // can be as large as 65535
 
@@ -284,13 +345,35 @@ private:
 		//NetworkCommand commands[maxCommandCount];
 		std::vector<NetworkCommand> commands;
 	};
+	void toEndianHeader();
+	void fromEndianHeader();
+	void toEndianDetail(uint16 totalCommand);
+	void fromEndianDetail();
 
 private:
 	Data data;
 
+protected:
+	virtual const char * getPackedMessageFormat() const { return NULL; }
+	virtual unsigned int getPackedSize() { return 0; }
+	virtual void unpackMessage(unsigned char *buf) { };
+	virtual unsigned char * packMessage() { return NULL; }
+
+	const char * getPackedMessageFormatHeader() const;
+	unsigned int getPackedSizeHeader();
+	void unpackMessageHeader(unsigned char *buf);
+	unsigned char * packMessageHeader();
+
+	const char * getPackedMessageFormatDetail() const;
+	unsigned int getPackedSizeDetail(int count);
+	void unpackMessageDetail(unsigned char *buf,int count);
+	unsigned char * packMessageDetail(uint16 totalCommand);
+
 public:
 	NetworkMessageCommandList(int32 frameCount= -1);
 
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
 	bool addCommand(const NetworkCommand* networkCommand);
 
 	void clear()									{data.header.commandCount= 0;}
@@ -299,7 +382,7 @@ public:
 	const NetworkCommand* getCommand(int i) const	{return &data.commands[i];}
 
 	virtual bool receive(Socket* socket);
-	virtual void send(Socket* socket) const;
+	virtual void send(Socket* socket);
 };
 #pragma pack(pop)
 
@@ -322,22 +405,32 @@ private:
 		int8 playerIndex;
 		NetworkString<maxLanguageStringSize> targetLanguage;
 	};
+	void toEndian();
+	void fromEndian();
 
 private:
 	Data data;
 
+protected:
+	virtual const char * getPackedMessageFormat() const;
+	virtual unsigned int getPackedSize();
+	virtual void unpackMessage(unsigned char *buf);
+	virtual unsigned char * packMessage();
+
 public:
 	NetworkMessageText(){}
 	NetworkMessageText(const string &text, int teamIndex, int playerIndex,
 			const string targetLanguage);
 
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
 	string getText() const		{return data.text.getString();}
 	int getTeamIndex() const	{return data.teamIndex;}
 	int getPlayerIndex() const  {return data.playerIndex;}
 	string getTargetLanguage() const  {return data.targetLanguage.getString();}
 
 	virtual bool receive(Socket* socket);
-	virtual void send(Socket* socket) const;
+	virtual void send(Socket* socket);
 	NetworkMessageText * getCopy() const;
 };
 #pragma pack(pop)
@@ -354,15 +447,25 @@ private:
 	struct Data{
 		int8 messageType;
 	};
+	void toEndian();
+	void fromEndian();
 
 private:
 	Data data;
 
+protected:
+	virtual const char * getPackedMessageFormat() const;
+	virtual unsigned int getPackedSize();
+	virtual void unpackMessage(unsigned char *buf);
+	virtual unsigned char * packMessage();
+
 public:
 	NetworkMessageQuit();
 
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
 	virtual bool receive(Socket* socket);
-	virtual void send(Socket* socket) const;
+	virtual void send(Socket* socket);
 };
 #pragma pack(pop)
 
@@ -390,51 +493,72 @@ private:
 		NetworkString<maxStringSize> tileset;
 		NetworkString<maxStringSize> tech;
 
-		int32 mapCRC;
-		int32 tilesetCRC;
-		int32 techCRC;
+		uint32 mapCRC;
+		uint32 tilesetCRC;
+		uint32 techCRC;
 
-		int32 techCRCFileCount;
+		uint32 techCRCFileCount;
 	};
 
 	static const int32 HeaderSize = sizeof(DataHeader);
 
 	struct DataDetail {
 		NetworkString<maxStringSize> techCRCFileList[maxFileCRCCount];
-		int32 techCRCFileCRCList[maxFileCRCCount];
+		uint32 techCRCFileCRCList[maxFileCRCCount];
 	};
 
 	static const int32 DetailSize1 = sizeof(NetworkString<maxStringSize>);
-	static const int32 DetailSize2 = sizeof(int32);
+	static const int32 DetailSize2 = sizeof(uint32);
 
 	struct Data {
 		DataHeader header;
 		DataDetail detail;
 	};
+	void toEndianHeader();
+	void fromEndianHeader();
+	void toEndianDetail(uint32 totalFileCount);
+	void fromEndianDetail();
 
 private:
 	Data data;
 
+protected:
+	virtual const char * getPackedMessageFormat() const { return NULL; }
+	virtual unsigned int getPackedSize() { return 0; }
+	virtual void unpackMessage(unsigned char *buf) { };
+	virtual unsigned char * packMessage() { return NULL; }
+
+	const char * getPackedMessageFormatHeader() const;
+	unsigned int getPackedSizeHeader();
+	void unpackMessageHeader(unsigned char *buf);
+	unsigned char * packMessageHeader();
+
+	unsigned int getPackedSizeDetail();
+	void unpackMessageDetail(unsigned char *buf);
+	unsigned char * packMessageDetail();
+
 public:
     NetworkMessageSynchNetworkGameData() {};
 	NetworkMessageSynchNetworkGameData(const GameSettings *gameSettings);
 
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
 	virtual bool receive(Socket* socket);
-	virtual void send(Socket* socket) const;
+	virtual void send(Socket* socket);
 
 	string getMap() const		{return data.header.map.getString();}
 	string getTileset() const   {return data.header.tileset.getString();}
 	string getTech() const		{return data.header.tech.getString();}
 
-	int32 getMapCRC() const		{return data.header.mapCRC;}
-	int32 getTilesetCRC() const	{return data.header.tilesetCRC;}
-	int32 getTechCRC() const	{return data.header.techCRC;}
+	uint32 getMapCRC() const		{return data.header.mapCRC;}
+	uint32 getTilesetCRC() const	{return data.header.tilesetCRC;}
+	uint32 getTechCRC() const	{return data.header.techCRC;}
 
-	int32 getTechCRCFileCount() const {return data.header.techCRCFileCount;}
+	uint32 getTechCRCFileCount() const {return data.header.techCRCFileCount;}
 	const NetworkString<maxStringSize> * getTechCRCFileList() const {return &data.detail.techCRCFileList[0];}
-	const int32 * getTechCRCFileCRCList() const {return data.detail.techCRCFileCRCList;}
+	const uint32 * getTechCRCFileCRCList() const {return data.detail.techCRCFileCRCList;}
 
-	string getTechCRCFileMismatchReport(vector<std::pair<string,int32> > &vctFileList);
+	string getTechCRCFileMismatchReport(vector<std::pair<string,uint32> > &vctFileList);
 };
 #pragma pack(pop)
 
@@ -458,46 +582,58 @@ private:
 	struct DataHeader {
 		int8 messageType;
 
-		int32 mapCRC;
-		int32 tilesetCRC;
-		int32 techCRC;
+		uint32 mapCRC;
+		uint32 tilesetCRC;
+		uint32 techCRC;
 
-		int32 techCRCFileCount;
+		uint32 techCRCFileCount;
 	};
 	static const int32 HeaderSize = sizeof(DataHeader);
 
 	struct DataDetail {
 		NetworkString<maxStringSize> techCRCFileList[maxFileCRCCount];
-		int32 techCRCFileCRCList[maxFileCRCCount];
+		uint32 techCRCFileCRCList[maxFileCRCCount];
 	};
 
 	static const int32 DetailSize1 = sizeof(NetworkString<maxStringSize>);
-	static const int32 DetailSize2 = sizeof(int32);
+	static const int32 DetailSize2 = sizeof(uint32);
 
 	struct Data {
 		DataHeader header;
 		DataDetail detail;
 	};
+	void toEndianHeader();
+	void fromEndianHeader();
+	void toEndianDetail(uint32 totalFileCount);
+	void fromEndianDetail();
 
 private:
 	Data data;
 
+protected:
+	virtual const char * getPackedMessageFormat() const { return NULL; }
+	virtual unsigned int getPackedSize() { return 0; }
+	virtual void unpackMessage(unsigned char *buf) { };
+	virtual unsigned char * packMessage() { return NULL; }
+
 public:
     NetworkMessageSynchNetworkGameDataStatus() {};
-	NetworkMessageSynchNetworkGameDataStatus(int32 mapCRC, int32 tilesetCRC, int32 techCRC, vector<std::pair<string,int32> > &vctFileList);
+	NetworkMessageSynchNetworkGameDataStatus(uint32 mapCRC, uint32 tilesetCRC, uint32 techCRC, vector<std::pair<string,uint32> > &vctFileList);
+
+	virtual size_t getDataSize() const { return sizeof(Data); }
 
 	virtual bool receive(Socket* socket);
-	virtual void send(Socket* socket) const;
+	virtual void send(Socket* socket);
 
-	int32 getMapCRC() const		{return data.header.mapCRC;}
-	int32 getTilesetCRC() const	{return data.header.tilesetCRC;}
-	int32 getTechCRC() const	{return data.header.techCRC;}
+	uint32 getMapCRC() const		{return data.header.mapCRC;}
+	uint32 getTilesetCRC() const	{return data.header.tilesetCRC;}
+	uint32 getTechCRC() const	{return data.header.techCRC;}
 
-	int32 getTechCRCFileCount() const {return data.header.techCRCFileCount;}
+	uint32 getTechCRCFileCount() const {return data.header.techCRCFileCount;}
 	const NetworkString<maxStringSize> * getTechCRCFileList() const {return &data.detail.techCRCFileList[0];}
-	const int32 * getTechCRCFileCRCList() const {return data.detail.techCRCFileCRCList;}
+	const uint32 * getTechCRCFileCRCList() const {return data.detail.techCRCFileCRCList;}
 
-	string getTechCRCFileMismatchReport(string techtree, vector<std::pair<string,int32> > &vctFileList);
+	string getTechCRCFileMismatchReport(string techtree, vector<std::pair<string,uint32> > &vctFileList);
 
 };
 #pragma pack(pop)
@@ -519,25 +655,35 @@ private:
 	struct Data{
 		int8 messageType;
 
-		int32 totalFileCount;
-		int32 fileIndex;
-		int32 fileCRC;
+		uint32 totalFileCount;
+		uint32 fileIndex;
+		uint32 fileCRC;
 		NetworkString<maxStringSize> fileName;
 	};
+	void toEndian();
+	void fromEndian();
 
 private:
 	Data data;
 
+protected:
+	virtual const char * getPackedMessageFormat() const;
+	virtual unsigned int getPackedSize();
+	virtual void unpackMessage(unsigned char *buf);
+	virtual unsigned char * packMessage();
+
 public:
     NetworkMessageSynchNetworkGameDataFileCRCCheck() {};
-	NetworkMessageSynchNetworkGameDataFileCRCCheck(int32 totalFileCount, int32 fileIndex, int32 fileCRC, const string fileName);
+	NetworkMessageSynchNetworkGameDataFileCRCCheck(uint32 totalFileCount, uint32 fileIndex, uint32 fileCRC, const string fileName);
+
+	virtual size_t getDataSize() const { return sizeof(Data); }
 
 	virtual bool receive(Socket* socket);
-	virtual void send(Socket* socket) const;
+	virtual void send(Socket* socket);
 
-	int32 getTotalFileCount() const	{return data.totalFileCount;}
-	int32 getFileIndex() const	    {return data.fileIndex;}
-	int32 getFileCRC() const	    {return data.fileCRC;}
+	uint32 getTotalFileCount() const	{return data.totalFileCount;}
+	uint32 getFileIndex() const	    {return data.fileIndex;}
+	uint32 getFileCRC() const	    {return data.fileCRC;}
 	string getFileName() const		{return data.fileName.getString();}
 };
 #pragma pack(pop)
@@ -561,16 +707,26 @@ private:
 
 		NetworkString<maxStringSize> fileName;
 	};
+	void toEndian();
+	void fromEndian();
 
 private:
 	Data data;
 
+protected:
+	virtual const char * getPackedMessageFormat() const;
+	virtual unsigned int getPackedSize();
+	virtual void unpackMessage(unsigned char *buf);
+	virtual unsigned char * packMessage();
+
 public:
     NetworkMessageSynchNetworkGameDataFileGet() {};
 	NetworkMessageSynchNetworkGameDataFileGet(const string fileName);
 
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
 	virtual bool receive(Socket* socket);
-	virtual void send(Socket* socket) const;
+	virtual void send(Socket* socket);
 
 	string getFileName() const		{return data.fileName.getString();}
 };
@@ -594,13 +750,6 @@ enum SwitchSetupRequestFlagType {
 	ssrft_PlayerStatus			= 0x20
 };
 
-enum NetworkPlayerStatusType {
-	npst_None					= 0x00,
-	npst_PickSettings			= 0x01,
-	npst_BeRightBack			= 0x02,
-	npst_Ready					= 0x04
-};
-
 #pragma pack(push, 1)
 class SwitchSetupRequest: public NetworkMessage{
 private:
@@ -619,10 +768,18 @@ private:
 		int8 switchFlags;
 		NetworkString<maxLanguageStringSize> language;
 	};
+	void toEndian();
+	void fromEndian();
 
 private:
 	Data data;
 
+public:
+	virtual const char * getPackedMessageFormat() const;
+	virtual unsigned int getPackedSize();
+	virtual void unpackMessage(unsigned char *buf);
+	virtual unsigned char * packMessage();
+
 public:
 	SwitchSetupRequest();
 	SwitchSetupRequest( string selectedFactionName, int8 currentFactionIndex,
@@ -630,6 +787,8 @@ public:
 						int8 networkPlayerStatus, int8 flags,
 						string language);
 
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
 	string getSelectedFactionName() const	{return data.selectedFactionName.getString();}
 	int getCurrentFactionIndex() const		{return data.currentFactionIndex;}
 	int getToFactionIndex() const			{return data.toFactionIndex;}
@@ -643,7 +802,7 @@ public:
 	string getNetworkPlayerLanguage() const	{ return data.language.getString(); }
 
 	virtual bool receive(Socket* socket);
-	virtual void send(Socket* socket) const;
+	virtual void send(Socket* socket);
 };
 #pragma pack(pop)
 
@@ -658,21 +817,31 @@ public:
 class PlayerIndexMessage: public NetworkMessage{
 
 private:
-	struct Data{
+	struct Data {
 		int8 messageType;
 		int16 playerIndex;
 	};
+	void toEndian();
+	void fromEndian();
 
 private:
 	Data data;
 
+protected:
+	virtual const char * getPackedMessageFormat() const;
+	virtual unsigned int getPackedSize();
+	virtual void unpackMessage(unsigned char *buf);
+	virtual unsigned char * packMessage();
+
 public:
 	PlayerIndexMessage( int16 playerIndex);
 
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
 	int16 getPlayerIndex() const	{return data.playerIndex;}
 
 	virtual bool receive(Socket* socket);
-	virtual void send(Socket* socket) const;
+	virtual void send(Socket* socket);
 };
 #pragma pack(pop)
 
@@ -707,22 +876,173 @@ enum NetworkMessageLoadingStatusType {
 #pragma pack(push, 1)
 class NetworkMessageLoadingStatus : public NetworkMessage {
 private:
-	struct Data{
+	struct Data {
 		int8 messageType;
 		uint32 status;
 	};
+	void toEndian();
+	void fromEndian();
 
 private:
 	Data data;
 
+protected:
+	virtual const char * getPackedMessageFormat() const;
+	virtual unsigned int getPackedSize();
+	virtual void unpackMessage(unsigned char *buf);
+	virtual unsigned char * packMessage();
+
 public:
 	NetworkMessageLoadingStatus();
 	NetworkMessageLoadingStatus(uint32 status);
 
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
 	uint32 getStatus() const	{return data.status;}
 
 	virtual bool receive(Socket* socket);
-	virtual void send(Socket* socket) const;
+	virtual void send(Socket* socket);
+};
+#pragma pack(pop)
+
+
+// =====================================================
+//	class NetworkMessageMarkCell
+//
+//	Mark a Cell message nmtMarkCell
+// =====================================================
+
+#pragma pack(push, 1)
+class NetworkMessageMarkCell: public NetworkMessage {
+private:
+	static const int maxTextStringSize= 500;
+
+private:
+	struct Data{
+		int8 messageType;
+
+		int16 targetX;
+		int16 targetY;
+		int8 factionIndex;
+		int8 playerIndex;
+		NetworkString<maxTextStringSize> text;
+	};
+	void toEndian();
+	void fromEndian();
+
+private:
+	Data data;
+
+protected:
+	virtual const char * getPackedMessageFormat() const;
+	virtual unsigned int getPackedSize();
+	virtual void unpackMessage(unsigned char *buf);
+	virtual unsigned char * packMessage();
+
+public:
+	NetworkMessageMarkCell(){}
+	NetworkMessageMarkCell(Vec2i target, int factionIndex, const string &text, int playerIndex);
+
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
+	string getText() const			{ return data.text.getString(); }
+	Vec2i getTarget() const		{ return Vec2i(data.targetX,data.targetY); }
+	int getFactionIndex() const  { return data.factionIndex; }
+	int getPlayerIndex() const { return data.playerIndex; }
+
+	virtual bool receive(Socket* socket);
+	virtual void send(Socket* socket);
+	NetworkMessageMarkCell * getCopy() const;
+};
+#pragma pack(pop)
+
+// =====================================================
+//	class NetworkUnMessageMarkCell
+//
+//	Mark a Cell message nmtUnMarkCell
+// =====================================================
+
+#pragma pack(push, 1)
+class NetworkMessageUnMarkCell: public NetworkMessage {
+
+private:
+	struct Data {
+		int8 messageType;
+
+		int16 targetX;
+		int16 targetY;
+		int8 factionIndex;
+	};
+	void toEndian();
+	void fromEndian();
+
+private:
+	Data data;
+
+protected:
+	virtual const char * getPackedMessageFormat() const;
+	virtual unsigned int getPackedSize();
+	virtual void unpackMessage(unsigned char *buf);
+	virtual unsigned char * packMessage();
+
+public:
+	NetworkMessageUnMarkCell(){}
+	NetworkMessageUnMarkCell(Vec2i target, int factionIndex);
+
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
+	Vec2i getTarget() const		{ return Vec2i(data.targetX,data.targetY); }
+	int getFactionIndex() const  { return data.factionIndex; }
+
+	virtual bool receive(Socket* socket);
+	virtual void send(Socket* socket);
+	NetworkMessageUnMarkCell * getCopy() const;
+};
+#pragma pack(pop)
+
+// =====================================================
+//	class NetworkHighlightCellMessage
+//
+//	Highlight a Cell message
+// =====================================================
+
+
+#pragma pack(push, 1)
+class NetworkMessageHighlightCell: public NetworkMessage {
+private:
+	static const int maxTextStringSize= 500;
+
+private:
+	struct Data{
+		int8 messageType;
+
+		int16 targetX;
+		int16 targetY;
+		int8 factionIndex;
+	};
+	void toEndian();
+	void fromEndian();
+
+private:
+	Data data;
+
+protected:
+	virtual const char * getPackedMessageFormat() const;
+	virtual unsigned int getPackedSize();
+	virtual void unpackMessage(unsigned char *buf);
+	virtual unsigned char * packMessage();
+
+public:
+	NetworkMessageHighlightCell(){}
+	NetworkMessageHighlightCell(Vec2i target, int factionIndex);
+
+	virtual size_t getDataSize() const { return sizeof(Data); }
+
+	Vec2i getTarget() const		{ return Vec2i(data.targetX,data.targetY); }
+	int getFactionIndex() const  { return data.factionIndex; }
+
+	virtual bool receive(Socket* socket);
+	virtual void send(Socket* socket);
 };
 #pragma pack(pop)
 
