@@ -12,8 +12,12 @@
 #ifndef _GLEST_GAME_SERVERINTERFACE_H_
 #define _GLEST_GAME_SERVERINTERFACE_H_
 
-#include <vector>
+#ifdef WIN32
+    #include <winsock2.h>
+    #include <winsock.h>
+#endif
 
+#include <vector>
 #include "game_constants.h"
 #include "network_interface.h"
 #include "connection_slot.h"
@@ -35,6 +39,7 @@ namespace Glest{ namespace Game{
 
 class ServerInterface: public GameNetworkInterface,
                        public ConnectionSlotCallbackInterface,
+                       // This is for publishing game status to the masterserver
                        public SimpleTaskCallbackInterface,
                        public FTPClientValidationInterface {
 
@@ -74,7 +79,7 @@ private:
     vector<TextMessageQueue> textMessageQueue;
 
     Mutex *broadcastMessageQueueThreadAccessor;
-    vector<pair<const NetworkMessage *,int> > broadcastMessageQueue;
+    vector<pair<NetworkMessage *,int> > broadcastMessageQueue;
 
     Mutex *inBroadcastMessageThreadAccessor;
     bool inBroadcastMessage;
@@ -87,6 +92,8 @@ private:
 
 	map<string,pair<uint64,time_t> > badClientConnectIPList;
 
+	ServerSocket *serverSocketAdmin;
+
 public:
 	ServerInterface(bool publishEnabled);
 	virtual ~ServerInterface();
@@ -97,14 +104,29 @@ public:
     //    return &serverSocket;
     //}
 
+	time_t getGameStartTime() const { return gameStartTime; }
+
     virtual void close();
     virtual void update();
     virtual void updateLobby()  { };
     virtual void updateKeyframe(int frameCount);
+    virtual void setKeyframe(int frameCount) { currentFrameCount = frameCount; }
+
     virtual void waitUntilReady(Checksum *checksum);
     virtual void sendTextMessage(const string & text, int teamIndex, bool echoLocal, string targetLanguage);
     void sendTextMessage(const string & text, int teamIndex, bool echoLocal, string targetLanguage, int lockedSlotIndex);
+
     void queueTextMessage(const string & text, int teamIndex, bool echoLocal, string targetLanguage);
+
+    virtual void sendMarkCellMessage(Vec2i targetPos, int factionIndex, string note,int playerIndex);
+    void sendMarkCellMessage(Vec2i targetPos, int factionIndex, string note, int playerIndex, int lockedSlotIndex);
+
+	virtual void sendHighlightCellMessage(Vec2i targetPos, int factionIndex);
+    void sendHighlightCellMessage(Vec2i targetPos, int factionIndex, int lockedSlotIndex);
+
+    virtual void sendUnMarkCellMessage(Vec2i targetPos, int factionIndex);
+    void sendUnMarkCellMessage(Vec2i targetPos, int factionIndex, int lockedSlotIndex);
+
     virtual void quitGame(bool userManuallyQuit);
     virtual string getNetworkStatus();
     ServerSocket *getServerSocket()
@@ -121,7 +143,9 @@ public:
     bool switchSlot(int fromPlayerIndex, int toPlayerIndex);
     void removeSlot(int playerIndex, int lockedSlotIndex = -1);
     ConnectionSlot *getSlot(int playerIndex);
+    int getSlotCount();
     int getConnectedSlotCount();
+
     int getOpenSlotCount();
     bool launchGame(const GameSettings *gameSettings);
 	void validateGameSettings(GameSettings *serverGameSettings);
@@ -161,12 +185,12 @@ public:
         return serverSocket.getBindPort();
     }
 
-    void broadcastPing(const NetworkMessagePing *networkMessage, int excludeSlot = -1)
+    void broadcastPing(NetworkMessagePing *networkMessage, int excludeSlot = -1)
     {
         this->broadcastMessage(networkMessage, excludeSlot);
     }
 
-    void queueBroadcastMessage(const NetworkMessage *networkMessage, int excludeSlot = -1);
+    void queueBroadcastMessage(NetworkMessage *networkMessage, int excludeSlot = -1);
     virtual string getHumanPlayerName(int index = -1);
     virtual int getHumanPlayerIndex() const;
     bool getNeedToRepublishToMasterserver() const
@@ -194,10 +218,13 @@ public:
     virtual int isClientAllowedToGetFile(uint32 clientIp, const char *username, const char *filename);
 
     void notifyBadClientConnectAttempt(string ipAddress);
+    std::string DumpStatsToLog(bool dumpToStringOnly) const;
+
+    virtual void saveGame(XmlNode *rootNode);
 
 private:
-    void broadcastMessage(const NetworkMessage *networkMessage, int excludeSlot = -1, int lockedSlotIndex = -1);
-    void broadcastMessageToConnectedClients(const NetworkMessage *networkMessage, int excludeSlot = -1);
+    void broadcastMessage(NetworkMessage *networkMessage, int excludeSlot = -1, int lockedSlotIndex = -1);
+    void broadcastMessageToConnectedClients(NetworkMessage *networkMessage, int excludeSlot = -1);
     bool shouldDiscardNetworkMessage(NetworkMessageType networkMessageType, ConnectionSlot *connectionSlot);
     void updateSlot(ConnectionSlotEvent *event);
     void validateConnectedClients();
@@ -212,6 +239,12 @@ protected:
     void checkForLaggingClients(std::map<int,bool> &mapSlotSignalledList, std::map<int,ConnectionSlotEvent> &eventList, std::map<PLATFORM_SOCKET,bool> &socketTriggeredList,std::vector <string> &errorMsgList);
     void executeNetworkCommandsFromClients();
     void dispatchPendingChatMessages(std::vector <string> &errorMsgList);
+    void dispatchPendingMarkCellMessages(std::vector <string> &errorMsgList);
+    void dispatchPendingUnMarkCellMessages(std::vector <string> &errorMsgList);
+    void dispatchPendingHighlightCellMessages(std::vector <string> &errorMsgList);
+
+
+    void shutdownMasterserverPublishThread();
 };
 
 }}//end namespace
