@@ -12,6 +12,11 @@
 #ifndef _GLEST_GAME_SOUNDCONTAINER_H_
 #define _GLEST_GAME_SOUNDCONTAINER_H_
 
+#ifdef WIN32
+    #include <winsock2.h>
+    #include <winsock.h>
+#endif
+
 #include <vector>
 #include "sound.h"
 #include "randomgen.h"
