@@ -11,6 +11,11 @@
 #ifndef _GLEST_GAME_UNIT_H_
 #define _GLEST_GAME_UNIT_H_
 
+#ifdef WIN32
+    #include <winsock2.h>
+    #include <winsock.h>
+#endif
+
 #include "model.h"
 #include "upgrade_type.h"
 #include "particle.h"
@@ -18,6 +23,7 @@
 #include "game_constants.h"
 #include "platform_common.h"
 #include <vector>
+#include "faction.h"
 #include "leak_dumper.h"
 
 //#define LEAK_CHECK_UNITS
@@ -35,7 +41,7 @@ using Shared::PlatformCommon::Chrono;
 using Shared::PlatformCommon::ValueCheckerVault;
 
 class Map;
-class Faction;
+//class Faction;
 class Unit;
 class Command;
 class SkillType;
@@ -49,6 +55,8 @@ class Level;
 class MorphCommandType;
 class Game;
 class Unit;
+class GameSettings;
+class World;
 
 enum CommandResult {
 	crSuccess,
@@ -75,6 +83,19 @@ enum CauseOfDeathType {
 	ucodStarvedRegeneration
 };
 
+class UnitBuildInfo {
+public:
+	UnitBuildInfo() {
+		unit = NULL;
+		//pos;
+		buildUnit = NULL;
+	}
+	const Unit *unit;
+	CardinalDir facing;
+	Vec2i pos;
+	const UnitType *buildUnit;
+};
+
 // =====================================================
 // 	class UnitObserver
 // =====================================================
@@ -88,6 +109,8 @@ public:
 public:
 	virtual ~UnitObserver() {}
 	virtual void unitEvent(Event event, const Unit *unit)=0;
+
+	virtual void saveGame(XmlNode *rootNode) const = 0;
 };
 
 // =====================================================
@@ -107,6 +130,9 @@ public:
 
 	int getUnitId() const			{ return id; }
 	Faction *getUnitFaction() const	{ return faction; }
+
+	void saveGame(XmlNode *rootNode);
+	void loadGame(const XmlNode *rootNode,World *world);
 };
 
 class UnitPathInterface {
@@ -133,6 +159,8 @@ public:
 
 	virtual void setMap(Map *value) = 0;
 	virtual Map * getMap() = 0;
+
+	virtual void saveGame(XmlNode *rootNode) = 0;
 };
 
 class UnitPathBasic : public UnitPathInterface {
@@ -179,6 +207,8 @@ public:
 	virtual Map * getMap() { return map; }
 
 	virtual std::string toString() const;
+
+	virtual void saveGame(XmlNode *rootNode);
 };
 
 // =====================================================
@@ -215,7 +245,7 @@ public:
 	Vec2i peek()			{return back();}	 /**< peek at the next position			 */
 	void pop()				{this->pop_back();}	/**< pop the next position off the path */
 #else
-	// new style, for the new RoutePlanner
+	// new style
 	Vec2i peek()			{return front();}	 /**< peek at the next position			 */
 	//virtual Vec2i pop()		{ Vec2i p= front(); erase(begin()); return p; }	/**< pop the next position off the path */
 	void pop()		{ erase(begin()); }	/**< pop the next position off the path */
@@ -235,6 +265,8 @@ public:
 	virtual Map * getMap() { return map; }
 
 	virtual std::string toString() const;
+
+	virtual void saveGame(XmlNode *rootNode) {};
 };
 
 class WaypointPath : public list<Vec2i> {
@@ -257,24 +289,27 @@ class UnitAttackBoostEffect {
 public:
 
 	UnitAttackBoostEffect();
-	~UnitAttackBoostEffect();
+	virtual ~UnitAttackBoostEffect();
 
 	const AttackBoost *boost;
 	const Unit *source;
 	UnitParticleSystem *ups;
 	UnitParticleSystemType *upst;
 
+	virtual void saveGame(XmlNode *rootNode);
 };
 
 class UnitAttackBoostEffectOriginator {
 public:
 
 	UnitAttackBoostEffectOriginator();
-	~UnitAttackBoostEffectOriginator();
+	virtual ~UnitAttackBoostEffectOriginator();
 
 	const SkillType *skillType;
 	std::vector<int> currentAttackBoostUnits;
 	UnitAttackBoostEffect *currentAppliedEffect;
+
+	virtual void saveGame(XmlNode *rootNode);
 };
 
 class Unit : public BaseColorPickEntity, ValueCheckerVault {
@@ -290,7 +325,6 @@ private:
 public:
 	static const float speedDivider;
 	static const int maxDeadCount;
-	static const float highlightTime;
 	static const int invalidId;
 
 #ifdef LEAK_CHECK_UNITS
@@ -311,6 +345,7 @@ private:
 	int progress2;
 	int kills;
 	int enemyKills;
+	bool morphFieldsBlocked;
 
 	UnitReference targetRef;
 
@@ -415,105 +450,134 @@ private:
 	int lastAttackedUnitId;
 	CauseOfDeathType causeOfDeath;
 
+	uint32 pathfindFailedConsecutiveFrameCount;
+	Vec2i currentPathFinderDesiredFinalPos;
+
+	RandomGen random;
+	int pathFindRefreshCellCount;
+
+
 public:
     Unit(int id, UnitPathInterface *path, const Vec2i &pos, const UnitType *type, Faction *faction, Map *map, CardinalDir placeFacing);
-    ~Unit();
+    virtual ~Unit();
 
     //static bool isUnitDeleted(void *unit);
 
     static void setGame(Game *value) { game=value;}
 
+    inline int getPathFindRefreshCellCount() const { return pathFindRefreshCellCount; }
+
+    void setCurrentPathFinderDesiredFinalPos(const Vec2i &finalPos) { currentPathFinderDesiredFinalPos = finalPos; }
+    Vec2i getCurrentPathFinderDesiredFinalPos() const { return currentPathFinderDesiredFinalPos; }
+
     //const std::pair<const SkillType *,std::vector<Unit *> > & getCurrentAttackBoostUnits() const { return currentAttackBoostUnits; }
     const UnitAttackBoostEffectOriginator & getAttackBoostOriginatorEffect() const { return currentAttackBoostOriginatorEffect; }
     bool unitHasAttackBoost(const AttackBoost *boost, const Unit *source) const;
 
+    inline uint32 getPathfindFailedConsecutiveFrameCount() const { return pathfindFailedConsecutiveFrameCount; }
+    inline void incrementPathfindFailedConsecutiveFrameCount() { pathfindFailedConsecutiveFrameCount++; }
+    inline void resetPathfindFailedConsecutiveFrameCount() { pathfindFailedConsecutiveFrameCount=0; }
+
     //queries
     Command *getCurrrentCommandThreadSafe();
     void setIgnoreCheckCommand(bool value)      { ignoreCheckCommand=value;}
-    bool getIgnoreCheckCommand() const			{return ignoreCheckCommand;}
-	int getId() const							{return id;}
-	Field getCurrField() const					{return currField;}
-	int getLoadCount() const					{return loadCount;}
-	float getLastAnimProgress() const			{return lastAnimProgress;}
+    inline bool getIgnoreCheckCommand() const			{return ignoreCheckCommand;}
+    inline int getId() const							{return id;}
+    inline Field getCurrField() const					{return currField;}
+    inline int getLoadCount() const					{return loadCount;}
+    inline float getLastAnimProgress() const			{return lastAnimProgress;}
 	//float getProgress() const					{return progress;}
-	float getAnimProgress() const				{return animProgress;}
-	float getHightlight() const					{return highlight;}
-	int getProgress2() const					{return progress2;}
-	int getFactionIndex() const;
-	int getTeam() const;
-	int getHp() const							{return hp;}
-	int getEp() const							{return ep;}
+    inline float getAnimProgress() const				{return animProgress;}
+    inline float getHightlight() const					{return highlight;}
+    inline int getProgress2() const					{return progress2;}
+	inline int getFactionIndex() const {
+		return faction->getIndex();
+	}
+	inline int getTeam() const {
+		return faction->getTeam();
+	}
+	inline int getHp() const							{return hp;}
+	inline int getEp() const							{return ep;}
 	int getProductionPercent() const;
 	float getProgressRatio() const;
 	float getHpRatio() const;
 	float getEpRatio() const;
-	bool getToBeUndertaken() const				{return toBeUndertaken;}
-	Vec2i getTargetPos() const					{return targetPos;}
-	Vec3f getTargetVec() const					{return targetVec;}
-	Field getTargetField() const				{return targetField;}
-	Vec2i getMeetingPos() const					{return meetingPos;}
-	Faction *getFaction() const					{return faction;}
-    const ResourceType *getLoadType() const		{return loadType;}
-	const UnitType *getType() const				{return type;}
-	const SkillType *getCurrSkill() const		{return currSkill;}
-	const TotalUpgrade *getTotalUpgrade() const	{return &totalUpgrade;}
-	float getRotation() const					{return rotation;}
+	inline bool getToBeUndertaken() const				{return toBeUndertaken;}
+	inline Vec2i getTargetPos() const					{return targetPos;}
+	inline Vec3f getTargetVec() const					{return targetVec;}
+	inline Field getTargetField() const				{return targetField;}
+	inline Vec2i getMeetingPos() const					{return meetingPos;}
+	inline Faction *getFaction() const					{return faction;}
+	inline const ResourceType *getLoadType() const		{return loadType;}
+	inline const UnitType *getType() const				{return type;}
+	inline const SkillType *getCurrSkill() const		{return currSkill;}
+	inline const TotalUpgrade *getTotalUpgrade() const	{return &totalUpgrade;}
+	inline float getRotation() const					{return rotation;}
 	float getRotationX() const;
 	float getRotationZ() const;
-	ParticleSystem *getFire() const				{return fire;}
-	int getKills() const						{return kills;}
-	int getEnemyKills() const					{return enemyKills;}
-	const Level *getLevel() const				{return level;}
+	inline ParticleSystem *getFire() const				{return fire;}
+	inline int getKills() const						{return kills;}
+	inline int getEnemyKills() const					{return enemyKills;}
+	inline const Level *getLevel() const				{return level;}
 	const Level *getNextLevel() const;
 	string getFullName() const;
-	const UnitPathInterface *getPath() const	{return unitPath;}
-	UnitPathInterface *getPath()				{return unitPath;}
-	WaypointPath *getWaypointPath()				{return &waypointPath;}
+	inline const UnitPathInterface *getPath() const	{return unitPath;}
+	inline UnitPathInterface *getPath()				{return unitPath;}
+	inline WaypointPath *getWaypointPath()				{return &waypointPath;}
 
-	int getLastAttackerUnitId() const { return lastAttackerUnitId; }
-	void setLastAttackerUnitId(int unitId) { lastAttackerUnitId = unitId; }
+	inline int getLastAttackerUnitId() const { return lastAttackerUnitId; }
+	inline void setLastAttackerUnitId(int unitId) { lastAttackerUnitId = unitId; }
 
-	int getLastAttackedUnitId() const { return lastAttackedUnitId; }
-	void setLastAttackedUnitId(int unitId) { lastAttackedUnitId = unitId; }
+	inline int getLastAttackedUnitId() const { return lastAttackedUnitId; }
+	inline void setLastAttackedUnitId(int unitId) { lastAttackedUnitId = unitId; }
 
-	CauseOfDeathType getCauseOfDeath() const { return causeOfDeath; }
-	void setCauseOfDeath(CauseOfDeathType cause) { causeOfDeath = cause; }
+	inline CauseOfDeathType getCauseOfDeath() const { return causeOfDeath; }
+	inline void setCauseOfDeath(CauseOfDeathType cause) { causeOfDeath = cause; }
 
     //pos
-	Vec2i getPos() const				{return pos;}
+	inline Vec2i getPos() const				{return pos;}
 	Vec2i getPosWithCellMapSet() const;
-	Vec2i getLastPos() const			{return lastPos;}
+	inline Vec2i getLastPos() const			{return lastPos;}
 	Vec2i getCenteredPos() const;
     Vec2f getFloatCenteredPos() const;
 	Vec2i getCellPos() const;
 
     //is
-	bool isHighlighted() const			{return highlight>0.f;}
-	bool isDead() const					{return !alive;}
-	bool isAlive() const				{return alive;}
+	inline bool isHighlighted() const			{return highlight>0.f;}
+	inline bool isDead() const					{return !alive;}
+	inline bool isAlive() const				{return alive;}
     bool isOperative() const;
     bool isBeingBuilt() const;
     bool isBuilt() const;
+    bool isBuildCommandPending() const;
+    UnitBuildInfo getBuildCommandPendingInfo() const;
+
     bool isAnimProgressBound() const;
-    bool isPutrefacting() const;
+    bool isPutrefacting() const {
+    	return deadCount!=0;
+    }
 	bool isAlly(const Unit *unit) const;
 	bool isDamaged() const;
 	bool isInteresting(InterestingUnitType iut) const;
 
     //set
-	void setCurrField(Field currField)					{this->currField= currField;}
+	inline void setCurrField(Field currField)					{this->currField= currField;}
     void setCurrSkill(const SkillType *currSkill);
     void setCurrSkill(SkillClass sc);
-	void setLoadCount(int loadCount)					{this->loadCount= loadCount;}
-	void setLoadType(const ResourceType *loadType)		{this->loadType= loadType;}
-	void setProgress2(int progress2)					{this->progress2= progress2;}
-	void setPos(const Vec2i &pos);
+
+    void setMorphFieldsBlocked ( bool value ) {this->morphFieldsBlocked=value;}
+	bool getMorphFieldsBlocked() const { return morphFieldsBlocked; }
+
+    inline void setLoadCount(int loadCount)					{this->loadCount= loadCount;}
+    inline void setLoadType(const ResourceType *loadType)		{this->loadType= loadType;}
+    inline void setProgress2(int progress2)					{this->progress2= progress2;}
+	void setPos(const Vec2i &pos,bool clearPathFinder=false);
 	void setTargetPos(const Vec2i &targetPos);
 	void setTarget(const Unit *unit);
 	void setTargetVec(const Vec3f &targetVec);
 	void setMeetingPos(const Vec2i &meetingPos);
 	void setVisible(const bool visible);
-	bool getVisible() const { return visible; }
+	inline bool getVisible() const { return visible; }
 
 	//render related
     const Model *getCurrentModel();
@@ -524,17 +588,22 @@ public:
 
     //command related
 	bool anyCommand(bool validateCommandtype=false) const;
-	Command *getCurrCommand() const;
+	inline Command *getCurrCommand() const {
+		if(commands.empty() == false) {
+			return commands.front();
+		}
+		return NULL;
+	}
 	void replaceCurrCommand(Command *cmd);
 	int getCountOfProducedUnits(const UnitType *ut) const;
 	unsigned int getCommandSize() const;
-	CommandResult giveCommand(Command *command, bool tryQueue = false);		//give a command
+	std::pair<CommandResult,string> giveCommand(Command *command, bool tryQueue = false);		//give a command
 	CommandResult finishCommand();						//command finished
 	CommandResult cancelCommand();						//cancel canceled
 
 	//lifecycle
 	void create(bool startingUnit= false);
-	void born();
+	void born(const CommandType *ct);
 	void kill();
 	void undertake();
 
@@ -555,6 +624,7 @@ public:
     int update2();
     bool update();
 	void tick();
+	RandomGen* getRandom() { return &random; }
 
 	bool applyAttackBoost(const AttackBoost *boost, const Unit *source);
 	void deapplyAttackBoost(const AttackBoost *boost, const Unit *source);
@@ -563,40 +633,67 @@ public:
 	void computeTotalUpgrade();
 	void incKills(int team);
 	bool morph(const MorphCommandType *mct);
-	CommandResult checkCommand(Command *command) const;
+	std::pair<CommandResult,string> checkCommand(Command *command) const;
 	void applyCommand(Command *command);
 
 	void setModelFacing(CardinalDir value);
-	CardinalDir getModelFacing() const { return modelFacing; }
+	inline CardinalDir getModelFacing() const { return modelFacing; }
 
 	bool isMeetingPointSettable() const;
 
-	int getLastRenderFrame() const { return lastRenderFrame; }
-	void setLastRenderFrame(int value) { lastRenderFrame = value; }
+	inline int getLastRenderFrame() const { return lastRenderFrame; }
+	inline void setLastRenderFrame(int value) { lastRenderFrame = value; }
 
-	int getRetryCurrCommandCount() const { return retryCurrCommandCount; }
-	void setRetryCurrCommandCount(int value) { retryCurrCommandCount = value; }
+	inline int getRetryCurrCommandCount() const { return retryCurrCommandCount; }
+	inline void setRetryCurrCommandCount(int value) { retryCurrCommandCount = value; }
 
-	Vec3f getScreenPos() const { return screenPos; }
+	inline Vec3f getScreenPos() const { return screenPos; }
 	void setScreenPos(Vec3f value) { screenPos = value; }
 
-	string getCurrentUnitTitle() const {return currentUnitTitle;}
+	inline string getCurrentUnitTitle() const {return currentUnitTitle;}
 	void setCurrentUnitTitle(string value) { currentUnitTitle = value;}
 
 	void exploreCells();
 
-	bool getInBailOutAttempt() const { return inBailOutAttempt; }
-	void setInBailOutAttempt(bool value) { inBailOutAttempt = value; }
+	inline bool getInBailOutAttempt() const { return inBailOutAttempt; }
+	inline void setInBailOutAttempt(bool value) { inBailOutAttempt = value; }
 
 	//std::vector<std::pair<Vec2i,Chrono> > getBadHarvestPosList() const { return badHarvestPosList; }
 	//void setBadHarvestPosList(std::vector<std::pair<Vec2i,Chrono> > value) { badHarvestPosList = value; }
 	void addBadHarvestPos(const Vec2i &value);
 	void removeBadHarvestPos(const Vec2i &value);
-	bool isBadHarvestPos(const Vec2i &value,bool checkPeerUnits=true) const;
+	inline bool isBadHarvestPos(const Vec2i &value,bool checkPeerUnits=true) const {
+		bool result = false;
+		if(badHarvestPosList.empty() == true) {
+			return result;
+		}
+
+		std::map<Vec2i,int>::const_iterator iter = badHarvestPosList.find(value);
+		if(iter != badHarvestPosList.end()) {
+			result = true;
+		}
+		else if(checkPeerUnits == true) {
+			// Check if any other units of similar type have this position tagged
+			// as bad?
+			for(int i = 0; i < this->getFaction()->getUnitCount(); ++i) {
+				Unit *peerUnit = this->getFaction()->getUnit(i);
+				if( peerUnit != NULL && peerUnit->getId() != this->getId() &&
+					peerUnit->getType()->hasCommandClass(ccHarvest) == true &&
+					peerUnit->getType()->getSize() <= this->getType()->getSize()) {
+					if(peerUnit->isBadHarvestPos(value,false) == true) {
+						result = true;
+						break;
+					}
+				}
+			}
+		}
+
+		return result;
+	}
 	void cleanupOldBadHarvestPos();
 
 	void setLastHarvestResourceTarget(const Vec2i *pos);
-	std::pair<Vec2i,int> getLastHarvestResourceTarget() const { return lastHarvestResourceTarget;}
+	inline std::pair<Vec2i,int> getLastHarvestResourceTarget() const { return lastHarvestResourceTarget;}
 
 	//std::pair<Vec2i,std::vector<Vec2i> > getCurrentTargetPathTaken() const { return currentTargetPathTaken; }
 	//void addCurrentTargetPathTakenCell(const Vec2i &target,const Vec2i &cell);
@@ -606,27 +703,30 @@ public:
 	bool needToUpdate();
 
 	bool isLastStuckFrameWithinCurrentFrameTolerance() const;
-	uint32 getLastStuckFrame() const { return lastStuckFrame; }
-	void setLastStuckFrame(uint32 value) { lastStuckFrame = value; }
+	inline uint32 getLastStuckFrame() const { return lastStuckFrame; }
+	inline void setLastStuckFrame(uint32 value) { lastStuckFrame = value; }
 	void setLastStuckFrameToCurrentFrame();
 
-	Vec2i getLastStuckPos() const { return lastStuckPos; }
-	void setLastStuckPos(Vec2i pos) { lastStuckPos = pos; }
+	inline Vec2i getLastStuckPos() const { return lastStuckPos; }
+	inline void setLastStuckPos(Vec2i pos) { lastStuckPos = pos; }
 
 	bool isLastPathfindFailedFrameWithinCurrentFrameTolerance() const;
-	uint32 getLastPathfindFailedFrame() const { return lastPathfindFailedFrame; }
-	void setLastPathfindFailedFrame(uint32 value) { lastPathfindFailedFrame = value; }
+	inline uint32 getLastPathfindFailedFrame() const { return lastPathfindFailedFrame; }
+	inline void setLastPathfindFailedFrame(uint32 value) { lastPathfindFailedFrame = value; }
 	void setLastPathfindFailedFrameToCurrentFrame();
 
-	Vec2i getLastPathfindFailedPos() const { return lastPathfindFailedPos; }
-	void setLastPathfindFailedPos(Vec2i pos) { lastPathfindFailedPos = pos; }
+	inline Vec2i getLastPathfindFailedPos() const { return lastPathfindFailedPos; }
+	inline void setLastPathfindFailedPos(Vec2i pos) { lastPathfindFailedPos = pos; }
 
-	bool getUsePathfinderExtendedMaxNodes() const { return usePathfinderExtendedMaxNodes; }
-	void setUsePathfinderExtendedMaxNodes(bool value) { usePathfinderExtendedMaxNodes = value; }
+	inline bool getUsePathfinderExtendedMaxNodes() const { return usePathfinderExtendedMaxNodes; }
+	inline void setUsePathfinderExtendedMaxNodes(bool value) { usePathfinderExtendedMaxNodes = value; }
 
 	void updateTimedParticles();
 
 	virtual string getUniquePickName() const;
+	void saveGame(XmlNode *rootNode);
+	static Unit * loadGame(const XmlNode *rootNode,GameSettings *settings,Faction *faction, World *world);
+
 private:
 	float computeHeight(const Vec2i &pos) const;
 	void calculateXZRotation();
@@ -639,6 +739,10 @@ private:
 
 	int getFrameCount() const;
 	void checkCustomizedParticleTriggers(bool force);
+	bool checkModelStateInfoForNewHpValue();
+	void checkUnitLevel();
+
+	void morphAttackBoosts(Unit *unit);
 };
 
 }}// end namespace
