@@ -106,6 +106,7 @@ string StopCommandType::getDesc(const TotalUpgrade *totalUpgrade) const{
         str+= lang.get("EpCost")+": "+intToStr(stopSkillType->getEpCost())+"\n";
  if(stopSkillType->getHpCost()!=0)
         str+= lang.get("HpCost")+": "+intToStr(stopSkillType->getHpCost())+"\n";
+	str+=stopSkillType->getBoostDesc();
     return str;
 }
 
@@ -167,7 +168,7 @@ string MoveCommandType::getDesc(const TotalUpgrade *totalUpgrade) const{
 		if(moveSkillType->getHpCost()!=0){
         str+= lang.get("HpCost")+": "+intToStr(moveSkillType->getHpCost())+"\n";
 	}
-
+	str+=moveSkillType->getBoostDesc();
     return str;
 }
 
@@ -261,7 +262,7 @@ string AttackCommandType::getDesc(const TotalUpgrade *totalUpgrade) const{
     str+="\n";
 
     str+= lang.get("AttackSpeed")+": "+ intToStr(attackSkillType->getSpeed()) +"\n";
-
+	str+=attackSkillType->getBoostDesc();
     return str;
 }
 
@@ -345,7 +346,7 @@ string AttackStoppedCommandType::getDesc(const TotalUpgrade *totalUpgrade) const
 		}
 	}
 	str+="\n";
-
+	str+=attackSkillType->getBoostDesc();
     return str;
 }
 
@@ -448,7 +449,7 @@ string BuildCommandType::getDesc(const TotalUpgrade *totalUpgrade) const{
 		if(buildSkillType->getHpCost()!=0){
         str+= lang.get("HpCost")+": "+intToStr(buildSkillType->getHpCost())+"\n";
 	}
-
+	str+=buildSkillType->getBoostDesc();
     return str;
 }
 
@@ -526,9 +527,9 @@ string HarvestCommandType::getDesc(const TotalUpgrade *totalUpgrade) const{
 	}
 	str+=lang.get("Resources")+":\n";
 	for(int i=0; i<getHarvestedResourceCount(); ++i){
-		str+= getHarvestedResource(i)->getName()+"\n";
+		str+= getHarvestedResource(i)->getName(true)+"\n";
 	}
-
+	str+=harvestSkillType->getBoostDesc();
     return str;
 }
 
@@ -600,9 +601,9 @@ string RepairCommandType::getDesc(const TotalUpgrade *totalUpgrade) const{
 	}
     str+="\n"+lang.get("CanRepair")+":\n";
     for(int i=0; i<repairableUnits.size(); ++i){
-        str+= (static_cast<const UnitType*>(repairableUnits[i]))->getName()+"\n";
+        str+= (static_cast<const UnitType*>(repairableUnits[i]))->getName(true)+"\n";
     }
-
+	str+=repairSkillType->getBoostDesc();
     return str;
 }
 
@@ -673,7 +674,7 @@ string ProduceCommandType::getDesc(const TotalUpgrade *totalUpgrade) const{
         str+= lang.get("hpCost")+": "+intToStr(produceSkillType->getHpCost())+"\n";
 	}
     str+= "\n" + getProducedUnit()->getReqDesc();
-
+	str+=produceSkillType->getBoostDesc();
     return str;
 }
 
@@ -732,7 +733,7 @@ string UpgradeCommandType::getDesc(const TotalUpgrade *totalUpgrade) const{
 if(upgradeSkillType->getHpCost()!=0)
         str+= lang.get("HpCost")+": "+intToStr(upgradeSkillType->getHpCost())+"\n";
     str+= "\n"+getProducedUpgrade()->getReqDesc();
-
+	str+=upgradeSkillType->getBoostDesc();
     return str;
 }
 
@@ -760,6 +761,7 @@ MorphCommandType::MorphCommandType(){
     morphSkillType=NULL;
     morphUnit=NULL;
     discount=0;
+    ignoreResourceRequirements=false;
 }
 
 void MorphCommandType::update(UnitUpdater *unitUpdater, Unit *unit, int frameIndex) const {
@@ -782,6 +784,13 @@ void MorphCommandType::load(int id, const XmlNode *n, const string &dir,
     //discount
 	discount= n->getChild("discount")->getAttribute("value")->getIntValue();
 
+	ignoreResourceRequirements = false;
+	if(n->hasChild("ignore-resource-requirements") == true) {
+		ignoreResourceRequirements= n->getChild("ignore-resource-requirements")->getAttribute("value")->getBoolValue();
+
+		//printf("ignoreResourceRequirements = %d\n",ignoreResourceRequirements);
+	}
+
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
 }
 
@@ -805,7 +814,8 @@ string MorphCommandType::getDesc(const TotalUpgrade *totalUpgrade) const{
         str+= lang.get("Discount")+": "+intToStr(discount)+"%\n";
 	}
 
-    str+= "\n"+getProduced()->getReqDesc();
+    str+= "\n"+getProduced()->getReqDesc(ignoreResourceRequirements);
+	str+=morphSkillType->getBoostDesc();
 
     return str;
 }
