@@ -33,30 +33,98 @@ namespace Glest{ namespace Game{
 FactionType::FactionType() {
 	music			= NULL;
 	personalityType = fpt_Normal;
+	isLinked		= false;
 }
 
 //load a faction, given a directory
-void FactionType::load(const string &dir, const TechTree *techTree, Checksum* checksum,
+void FactionType::load(const string &factionName, const TechTree *techTree, Checksum* checksum,
 		Checksum *techtreeChecksum, std::map<string,vector<pair<string, string> > > &loadedFileList) {
 
-	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
-	string currentPath = dir;
-	endPathWithSlash(currentPath);
+	string techTreePath = techTree->getPath();
+	string techTreeName=techTree->getName();
+	string currentPath = "";
 
-    name= lastDir(currentPath);
+	//open xml file
+	string path="";
+	XmlTree xmlTree;
+	const XmlNode *factionNode;
 
-    // Add special Observer Faction
-    //Lang &lang= Lang::getInstance();
-    if(name == formatString(GameConstants::OBSERVER_SLOTNAME)) {
-    	personalityType = fpt_Observer;
-    }
+	//printf("\n>>> factionname=%s\n",factionName.c_str());
+	for(bool realFactionPathFound=false;realFactionPathFound == false;) {
+		currentPath = techTreePath + "factions/" + factionName;
+		endPathWithSlash(currentPath);
 
-	char szBuf[1024]="";
-	sprintf(szBuf,Lang::getInstance().get("LogScreenGameLoadingFactionType","",true).c_str(),formatString(name).c_str());
+	    name= lastDir(currentPath);
+
+	    // Add special Observer Faction
+	    //Lang &lang= Lang::getInstance();
+	    if(name == formatString(GameConstants::OBSERVER_SLOTNAME)) {
+	    	personalityType = fpt_Observer;
+	    }
+
+	    if(personalityType == fpt_Normal) {
+			string tmppath= currentPath + factionName +".xml";
+			std::map<string,string> mapExtraTagReplacementValues;
+	//		mapExtraTagReplacementValues["$COMMONDATAPATH"] = techTreePath + "/commondata/";
+			//printf("current $COMMONDATAPATH = %s\n",mapExtraTagReplacementValues["$COMMONDATAPATH"].c_str());
+			XmlTree xmlTree;
+			xmlTree.load(tmppath, Properties::getTagReplacementValues(&mapExtraTagReplacementValues));
+
+
+			const XmlNode *rootNode= xmlTree.getRootNode();
+
+			if(rootNode->getName()=="link") {
+				if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"Faction [%s] is a linked faction\n",name.c_str());
+
+				isLinked = true;
+				const XmlNode *techTreeNode= rootNode->getChild("techtree");
+				const string linkedTechTreeName=techTreeNode->getAttribute("name")->getRestrictedValue();
+	//			const XmlNode *factionLinkNode= rootNode->getChild("faction");
+	//			string linkedFactionName=factionLinkNode->getAttribute("name")->getRestrictedValue();
+				string linkedTechTreePath=techTree->findPath(linkedTechTreeName);
+				techTreePath=linkedTechTreePath;
+				endPathWithSlash(techTreePath);
+				techTreeName=linkedTechTreeName;
+
+				string linkedCurrentPath = techTreePath + "factions/" + factionName;
+				endPathWithSlash(linkedCurrentPath);
+				string linkedTmppath= linkedCurrentPath + factionName +".xml";
+
+				//printf("linkedTmppath [%s] linkedCurrentPath [%s]\n",linkedTmppath.c_str(),linkedCurrentPath.c_str());
+
+				loadedFileList[linkedTmppath].push_back(make_pair(linkedCurrentPath,linkedCurrentPath));
+				loadedFileList[tmppath].push_back(make_pair(currentPath,currentPath));
+				if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"techTreePath [%s] techTreeName [%s]\n",techTreePath.c_str(),techTreeName.c_str());
+			}
+			else {
+				// stop looking for new path, no more links ...
+				//xmlTree.load(tmppath, Properties::getTagReplacementValues(&mapExtraTagReplacementValues));
+
+				loadedFileList[tmppath].push_back(make_pair(currentPath,currentPath));
+
+				realFactionPathFound=true;
+				//printf("techPath found! %s\n",tmppath.c_str());
+
+				path=tmppath;
+			}
+	    }
+	    else {
+	    	break;
+	    }
+	}
+
+	char szBuf[8096]="";
+	snprintf(szBuf,8096,Lang::getInstance().get("LogScreenGameLoadingFactionType","",true).c_str(),formatString(name).c_str());
 	Logger::getInstance().add(szBuf, true);
 
 	if(personalityType == fpt_Normal) {
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"Loading faction [%s] currentPath [%s]\n",path.c_str(),currentPath.c_str());
+
+		checksum->addFile(path);
+		techtreeChecksum->addFile(path);
+
 		// a1) preload units
 		//string unitsPath= currentPath + "units/*.";
 		string unitsPath= currentPath + "units/";
@@ -94,15 +162,21 @@ void FactionType::load(const string &dir, const TechTree *techTree, Checksum* ch
 			int progressBaseValue=logger.getProgress();
 			for(int i = 0; i < unitTypes.size(); ++i) {
 				string str= currentPath + "units/" + unitTypes[i].getName();
-				unitTypes[i].load(i, str, techTree, this, checksum,techtreeChecksum,
+				unitTypes[i].loaddd(i, str, techTree,techTreePath, this, checksum,techtreeChecksum,
 						loadedFileList);
 				logger.setProgress(progressBaseValue+(int)((((double)i + 1.0) / (double)unitTypes.size()) * 100.0/techTree->getTypeCount()));
 				SDL_PumpEvents();
 			}
 		}
+	    catch(megaglest_runtime_error& ex) {
+	    	//printf("1111111b ex.wantStackTrace() = %d\n",ex.wantStackTrace());
+			SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.what());
+			//printf("222222b\n");
+			throw megaglest_runtime_error("Error loading units: "+ currentPath + "\n" + ex.what(),!ex.wantStackTrace());
+	    }
 		catch(const exception &e) {
-			SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",__FILE__,__FUNCTION__,__LINE__,e.what());
-			throw runtime_error("Error loading units: "+ currentPath + "\n" + e.what());
+			SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.what());
+			throw megaglest_runtime_error("Error loading units: "+ currentPath + "\n" + e.what());
 		}
 
 		// b2) load upgrades
@@ -116,24 +190,23 @@ void FactionType::load(const string &dir, const TechTree *techTree, Checksum* ch
 			}
 		}
 		catch(const exception &e){
-			SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",__FILE__,__FUNCTION__,__LINE__,e.what());
-			throw runtime_error("Error loading upgrades: "+ currentPath + "\n" + e.what());
+			SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.what());
+			throw megaglest_runtime_error("Error loading upgrades: "+ currentPath + "\n" + e.what());
 		}
 
-		//open xml file
-		string path= currentPath + name + ".xml";
-		checksum->addFile(path);
-		techtreeChecksum->addFile(path);
+		string tmppath= currentPath + factionName +".xml";
+
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"Loading faction xml [%s]\n",tmppath.c_str());
 
-		XmlTree xmlTree;
 		std::map<string,string> mapExtraTagReplacementValues;
-		mapExtraTagReplacementValues["$COMMONDATAPATH"] = techTree->getPath() + "/commondata/";
-		xmlTree.load(path, Properties::getTagReplacementValues(&mapExtraTagReplacementValues));
-		loadedFileList[path].push_back(make_pair(currentPath,currentPath));
+		mapExtraTagReplacementValues["$COMMONDATAPATH"] = techTreePath + "/commondata/";
+		//printf("current $COMMONDATAPATH = %s\n",mapExtraTagReplacementValues["$COMMONDATAPATH"].c_str());
+		xmlTree.load(tmppath, Properties::getTagReplacementValues(&mapExtraTagReplacementValues));
 
-		const XmlNode *factionNode= xmlTree.getRootNode();
 
+		factionNode=xmlTree.getRootNode();
 		//read starting resources
+		//printf("factionNode->getName()=%s",factionNode->getName().c_str());
 		const XmlNode *startingResourcesNode= factionNode->getChild("starting-resources");
 
 		startingResources.resize(startingResourcesNode->getChildCount());
@@ -169,6 +242,31 @@ void FactionType::load(const string &dir, const TechTree *techTree, Checksum* ch
 		//read ai behavior
 		if(factionNode->hasChild("ai-behavior") == true) {
 			const XmlNode *aiNode= factionNode->getChild("ai-behavior");
+			if(aiNode->hasAttribute("min-static-resource-count") == true) {
+				mapAIBehaviorStaticOverrideValues[aibsvcMinStaticResourceCount] = aiNode->getAttribute("min-static-resource-count")->getIntValue();
+			}
+
+			if(aiNode->hasChild("static-values") == true) {
+				const XmlNode *aiNodeUnits= aiNode->getChild("static-values");
+				for(int i = 0; i < aiNodeUnits->getChildCount(); ++i) {
+					const XmlNode *unitNode= aiNodeUnits->getChild("static", i);
+					AIBehaviorStaticValueCategory type  = aibsvcMaxBuildRadius;
+					if(unitNode->hasAttribute("type") == true) {
+						type  = static_cast<AIBehaviorStaticValueCategory>(
+								unitNode->getAttribute("type")->getIntValue());
+					}
+					else {
+						type = EnumParser<AIBehaviorStaticValueCategory>::getEnum(
+								unitNode->getAttribute("type-name")->getValue());
+						//printf("Discovered overriden static value for AI, type = %d, value = %d\n",type,value);
+					}
+
+					int value = unitNode->getAttribute("value")->getIntValue();
+					mapAIBehaviorStaticOverrideValues[type]=value;
+					//printf("Discovered overriden static value for AI, type = %d, value = %d\n",type,value);
+				}
+			}
+
 			if(aiNode->hasChild("worker-units") == true) {
 				const XmlNode *aiNodeUnits= aiNode->getChild("worker-units");
 				for(int i = 0; i < aiNodeUnits->getChildCount(); ++i) {
@@ -221,7 +319,17 @@ void FactionType::load(const string &dir, const TechTree *techTree, Checksum* ch
 			}
 		}
 	}
-	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
+}
+
+int FactionType::getAIBehaviorStaticOverideValue(AIBehaviorStaticValueCategory type) const {
+	int result = INT_MAX;
+	std::map<AIBehaviorStaticValueCategory, int >::const_iterator iterFind =
+			mapAIBehaviorStaticOverrideValues.find(type);
+	if(iterFind != mapAIBehaviorStaticOverrideValues.end()) {
+		result = iterFind->second;
+	}
+	return result;
 }
 
 const std::vector<FactionType::PairPUnitTypeInt> FactionType::getAIBehaviorUnits(AIBehaviorUnitCategory category) const {
@@ -243,8 +351,8 @@ std::vector<std::string> FactionType::validateFactionType() {
 	const uint32 MAX_BITRATE_WARNING = 200000;
 	StrSound *factionMusic = getMusic();
 	if(factionMusic != NULL && factionMusic->getInfo()->getBitRate() > MAX_BITRATE_WARNING) {
-		char szBuf[4096]="";
-		sprintf(szBuf,"The Faction [%s] has the music [%s]\nwhich has a bitrate of [%u] which may cause some sound drivers to crash, please use a bitrate of %d or less!",this->getName().c_str(),factionMusic->getFileName().c_str(),factionMusic->getInfo()->getBitRate(),MAX_BITRATE_WARNING);
+		char szBuf[8096]="";
+		snprintf(szBuf,8096,"The Faction [%s] has the music [%s]\nwhich has a bitrate of [%u] which may cause some sound drivers to crash, please use a bitrate of %d or less!",this->getName().c_str(),factionMusic->getFileName().c_str(),factionMusic->getInfo()->getBitRate(),MAX_BITRATE_WARNING);
 		results.push_back(szBuf);
 	}
 
@@ -254,16 +362,16 @@ std::vector<std::string> FactionType::validateFactionType() {
     	for(int i = 0; i < unitType.getSelectionSounds().getSounds().size(); ++i) {
     		StaticSound *sound = unitType.getSelectionSounds().getSounds()[i];
     		if(sound != NULL && sound->getInfo()->getBitRate() > MAX_BITRATE_WARNING) {
-				char szBuf[4096]="";
-				sprintf(szBuf,"The Unit [%s] in Faction [%s] has the sound [%s]\nwhich has a bitrate of [%u] which may cause some sound drivers to crash, please use a bitrate of %d or less!",unitType.getName().c_str(),this->getName().c_str(),sound->getFileName().c_str(),sound->getInfo()->getBitRate(),MAX_BITRATE_WARNING);
+				char szBuf[8096]="";
+				snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has the sound [%s]\nwhich has a bitrate of [%u] which may cause some sound drivers to crash, please use a bitrate of %d or less!",unitType.getName().c_str(),this->getName().c_str(),sound->getFileName().c_str(),sound->getInfo()->getBitRate(),MAX_BITRATE_WARNING);
 				results.push_back(szBuf);
     		}
     	}
     	for(int i = 0; i < unitType.getCommandSounds().getSounds().size(); ++i) {
     		StaticSound *sound = unitType.getCommandSounds().getSounds()[i];
     		if(sound != NULL && sound->getInfo()->getBitRate() > MAX_BITRATE_WARNING) {
-				char szBuf[4096]="";
-				sprintf(szBuf,"The Unit [%s] in Faction [%s] has the sound [%s]\nwhich has a bitrate of [%u] which may cause some sound drivers to crash, please use a bitrate of %d or less!",unitType.getName().c_str(),this->getName().c_str(),sound->getFileName().c_str(),sound->getInfo()->getBitRate(),MAX_BITRATE_WARNING);
+				char szBuf[8096]="";
+				snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has the sound [%s]\nwhich has a bitrate of [%u] which may cause some sound drivers to crash, please use a bitrate of %d or less!",unitType.getName().c_str(),this->getName().c_str(),sound->getFileName().c_str(),sound->getInfo()->getBitRate(),MAX_BITRATE_WARNING);
 				results.push_back(szBuf);
     		}
     	}
@@ -296,8 +404,8 @@ std::vector<std::string> FactionType::validateFactionType() {
 						}
 
 						if(foundUpgraderUnit == false) {
-							char szBuf[4096]="";
-							sprintf(szBuf,"The Unit [%s] in Faction [%s] has the command [%s]\nwhich has upgrade requirement [%s] but there are no units able to perform the upgrade!",unitType.getName().c_str(),this->getName().c_str(),cmdType->getName().c_str(),upgradeType->getName().c_str());
+							char szBuf[8096]="";
+							snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has the command [%s]\nwhich has upgrade requirement [%s] but there are no units able to perform the upgrade!",unitType.getName().c_str(),this->getName().c_str(),cmdType->getName().c_str(),upgradeType->getName().c_str());
 							results.push_back(szBuf);
 						}
 					}
@@ -320,8 +428,8 @@ std::vector<std::string> FactionType::validateFactionType() {
 								// Now also validate the the unit to be built
 								// has a be_built_skill
 						    	if(buildUnit->hasSkillClass(scBeBuilt) == false) {
-									char szBuf[4096]="";
-									sprintf(szBuf,"The Unit [%s] in Faction [%s] has the command [%s]\nwhich can build the Unit [%s] but the Unit to be built\ndoes not have the skill class [be_built_skill] in this faction!",unitType.getName().c_str(),this->getName().c_str(),cmdType->getName().c_str(),buildUnit->getName().c_str());
+									char szBuf[8096]="";
+									snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has the command [%s]\nwhich can build the Unit [%s] but the Unit to be built\ndoes not have the skill class [be_built_skill] in this faction!",unitType.getName().c_str(),this->getName().c_str(),cmdType->getName().c_str(),buildUnit->getName().c_str());
 									results.push_back(szBuf);
 						    	}
 
@@ -330,8 +438,8 @@ std::vector<std::string> FactionType::validateFactionType() {
 						}
 
 						if(foundUnit == false) {
-							char szBuf[4096]="";
-							sprintf(szBuf,"The Unit [%s] in Faction [%s] has the command [%s]\nwhich can build the Unit [%s] but the Unit to be built does not exist in this faction!",unitType.getName().c_str(),this->getName().c_str(),cmdType->getName().c_str(),buildUnit->getName().c_str());
+							char szBuf[8096]="";
+							snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has the command [%s]\nwhich can build the Unit [%s] but the Unit to be built does not exist in this faction!",unitType.getName().c_str(),this->getName().c_str(),cmdType->getName().c_str(),buildUnit->getName().c_str());
 							results.push_back(szBuf);
 						}
 					}
@@ -354,8 +462,8 @@ std::vector<std::string> FactionType::validateFactionType() {
 						}
 
 						if(foundUnit == false) {
-							char szBuf[4096]="";
-							sprintf(szBuf,"The Unit [%s] in Faction [%s] has the command [%s]\nwhich can repair the Unit [%s] but the Unit to be repaired does not exist in this faction!",unitType.getName().c_str(),this->getName().c_str(),cmdType->getName().c_str(),repairUnit->getName().c_str());
+							char szBuf[8096]="";
+							snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has the command [%s]\nwhich can repair the Unit [%s] but the Unit to be repaired does not exist in this faction!",unitType.getName().c_str(),this->getName().c_str(),cmdType->getName().c_str(),repairUnit->getName().c_str());
 							results.push_back(szBuf);
 						}
 					}
@@ -380,8 +488,8 @@ std::vector<std::string> FactionType::validateFactionType() {
 						}
 
 						if(foundUnit == false) {
-							char szBuf[4096]="";
-							sprintf(szBuf,"The Unit [%s] in Faction [%s] has the command [%s]\nwhich can morph into the Unit [%s] but the Unit to be morphed to does not exist in this faction!",unitType.getName().c_str(),this->getName().c_str(),cmdType->getName().c_str(),morphUnit->getName().c_str());
+							char szBuf[8096]="";
+							snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has the command [%s]\nwhich can morph into the Unit [%s] but the Unit to be morphed to does not exist in this faction!",unitType.getName().c_str(),this->getName().c_str(),cmdType->getName().c_str(),morphUnit->getName().c_str());
 							results.push_back(szBuf);
 						}
 					}
@@ -391,8 +499,8 @@ std::vector<std::string> FactionType::validateFactionType() {
 
     	const int maxMorphsAllowed = 6;
 		if(morphCommandCount > maxMorphsAllowed) {
-			char szBuf[4096]="";
-			sprintf(szBuf,"The Unit [%s] in Faction [%s] has more than %d morph commands which is too many to display in the UI!",unitType.getName().c_str(),this->getName().c_str(),maxMorphsAllowed);
+			char szBuf[8096]="";
+			snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has more than %d morph commands which is too many to display in the UI!",unitType.getName().c_str(),this->getName().c_str(),maxMorphsAllowed);
 			results.push_back(szBuf);
 		}
 
@@ -413,8 +521,8 @@ std::vector<std::string> FactionType::validateFactionType() {
 				}
 
 				if(foundUnit == false) {
-					char szBuf[4096]="";
-					sprintf(szBuf,"The Unit [%s] in Faction [%s] has the required Unit [%s]\nbut the required unit does not exist in this faction!",unitType.getName().c_str(),this->getName().c_str(),unitType2->getName().c_str());
+					char szBuf[8096]="";
+					snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has the required Unit [%s]\nbut the required unit does not exist in this faction!",unitType.getName().c_str(),this->getName().c_str(),unitType2->getName().c_str());
 					results.push_back(szBuf);
 				}
 			}
@@ -455,18 +563,34 @@ std::vector<std::string> FactionType::validateFactionType() {
 								 break;
 							}
 						}
+						if(foundUnit == true) {
+							break;
+						}
+					}
+					// Check if this is a morph command
+					if(cmdType->getClass() == ccMorph) {
+							const MorphCommandType *morph = dynamic_cast<const MorphCommandType *>(cmdType);
+							const UnitType *morphUnit = morph->getMorphUnit();
+
+							if( morphUnit != NULL &&
+									unitType.getId() != unitType2.getId() &&
+									unitType.getName() == morphUnit->getName()) {
+									 foundUnit = true;
+									 break;
+							}
 					}
 
-	    			// Check if this is a morph command
-					if(cmdType->getClass() == ccMorph) {
-						const MorphCommandType *morph = dynamic_cast<const MorphCommandType *>(cmdType);
-						const UnitType *morphUnit = morph->getMorphUnit();
+	    			// Check if this is an attack command with spawned units on attack
+					if(cmdType->getClass() == ccAttack) {
+						const AttackCommandType *act= dynamic_cast<const AttackCommandType*>(cmdType);
+						if( act != NULL && act->getAttackSkillType() != NULL &&
+							act->getAttackSkillType()->getSpawnUnit() != "" && act->getAttackSkillType()->getSpawnUnitCount() > 0) {
 
-						if( morphUnit != NULL &&
-							unitType.getId() != unitType2.getId() &&
-							unitType.getName() == morphUnit->getName()) {
-							 foundUnit = true;
-							 break;
+							if( unitType.getId() != unitType2.getId() &&
+								unitType.getName() == act->getAttackSkillType()->getSpawnUnit()) {
+								 foundUnit = true;
+								 break;
+							}
 						}
 					}
 	    		}
@@ -474,8 +598,10 @@ std::vector<std::string> FactionType::validateFactionType() {
 		}
 
 		if(foundUnit == false) {
-			char szBuf[4096]="";
-			sprintf(szBuf,"The Unit [%s] in Faction [%s] has no other units that can produce, build or morph into it in this faction!",unitType.getName().c_str(),this->getName().c_str());
+			//printf("Problem for unit [%s] unitTypes.size() = " MG_SIZE_T_SPECIFIER "\n",unitType.getName().c_str(),unitTypes.size());
+
+			char szBuf[8096]="";
+			snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has no other units that can produce, build or morph into it in this faction!",unitType.getName().c_str(),this->getName().c_str());
 			results.push_back(szBuf);
 		}
 
@@ -486,8 +612,8 @@ std::vector<std::string> FactionType::validateFactionType() {
                 if(st != NULL && dynamic_cast<const AttackSkillType *>(st) != NULL) {
                     const AttackSkillType *ast = dynamic_cast<const AttackSkillType *>(st);
                     if(ast->getAttackVar() < 0) {
-                        char szBuf[4096]="";
-                        sprintf(szBuf,"The Unit [%s] in Faction [%s] has the skill [%s] with an INVALID attack var value which is < 0 [%d]!",unitType.getName().c_str(),this->getName().c_str(),ast->getName().c_str(),ast->getAttackVar());
+                        char szBuf[8096]="";
+                        snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has the skill [%s] with an INVALID attack var value which is < 0 [%d]!",unitType.getName().c_str(),this->getName().c_str(),ast->getName().c_str(),ast->getAttackVar());
                         results.push_back(szBuf);
                     }
                 }
@@ -498,8 +624,8 @@ std::vector<std::string> FactionType::validateFactionType() {
         // Check if the unit has both be_built and harvest skills, this may cause issues
         // with the AI
         if(unitType.hasSkillClass(scBeBuilt) == true && unitType.hasSkillClass(scHarvest) == true) {
-			char szBuf[4096]="";
-			sprintf(szBuf,"The Unit [%s] in Faction [%s] has both a bebuilt and harvest skill which will cause AI problems for CPU players!",unitType.getName().c_str(),this->getName().c_str());
+			char szBuf[8096]="";
+			snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has both a bebuilt and harvest skill which will cause AI problems for CPU players!",unitType.getName().c_str(),this->getName().c_str());
 			results.push_back(szBuf);
         }
         // end
@@ -507,8 +633,8 @@ std::vector<std::string> FactionType::validateFactionType() {
         // Check if the unit has harvest skills but not move, meaning they cannot
         // harvest the resource
         if(unitType.hasSkillClass(scHarvest) == true && unitType.hasSkillClass(scMove) == false) {
-			char szBuf[4096]="";
-			sprintf(szBuf,"The Unit [%s] in Faction [%s] has a harvest skill but no move skill so it cannot harvest!",unitType.getName().c_str(),this->getName().c_str());
+			char szBuf[8096]="";
+			snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has a harvest skill but no move skill so it cannot harvest!",unitType.getName().c_str(),this->getName().c_str());
 			results.push_back(szBuf);
         }
         // end
@@ -541,8 +667,8 @@ std::vector<std::string> FactionType::validateFactionTypeResourceTypes(vector<Re
 				}
 
 				if(foundResourceType == false) {
-					char szBuf[4096]="";
-					sprintf(szBuf,"The Unit [%s] in Faction [%s] has the resource req [%s]\nbut there are no such resources in this tech!",unitType.getName().c_str(),this->getName().c_str(),r->getType()->getName().c_str());
+					char szBuf[8096]="";
+					snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has the resource req [%s]\nbut there are no such resources in this tech!",unitType.getName().c_str(),this->getName().c_str(),r->getType()->getName().c_str());
 					results.push_back(szBuf);
 				}
     		}
@@ -565,8 +691,8 @@ std::vector<std::string> FactionType::validateFactionTypeResourceTypes(vector<Re
 				}
 
 				if(foundResourceType == false) {
-					char szBuf[4096]="";
-					sprintf(szBuf,"The Unit [%s] in Faction [%s] has the stored resource [%s]\nbut there are no such resources in this tech!",unitType.getName().c_str(),this->getName().c_str(),r->getType()->getName().c_str());
+					char szBuf[8096]="";
+					snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has the stored resource [%s]\nbut there are no such resources in this tech!",unitType.getName().c_str(),this->getName().c_str(),r->getType()->getName().c_str());
 					results.push_back(szBuf);
 				}
 			}
@@ -594,8 +720,8 @@ std::vector<std::string> FactionType::validateFactionTypeResourceTypes(vector<Re
 						}
 
 						if(foundResourceType == false) {
-							char szBuf[4096]="";
-							sprintf(szBuf,"The Unit [%s] in Faction [%s] has the command [%s] which can harvest the resource [%s]\nbut there are no such resources in this tech!",unitType.getName().c_str(),this->getName().c_str(),cmdType->getName().c_str(),harvestResource->getName().c_str());
+							char szBuf[8096]="";
+							snprintf(szBuf,8096,"The Unit [%s] in Faction [%s] has the command [%s] which can harvest the resource [%s]\nbut there are no such resources in this tech!",unitType.getName().c_str(),this->getName().c_str(),cmdType->getName().c_str(),harvestResource->getName().c_str());
 							results.push_back(szBuf);
 						}
 					}
@@ -640,8 +766,8 @@ std::vector<std::string> FactionType::validateFactionTypeUpgradeTypes() {
 	    }
 
 		if(foundUnit == false) {
-			char szBuf[4096]="";
-			sprintf(szBuf,"The Upgrade Type [%s] in Faction [%s] has no Unit able to produce this upgrade in this faction!",upgradeType.getName().c_str(),this->getName().c_str());
+			char szBuf[8096]="";
+			snprintf(szBuf,8096,"The Upgrade Type [%s] in Faction [%s] has no Unit able to produce this upgrade in this faction!",upgradeType.getName().c_str(),this->getName().c_str());
 			results.push_back(szBuf);
 		}
 	}
@@ -658,12 +784,37 @@ const UnitType *FactionType::getUnitType(const string &name) const{
 		}
     }
 
-    if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] size = %d\n",__FILE__,__FUNCTION__,__LINE__,name.c_str(),unitTypes.size());
+    printf("In [%s::%s Line: %d] scanning [%s] size = " MG_SIZE_T_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),unitTypes.size());
     for(int i=0; i<unitTypes.size();i++){
-    	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",__FILE__,__FUNCTION__,__LINE__,name.c_str(),i,unitTypes[i].getName().c_str());
+    	printf("In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,unitTypes[i].getName().c_str());
     }
 
-	throw runtime_error("Unit not found: [" + name + "]");
+    if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] size = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),unitTypes.size());
+    for(int i=0; i<unitTypes.size();i++){
+    	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,unitTypes[i].getName().c_str());
+    }
+
+	throw megaglest_runtime_error("Unit type not found: [" + name + "] in faction type [" + this->name + "]");
+}
+
+const UnitType *FactionType::getUnitTypeById(int id) const{
+    for(int i=0; i<unitTypes.size();i++){
+		if(unitTypes[i].getId() == id) {
+            return &unitTypes[i];
+		}
+    }
+
+    printf("In [%s::%s Line: %d] scanning [%d] size = " MG_SIZE_T_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,id,unitTypes.size());
+    for(int i=0; i<unitTypes.size();i++){
+    	printf("In [%s::%s Line: %d] scanning [%s] idx = %d [%s][%d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,unitTypes[i].getName().c_str(),unitTypes[i].getId());
+    }
+
+    if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] size = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),unitTypes.size());
+    for(int i=0; i<unitTypes.size();i++){
+    	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,unitTypes[i].getName().c_str());
+    }
+
+	throw megaglest_runtime_error("Unit type not found: [" + intToStr(id) + "] in faction type [" + this->name + "]");
 }
 
 const UpgradeType *FactionType::getUpgradeType(const string &name) const{
@@ -673,12 +824,17 @@ const UpgradeType *FactionType::getUpgradeType(const string &name) const{
 		}
     }
 
-    if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] size = %d\n",__FILE__,__FUNCTION__,__LINE__,name.c_str(),unitTypes.size());
+    printf("In [%s::%s Line: %d] scanning [%s] size = " MG_SIZE_T_SPECIFIER "\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),unitTypes.size());
     for(int i=0; i<upgradeTypes.size();i++){
-    	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",__FILE__,__FUNCTION__,__LINE__,name.c_str(),i,upgradeTypes[i].getName().c_str());
+    	printf("In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,upgradeTypes[i].getName().c_str());
     }
 
-	throw runtime_error("Upgrade not found: "+name);
+    if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] size = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),unitTypes.size());
+    for(int i=0; i<upgradeTypes.size();i++){
+    	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] scanning [%s] idx = %d [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,name.c_str(),i,upgradeTypes[i].getName().c_str());
+    }
+
+	throw megaglest_runtime_error("Upgrade type not found: [" + name + "] in faction type [" + this->name + "]");
 }
 
 int FactionType::getStartingResourceAmount(const ResourceType *resourceType) const{
@@ -737,9 +893,7 @@ bool FactionType::factionUsesResourceType(const ResourceType *rt) const {
 }
 
 std::string FactionType::toString() const {
-	std::string result = "";
-
-	result = "Faction Name: " + name + "\n";
+	std::string result = "Faction Name: " + name + "\n";
 
 	result += "Unit Type List count = " + intToStr(this->getUnitTypeCount()) + "\n";
     for(int i=0; i<unitTypes.size();i++) {
@@ -754,4 +908,56 @@ std::string FactionType::toString() const {
 	return result;
 }
 
+void FactionType::saveGame(XmlNode *rootNode) {
+	std::map<string,string> mapTagReplacements;
+	XmlNode *factionTypeNode = rootNode->addChild("FactionType");
+
+//    string name;
+	factionTypeNode->addAttribute("name",name, mapTagReplacements);
+//    UnitTypes unitTypes;
+	for(unsigned int i = 0; i < unitTypes.size(); ++i) {
+		XmlNode *unitTypesNode = factionTypeNode->addChild("unitTypes");
+		unitTypesNode->addAttribute("name",unitTypes[i].getName(), mapTagReplacements);
+	}
+//    UpgradeTypes upgradeTypes;
+	for(unsigned int i = 0; i < upgradeTypes.size(); ++i) {
+		XmlNode *upgradeTypesNode = factionTypeNode->addChild("upgradeTypes");
+		upgradeTypesNode->addAttribute("name",upgradeTypes[i].getName(), mapTagReplacements);
+	}
+
+//	StartingUnits startingUnits;
+	for(unsigned int i = 0; i < startingUnits.size(); ++i) {
+		XmlNode *startingUnitsNode = factionTypeNode->addChild("startingUnits");
+		startingUnitsNode->addAttribute("name",startingUnits[i].first->getName(), mapTagReplacements);
+		startingUnitsNode->addAttribute("count",intToStr(startingUnits[i].second), mapTagReplacements);
+	}
+
+//	Resources startingResources;
+	for(unsigned int i = 0; i < startingResources.size(); ++i) {
+		startingResources[i].saveGame(factionTypeNode);
+	}
+
+//	StrSound *music;
+//	FactionPersonalityType personalityType;
+	factionTypeNode->addAttribute("personalityType",intToStr(personalityType), mapTagReplacements);
+//	std::map<AIBehaviorUnitCategory, std::vector<PairPUnitTypeInt> > mapAIBehaviorUnitCategories;
+	for(std::map<AIBehaviorUnitCategory, std::vector<PairPUnitTypeInt> >::iterator iterMap = mapAIBehaviorUnitCategories.begin();
+			iterMap != mapAIBehaviorUnitCategories.end(); ++iterMap) {
+
+		std::vector<PairPUnitTypeInt> &vct = iterMap->second;
+		for(unsigned int i = 0; i < vct.size(); ++i) {
+			PairPUnitTypeInt &item = vct[i];
+
+			XmlNode *mapAIBehaviorUnitCategoriesNode = factionTypeNode->addChild("mapAIBehaviorUnitCategories");
+			mapAIBehaviorUnitCategoriesNode->addAttribute("key",intToStr(iterMap->first), mapTagReplacements);
+			mapAIBehaviorUnitCategoriesNode->addAttribute("unitType",item.first->getName(), mapTagReplacements);
+			mapAIBehaviorUnitCategoriesNode->addAttribute("count",intToStr(item.second), mapTagReplacements);
+		}
+	}
+//	std::vector<const UpgradeType*> vctAIBehaviorUpgrades;
+	//for(unsigned int i = 0; i < vctAIBehaviorUpgrades.size(); ++i) {
+	//	vctAIBehaviorUpgrades[i]->saveGame(factionTypeNode);
+	//}
+}
+
 }}//end namespace
