@@ -150,6 +150,95 @@ bool AttackBoost::isAffected(const Unit *source, const Unit *dest) const {
 	return result;
 }
 
+string AttackBoost::getDesc() const{
+	Lang &lang= Lang::getInstance();
+    string str= "";
+    string indent="  ";
+    if(enabled) {
+    	if(boostUnitList.empty() == false) {
+    		str+= "\n"+ lang.get("Effects")+":\n";
+    	}
+
+    	str += indent+lang.get("effectRadius") + ": " + intToStr(radius) +"\n";
+    	if(allowMultipleBoosts==false) {
+    		string allowIt=lang.get("No");
+    		if(allowMultipleBoosts==true)
+    			allowIt=lang.get("False");
+    		str += indent+lang.get("allowMultiBoost") + ": " + allowIt +"\n";
+    	}
+    	str+=boostUpgrade.getDesc();
+
+    	if(targetType==abtAlly)
+    	{
+    		str+= lang.get("AffectedUnitsFromTeam") +":\n";
+    	}
+    	else if(targetType==abtFoe)
+    	{
+    		str+= lang.get("AffectedUnitsFromFoe") +":\n";
+    	}
+    	else if(targetType==abtFaction)
+    	{
+    		str+= lang.get("AffectedUnitsFromYourFaction") +":\n";
+    	}
+    	else if(targetType==abtUnitTypes)
+    	{
+    		str+= lang.get("AffectedUnitsFromAll") +":\n";
+    	}
+    	else if(targetType==abtAll)
+    	{
+    		str+= lang.get("AffectedUnitsFromAll") +":\n";
+    	}
+
+    	if(boostUnitList.empty() == false) {
+			for(int i=0; i<boostUnitList.size(); ++i){
+				str+= "  "+boostUnitList[i]->getName(true)+"\n";
+			}
+    	}
+    	else
+    	{
+    		str+= lang.get("All")+"\n";
+    	}
+
+    	return str;
+    }
+    else
+    	return "";
+}
+
+void AttackBoost::saveGame(XmlNode *rootNode) const {
+	std::map<string,string> mapTagReplacements;
+	XmlNode *attackBoostNode = rootNode->addChild("AttackBoost");
+
+//	bool enabled;
+	attackBoostNode->addAttribute("enabled",intToStr(enabled), mapTagReplacements);
+//	bool allowMultipleBoosts;
+	attackBoostNode->addAttribute("allowMultipleBoosts",intToStr(allowMultipleBoosts), mapTagReplacements);
+//	int radius;
+	attackBoostNode->addAttribute("radius",intToStr(radius), mapTagReplacements);
+//	AttackBoostTargetType targetType;
+	attackBoostNode->addAttribute("targetType",intToStr(targetType), mapTagReplacements);
+//	vector<const UnitType *> boostUnitList;
+	for(unsigned int i = 0; i < boostUnitList.size(); ++i) {
+		const UnitType *ut = boostUnitList[i];
+		XmlNode *unitTypeNode = attackBoostNode->addChild("UnitType");
+		unitTypeNode->addAttribute("name",ut->getName(), mapTagReplacements);
+	}
+//	UpgradeTypeBase boostUpgrade;
+	boostUpgrade.saveGame(attackBoostNode);
+//	UnitParticleSystemType *unitParticleSystemTypeForSourceUnit;
+	if(unitParticleSystemTypeForSourceUnit != NULL) {
+		unitParticleSystemTypeForSourceUnit->saveGame(attackBoostNode);
+	}
+//	UnitParticleSystemType *unitParticleSystemTypeForAffectedUnit;
+	if(unitParticleSystemTypeForAffectedUnit != NULL) {
+		unitParticleSystemTypeForAffectedUnit->saveGame(attackBoostNode);
+	}
+
+//	bool includeSelf;
+	attackBoostNode->addAttribute("includeSelf",intToStr(includeSelf), mapTagReplacements);
+//	string name;
+	attackBoostNode->addAttribute("name",name, mapTagReplacements);
+}
 
 // =====================================================
 // 	class SkillType
@@ -242,12 +331,12 @@ void SkillType::loadAttackBoost(const XmlNode *attackBoostsNode, const XmlNode *
 		}
 	}
 	else {
-		char szBuf[4096] = "";
-		sprintf(szBuf, "Unsupported target [%s] specified for attack boost for skill [%s] in [%s]", targetType.c_str(), name.c_str(), parentLoader.c_str());
-		throw runtime_error(szBuf);
+		char szBuf[8096] = "";
+		snprintf(szBuf, 8096,"Unsupported target [%s] specified for attack boost for skill [%s] in [%s]", targetType.c_str(), name.c_str(), parentLoader.c_str());
+		throw megaglest_runtime_error(szBuf);
 	}
 
-    attackBoost.boostUpgrade.load(attackBoostNode);
+    attackBoost.boostUpgrade.load(attackBoostNode,attackBoost.name);
     if(attackBoostNode->hasChild("particles") == true) {
         const XmlNode *particleNode = attackBoostNode->getChild("particles");
         bool particleEnabled = particleNode->getAttribute("value")->getBoolValue();
@@ -328,9 +417,9 @@ void SkillType::load(const XmlNode *sn, const XmlNode *attackBoostsNode,
 		}
 	}
 	if(animations.empty() == true) {
-		char szBuf[4096]="";
-		sprintf(szBuf,"Error no animations found for skill [%s] for parentLoader [%s]",name.c_str(),parentLoader.c_str());
-		throw runtime_error(szBuf);
+		char szBuf[8096]="";
+		snprintf(szBuf,8096,"Error no animations found for skill [%s] for parentLoader [%s]",name.c_str(),parentLoader.c_str());
+		throw megaglest_runtime_error(szBuf);
 	}
 
 	//particles
@@ -397,11 +486,16 @@ bool SkillType::CanCycleNextRandomAnimation(const int *animationRandomCycleCount
 	return result;
 }
 
+const AnimationAttributes SkillType::getAnimationAttribute(int index) const {
+	return animationAttributes[index];
+}
+
 Model *SkillType::getAnimation(float animProgress, const Unit *unit,
 		int *lastAnimationIndex, int *animationRandomCycleCount) const {
 	int modelIndex = 0;
+	//printf("Count [%d] animProgress = [%f] for skill [%s] animationRandomCycleCount = %d\n",animations.size(),animProgress,name.c_str(),*animationRandomCycleCount);
 	if(animations.size() > 1) {
-		//printf("animProgress = [%f] for skill [%s]\n",animProgress,name.c_str());
+		//printf("animProgress = [%f] for skill [%s] animationRandomCycleCount = %d\n",animProgress,name.c_str(),*animationRandomCycleCount);
 
 		if(lastAnimationIndex) {
 			modelIndex = *lastAnimationIndex;
@@ -442,14 +536,18 @@ Model *SkillType::getAnimation(float animProgress, const Unit *unit,
 
 				if(foundSpecificAnimation == false) {
 					//int modelIndex = random.randRange(0,animations.size()-1);
-					srand(time(NULL) + unit->getId());
+					Chrono seed(true);
+					srand((unsigned int)seed.getCurTicks() + unit->getId());
+
 					modelIndex = rand() % animations.size();
 
 					//const AnimationAttributes &attributes = animationAttributes[modelIndex];
 					//printf("SELECTING RANDOM Model index = %d [%s] model attributes [%d to %d] for unit [%s - %d] with HP = %d\n",modelIndex,animations[modelIndex]->getFileName().c_str(),attributes.fromHp,attributes.toHp,unit->getType()->getName().c_str(),unit->getId(),unit->getHp());
 				}
 				else {
-					srand(time(NULL) + unit->getId());
+					Chrono seed(true);
+					srand((unsigned int)seed.getCurTicks() + unit->getId());
+
 					int filteredModelIndex = rand() % filteredAnimations.size();
 					modelIndex = filteredAnimations[filteredModelIndex];
 				}
@@ -457,6 +555,9 @@ Model *SkillType::getAnimation(float animProgress, const Unit *unit,
 		}
 	}
 	if(lastAnimationIndex) {
+		if(*lastAnimationIndex != modelIndex) {
+			//printf("Switching model from [%s] to [%s]\n",(*lastAnimationIndex >= 0 ? animations[*lastAnimationIndex]->getFileName().c_str() : "none"),animations[modelIndex]->getFileName().c_str());
+		}
 		*lastAnimationIndex = modelIndex;
 	}
 
@@ -478,20 +579,77 @@ string SkillType::skillClassToStr(SkillClass skillClass) {
 	case scUpgrade: return "Upgrade";
 	default:
 		assert(false);
-		return "";
-	};
+		break;
+	}
+	return "";
 }
 
-string SkillType::fieldToStr(Field field){
-	switch(field){
-	case fLand: return "Land";
-	case fAir: return "Air";
-	default:
-		assert(false);
-		return "";
-	};
+string SkillType::fieldToStr(Field field) {
+	Lang &lang= Lang::getInstance();
+	string fieldName = "";
+	switch(field) {
+		case fLand:
+			if(lang.hasString("FieldLand") == true) {
+				fieldName = lang.get("FieldLand");
+			}
+			else {
+				fieldName = "Land";
+			}
+			//return "Land";
+			return lang.getTilesetString("FieldLandName",fieldName.c_str());
+
+		case fAir:
+			if(lang.hasString("FieldAir") == true) {
+				fieldName = lang.get("FieldAir");
+			}
+			else {
+				fieldName = "Air";
+			}
+
+			//return "Air";
+			return lang.getTilesetString("FieldAirName",fieldName.c_str());
+		default:
+			assert(false);
+			break;
+	}
+	return "";
 }
 
+void SkillType::saveGame(XmlNode *rootNode) {
+	std::map<string,string> mapTagReplacements;
+	XmlNode *skillTypeNode = rootNode->addChild("SkillType");
+
+//    SkillClass skillClass;
+	skillTypeNode->addAttribute("skillClass",intToStr(skillClass), mapTagReplacements);
+//	string name;
+	skillTypeNode->addAttribute("name",name, mapTagReplacements);
+//	int mpCost;
+	skillTypeNode->addAttribute("mpCost",intToStr(mpCost), mapTagReplacements);
+//	int hpCost;
+	skillTypeNode->addAttribute("hpCost",intToStr(hpCost), mapTagReplacements);
+//    int speed;
+	skillTypeNode->addAttribute("speed",intToStr(speed), mapTagReplacements);
+//    int animSpeed;
+	skillTypeNode->addAttribute("animSpeed",intToStr(animSpeed), mapTagReplacements);
+//    int animationRandomCycleMaxcount;
+	skillTypeNode->addAttribute("animationRandomCycleMaxcount",intToStr(animationRandomCycleMaxcount), mapTagReplacements);
+//    vector<Model *> animations;
+//    vector<AnimationAttributes> animationAttributes;
+//
+//    SoundContainer sounds;
+//	float soundStartTime;
+	skillTypeNode->addAttribute("soundStartTime",floatToStr(soundStartTime), mapTagReplacements);
+//	RandomGen random;
+	skillTypeNode->addAttribute("random",intToStr(random.getLastNumber()), mapTagReplacements);
+//	AttackBoost attackBoost;
+	attackBoost.saveGame(skillTypeNode);
+//	static int nextAttackBoostId;
+	skillTypeNode->addAttribute("nextAttackBoostId",intToStr(nextAttackBoostId), mapTagReplacements);
+//	UnitParticleSystemTypes unitParticleSystemTypes;
+	for(UnitParticleSystemTypes::iterator it = unitParticleSystemTypes.begin(); it != unitParticleSystemTypes.end(); ++it) {
+		(*it)->saveGame(skillTypeNode);
+	}
+}
 
 // =====================================================
 // 	class StopSkillType
@@ -573,9 +731,9 @@ void AttackSkillType::load(const XmlNode *sn, const XmlNode *attackBoostsNode,
     attackVar= sn->getChild("attack-var")->getAttribute("value")->getIntValue();
 
     if(attackVar < 0) {
-        char szBuf[4096]="";
-        sprintf(szBuf,"The attack skill has an INVALID attack var value which is < 0 [%d] in file [%s]!",attackVar,dir.c_str());
-        throw runtime_error(szBuf);
+        char szBuf[8096]="";
+        snprintf(szBuf,8096,"The attack skill has an INVALID attack var value which is < 0 [%d] in file [%s]!",attackVar,dir.c_str());
+        throw megaglest_runtime_error(szBuf);
     }
 
     attackRange= sn->getChild("attack-range")->getAttribute("value")->getIntValue();
@@ -603,7 +761,7 @@ void AttackSkillType::load(const XmlNode *sn, const XmlNode *attackBoostsNode,
 			attackFields[fAir]= true;
 		}
 		else{
-			throw runtime_error("Not a valid field: "+fieldName+": "+ dir);
+			throw megaglest_runtime_error("Not a valid field: "+fieldName+": "+ dir);
 		}
 	}
 
@@ -678,6 +836,52 @@ int AttackSkillType::getTotalAttackRange(const TotalUpgrade *totalUpgrade) const
 	return result;
 }
 
+void AttackSkillType::saveGame(XmlNode *rootNode) {
+	SkillType::saveGame(rootNode);
+	std::map<string,string> mapTagReplacements;
+	XmlNode *attackSkillTypeNode = rootNode->addChild("AttackSkillType");
+
+//    int attackStrength;
+	attackSkillTypeNode->addAttribute("attackStrength",intToStr(attackStrength), mapTagReplacements);
+//    int attackVar;
+	attackSkillTypeNode->addAttribute("attackVar",intToStr(attackVar), mapTagReplacements);
+//    int attackRange;
+	attackSkillTypeNode->addAttribute("attackRange",intToStr(attackRange), mapTagReplacements);
+//	const AttackType *attackType;
+	if(attackType != NULL) {
+		attackSkillTypeNode->addAttribute("attackType",attackType->getName(), mapTagReplacements);
+	}
+//	bool attackFields[fieldCount];
+	for(unsigned int i = 0; i < fieldCount; ++i) {
+		XmlNode *attackFieldsNode = attackSkillTypeNode->addChild("attackFields");
+		attackFieldsNode->addAttribute("key",intToStr(i), mapTagReplacements);
+		attackFieldsNode->addAttribute("value",intToStr(attackFields[i]), mapTagReplacements);
+	}
+//	float attackStartTime;
+	attackSkillTypeNode->addAttribute("attackStartTime",floatToStr(attackStartTime), mapTagReplacements);
+//	string spawnUnit;
+	attackSkillTypeNode->addAttribute("spawnUnit",spawnUnit, mapTagReplacements);
+//	int spawnUnitcount;
+	attackSkillTypeNode->addAttribute("spawnUnitcount",intToStr(spawnUnitcount), mapTagReplacements);
+//    bool projectile;
+	attackSkillTypeNode->addAttribute("projectile",intToStr(projectile), mapTagReplacements);
+//    ParticleSystemTypeProjectile* projectileParticleSystemType;
+	if(projectileParticleSystemType != NULL) {
+		projectileParticleSystemType->saveGame(attackSkillTypeNode);
+	}
+//	SoundContainer projSounds;
+//
+//    bool splash;
+	attackSkillTypeNode->addAttribute("splash",intToStr(splash), mapTagReplacements);
+//    int splashRadius;
+	attackSkillTypeNode->addAttribute("splashRadius",intToStr(splashRadius), mapTagReplacements);
+//    bool splashDamageAll;
+	attackSkillTypeNode->addAttribute("splashDamageAll",intToStr(splashDamageAll), mapTagReplacements);
+//    ParticleSystemTypeSplash* splashParticleSystemType;
+	if(splashParticleSystemType != NULL) {
+		splashParticleSystemType->saveGame(attackSkillTypeNode);
+	}
+}
 // =====================================================
 // 	class BuildSkillType
 // =====================================================
@@ -748,6 +952,13 @@ int ProduceSkillType::getTotalSpeed(const TotalUpgrade *totalUpgrade) const{
 	return result;
 }
 
+void ProduceSkillType::saveGame(XmlNode *rootNode) {
+	SkillType::saveGame(rootNode);
+	std::map<string,string> mapTagReplacements;
+	XmlNode *produceSkillTypeNode = rootNode->addChild("ProduceSkillType");
+
+	produceSkillTypeNode->addAttribute("animProgressBound",intToStr(animProgressBound), mapTagReplacements);
+}
 // =====================================================
 // 	class UpgradeSkillType
 // =====================================================
@@ -781,6 +992,14 @@ int UpgradeSkillType::getTotalSpeed(const TotalUpgrade *totalUpgrade) const{
 	return result;
 }
 
+void UpgradeSkillType::saveGame(XmlNode *rootNode) {
+	SkillType::saveGame(rootNode);
+	std::map<string,string> mapTagReplacements;
+	XmlNode *upgradeSkillTypeNode = rootNode->addChild("UpgradeSkillType");
+
+	upgradeSkillTypeNode->addAttribute("animProgressBound",intToStr(animProgressBound), mapTagReplacements);
+}
+
 // =====================================================
 // 	class BeBuiltSkillType
 // =====================================================
@@ -812,6 +1031,14 @@ string BeBuiltSkillType::toString() const{
 	return "Be built";
 }
 
+void BeBuiltSkillType::saveGame(XmlNode *rootNode) {
+	SkillType::saveGame(rootNode);
+	std::map<string,string> mapTagReplacements;
+	XmlNode *beBuiltSkillTypeNode = rootNode->addChild("BeBuiltSkillType");
+
+	beBuiltSkillTypeNode->addAttribute("animProgressBound",intToStr(animProgressBound), mapTagReplacements);
+}
+
 // =====================================================
 // 	class MorphSkillType
 // =====================================================
@@ -845,6 +1072,14 @@ int MorphSkillType::getTotalSpeed(const TotalUpgrade *totalUpgrade) const{
 	return result;
 }
 
+void MorphSkillType::saveGame(XmlNode *rootNode) {
+	SkillType::saveGame(rootNode);
+	std::map<string,string> mapTagReplacements;
+	XmlNode *morphSkillTypeNode = rootNode->addChild("MorphSkillType");
+
+	morphSkillTypeNode->addAttribute("animProgressBound",intToStr(animProgressBound), mapTagReplacements);
+}
+
 // =====================================================
 // 	class DieSkillType
 // =====================================================
@@ -867,6 +1102,14 @@ string DieSkillType::toString() const{
 	return "Die";
 }
 
+void DieSkillType::saveGame(XmlNode *rootNode) {
+	SkillType::saveGame(rootNode);
+	std::map<string,string> mapTagReplacements;
+	XmlNode *dieSkillTypeNode = rootNode->addChild("DieSkillType");
+
+	dieSkillTypeNode->addAttribute("fade",intToStr(fade), mapTagReplacements);
+}
+
 // =====================================================
 // 	class SkillTypeFactory
 // =====================================================
