@@ -9,10 +9,13 @@
 //	License, or (at your option) any later version
 // ==============================================================
 
+#ifdef WIN32
+    #include <winsock2.h>
+    #include <winsock.h>
+#endif
+
 #include "unit_type.h"
-
 #include <cassert>
-
 #include "util.h"
 #include "upgrade_type.h"
 #include "resource_type.h"
@@ -42,6 +45,25 @@ void Level::init(string name, int kills){
 	this->kills= kills;
 }
 
+void Level::saveGame(XmlNode *rootNode) const {
+	std::map<string,string> mapTagReplacements;
+	XmlNode *levelNode = rootNode->addChild("Level");
+
+	levelNode->addAttribute("name",name, mapTagReplacements);
+	levelNode->addAttribute("kills",intToStr(kills), mapTagReplacements);
+}
+
+const Level * Level::loadGame(const XmlNode *rootNode,const UnitType *ut) {
+	const Level *result = NULL;
+	if(rootNode->hasChild("Level") == true) {
+		const XmlNode *levelNode = rootNode->getChild("Level");
+
+		result = ut->getLevel(levelNode->getAttribute("name")->getValue());
+	}
+
+	return result;
+}
+
 // =====================================================
 // 	class UnitType
 // =====================================================
@@ -54,6 +76,7 @@ const char *UnitType::propertyNames[]= {"burnable", "rotated_climb"};
 
 UnitType::UnitType() : ProducibleType() {
 
+	countInVictoryConditions = ucvcNotSet;
 	meetingPointImage = NULL;
     lightColor= Vec3f(0.f);
     light= false;
@@ -61,6 +84,17 @@ UnitType::UnitType() : ProducibleType() {
 	armorType= NULL;
 	rotatedBuildPos=0;
 
+	field = fLand;
+	id = 0;
+	meetingPoint = false;
+	rotationAllowed = false;
+
+	countUnitDeathInStats=false;
+	countUnitProductionInStats=false;
+	countUnitKillInStats=false;
+	countKillForUnitUpgrade=false;
+
+
     for(int i=0; i<ccCount; ++i){
         firstCommandTypeOfClass[i]= NULL;
     }
@@ -117,11 +151,11 @@ void UnitType::preLoad(const string &dir) {
 	name= lastDir(dir);
 }
 
-void UnitType::load(int id,const string &dir, const TechTree *techTree,
+void UnitType::loaddd(int id,const string &dir, const TechTree *techTree, const string &techTreePath,
 		const FactionType *factionType, Checksum* checksum,
 		Checksum* techtreeChecksum, std::map<string,vector<pair<string, string> > > &loadedFileList) {
 
-	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
 	string currentPath = dir;
 	endPathWithSlash(currentPath);
@@ -133,8 +167,8 @@ void UnitType::load(int id,const string &dir, const TechTree *techTree,
 	try {
 		//Lang &lang= Lang::getInstance();
 
-		char szBuf[1024]="";
-		sprintf(szBuf,Lang::getInstance().get("LogScreenGameLoadingUnitType","",true).c_str(),formatString(name).c_str());
+		char szBuf[8096]="";
+		snprintf(szBuf,8096,Lang::getInstance().get("LogScreenGameLoadingUnitType","",true).c_str(),formatString(name).c_str());
 		Logger::getInstance().add(szBuf, true);
 
 		//file load
@@ -143,7 +177,7 @@ void UnitType::load(int id,const string &dir, const TechTree *techTree,
 
 		XmlTree xmlTree;
 		std::map<string,string> mapExtraTagReplacementValues;
-		mapExtraTagReplacementValues["$COMMONDATAPATH"] = techTree->getPath() + "/commondata/";
+		mapExtraTagReplacementValues["$COMMONDATAPATH"] = techTreePath + "/commondata/";
 		xmlTree.load(path, Properties::getTagReplacementValues(&mapExtraTagReplacementValues));
 		loadedFileList[path].push_back(make_pair(dir,dir));
 
@@ -151,6 +185,16 @@ void UnitType::load(int id,const string &dir, const TechTree *techTree,
 
 		const XmlNode *parametersNode= unitNode->getChild("parameters");
 
+		if(parametersNode->hasChild("count-in-victory-conditions") == true) {
+			bool countUnit = parametersNode->getChild("count-in-victory-conditions")->getAttribute("value")->getBoolValue();
+			if(countUnit == true) {
+				countInVictoryConditions = ucvcTrue;
+			}
+			else {
+				countInVictoryConditions = ucvcFalse;
+			}
+		}
+
 		//size
 		//checkItemInVault(&(this->size),this->size);
 		size= parametersNode->getChild("size")->getAttribute("value")->getIntValue();
@@ -224,7 +268,7 @@ void UnitType::load(int id,const string &dir, const TechTree *techTree,
 				const XmlNode *rowNode= cellMapNode->getChild("row", i);
 				string row= rowNode->getAttribute("value")->getRestrictedValue();
 				if(row.size()!=size){
-					throw runtime_error("Cellmap row has not the same length as unit size");
+					throw megaglest_runtime_error("Cellmap row has not the same length as unit size");
 				}
 				for(int j=0; j<row.size(); ++j){
 					cellMap[i*size+j]= row[j]=='0'? false: true;
@@ -254,7 +298,7 @@ void UnitType::load(int id,const string &dir, const TechTree *techTree,
 				fields[fAir]= true;
 			}
 			else{
-				throw runtime_error("Not a valid field: "+fieldName+": "+ path);
+				throw megaglest_runtime_error("Not a valid field: "+fieldName+": "+ path);
 			}
 		}
 
@@ -265,7 +309,7 @@ void UnitType::load(int id,const string &dir, const TechTree *techTree,
 			field = fAir;
 		}
 		else {
-			throw runtime_error("Unit has no field: " + path);
+			throw megaglest_runtime_error("Unit has no field: " + path);
 		}
 
 		//properties
@@ -282,7 +326,7 @@ void UnitType::load(int id,const string &dir, const TechTree *techTree,
 				}
 			}
 			if(!found) {
-				throw runtime_error("Unknown property: " + propertyName);
+				throw megaglest_runtime_error("Unknown property: " + propertyName);
 			}
 		}
 		//damage-particles
@@ -483,6 +527,41 @@ void UnitType::load(int id,const string &dir, const TechTree *techTree,
 			loadedFileList[meetingPointNode->getAttribute("image-path")->getRestrictedValue(currentPath)].push_back(make_pair(sourceXMLFile,meetingPointNode->getAttribute("image-path")->getRestrictedValue()));
 		}
 
+		//countUnitDeathInStats
+		if(parametersNode->hasChild("count-unit-death-in-stats")){
+			const XmlNode *countUnitDeathInStatsNode= parametersNode->getChild("count-unit-death-in-stats");
+			countUnitDeathInStats= countUnitDeathInStatsNode->getAttribute("value")->getBoolValue();
+		} else {
+			countUnitDeathInStats=true;
+		}
+		//countUnitProductionInStats
+		if(parametersNode->hasChild("count-unit-production-in-stats")){
+			const XmlNode *countUnitProductionInStatsNode= parametersNode->getChild("count-unit-production-in-stats");
+			countUnitProductionInStats= countUnitProductionInStatsNode->getAttribute("value")->getBoolValue();
+		} else {
+			countUnitProductionInStats=true;
+		}
+		//countUnitKillInStats
+		if(parametersNode->hasChild("count-unit-kill-in-stats")){
+			const XmlNode *countUnitKillInStatsNode= parametersNode->getChild("count-unit-kill-in-stats");
+			countUnitKillInStats= countUnitKillInStatsNode->getAttribute("value")->getBoolValue();
+		} else {
+			countUnitKillInStats=true;
+		}
+
+		//countKillForUnitUpgrade
+		if(parametersNode->hasChild("count-kill-for-unit-upgrade")){
+			const XmlNode *countKillForUnitUpgradeNode= parametersNode->getChild("count-kill-for-unit-upgrade");
+			countKillForUnitUpgrade= countKillForUnitUpgradeNode->getAttribute("value")->getBoolValue();
+		} else {
+			countKillForUnitUpgrade=true;
+		}
+
+		if(countKillForUnitUpgrade == false){
+			// it makes no sense if we count it in stats but not for upgrades
+			countUnitKillInStats=false;
+		}
+
 		//selection sounds
 		const XmlNode *selectionSoundNode= parametersNode->getChild("selection-sounds");
 		if(selectionSoundNode->getAttribute("enabled")->getBoolValue()){
@@ -520,6 +599,10 @@ void UnitType::load(int id,const string &dir, const TechTree *techTree,
 
 		const XmlNode *skillsNode= unitNode->getChild("skills");
 		skillTypes.resize(skillsNode->getChildCount());
+
+		snprintf(szBuf,8096,Lang::getInstance().get("LogScreenGameLoadingUnitTypeSkills","",true).c_str(),formatString(name).c_str(),skillTypes.size());
+		Logger::getInstance().add(szBuf, true);
+
 		for(int i = 0; i < skillTypes.size(); ++i) {
 			const XmlNode *sn= skillsNode->getChild("skill", i);
 			const XmlNode *typeNode= sn->getChild("type");
@@ -528,6 +611,7 @@ void UnitType::load(int id,const string &dir, const TechTree *techTree,
 
 			skillType->load(sn, attackBoostsNode, dir, techTree, factionType, loadedFileList,sourceXMLFile);
 			skillTypes[i]= skillType;
+
 		}
 
 		//commands
@@ -547,27 +631,46 @@ void UnitType::load(int id,const string &dir, const TechTree *techTree,
 		computeFirstCtOfClass();
 
 		if(getFirstStOfClass(scStop)==NULL){
-			throw runtime_error("Every unit must have at least one stop skill: "+ path);
+			throw megaglest_runtime_error("Every unit must have at least one stop skill: "+ path);
 		}
 		if(getFirstStOfClass(scDie)==NULL){
-			throw runtime_error("Every unit must have at least one die skill: "+ path);
+			throw megaglest_runtime_error("Every unit must have at least one die skill: "+ path);
 		}
 
 	}
 	//Exception handling (conversions and so on);
+    catch(megaglest_runtime_error& ex) {
+    	//printf("1111111a ex.wantStackTrace() = %d\n",ex.wantStackTrace());
+		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.what());
+		//printf("222222a\n");
+		throw megaglest_runtime_error("Error loading UnitType: " + path + "\n" + ex.what(),!ex.wantStackTrace());
+    }
 	catch(const exception &e){
-		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",__FILE__,__FUNCTION__,__LINE__,e.what());
-		throw runtime_error("Error loading UnitType: " + path + "\n" + e.what());
+		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.what());
+		throw megaglest_runtime_error("Error loading UnitType: " + path + "\n" + e.what());
 	}
 
-	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 }
 
 // ==================== get ====================
 
+const Level *UnitType::getLevel(string name) const {
+	const Level *result = NULL;
+	for(unsigned int i = 0; i < levels.size(); ++i) {
+		const Level &level = levels[i];
+		if(level.getName() == name) {
+			result = &level;
+			break;
+		}
+	}
+
+	return result;
+}
+
 const CommandType *UnitType::getFirstCtOfClass(CommandClass commandClass) const{
 	if(firstCommandTypeOfClass[commandClass] == NULL) {
-		//if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] commandClass = %d\n",__FILE__,__FUNCTION__,__LINE__,commandClass);
+		//if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] commandClass = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,commandClass);
 
 		/*
 	    for(int j=0; j<ccCount; ++j){
@@ -579,7 +682,7 @@ const CommandType *UnitType::getFirstCtOfClass(CommandClass commandClass) const{
 	    }
 	    */
 
-		//if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+		//if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 	}
     return firstCommandTypeOfClass[commandClass];
 }
@@ -595,7 +698,7 @@ const SkillType *UnitType::getFirstStOfClass(SkillClass skillClass) const{
 	        }
 	    }
 	    */
-		//SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+		//SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 	}
     return firstSkillTypeOfClass[skillClass];
 }
@@ -622,7 +725,7 @@ const AttackCommandType *UnitType::getFirstAttackCommand(Field field) const{
 
 	for(int i = 0; i < commandTypes.size(); ++i){
 		if(commandTypes[i] == NULL) {
-			throw runtime_error("commandTypes[i] == NULL");
+			throw megaglest_runtime_error("commandTypes[i] == NULL");
 		}
 
 		//printf("$$$ Unit [%s] i = %d, commandTypes[i] [%s]\n",this->getName().c_str(),(int)i, commandTypes[i]->toString().c_str());
@@ -643,7 +746,7 @@ const AttackStoppedCommandType *UnitType::getFirstAttackStoppedCommand(Field fie
 
 	for(int i = 0; i < commandTypes.size(); ++i){
 		if(commandTypes[i] == NULL) {
-			throw runtime_error("commandTypes[i] == NULL");
+			throw megaglest_runtime_error("commandTypes[i] == NULL");
 		}
 
 		//printf("$$$ Unit [%s] i = %d, commandTypes[i] [%s]\n",this->getName().c_str(),(int)i, commandTypes[i]->toString().c_str());
@@ -709,7 +812,7 @@ Vec2i UnitType::getFirstOccupiedCellInCellMap(Vec2i currentPos) const {
 bool UnitType::getCellMapCell(int x, int y, CardinalDir facing) const {
 	assert(cellMap);
 	if(cellMap == NULL) {
-		throw runtime_error("cellMap == NULL");
+		throw megaglest_runtime_error("cellMap == NULL");
 	}
 
 	//checkItemInVault(&(this->size),this->size);
@@ -751,11 +854,11 @@ const SkillType *UnitType::getSkillType(const string &skillName, SkillClass skil
 				return skillTypes[i];
 			}
 			else{
-				throw runtime_error("Skill \""+skillName+"\" is not of class \""+SkillType::skillClassToStr(skillClass));
+				throw megaglest_runtime_error("Skill \""+skillName+"\" is not of class \""+SkillType::skillClassToStr(skillClass));
 			}
 		}
 	}
-	throw runtime_error("No skill named \""+skillName+"\"");
+	throw megaglest_runtime_error("No skill named \""+skillName+"\"");
 }
 
 // ==================== totals ====================
@@ -770,7 +873,7 @@ int UnitType::getTotalMaxHp(const TotalUpgrade *totalUpgrade) const {
 int UnitType::getTotalMaxHpRegeneration(const TotalUpgrade *totalUpgrade) const {
 	checkItemInVault(&(this->hpRegeneration),this->hpRegeneration);
 	int result = hpRegeneration + totalUpgrade->getMaxHpRegeneration();
-	result = max(0,result);
+	//result = max(0,result);
 	return result;
 }
 
@@ -784,7 +887,7 @@ int UnitType::getTotalMaxEp(const TotalUpgrade *totalUpgrade) const {
 int UnitType::getTotalMaxEpRegeneration(const TotalUpgrade *totalUpgrade) const {
 	checkItemInVault(&(this->epRegeneration),this->epRegeneration);
 	int result = epRegeneration + totalUpgrade->getMaxEpRegeneration();
-	result = max(0,result);
+	//result = max(0,result);
 	return result;
 }
 
@@ -818,10 +921,6 @@ bool UnitType::hasCommandType(const CommandType *commandType) const {
     return false;
 }
 
-bool UnitType::hasCommandClass(CommandClass commandClass) const {
-	return firstCommandTypeOfClass[commandClass]!=NULL;
-}
-
 bool UnitType::hasSkillType(const SkillType *skillType) const {
     assert(skillType!=NULL);
     for(int i=0; i<skillTypes.size(); ++i) {
@@ -842,6 +941,7 @@ bool UnitType::isOfClass(UnitClass uc) const{
 		return hasSkillClass(scBeBuilt);
 	default:
 		assert(false);
+		break;
 	}
 	return false;
 }
@@ -884,9 +984,9 @@ const CommandType* UnitType::findCommandTypeById(int id) const{
 
 const CommandType *UnitType::getCommandType(int i) const {
 	if(i >= commandTypes.size()) {
-		char szBuf[1024]="";
-		sprintf(szBuf,"In [%s::%s Line: %d] i >= commandTypes.size(), i = %d, commandTypes.size() = %lu",__FILE__,__FUNCTION__,__LINE__,i,(unsigned long)commandTypes.size());
-		throw runtime_error(szBuf);
+		char szBuf[8096]="";
+		snprintf(szBuf,8096,"In [%s::%s Line: %d] i >= commandTypes.size(), i = %d, commandTypes.size() = " MG_SIZE_T_SPECIFIER "",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,i,commandTypes.size());
+		throw megaglest_runtime_error(szBuf);
 	}
 	return commandTypes[i];
 }
@@ -916,10 +1016,15 @@ string UnitType::getReqDesc() const{
 		return ProducibleType::getReqDesc()+"\nLimits: "+resultTxt;
 }
 
-std::string UnitType::toString() const {
-	std::string result = "";
+string UnitType::getName(bool translatedValue) const {
+	if(translatedValue == false) return name;
 
-	result = "Unit Name: [" + name + "] id = " + intToStr(id);
+	Lang &lang = Lang::getInstance();
+	return lang.getTechTreeString("UnitTypeName_" + name,name.c_str());
+}
+
+std::string UnitType::toString() const {
+	std::string result = "Unit Name: [" + name + "] id = " + intToStr(id);
 	result += " maxHp = " + intToStr(maxHp);
 	result += " hpRegeneration = " + intToStr(hpRegeneration);
 	result += " maxEp = " + intToStr(maxEp);
@@ -950,7 +1055,7 @@ std::string UnitType::toString() const {
 	result += " rotationAllowed = " + intToStr(rotationAllowed);
 
 	if(cellMap != NULL) {
-		result += " cellMap:";
+		result += " cellMap: [" + intToStr(size) + "]";
 		for(int i = 0; i < size; ++i) {
 			for(int j = 0; j < size; ++j){
 				result += " i = " + intToStr(i) + " j = " + intToStr(j) + " value = " + intToStr(cellMap[i*size+j]);
@@ -958,28 +1063,30 @@ std::string UnitType::toString() const {
 		}
 	}
 
-	result += " skillTypes:";
+	result += " skillTypes: [" + intToStr(skillTypes.size()) + "]";
 	for(int i = 0; i < skillTypes.size(); ++i) {
 		result += " i = " + intToStr(i) + " " + skillTypes[i]->toString();
 	}
 
-	result += " commandTypes:";
+	result += " commandTypes: [" + intToStr(commandTypes.size()) + "]";
 	for(int i = 0; i < commandTypes.size(); ++i) {
 		result += " i = " + intToStr(i) + " " + commandTypes[i]->toString();
 	}
 
-	result += " storedResources:";
+	result += " storedResources: [" + intToStr(storedResources.size()) + "]";
 	for(int i = 0; i < storedResources.size(); ++i) {
 		result += " i = " + intToStr(i) + " " + storedResources[i].getDescription();
 	}
 
-	result += " levels:";
+	result += " levels: [" + intToStr(levels.size()) + "]";
 	for(int i = 0; i < levels.size(); ++i) {
 		result += " i = " + intToStr(i) + " " + levels[i].getName();
 	}
 
 	result += " meetingPoint = " + intToStr(meetingPoint);
 
+	result += " countInVictoryConditions = " + intToStr(countInVictoryConditions);
+
 	return result;
 }
 
