@@ -80,6 +80,7 @@ MainWindow::MainWindow(string appPath)
 		, menuBar(NULL)
 		, panel(NULL)
         , glCanvas(NULL)
+		, fileDialog(NULL)
         , program(NULL), boxsizer(NULL), startupSettingsInited(false) {
 
 	menuFile=NULL;
@@ -430,12 +431,22 @@ void MainWindow::init(string fname) {
 }
 
 void MainWindow::onClose(wxCloseEvent &event) {
-	if( wxMessageDialog(NULL, ToUnicode("Do you want to save the current map?"),
-		ToUnicode("Question"), wxYES_NO | wxYES_DEFAULT).ShowModal() == wxID_YES) {
-		wxCommandEvent ev;
-		MainWindow::onMenuFileSave(ev);
+	if(program != NULL && program->getMap()->getHasChanged() == true) {
+		if( wxMessageDialog(NULL, ToUnicode("Do you want to save the current map?"),
+			ToUnicode("Question"), wxYES_NO | wxYES_DEFAULT).ShowModal() == wxID_YES) {
+			wxCommandEvent ev;
+			MainWindow::onMenuFileSave(ev);
+		}
 	}
-	delete this;
+	delete program;
+	program = NULL;
+
+	//delete glCanvas;
+	if(glCanvas) glCanvas->Destroy();
+	glCanvas = NULL;
+
+	//delete this;
+	this->Destroy();
 }
 
 void MainWindow::setupStartupSettings() {
@@ -459,6 +470,9 @@ void MainWindow::setupStartupSettings() {
 }
 
 MainWindow::~MainWindow() {
+	delete fileDialog;
+	fileDialog = NULL;
+
 	delete program;
 	program = NULL;
 
@@ -971,8 +985,7 @@ void MainWindow::onMenuEditInfo(wxCommandEvent &event) {
 	simpleDialog.addValue("Author", program->getMap()->getAuthor());
 	if (!simpleDialog.show("Info",true)) return;
 
-    bool ischanged = false;
-	ischanged = program->setMapTitle(simpleDialog.getValue("Title"));
+    bool ischanged = program->setMapTitle(simpleDialog.getValue("Title"));
 	ischanged = (program->setMapDesc(simpleDialog.getValue("Description")) || ischanged);
 	ischanged = (program->setMapAuthor(simpleDialog.getValue("Author")) || ischanged);
 	if (ischanged)
@@ -988,10 +1001,10 @@ void MainWindow::onMenuEditAdvanced(wxCommandEvent &event) {
 	}
 
 	SimpleDialog simpleDialog;
-	simpleDialog.addValue("Height Factor", intToStr(program->getMap()->getHeightFactor()),"(lower means map is more more zoomed in)");
-	simpleDialog.addValue("Water Level", intToStr(program->getMap()->getWaterLevel()),"(water is visible below this, and walkable until 1.5 less)");
-	simpleDialog.addValue("Cliff Level", intToStr(program->getMap()->getCliffLevel()),"(neighboring fields with at least this heights difference are cliffs)");
-	simpleDialog.addValue("Camera Height", intToStr(program->getMap()->getCameraHeight()),"(you can give a camera heigth here default is 0 ;ignored if <20)");
+	simpleDialog.addValue("Height Factor", intToStr(program->getMap()->getHeightFactor()),"lower means more hill effect. Numbers above 100 are handled like this:\nx=x/100 ,so a 150 will mean 1.5 in the game.");
+	simpleDialog.addValue("Water Level", intToStr(program->getMap()->getWaterLevel()),"water is visible below this, and walkable until 1.5 less");
+	simpleDialog.addValue("Cliff Level", intToStr(program->getMap()->getCliffLevel()),"neighboring fields with at least this heights difference are cliffs");
+	simpleDialog.addValue("Camera Height", intToStr(program->getMap()->getCameraHeight()),"you can give a camera heigth here default is 0 ;ignored if <20");
 	if (!simpleDialog.show("Advanced")) return;
 
 	try {
@@ -1455,7 +1468,7 @@ bool SimpleDialog::show(const string &title, bool wide) {
 
 	vector<wxTextCtrl*> texts;
 	for (Values::iterator it = values.begin(); it != values.end(); ++it) {
-	    int helptextpos = it->second.find_first_of('|');
+	    size_t helptextpos = it->second.find_first_of('|');
 		sizer->Add(new wxStaticText(this, -1, ToUnicode(it->first)), 0, wxALL, 5);
 		wxTextCtrl *text = new wxTextCtrl(this, -1, ToUnicode(it->second.substr(0,helptextpos)));
 		if(wide) text->SetMinSize( wxSize((text->GetSize().GetWidth())*4, text->GetSize().GetHeight()));  // 4 time as wide as default
@@ -1559,7 +1572,7 @@ bool App::OnInit() {
 
 int App::MainLoop() {
 	try {
-		//throw runtime_error("test");
+		//throw megaglest_runtime_error("test");
 		return wxApp::MainLoop();
 	}
 	catch (const exception &e) {
@@ -1582,7 +1595,6 @@ MsgDialog::MsgDialog(wxWindow *parent,
 					 const wxPoint& pos) {
 
 	m_sizerText = NULL;
-	// TODO: should we use main frame as parent by default here?
     if ( !wxDialog::Create(parent, wxID_ANY, caption,
                            pos, wxDefaultSize,
 						   style) ) {
