@@ -12,6 +12,11 @@
 #ifndef _MAPEDITOR_MAIN_H_
 #define _MAPEDITOR_MAIN_H_
 
+#ifdef WIN32
+    #include <winsock2.h>
+    #include <winsock.h>
+#endif
+
 #include <string>
 #include <vector>
 
@@ -333,6 +338,10 @@ private:
 	MainWindow *mainWindow;
 
 public:
+	App() : wxApp() {
+		mainWindow = NULL;
+	}
+	virtual ~App() {}
 	virtual bool OnInit();
 	virtual int MainLoop();
 	virtual int OnExit();
