@@ -24,7 +24,7 @@ namespace Shared { namespace Graphics {
 class BaseRenderer : public RendererMapInterface {
 public:
 	BaseRenderer() { }
-	~BaseRenderer() { }
+	virtual ~BaseRenderer() { }
 
 	virtual void initMapSurface(int clientW, int clientH);
 	virtual void renderMap(MapPreview *map, int x, int y, int clientW, int clientH, int cellSize, bool grid=false);
