@@ -18,10 +18,12 @@
 #include "pixmap.h"
 #include "texture_manager.h"
 #include "randomgen.h"
+#include "xml_parser.h"
 #include "leak_dumper.h"
 
 using std::list;
 using Shared::Util::RandomGen;
+using Shared::Xml::XmlNode;
 
 namespace Shared{ namespace Graphics{
 
@@ -64,6 +66,9 @@ public:
 	Vec4f getColor() const		{return color;}
 	float getSize() const		{return size;}
 	int getEnergy()	const		{return energy;}
+
+	void saveGame(XmlNode *rootNode);
+	void loadGame(const XmlNode *rootNode);
 };
 
 // =====================================================
@@ -74,6 +79,8 @@ class ParticleObserver{
 public:
 	virtual ~ParticleObserver(){};
 	virtual void update(ParticleSystem *particleSystem)= 0;
+	virtual void saveGame(XmlNode *rootNode) = 0;
+	virtual void loadGame(const XmlNode *rootNode, void *genericData) = 0;
 };
 
 // =====================================================
@@ -117,6 +124,10 @@ protected:
 	int aliveParticleCount;
 	int particleCount;
 	
+	string textureFileLoadDeferred;
+	int textureFileLoadDeferredSystemId;
+	Texture::Format textureFileLoadDeferredFormat;
+	int textureFileLoadDeferredComponents;
 
 	Texture *texture;
 	Vec3f pos;
@@ -156,6 +167,11 @@ public:
 	bool getActive() const						{return active;}
 	virtual bool getVisible() const				{return visible;}
 
+	virtual string getTextureFileLoadDeferred();
+	virtual int getTextureFileLoadDeferredSystemId();
+	virtual Texture::Format getTextureFileLoadDeferredFormat();
+	virtual int getTextureFileLoadDeferredComponents();
+
 	//set
 	virtual void setState(State state);
 	void setTexture(Texture *texture);
@@ -186,6 +202,11 @@ public:
 	virtual int getChildCount() { return 0; }
 	virtual ParticleSystem* getChild(int i);
 
+	string toString() const;
+
+	virtual void saveGame(XmlNode *rootNode);
+	virtual void loadGame(const XmlNode *rootNode);
+
 protected:
 	//protected
 	Particle *createParticle();
@@ -218,6 +239,9 @@ public:
 	//set params
 	void setRadius(float radius);
 	void setWind(float windAngle, float windSpeed);
+
+	virtual void saveGame(XmlNode *rootNode);
+	virtual void loadGame(const XmlNode *rootNode);
 };
 
 // =====================================================
@@ -246,13 +270,21 @@ public:
 	virtual void render(ParticleRenderer *pr, ModelRenderer *mr);
 	float getTween() { return tween; }  // 0.0 -> 1.0 for animation of model
 	Model *getModel() const {return model;}
+	virtual string getModelFileLoadDeferred();
+
 	void setPrimitive(Primitive primitive) {this->primitive= primitive;}
 	Vec3f getDirection() const {return direction;}
 	void setModelCycle(float modelCycle) {this->modelCycle= modelCycle;}
+
+	virtual void saveGame(XmlNode *rootNode);
+	virtual void loadGame(const XmlNode *rootNode);
+
 protected:
 	typedef std::vector<UnitParticleSystem*> Children;
 	Children children;
 	Primitive primitive;
+
+	string modelFileLoadDeferred;
 	Model *model;
 	float modelCycle;
 	Vec3f offset;
@@ -355,6 +387,9 @@ public:
 	void setParentDirection(Vec3f parentDirection);
 	
 	static Shape strToShape(const string& str);
+
+	virtual void saveGame(XmlNode *rootNode);
+	virtual void loadGame(const XmlNode *rootNode);
 };
 
 // =====================================================
@@ -407,7 +442,7 @@ public:
 /// Base class for Projectiles and Splashes
 // ===========================================================================
 
-class AttackParticleSystem: public GameParticleSystem{
+class AttackParticleSystem: public GameParticleSystem {
 
 protected:
 	float sizeNoEnergy;
@@ -421,6 +456,9 @@ public:
 	void setGravity(float gravity)				{this->gravity= gravity;}
 	
 	virtual void initParticleSystem() {} // opportunity to do any initialization when the system has been created and all settings set
+
+	virtual void saveGame(XmlNode *rootNode);
+	virtual void loadGame(const XmlNode *rootNode);
 };
 
 // =====================================================
@@ -478,6 +516,9 @@ public:
 	void setPath(Vec3f startPos, Vec3f endPos);
 
 	static Trajectory strToTrajectory(const string &str);
+
+	virtual void saveGame(XmlNode *rootNode);
+	virtual void loadGame(const XmlNode *rootNode);
 };
 
 // =====================================================
@@ -514,6 +555,9 @@ public:
 	void setVerticalSpreadB(float verticalSpreadB)		{this->verticalSpreadB= verticalSpreadB;}
 	void setHorizontalSpreadA(float horizontalSpreadA)	{this->horizontalSpreadA= horizontalSpreadA;}
 	void setHorizontalSpreadB(float horizontalSpreadB)	{this->horizontalSpreadB= horizontalSpreadB;}
+
+	virtual void saveGame(XmlNode *rootNode);
+	virtual void loadGame(const XmlNode *rootNode);
 	
 };
 
