@@ -20,7 +20,7 @@
 #include <stdexcept>
 #include <list>
 
-#include "types.h"
+#include "data_types.h"
 #include "checksum.h"
 #include <utility>
 #include <SDL.h>
@@ -82,6 +82,14 @@ const char vkDelete = -26;
 const char vkPrint = -27;
 const char vkPause = -29;
 
+class ShellCommandOutputCallbackInterface {
+public:
+	virtual ~ShellCommandOutputCallbackInterface() {}
+
+	virtual void * getShellCommandOutput_UserData(string cmd) = 0;
+    virtual void ShellCommandOutput_CallbackEvent(string cmd,char *output,void *userdata) = 0;
+};
+
 // =====================================================
 //	class PerformanceTimer
 // =====================================================
@@ -168,6 +176,9 @@ public:
 // =====================================================
 void Tokenize(const string& str,vector<string>& tokens,const string& delimiters = " ");
 bool isdir(const char *path);
+bool fileExists(const string &path);
+inline bool folderExists(const string &path) { return isdir(path.c_str()); }
+
 void findDirs(string path, vector<string> &results, bool errorOnNotFound,bool keepDuplicates);
 void findDirs(const vector<string> &paths, vector<string> &results, bool errorOnNotFound=false,bool keepDuplicates=false);
 void findAll(const vector<string> &paths, const string &fileFilter, vector<string> &results, bool cutExtension=false, bool errorOnNotFound=true,bool keepDuplicates=false);
@@ -177,22 +188,22 @@ vector<string> getFolderTreeContentsListRecursively(const string &path, const st
 string getCRCCacheFilePath();
 void setCRCCacheFilePath(string path);
 
-std::pair<string,string> getFolderTreeContentsCheckSumCacheKey(vector<string> paths, string pathSearchString, const string filterFileExt);
-void clearFolderTreeContentsCheckSum(vector<string> paths, string pathSearchString, const string filterFileExt);
-int32 getFolderTreeContentsCheckSumRecursively(vector<string> paths, string pathSearchString, const string filterFileExt, Checksum *recursiveChecksum,bool forceNoCache=false);
-time_t getFolderTreeContentsCheckSumRecursivelyLastGenerated(vector<string> paths, string pathSearchString, const string filterFileExt);
+std::pair<string,string> getFolderTreeContentsCheckSumCacheKey(vector<string> paths, string pathSearchString, const string &filterFileExt);
+void clearFolderTreeContentsCheckSum(vector<string> paths, string pathSearchString, const string &filterFileExt);
+uint32 getFolderTreeContentsCheckSumRecursively(vector<string> paths, string pathSearchString, const string &filterFileExt, Checksum *recursiveChecksum,bool forceNoCache=false);
+time_t getFolderTreeContentsCheckSumRecursivelyLastGenerated(vector<string> paths, string pathSearchString, const string &filterFileExt);
 
-std::pair<string,string> getFolderTreeContentsCheckSumCacheKey(const string &path, const string filterFileExt);
-void clearFolderTreeContentsCheckSum(const string &path, const string filterFileExt);
-int32 getFolderTreeContentsCheckSumRecursively(const string &path, const string &filterFileExt, Checksum *recursiveChecksum,bool forceNoCache=false);
+std::pair<string,string> getFolderTreeContentsCheckSumCacheKey(const string &path, const string &filterFileExt);
+void clearFolderTreeContentsCheckSum(const string &path, const string &filterFileExt);
+uint32 getFolderTreeContentsCheckSumRecursively(const string &path, const string &filterFileExt, Checksum *recursiveChecksum,bool forceNoCache=false);
 
-std::pair<string,string> getFolderTreeContentsCheckSumListCacheKey(vector<string> paths, string pathSearchString, const string filterFileExt);
-void clearFolderTreeContentsCheckSumList(vector<string> paths, string pathSearchString, const string filterFileExt);
-vector<std::pair<string,int32> > getFolderTreeContentsCheckSumListRecursively(vector<string> paths, string pathSearchString, const string filterFileExt, vector<std::pair<string,int32> > *recursiveMap);
+std::pair<string,string> getFolderTreeContentsCheckSumListCacheKey(vector<string> paths, string pathSearchString, const string &filterFileExt);
+void clearFolderTreeContentsCheckSumList(vector<string> paths, string pathSearchString, const string &filterFileExt);
+vector<std::pair<string,uint32> > getFolderTreeContentsCheckSumListRecursively(vector<string> paths, string pathSearchString, const string &filterFileExt, vector<std::pair<string,uint32> > *recursiveMap);
 
-std::pair<string,string> getFolderTreeContentsCheckSumListCacheKey(const string &path, const string filterFileExt);
-void clearFolderTreeContentsCheckSumList(const string &path, const string filterFileExt);
-vector<std::pair<string,int32> > getFolderTreeContentsCheckSumListRecursively(const string &path, const string &filterFileExt, vector<std::pair<string,int32> > *recursiveMap);
+std::pair<string,string> getFolderTreeContentsCheckSumListCacheKey(const string &path, const string &filterFileExt);
+void clearFolderTreeContentsCheckSumList(const string &path, const string &filterFileExt);
+vector<std::pair<string,uint32> > getFolderTreeContentsCheckSumListRecursively(const string &path, const string &filterFileExt, vector<std::pair<string,uint32> > *recursiveMap);
 
 void createDirectoryPaths(string  Path);
 string extractFileFromDirectoryPath(string filename);
@@ -213,10 +224,13 @@ void trimPathWithStartingSlash(string &path);
 void updatePathClimbingParts(string &path);
 string formatPath(string path);
 
+string replaceAllHTMLEntities(string& context);
 string replaceAll(string& context, const string& from, const string& to);
+vector<char> replaceAllBetweenTokens(vector<char>& context, const string &startToken, const string &endToken, const string &newText, bool removeTokens=true);
+string replaceAllBetweenTokens(string& context, const string &startToken, const string &endToken, const string &newText, bool removeTokens=true);
 bool removeFile(string file);
 bool renameFile(string oldFile, string newFile);
-void removeFolder(const string path);
+void removeFolder(const string &path);
 off_t getFileSize(string filename);
 bool searchAndReplaceTextInFile(string fileName, string findText, string replaceText, bool simulateOnly);
 void copyFileTo(string fromFileName, string toFileName);
@@ -234,6 +248,21 @@ string getCommandLine();
 
 #define SPACES " "
 
+inline string trim_at_delim (const string & s, const string &t)  {
+  string d (s);
+  string::size_type i(d.find(t));
+  //printf("Searching for [%s] in [%s] got " MG_SIZE_T_SPECIFIER "\n",t.c_str(),d.c_str(),i);
+
+  if (i == string::npos) {
+    return d;
+  }
+  else {
+	  d = d.erase (i) ;
+	  //printf("returning [%s]\n",d.c_str());
+	  return d;
+  }
+}
+
 inline string trim_right (const string & s, const string & t = SPACES)  {
   string d (s);
   string::size_type i (d.find_last_not_of (t));
@@ -255,15 +284,19 @@ inline string trim (const string & s, const string & t = SPACES) {
 
 string getFullFileArchiveExtractCommand(string fileArchiveExtractCommand,
 		string fileArchiveExtractCommandParameters, string outputpath, string archivename);
-bool executeShellCommand(string cmd,int expectedResult=IGNORE_CMD_RESULT_VALUE);
+bool executeShellCommand(string cmd,int expectedResult=IGNORE_CMD_RESULT_VALUE,ShellCommandOutputCallbackInterface *cb=NULL);
 string executable_path(string exeName,bool includeExeNameInPath=false);
 
+void saveDataToFile(string filename, string data);
+
 bool valid_utf8_file(const char* file_name);
 
+string getFileTextContents(string path);
+
 class ValueCheckerVault {
 
 protected:
-	std::map<const void *,int32> vaultList;
+	std::map<const void *,uint32> vaultList;
 
 	void addItemToVault(const void *ptr,int value);
 	void checkItemInVault(const void *ptr,int value) const;
