@@ -12,23 +12,11 @@
 #ifndef _SHARED_PLATFORM_SOCKET_H_
 #define _SHARED_PLATFORM_SOCKET_H_
 
-#include <string>
-
-#include <errno.h>
-#include <sys/types.h>
-#include <fcntl.h>
-#include <map>
-#include <vector>
-#include "base_thread.h"
-#include "simple_threads.h"
-#include "types.h"
-
-using std::string;
-
 #ifdef WIN32
     #ifdef __MINGW32__
 	   #include <winsock2.h>
     #else
+       #include <winsock2.h>
        #include <winsock.h>
     #endif
 
@@ -44,6 +32,18 @@ using std::string;
 
 #endif
 
+#include <string>
+#include <errno.h>
+#include <sys/types.h>
+#include <fcntl.h>
+#include <map>
+#include <vector>
+#include "base_thread.h"
+#include "simple_threads.h"
+#include "data_types.h"
+
+using std::string;
+
 #include "leak_dumper.h"
 
 using namespace Shared::PlatformCommon;
@@ -55,6 +55,8 @@ class FTPClientValidationInterface {
 public:
 	virtual int isValidClientType(uint32 clientIp) = 0;
 	virtual int isClientAllowedToGetFile(uint32 clientIp, const char *username, const char *filename) = 0;
+
+	virtual ~FTPClientValidationInterface() {}
 };
 
 
@@ -62,6 +64,8 @@ public:
 class UPNPInitInterface {
 public:
 	virtual void UPNPInitStatus(bool result) = 0;
+
+	virtual ~UPNPInitInterface() {}
 };
 
 //
@@ -71,6 +75,7 @@ public:
 class DiscoveredServersInterface {
 public:
 	virtual void DiscoveredServers(std::vector<string> serverList) = 0;
+	virtual ~DiscoveredServersInterface() {}
 };
 
 // =====================================================
@@ -102,7 +107,6 @@ public:
 };
 #endif
 
-//class Socket : public SimpleTaskCallbackInterface {
 class Socket {
 
 protected:
@@ -156,13 +160,13 @@ public:
 	int getDataToRead(bool wantImmediateReply=false);
 	int send(const void *data, int dataSize);
 	int receive(void *data, int dataSize, bool tryReceiveUntilDataSizeMet);
-	int peek(void *data, int dataSize, bool mustGetData=true);
+	int peek(void *data, int dataSize, bool mustGetData=true,int *pLastSocketError=NULL);
 
 	void setBlock(bool block);
 	static void setBlock(bool block, PLATFORM_SOCKET socket);
 
 	bool isReadable();
-	bool isWritable();
+	bool isWritable(struct timeval *timeVal=NULL);
 	bool isConnected();
 
 	static string getHostName();
@@ -216,9 +220,10 @@ class BroadCastSocketThread : public BaseThread
 private:
 	Mutex *mutexPauseBroadcast;
 	bool pauseBroadcast;
+	int boundPort;
 
 public:
-	BroadCastSocketThread();
+	BroadCastSocketThread(int boundPort);
 	virtual ~BroadCastSocketThread();
     virtual void execute();
     virtual bool canShutdown(bool deleteSelfIfShutdownDelayed=false);
@@ -235,6 +240,7 @@ protected:
 
 	bool portBound;
 	int boundPort;
+	string bindSpecificAddress;
 
 	static int externalPort;
 	static int ftpServerPort;
@@ -248,8 +254,10 @@ protected:
 	bool isBroadCastThreadRunning();
 	vector<string> blockIPList;
 
+	bool basicMode;
+
 public:
-	ServerSocket();
+	ServerSocket(bool basicMode = false);
 	virtual ~ServerSocket();
 	void bind(int port);
 	void listen(int connectionQueueSize= SOMAXCONN);
@@ -268,6 +276,8 @@ public:
 	int getBindPort() const { return boundPort; }
 	bool isPortBound() const { return portBound; }
 
+	void setBindSpecificAddress(string value) { bindSpecificAddress = value;}
+
 	static void setExternalPort(int port) { externalPort = port; }
 	static int getExternalPort() { return externalPort; }
 
