@@ -29,9 +29,12 @@ private:
 	ContextGl context;
 
 public:
-	void initGl(int colorBits, int depthBits, int stencilBits,bool hardware_acceleration, bool fullscreen_anti_aliasing);
+	void initGl(int colorBits, int depthBits, int stencilBits,
+			    bool hardware_acceleration, bool fullscreen_anti_aliasing,
+			    float gammaValue);
 	void makeCurrentGl();
 	void swapBuffersGl();
+	void setGamma(float gammaValue){context.setGammaValue(gammaValue);}
 };
 
 }}//end namespace
