@@ -1,7 +1,8 @@
 // ==============================================================
-//	This file is part of Glest Shared Library (www.glest.org)
+//	This file is part of MegaGlest Shared Library (www.megaglest.org)
 //
-//	Copyright (C) 2001-2008 Martiño Figueroa
+//	Copyright (C) 2012 Mark Vejvoda, Titus Tscharntke
+//                2001-2008 Martiño Figueroa
 //
 //	You can redistribute this code and/or modify it under 
 //	the terms of the GNU General Public License as published 
@@ -39,6 +40,9 @@ public:
 	int rand();
 	int randRange(int min, int max);
 	float randRange(float min, float max);
+
+	int getLastNumber() const { return lastNumber; }
+	void setLastNumber(int value) { lastNumber = value; }
 };
 
 }}//end namespace
