@@ -1,3 +1,10 @@
+// ==============================================================
+//	This file is part of MegaGlest Shared Library (www.megaglest.org)
+//
+//	Copyright (C) 2012 Mark Vejvoda, Titus Tscharntke
+//	The Megaglest Team, under GNU GPL v3.0
+// ==============================================================
+
 #include "font_d3d9.h"
 
 #include <stdexcept>
@@ -25,7 +32,7 @@ void Font2DD3d9::init(){
 
 	HRESULT result= D3DXCreateFont(d3dDevice, hFont, &d3dFont);
 	if(result!=D3D_OK){
-		throw runtime_error("FontD3d9::init() -> Can't create D3D font");
+		throw megaglest_runtime_error("FontD3d9::init() -> Can't create D3D font");
 	}
 	DeleteObject(hFont);
 }
@@ -35,4 +42,4 @@ void Font2DD3d9::end(){
 }
 
 
-}}}//end namespace
\ No newline at end of file
+}}}//end namespace
