@@ -24,13 +24,16 @@ using namespace Shared::Graphics::Gl;
 #include "font_text_freetypegl.h"
 #endif
 
+// If your compiler has a version older than 2.4.1 of fontconfig, you can tell cmake to
+// disable trying to use fontconfig via passing this to cmake:
+// -DWANT_FONTCONFIG=Off
 #ifdef HAVE_FONTCONFIG
 #include <fontconfig/fontconfig.h>
 #endif
 
 #include "util.h"
 #include "platform_common.h"
-
+#include "platform_util.h"
 #include "leak_dumper.h"
 
 using namespace std;
@@ -120,24 +123,42 @@ Text * FontMetrics::getTextHandler() {
 }
 
 float FontMetrics::getTextWidth(const string &str) {
+	string longestLine = "";
+	size_t found = str.find("\n");
+	if (found == string::npos) {
+		longestLine = str;
+	}
+	else {
+		vector<string> lineTokens;
+		Tokenize(str,lineTokens,"\n");
+		if(lineTokens.empty() == false) {
+			for(unsigned int i = 0; i < lineTokens.size(); ++i) {
+				string currentStr = lineTokens[i];
+				if(currentStr.length() > longestLine.length()) {
+					longestLine = currentStr;
+				}
+			}
+		}
+    }
+
 	if(textHandler != NULL) {
 		//printf("str [%s] textHandler->Advance = %f Font::scaleFontValue = %f\n",str.c_str(),textHandler->Advance(str.c_str()),Font::scaleFontValue);
-		return (textHandler->Advance(str.c_str()) * Font::scaleFontValue);
+		return (textHandler->Advance(longestLine.c_str()) * Font::scaleFontValue);
 		//return (textHandler->Advance(str.c_str()));
 	}
 	else {
 		float width= 0.f;
-		for(unsigned int i=0; i< str.size() && (int)i < Font::charCount; ++i){
-			if(str[i] >= Font::charCount) {
-				string sError = "str[i] >= Font::charCount, [" + str + "] i = " + intToStr(i);
-				throw runtime_error(sError);
+		for(unsigned int i=0; i< longestLine.size() && (int)i < Font::charCount; ++i){
+			if(longestLine[i] >= Font::charCount) {
+				string sError = "str[i] >= Font::charCount, [" + longestLine + "] i = " + uIntToStr(i);
+				throw megaglest_runtime_error(sError);
 			}
 			//Treat 2 byte characters as spaces
-			if(str[i] < 0) {
+			if(longestLine[i] < 0) {
 				width+= (widths[97]); // This is the letter a which is a normal wide character and good to use for spacing
 			}
 			else {
-				width+= widths[str[i]];
+				width+= widths[longestLine[i]];
 			}
 		}
 		return width;
@@ -155,6 +176,31 @@ float FontMetrics::getHeight(const string &str) const {
 	}
 }
 
+string FontMetrics::wordWrapText(string text, int maxWidth) {
+	// Strip newlines from source
+    //replaceAll(text, "\n", " ");
+
+	// Get all words (space separated text)
+	vector<string> words;
+	Tokenize(text,words," ");
+
+	string wrappedText = "";
+    float lineWidth = 0.0f;
+
+    for(unsigned int i = 0; i < words.size(); ++i) {
+    	string word = words[i];
+        float wordWidth = this->getTextWidth(word);
+        if (lineWidth + wordWidth > maxWidth) {
+            wrappedText += "\n";
+            lineWidth = 0;
+        }
+        lineWidth += wordWidth;
+        wrappedText += word + " ";
+    }
+
+    return wrappedText;
+}
+
 // ===============================================
 //	class Font
 // ===============================================
@@ -274,6 +320,11 @@ Font3D::Font3D(FontTextHandlerType type) : Font(type) {
 
 string findFontFamily(const char* font, const char *fontFamily) {
 	string resultFile = "";
+
+	// If your compiler has a version older than 2.4.1 of fontconfig, you can tell cmake to
+	// disable trying to use fontconfig via passing this to cmake:
+	// -DWANT_FONTCONFIG=Off
+
 #ifdef HAVE_FONTCONFIG
 	// Get default font via fontconfig
 	if( !font && FcInit() && fontFamily)	{
@@ -298,7 +349,7 @@ string findFontFamily(const char* font, const char *fontFamily) {
 		fs = FcFontSetCreate();
 		match = FcFontMatch(0, pat, &result);
 
-		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Trying fontconfig for fontfamily [%s]\n",fontFamily);
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Trying fontconfig for fontfamily [%s]\n",(fontFamily != NULL ? fontFamily : "null"));
 
 		if (match) FcFontSetAdd(fs, match);
 		if (pat) FcPatternDestroy(pat);
@@ -310,8 +361,14 @@ string findFontFamily(const char* font, const char *fontFamily) {
 			}
 			FcFontSetDestroy(fs);
 		}
+		// If your compiler has a version older than 2.4.1 of fontconfig, you can tell cmake to
+		// disable trying to use fontconfig via passing this to cmake:
+		// -DWANT_FONTCONFIG=Off
 		FcFini();
 	}
+	else {
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("******************* FONT CONFIG will not be called font [%s] fontFamily [%s]!\n",(font != NULL ? font : "null"),(fontFamily != NULL ? fontFamily : "null"));
+	}
 #else
 	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("******************* NO FONT CONFIG ENABLED!\n");
 #endif
@@ -326,25 +383,30 @@ const char* findFont(const char *firstFontToTry,const char *firstFontFamilyToTry
 	#define CHECK_FONT_PATH(filename,fontFamily) \
 	{ \
 		path = filename; \
-		if( !font && path && strlen(path) > 0 && fileExists(path) == true ) { \
+		if( font == NULL && path != NULL && strlen(path) > 0 && fileExists(path) == true ) { \
 			font = strdup(path); \
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#1 candidate font file exists [%s]\n",(font != NULL ? font : "null")); \
 		} \
-		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#1 Searching for font file [%s] result [%s]\n",path,font); \
-		if( !font && fontFamily && strlen(fontFamily) > 0) { \
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#1 Searching for font file [%s] result [%s]\n",(path != NULL ? path : "null"),(font != NULL ? font : "null")); \
+		if( font == NULL && fontFamily != NULL && strlen(fontFamily) > 0) { \
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#2 Searching for font [%s] family [%s]\n",(font != NULL ? font : "null"),(fontFamily != NULL ? fontFamily : "null")); \
 			string fileFound = findFontFamily(font, fontFamily); \
 			if(fileFound != "") { \
 				path = fileFound.c_str(); \
-				if( !font && path && strlen(path) > 0 && fileExists(path) == true ) { \
+				if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#2 candidate font file found [%s]\n",fileFound.c_str()); \
+				if( path != NULL && strlen(path) > 0 && fileExists(path) == true ) { \
+					if(font) free((void*)font); \
 					font = strdup(path); \
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#2 candidate font file has been set[%s]\n",(font != NULL ? font : "null")); \
 				} \
 			} \
-			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#2 Searching for font family [%s] result [%s]\n",fontFamily,font); \
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("#2 Searching for font family [%s] result [%s]\n",(fontFamily != NULL ? fontFamily : "null"),(font != NULL ? font : "null")); \
 		} \
 	}
 
 	string tryFont = "";
-	if(firstFontToTry || firstFontFamilyToTry) {
-		if(firstFontToTry && strlen(firstFontToTry) > 0) {
+	if(firstFontToTry != NULL || firstFontFamilyToTry != NULL) {
+		if(firstFontToTry != NULL && strlen(firstFontToTry) > 0) {
 			tryFont = firstFontToTry;
 			#ifdef WIN32
 			  replaceAll(tryFont, "/", "\\");
@@ -363,6 +425,10 @@ const char* findFont(const char *firstFontToTry,const char *firstFontFamilyToTry
 
 		if(getenv("MEGAGLEST_FONT") != NULL) {
 			tryFont = getenv("MEGAGLEST_FONT");
+
+			if(Text::DEFAULT_FONT_PATH_ABSOLUTE != "") {
+				tryFont = Text::DEFAULT_FONT_PATH_ABSOLUTE + "/" + extractFileFromDirectoryPath(tryFont);
+			}
 			#ifdef WIN32
 			  replaceAll(tryFont, "/", "\\");
 			#endif
@@ -376,16 +442,20 @@ const char* findFont(const char *firstFontToTry,const char *firstFontFamilyToTry
 
 	string data_path = Text::DEFAULT_FONT_PATH;
 	string defaultFont = data_path + "data/core/fonts/LinBiolinum_RB.ttf";//LinBiolinum_Re-0.6.4.ttf
+	if(Text::DEFAULT_FONT_PATH_ABSOLUTE != "") {
+		data_path = Text::DEFAULT_FONT_PATH;
+		defaultFont = data_path + "/LinBiolinum_RB.ttf";//LinBiolinum_Re-0.6.4.ttf
+	}
 	tryFont = defaultFont;
 	#ifdef WIN32
 	  replaceAll(tryFont, "/", "\\");
 	#endif
 	CHECK_FONT_PATH(tryFont.c_str(),"Linux Biolinum O:style=Bold")
 
-#ifdef FONT_PATH
+	#ifdef FONT_PATH
 	// Get distro-specified font path
 	CHECK_FONT_PATH(FONT_PATH)
-#endif
+	#endif
 
 	CHECK_FONT_PATH("/usr/share/fonts/truetype/uralic/gothub__.ttf","Gothic Uralic:style=Regular")
 
@@ -429,6 +499,17 @@ const char* findFont(const char *firstFontToTry,const char *firstFontFamilyToTry
 	CHECK_FONT_PATH("/usr/share/fonts/truetype/freefont/FreeSans.ttf","FreeSans")
 	CHECK_FONT_PATH("/usr/share/fonts/truetype/freefont/FreeMono.ttf","FreeMono")
 
+	//openSUSE
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/LinBiolinum_RB.otf","Bold")
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/FreeSerif.ttf","FreeSerif")
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/FreeSans.ttf","FreeSans")
+	CHECK_FONT_PATH("/usr/share/fonts/truetype/FreeMono.ttf","FreeMono")
+
+	// gentoo paths
+	CHECK_FONT_PATH("/usr/share/fonts/freefont-ttf/FreeSerif.ttf","FreeSerif")
+	CHECK_FONT_PATH("/usr/share/fonts/freefont-ttf/FreeSans.ttf","FreeSans")
+	CHECK_FONT_PATH("/usr/share/fonts/freefont-ttf/FreeMono.ttf","FreeMono")
+
 #ifdef _WIN32
 	CHECK_FONT_PATH("c:\\windows\\fonts\\verdana.ttf",NULL)
 	CHECK_FONT_PATH("c:\\windows\\fonts\\tahoma.ttf",NULL)
@@ -436,6 +517,8 @@ const char* findFont(const char *firstFontToTry,const char *firstFontFamilyToTry
 	CHECK_FONT_PATH("\\windows\\fonts\\arial.ttf",NULL)
 #endif
 
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Final selection of font file is [%s]\n",(font != NULL ? font : "null")); \
+
 	return font;
 }
 
