@@ -176,11 +176,11 @@ void TextRenderer2DGl::render(const string &text, float x, float y, bool centere
 			else {
 				bool lastCharacterWasSpecial = true;
 				vector<string> parts;
-				char szBuf[4096]="";
+				char szBuf[8096]="";
 
 				for (int i=0; renderText[i] != '\0'; ++i) {
 					szBuf[0] = '\0';
-					sprintf(szBuf,"%c",renderText[i]);
+					snprintf(szBuf,8096,"%c",renderText[i]);
 
 					switch(renderText[i]) {
 						case '\t':
@@ -205,6 +205,7 @@ void TextRenderer2DGl::render(const string &text, float x, float y, bool centere
 								parts[parts.size()-1] += szBuf;
 							}
 							lastCharacterWasSpecial = false;
+							break;
 					}
 				}
 
@@ -221,6 +222,7 @@ void TextRenderer2DGl::render(const string &text, float x, float y, bool centere
 							break;
 						default:
 							font->getTextHandler()->Render(parts[i].c_str());
+							break;
 					}
 				}
 			}
@@ -270,11 +272,11 @@ void TextRenderer2DGl::render(const string &text, float x, float y, bool centere
 			else {
 				bool lastCharacterWasSpecial = true;
 				vector<string> parts;
-				char szBuf[4096]="";
+				char szBuf[8096]="";
 
 				for (int i=0; renderText[i] != '\0'; ++i) {
 					szBuf[0] = '\0';
-					sprintf(szBuf,"%c",renderText[i]);
+					snprintf(szBuf,8096,"%c",renderText[i]);
 
 					switch(renderText[i]) {
 						case '\t':
@@ -299,6 +301,7 @@ void TextRenderer2DGl::render(const string &text, float x, float y, bool centere
 								parts[parts.size()-1] += szBuf;
 							}
 							lastCharacterWasSpecial = false;
+							break;
 					}
 				}
 
@@ -315,6 +318,7 @@ void TextRenderer2DGl::render(const string &text, float x, float y, bool centere
 							break;
 						default:
 							font->getTextHandler()->Render(parts[i].c_str());
+							break;
 					}
 				}
 			}
@@ -567,11 +571,11 @@ void TextRenderer3DGl::internalRender(const string &text, float  x, float y, boo
 			int line=0;
 			bool lastCharacterWasSpecial = true;
 			vector<string> parts;
-			char szBuf[4096]="";
+			char szBuf[8096]="";
 
 			for (int i=0; renderText[i] != '\0'; ++i) {
 				szBuf[0] = '\0';
-				sprintf(szBuf,"%c",renderText[i]);
+				snprintf(szBuf,8096,"%c",renderText[i]);
 
 				switch(renderText[i]) {
 					case '\t':
@@ -590,6 +594,7 @@ void TextRenderer3DGl::internalRender(const string &text, float  x, float y, boo
 							parts[parts.size()-1] += szBuf;
 						}
 						lastCharacterWasSpecial = false;
+						break;
 				}
 			}
 
@@ -635,6 +640,7 @@ void TextRenderer3DGl::internalRender(const string &text, float  x, float y, boo
 							font->getTextHandler()->Render(parts[i].c_str());
 							specialFTGLErrorCheckWorkaround(parts[i]);
 						}
+						break;
 				}
 			}
 		}
