@@ -328,7 +328,7 @@ GLint toCompressionFormatGl(GLint format) {
 	}
 
 	static std::vector<int> supportedCompressionFormats = getSupportCompressedTextureFormats();
-	if(supportedCompressionFormats.size() <= 0) {
+	if(supportedCompressionFormats.empty() == true) {
 		return format;
 	}
 
@@ -486,7 +486,7 @@ bool TextureGl::supports_FBO_RBO() {
 
 void TextureGl::setup_FBO_RBO() {
 	if(getTextureWidth() < 0 || getTextureHeight() < 0) {
-		throw runtime_error("getTextureWidth() < 0 || getTextureHeight() < 0");
+		throw megaglest_runtime_error("getTextureWidth() < 0 || getTextureHeight() < 0");
 	}
 
 	// Need some work to get extensions properly working in Windows (use Glew lib)
@@ -689,11 +689,11 @@ void Texture1DGl::init(Filter filter, int maxAnisotropy) {
 			//
 
 			if(error != 0) {
-				//throw runtime_error("Error building texture 1D mipmaps");
+				//throw megaglest_runtime_error("Error building texture 1D mipmaps");
 				const char *errorString= reinterpret_cast<const char*>(gluErrorString(error));
-				char szBuf[1024]="";
-				sprintf(szBuf,"Error building texture 1D mipmaps, returned: %d [%s] for [%s] w = %d, glCompressionFormat = %d",error,errorString,pixmap.getPath().c_str(),pixmap.getW(),glCompressionFormat);
-				throw runtime_error(szBuf);
+				char szBuf[8096]="";
+				snprintf(szBuf,8096,"Error building texture 1D mipmaps, returned: %d [%s] for [%s] w = %d, glCompressionFormat = %d",error,errorString,pixmap.getPath().c_str(),pixmap.getW(),glCompressionFormat);
+				throw megaglest_runtime_error(szBuf);
 			}
 		}
 		else {
@@ -722,11 +722,11 @@ void Texture1DGl::init(Filter filter, int maxAnisotropy) {
 			//
 
 			if(error != GL_NO_ERROR) {
-				//throw runtime_error("Error creating texture 1D");
+				//throw megaglest_runtime_error("Error creating texture 1D");
 				const char *errorString= reinterpret_cast<const char*>(gluErrorString(error));
-				char szBuf[1024]="";
-				sprintf(szBuf,"Error creating texture 1D, returned: %d [%s] (%X) [%s] w = %d, glCompressionFormat = %d",error,errorString,error,pixmap.getPath().c_str(),pixmap.getW(),glCompressionFormat);
-				throw runtime_error(szBuf);
+				char szBuf[8096]="";
+				snprintf(szBuf,8096,"Error creating texture 1D, returned: %d [%s] (%X) [%s] w = %d, glCompressionFormat = %d",error,errorString,error,pixmap.getPath().c_str(),pixmap.getW(),glCompressionFormat);
+				throw megaglest_runtime_error(szBuf);
 			}
 		}
 		inited= true;
@@ -858,11 +858,11 @@ void Texture2DGl::init(Filter filter, int maxAnisotropy) {
 			//
 
 			if(error != GL_NO_ERROR) {
-				//throw runtime_error("Error building texture 2D mipmaps");
+				//throw megaglest_runtime_error("Error building texture 2D mipmaps");
 				const char *errorString= reinterpret_cast<const char*>(gluErrorString(error));
-				char szBuf[1024]="";
-				sprintf(szBuf,"Error building texture 2D mipmaps [%s], returned: %d [%s] for [%s] w = %d, h = %d, glCompressionFormat = %d",this->path.c_str(),error,errorString,(pixmap.getPath() != "" ? pixmap.getPath().c_str() : this->path.c_str()),pixmap.getW(),pixmap.getH(),glCompressionFormat);
-				throw runtime_error(szBuf);
+				char szBuf[8096]="";
+				snprintf(szBuf,8096,"Error building texture 2D mipmaps [%s], returned: %d [%s] for [%s] w = %d, h = %d, glCompressionFormat = %d",this->path.c_str(),error,errorString,(pixmap.getPath() != "" ? pixmap.getPath().c_str() : this->path.c_str()),pixmap.getW(),pixmap.getH(),glCompressionFormat);
+				throw megaglest_runtime_error(szBuf);
 			}
 		}
 		else {
@@ -925,13 +925,13 @@ void Texture2DGl::init(Filter filter, int maxAnisotropy) {
 
 			//
 
-			//throw runtime_error("TEST!");
+			//throw megaglest_runtime_error("TEST!");
 
 			if(error != GL_NO_ERROR) {
 				const char *errorString= reinterpret_cast<const char*>(gluErrorString(error));
-				char szBuf[1024]="";
-				sprintf(szBuf,"Error creating texture 2D [%s], returned: %d [%s] (%X) [%s] w = %d, h = %d, glInternalFormat = %d, glFormat = %d, glCompressionFormat = %d",this->path.c_str(),error,errorString,error,pixmap.getPath().c_str(),pixmap.getW(),pixmap.getH(),glInternalFormat,glFormat,glCompressionFormat);
-				throw runtime_error(szBuf);
+				char szBuf[8096]="";
+				snprintf(szBuf,8096,"Error creating texture 2D [%s], returned: %d [%s] (%X) [%s] w = %d, h = %d, glInternalFormat = %d, glFormat = %d, glCompressionFormat = %d",this->path.c_str(),error,errorString,error,pixmap.getPath().c_str(),pixmap.getW(),pixmap.getH(),glInternalFormat,glFormat,glCompressionFormat);
+				throw megaglest_runtime_error(szBuf);
 			}
 		}
 		inited= true;
@@ -1037,11 +1037,11 @@ void Texture3DGl::init(Filter filter, int maxAnisotropy) {
 		//
 
 		if(error != GL_NO_ERROR) {
-			//throw runtime_error("Error creating texture 3D");
+			//throw megaglest_runtime_error("Error creating texture 3D");
 			const char *errorString= reinterpret_cast<const char*>(gluErrorString(error));
-			char szBuf[1024]="";
-			sprintf(szBuf,"Error creating texture 3D, returned: %d [%s] (%X) [%s] w = %d, h = %d, d = %d, glCompressionFormat = %d",error,errorString,error,pixmap.getPath().c_str(),pixmap.getW(),pixmap.getH(),pixmap.getD(),glCompressionFormat);
-			throw runtime_error(szBuf);
+			char szBuf[8096]="";
+			snprintf(szBuf,8096,"Error creating texture 3D, returned: %d [%s] (%X) [%s] w = %d, h = %d, d = %d, glCompressionFormat = %d",error,errorString,error,pixmap.getPath().c_str(),pixmap.getW(),pixmap.getH(),pixmap.getD(),glCompressionFormat);
+			throw megaglest_runtime_error(szBuf);
 		}
 		inited= true;
 
@@ -1178,11 +1178,11 @@ void TextureCubeGl::init(Filter filter, int maxAnisotropy) {
 				//
 
 				if(error != GL_NO_ERROR) {
-					//throw runtime_error("Error building texture cube mipmaps");
+					//throw megaglest_runtime_error("Error building texture cube mipmaps");
 					const char *errorString= reinterpret_cast<const char*>(gluErrorString(error));
-					char szBuf[1024]="";
-					sprintf(szBuf,"Error building texture cube mipmaps, returned: %d [%s] for [%s] w = %d, h = %d, glCompressionFormat = %d",error,errorString,currentPixmap->getPath().c_str(),currentPixmap->getW(),currentPixmap->getH(),glCompressionFormat);
-					throw runtime_error(szBuf);
+					char szBuf[8096]="";
+					snprintf(szBuf,8096,"Error building texture cube mipmaps, returned: %d [%s] for [%s] w = %d, h = %d, glCompressionFormat = %d",error,errorString,currentPixmap->getPath().c_str(),currentPixmap->getW(),currentPixmap->getH(),glCompressionFormat);
+					throw megaglest_runtime_error(szBuf);
 				}
 			}
 			else {
@@ -1231,11 +1231,11 @@ void TextureCubeGl::init(Filter filter, int maxAnisotropy) {
 				//
 
 				if(error != GL_NO_ERROR) {
-					//throw runtime_error("Error creating texture cube");
+					//throw megaglest_runtime_error("Error creating texture cube");
 					const char *errorString= reinterpret_cast<const char*>(gluErrorString(error));
-					char szBuf[1024]="";
-					sprintf(szBuf,"Error creating texture cube, returned: %d [%s] (%X) [%s] w = %d, h = %d, glCompressionFormat = %d",error,errorString,error,currentPixmap->getPath().c_str(),currentPixmap->getW(),currentPixmap->getH(),glCompressionFormat);
-					throw runtime_error(szBuf);
+					char szBuf[8096]="";
+					snprintf(szBuf,8096,"Error creating texture cube, returned: %d [%s] (%X) [%s] w = %d, h = %d, glCompressionFormat = %d",error,errorString,error,currentPixmap->getPath().c_str(),currentPixmap->getW(),currentPixmap->getH(),glCompressionFormat);
+					throw megaglest_runtime_error(szBuf);
 				}
 			}
 
