@@ -37,26 +37,30 @@ namespace Shared{ namespace Platform{
 //	class PlatformContextGl
 // ======================================
 PlatformContextGl::PlatformContextGl() {
-	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 	icon = NULL;
 	screen = NULL;
 }
 
 PlatformContextGl::~PlatformContextGl() {
-	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
 	end();
 
-	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 }
 
-void PlatformContextGl::init(int colorBits, int depthBits, int stencilBits,bool hardware_acceleration, bool fullscreen_anti_aliasing) {
+void PlatformContextGl::init(int colorBits, int depthBits, int stencilBits,
+		                     bool hardware_acceleration,
+		                     bool fullscreen_anti_aliasing, float gammaValue) {
 	
-	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+    if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
 	Window::setupGraphicsScreen(depthBits, stencilBits, hardware_acceleration, fullscreen_anti_aliasing);
 
-	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
 	int flags = SDL_OPENGL;
 	if(PlatformCommon::Private::shouldBeFullscreen) {
@@ -74,76 +78,7 @@ void PlatformContextGl::init(int colorBits, int depthBits, int stencilBits,bool
 
 	if(GlobalStaticFlags::getIsNonGraphicalModeEnabled() == false) {
 
-	#ifndef WIN32
-		string mg_icon_file = "";
-	#if defined(CUSTOM_DATA_INSTALL_PATH_VALUE)
-		if(fileExists(formatPath(TOSTRING(CUSTOM_DATA_INSTALL_PATH_VALUE)) + "megaglest.png")) {
-			mg_icon_file = formatPath(TOSTRING(CUSTOM_DATA_INSTALL_PATH_VALUE)) + "megaglest.png";
-		}
-		else if(fileExists(formatPath(TOSTRING(CUSTOM_DATA_INSTALL_PATH_VALUE)) + "megaglest.bmp")) {
-			mg_icon_file = formatPath(TOSTRING(CUSTOM_DATA_INSTALL_PATH_VALUE)) + "megaglest.bmp";
-		}
-
-	#endif
-
-		if(mg_icon_file == "" && fileExists("megaglest.png")) {
-			mg_icon_file = "megaglest.png";
-		}
-		else if(mg_icon_file == "" && fileExists("megaglest.bmp")) {
-			mg_icon_file = "megaglest.bmp";
-		}
-		else if(mg_icon_file == "" && fileExists("/usr/share/pixmaps/megaglest.png")) {
-			mg_icon_file = "/usr/share/pixmaps/megaglest.png";
-		}
-		else if(mg_icon_file == "" && fileExists("/usr/share/pixmaps/megaglest.bmp")) {
-			mg_icon_file = "/usr/share/pixmaps/megaglest.bmp";
-		}
-
-		if(mg_icon_file != "") {
-
-			if(icon != NULL) {
-				SDL_FreeSurface(icon);
-				icon = NULL;
-			}
-
-			//printf("Loading icon [%s]\n",mg_icon_file.c_str());
-			if(extractExtension(mg_icon_file) == "bmp") {
-				icon = SDL_LoadBMP(mg_icon_file.c_str());
-			}
-			else {
-				Texture2D *texture2D = GraphicsInterface::getInstance().getFactory()->newTexture2D();
-				texture2D->load(mg_icon_file);
-				icon = texture2D->CreateSDLSurface(true);
-				delete texture2D;
-			}
-
-		//SDL_Surface *icon = IMG_Load("megaglest.ico");
-
-
-	//#if !defined(MACOSX)
-		// Set Icon (must be done before any sdl_setvideomode call)
-		// But don't set it on OS X, as we use a nicer external icon there.
-	//#if WORDS_BIGENDIAN
-	//	SDL_Surface* icon= SDL_CreateRGBSurfaceFrom((void*)logo,32,32,8,128,0xff000000,0x00ff0000,0x0000ff00,0);
-	//#else
-	//	SDL_Surface* icon= SDL_CreateRGBSurfaceFrom((void*)logo,32,32,32,128,0x000000ff,0x0000ff00,0x00ff0000,0xff000000);
-	//#endif
-
-			//printf("In [%s::%s Line: %d] icon = %p\n",__FILE__,__FUNCTION__,__LINE__,icon);
-			if(icon == NULL) {
-				printf("Error: %s\n", SDL_GetError());
-			}
-			if(icon != NULL) {
-
-				//uint32 colorkey = SDL_MapRGB(icon->format, 255, 0, 255);
-				//SDL_SetColorKey(icon, SDL_SRCCOLORKEY, colorkey);
-				SDL_SetColorKey(icon, SDL_SRCCOLORKEY, SDL_MapRGB(icon->format, 255, 0, 255));
-
-				SDL_WM_SetIcon(icon, NULL);
-			}
-		}
-	#endif
-
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] about to set resolution: %d x %d, colorBits = %d.\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,resW,resH,colorBits);
 
 		if(screen != NULL) {
@@ -151,9 +86,12 @@ void PlatformContextGl::init(int colorBits, int depthBits, int stencilBits,bool
 			screen = NULL;
 		}
 
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
 		screen = SDL_SetVideoMode(resW, resH, colorBits, flags);
 		if(screen == 0) {
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
+
 			std::ostringstream msg;
 			msg << "Couldn't set video mode "
 				<< resW << "x" << resH << " (" << colorBits
@@ -199,41 +137,143 @@ void PlatformContextGl::init(int colorBits, int depthBits, int stencilBits,bool
 			}
 		}
 
+#ifndef WIN32
+		string mg_icon_file = "";
+#if defined(CUSTOM_DATA_INSTALL_PATH_VALUE)
+		if(fileExists(formatPath(TOSTRING(CUSTOM_DATA_INSTALL_PATH_VALUE)) + "megaglest.png")) {
+			mg_icon_file = formatPath(TOSTRING(CUSTOM_DATA_INSTALL_PATH_VALUE)) + "megaglest.png";
+		}
+		else if(fileExists(formatPath(TOSTRING(CUSTOM_DATA_INSTALL_PATH_VALUE)) + "megaglest.bmp")) {
+			mg_icon_file = formatPath(TOSTRING(CUSTOM_DATA_INSTALL_PATH_VALUE)) + "megaglest.bmp";
+		}
+
+#endif
+
+		if(mg_icon_file == "" && fileExists("megaglest.png")) {
+			mg_icon_file = "megaglest.png";
+		}
+		else if(mg_icon_file == "" && fileExists("megaglest.bmp")) {
+			mg_icon_file = "megaglest.bmp";
+		}
+		else if(mg_icon_file == "" && fileExists("/usr/share/pixmaps/megaglest.png")) {
+			mg_icon_file = "/usr/share/pixmaps/megaglest.png";
+		}
+		else if(mg_icon_file == "" && fileExists("/usr/share/pixmaps/megaglest.bmp")) {
+			mg_icon_file = "/usr/share/pixmaps/megaglest.bmp";
+		}
+
+		if(mg_icon_file != "") {
+
+			if(icon != NULL) {
+				SDL_FreeSurface(icon);
+				icon = NULL;
+			}
+
+			//printf("Loading icon [%s]\n",mg_icon_file.c_str());
+			if(extractExtension(mg_icon_file) == "bmp") {
+				icon = SDL_LoadBMP(mg_icon_file.c_str());
+			}
+			else {
+				//printf("Loadng png icon\n");
+				Texture2D *texture2D = GraphicsInterface::getInstance().getFactory()->newTexture2D();
+				texture2D->load(mg_icon_file);
+				std::pair<SDL_Surface*,unsigned char*> result = texture2D->CreateSDLSurface(true);
+				icon = result.first;
+				delete texture2D;
+				delete [] result.second;
+			}
+
+		//SDL_Surface *icon = IMG_Load("megaglest.ico");
+
+
+	//#if !defined(MACOSX)
+		// Set Icon (must be done before any sdl_setvideomode call)
+		// But don't set it on OS X, as we use a nicer external icon there.
+	//#if WORDS_BIGENDIAN
+	//	SDL_Surface* icon= SDL_CreateRGBSurfaceFrom((void*)logo,32,32,8,128,0xff000000,0x00ff0000,0x0000ff00,0);
+	//#else
+	//	SDL_Surface* icon= SDL_CreateRGBSurfaceFrom((void*)logo,32,32,32,128,0x000000ff,0x0000ff00,0x00ff0000,0xff000000);
+	//#endif
+
+			//printf("In [%s::%s Line: %d] icon = %p\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,icon);
+			if(icon == NULL) {
+				printf("Icon Load Error #1: %s\n", SDL_GetError());
+			}
+			if(icon != NULL) {
+
+				//uint32 colorkey = SDL_MapRGB(icon->format, 255, 0, 255);
+				//SDL_SetColorKey(icon, SDL_SRCCOLORKEY, colorkey);
+				if(SDL_SetColorKey(icon, SDL_SRCCOLORKEY, SDL_MapRGB(icon->format, 255, 0, 255))) {
+					printf("Icon Load Error #2: %s\n", SDL_GetError());
+				}
+				else {
+					SDL_WM_SetIcon(icon, NULL);
+				}
+			}
+		}
+#endif
+
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 		SDL_WM_GrabInput(SDL_GRAB_OFF);
 
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s %d] BEFORE glewInit call\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
+		
 		GLuint err = glewInit();
+
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s %d] AFTER glewInit call err = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,err);
+
 		if (GLEW_OK != err) {
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
+
 			fprintf(stderr, "Error [main]: glewInit failed: %s\n", glewGetErrorString(err));
 			//return 1;
 			throw std::runtime_error((char *)glewGetErrorString(err));
 		}
 		//fprintf(stdout, "Status: Using GLEW %s\n", glewGetString(GLEW_VERSION));
 
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
+
 		int bufferSize = (resW * resH * BaseColorPickEntity::COLOR_COMPONENTS);
+
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
+
 		BaseColorPickEntity::init(bufferSize);
+
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
+
+		if(gammaValue != 0.0) {
+			//printf("Attempting to call SDL_SetGamma using value %f\n", gammaValue);
+			if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
+			if (SDL_SetGamma(gammaValue, gammaValue, gammaValue) < 0) {
+				printf("WARNING, SDL_SetGamma failed using value %f [%s]\n", gammaValue,SDL_GetError());
+			}
+		}
+
+        SDL_WM_GrabInput(SDL_GRAB_ON);
+        SDL_WM_GrabInput(SDL_GRAB_OFF);
 	}
 }
 
 void PlatformContextGl::end() {
-	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
 	if(icon != NULL) {
-		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
 		SDL_FreeSurface(icon);
 		icon = NULL;
 	}
 
-	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
 	if(screen != NULL) {
-		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 
 		SDL_FreeSurface(screen);
 		screen = NULL;
 	}
 
-	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",__FILE__,__FUNCTION__,__LINE__);
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__);
 }
 
 void PlatformContextGl::makeCurrent() {
