@@ -22,12 +22,15 @@ namespace Shared{ namespace Platform{
 //	class WindowGl
 // =====================================================
 
-void WindowGl::initGl(int colorBits, int depthBits, int stencilBits,bool hardware_acceleration, bool fullscreen_anti_aliasing){
+void WindowGl::initGl(int colorBits, int depthBits, int stencilBits,
+		             bool hardware_acceleration, bool fullscreen_anti_aliasing,
+		             float gammaValue) {
 	context.setColorBits(colorBits);
 	context.setDepthBits(depthBits);
 	context.setStencilBits(stencilBits);
 	context.setHardware_acceleration(hardware_acceleration);
 	context.setFullscreen_anti_aliasing(fullscreen_anti_aliasing);
+	context.setGammaValue(gammaValue);
 	
 	context.init();
 }
