@@ -19,7 +19,7 @@
 #include <string.h>
 #include "SDL_syswm.h"
 #include <iostream>
-
+#include <stdexcept>
 #include "leak_dumper.h"
 
 using namespace Shared::Util;
@@ -30,7 +30,8 @@ namespace Shared { namespace Platform {
 // =====================================================
 //	class PlatformExceptionHandler
 // =====================================================
-
+string PlatformExceptionHandler::application_binary="";
+bool PlatformExceptionHandler::disableBacktrace = false;
 PlatformExceptionHandler *PlatformExceptionHandler::thisPointer= NULL;
 
 // Constructs object and convert lpaszString to Unicode
@@ -53,7 +54,7 @@ LPWSTR Ansi2WideString(LPCSTR lpaszString) {
 }
 
 // Convert a wide Unicode string to an UTF8 string
-std::string utf8_encode(const std::wstring wstr) {
+std::string utf8_encode(const std::wstring &wstr) {
     int size_needed = WideCharToMultiByte(CP_UTF8, 0, &wstr[0], (int)wstr.size(), NULL, 0, NULL, NULL);
     std::string strTo( size_needed, 0 );
     WideCharToMultiByte(CP_UTF8, 0, &wstr[0], (int)wstr.size(), &strTo[0], size_needed, NULL, NULL);
@@ -64,7 +65,7 @@ std::string utf8_encode(const std::wstring wstr) {
 }
 
 // Convert an UTF8 string to a wide Unicode String
-std::wstring utf8_decode(const std::string str) {
+std::wstring utf8_decode(const std::string &str) {
 	string friendly_path = str;
 	replaceAll(friendly_path, "/", "\\");
 	replaceAll(friendly_path, "\\\\", "\\");
@@ -75,6 +76,32 @@ std::wstring utf8_decode(const std::string str) {
     return wstrTo;
 }
 
+/**
+* @param location The location of the registry key. For example "Software\\Bethesda Softworks\\Morrowind"
+* @param name the name of the registry key, for example "Installed Path"
+* @return the value of the key or an empty string if an error occured.
+*/
+std::string getRegKey(const std::string& location, const std::string& name){
+    HKEY key;
+    CHAR value[1024]; 
+    DWORD bufLen = 1024*sizeof(CHAR);
+    long ret;
+    ret = RegOpenKeyExA(HKEY_LOCAL_MACHINE, location.c_str(), 0, KEY_QUERY_VALUE, &key);
+    if( ret != ERROR_SUCCESS ){
+        return std::string();
+    }
+    ret = RegQueryValueExA(key, name.c_str(), 0, 0, (LPBYTE) value, &bufLen);
+    RegCloseKey(key);
+    if ( (ret != ERROR_SUCCESS) || (bufLen > 1024*sizeof(TCHAR)) ){
+        return std::string();
+    }
+    string stringValue = value;
+    size_t i = stringValue.length();
+    while( i > 0 && stringValue[i-1] == '\0' ){
+        --i;
+    }
+    return stringValue.substr(0,i); 
+}
 
 LONG WINAPI PlatformExceptionHandler::handler(LPEXCEPTION_POINTERS pointers){
 
@@ -124,6 +151,83 @@ void PlatformExceptionHandler::install(string dumpFileName){
 	SetUnhandledExceptionFilter(handler);
 }
 
+string PlatformExceptionHandler::getStackTrace() {
+	 string result = "\nStack Trace:\n";
+	 if(PlatformExceptionHandler::disableBacktrace == true) {
+		 result += "disabled...";
+		 return result;
+	 }
+#ifndef __MINGW32__
+
+    CONTEXT context = { 0 };
+    context.ContextFlags = CONTEXT_FULL;
+
+    IMAGEHLP_SYMBOL *pSym = (IMAGEHLP_SYMBOL*)new BYTE[sizeof(IMAGEHLP_SYMBOL) + 256];
+    pSym->SizeOfStruct = sizeof(IMAGEHLP_SYMBOL);
+    pSym->MaxNameLength = 256;
+
+    IMAGEHLP_LINE line = { 0 };
+    line.SizeOfStruct = sizeof(IMAGEHLP_LINE);
+
+    IMAGEHLP_MODULE module = { 0 };
+    module.SizeOfStruct = sizeof(IMAGEHLP_MODULE);
+
+    HANDLE hProcess = GetCurrentProcess();
+    HANDLE hThread = GetCurrentThread();
+    if (GetThreadContext(hThread, &context)) {
+        STACKFRAME stackframe = { 0 };
+        stackframe.AddrPC.Offset = context.Eip;
+        stackframe.AddrPC.Mode = AddrModeFlat;
+        stackframe.AddrFrame.Offset = context.Ebp;
+        stackframe.AddrFrame.Mode = AddrModeFlat;
+
+		SymInitialize(hProcess, NULL, TRUE);
+        BOOL fSuccess = TRUE;
+
+        do
+        {
+            fSuccess = StackWalk(IMAGE_FILE_MACHINE_I386,
+                                 GetCurrentProcess(),
+                                 GetCurrentThread(),
+                                 &stackframe,
+                                 &context,
+                                 NULL,
+                                 NULL,
+                                 NULL,
+                                 NULL);
+
+            DWORD dwDisplacement = 0;
+            SymGetSymFromAddr(hProcess, stackframe.AddrPC.Offset, &dwDisplacement, pSym);
+            SymGetLineFromAddr(hProcess, stackframe.AddrPC.Offset, &dwDisplacement, &line);
+            SymGetModuleInfo(hProcess, stackframe.AddrPC.Offset, &module);
+
+			// RetAddr Arg1 Arg2 Arg3 module!funtion FileName(line)+offset
+
+			char szBuf[8096]="";
+            snprintf(szBuf,8096,"%08lx %08lx %08lx %08lx %s!%s %s(%lu) %+ld\n",
+                   stackframe.AddrReturn.Offset,
+                   stackframe.Params[0],
+                   stackframe.Params[1],
+                   stackframe.Params[2],
+                   pSym->Name,
+                   module.ModuleName,
+                   line.FileName, 
+                   line.LineNumber, 
+                   dwDisplacement);
+			result += szBuf;
+
+        } while (fSuccess);
+
+        SymCleanup(hProcess);
+	}
+#endif
+	return result;
+}
+
+megaglest_runtime_error::megaglest_runtime_error(const string& __arg,bool noStackTrace)
+: std::runtime_error(noStackTrace == false ? __arg + PlatformExceptionHandler::getStackTrace() : __arg), noStackTrace(noStackTrace) {
+}
+
 // =====================================================
 //	class Misc
 // =====================================================
@@ -204,13 +308,13 @@ void message(string message){
 	std::cerr << "******************************************************\n";
 
 	LPWSTR wstr = Ansi2WideString(message.c_str());
-	MessageBox(NULL, wstr, L"Message", MB_OK);
+	MessageBox(NULL, wstr, L"Message", MB_OK | MB_SYSTEMMODAL);
 	delete [] wstr;
 }
 
 bool ask(string message){
 	LPWSTR wstr = Ansi2WideString(message.c_str());	
-	bool result = MessageBox(NULL, wstr, L"Confirmation", MB_YESNO)==IDYES;
+	bool result = MessageBox(NULL, wstr, L"Confirmation", MB_YESNO | MB_SYSTEMMODAL) == IDYES;
 	delete [] wstr;
 	return result;
 }
@@ -227,7 +331,7 @@ void exceptionMessage(const exception &excp){
 
 	LPWSTR wstr = Ansi2WideString(message.c_str());	
 	LPWSTR wstr1 = Ansi2WideString(title.c_str());	
-	MessageBox(NULL, wstr, wstr1, MB_ICONSTOP | MB_OK | MB_TASKMODAL);
+	MessageBox(NULL, wstr, wstr1, MB_ICONSTOP | MB_OK | MB_SYSTEMMODAL);
 	delete [] wstr;
 	delete [] wstr1;
 }
@@ -250,8 +354,7 @@ void init_win32() {
 
 	SDL_SysWMinfo wminfo;
 	SDL_VERSION(&wminfo.version)
-	if (SDL_GetWMInfo(&wminfo) != 1)
-	{
+	if (SDL_GetWMInfo(&wminfo) != 1) {
 		// error: wrong SDL version
 	}
 
@@ -266,7 +369,27 @@ void init_win32() {
 
 	::SetClassLong(hwnd, GCL_HICON, iconPtr);
 #endif
+
+	ontop_win32(0, 0);
 }
+
+void ontop_win32(int width, int height) {
+	SDL_SysWMinfo wminfo;
+	SDL_VERSION(&wminfo.version)
+	if (SDL_GetWMInfo(&wminfo) != 1) {
+		// error: wrong SDL version
+	}
+
+	HWND hwnd = wminfo.window;
+
+	SetWindowLong(hwnd, GWL_EXSTYLE, 0);
+    SetWindowLong(hwnd, GWL_STYLE, WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS);
+    SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_FRAMECHANGED | SWP_SHOWWINDOW);
+    if(width > 0 && height > 0) {
+    	SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, width, height, SWP_SHOWWINDOW);
+    }
+}
+
 void done_win32() {
 	::DestroyIcon(icon);
 }
