@@ -136,6 +136,7 @@ ALenum SoundSource::getFormat(Sound* sound)
 
 StaticSoundSource::StaticSoundSource() {
 	bufferAllocated = false;
+	buffer = 0;
 }
 
 StaticSoundSource::~StaticSoundSource() {
@@ -179,6 +180,8 @@ void StaticSoundSource::play(StaticSound* sound) {
 StreamSoundSource::StreamSoundSource()
 {
 	sound = 0;
+	fadeState = NoFading;
+	format = 0;
 	alGenBuffers(STREAMFRAGMENTS, buffers);
 	SoundPlayerOpenAL::checkAlError(string(__FILE__) + string(" ") + string(__FUNCTION__) + string(" ") + intToStr(__LINE__));
 }
@@ -331,6 +334,7 @@ SoundPlayerOpenAL::SoundPlayerOpenAL() {
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSound).enabled) SystemFlags::OutputDebug(SystemFlags::debugSound,"In [%s::%s %d]\n",__FILE__,__FUNCTION__,__LINE__);
 
 	device = 0;
+	context = 0;
 	initOk = false;
 
 	if(SystemFlags::getSystemSettingType(SystemFlags::debugSound).enabled) SystemFlags::OutputDebug(SystemFlags::debugSound,"In [%s::%s %d]\n",__FILE__,__FUNCTION__,__LINE__);
@@ -545,7 +549,7 @@ void SoundPlayerOpenAL::updateStreams() {
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSound).enabled) SystemFlags::OutputDebug(SystemFlags::debugSound,"In [%s::%s Line: %d] Error [%s]\n",__FILE__,__FUNCTION__,__LINE__,ex.what());
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",__FILE__,__FUNCTION__,__LINE__,ex.what());
 		printOpenALInfo();
-		throw runtime_error(ex.what());
+		throw megaglest_runtime_error(ex.what());
 	}
 	catch(...) {
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSound).enabled) SystemFlags::OutputDebug(SystemFlags::debugSound,"In [%s::%s Line: %d] UNKNOWN Error\n",__FILE__,__FUNCTION__,__LINE__);
@@ -571,7 +575,7 @@ StaticSoundSource* SoundPlayerOpenAL::findStaticSoundSource() {
 		}
 	}
 
-	if(SystemFlags::getSystemSettingType(SystemFlags::debugSound).enabled) SystemFlags::OutputDebug(SystemFlags::debugSound,"In [%s::%s %d] playingCount = %d, staticSources.size() = %lu, params.staticBufferCount = %u\n",__FILE__,__FUNCTION__,__LINE__,playingCount,(unsigned long)staticSources.size(),params.staticBufferCount);
+	if(SystemFlags::getSystemSettingType(SystemFlags::debugSound).enabled) SystemFlags::OutputDebug(SystemFlags::debugSound,"In [%s::%s %d] playingCount = %d, staticSources.size() = " MG_SIZE_T_SPECIFIER ", params.staticBufferCount = %u\n",__FILE__,__FUNCTION__,__LINE__,playingCount,(unsigned long)staticSources.size(),params.staticBufferCount);
 
 	// create a new source
 	if(staticSources.size() >= params.staticBufferCount) {
@@ -618,8 +622,8 @@ StreamSoundSource* SoundPlayerOpenAL::findStreamSoundSource() {
 void SoundPlayerOpenAL::checkAlcError(string message) {
 	int err = alcGetError(device);
 	if(err != ALC_NO_ERROR) {
-		char szBuf[4096]="";
-		sprintf(szBuf,"%s [%s]",message.c_str(),alcGetString(device, err));
+		char szBuf[8096]="";
+		snprintf(szBuf,8096,"%s [%s]",message.c_str(),alcGetString(device, err));
 
 		//std::stringstream msg;
 		//msg << message.c_str() << alcGetString(device, err);
@@ -631,8 +635,8 @@ void SoundPlayerOpenAL::checkAlcError(string message) {
 void SoundPlayerOpenAL::checkAlError(string message) {
 	int err = alGetError();
 	if(err != AL_NO_ERROR) {
-		char szBuf[4096]="";
-		sprintf(szBuf,"%s [%s]",message.c_str(),alGetString(err));
+		char szBuf[8096]="";
+		snprintf(szBuf,8096,"%s [%s]",message.c_str(),alGetString(err));
 		//std::stringstream msg;
 		//msg << message.c_str() << alGetString(err);
 		printf("openal error [%s]\n",szBuf);
