@@ -18,6 +18,7 @@
 #include <sstream>
 #include <iostream>
 #include <locale>
+#include "platform_util.h"
 #include "leak_dumper.h"
 
 using namespace std;
@@ -33,7 +34,7 @@ bool strToBool(const string &s) {
 	if(s=="1" || s=="true") {
 		return true;
 	}
-	throw runtime_error("Error converting string to bool, expected 0 or 1, found: [" + s + "]");
+	throw megaglest_runtime_error("Error converting string to bool, expected 0 or 1, found: [" + s + "]");
 }
 
 int strToInt(const string &s){
@@ -42,7 +43,19 @@ int strToInt(const string &s){
 	int intValue= strtol(s.c_str(), &endChar, 10);
 
 	if(*endChar!='\0'){
-		throw runtime_error("Error converting from string to int, found: [" + s + "]");
+		throw megaglest_runtime_error("Error converting from string to int, found: [" + s + "]");
+	}
+
+	return intValue;
+}
+
+uint32 strToUInt(const string &s){
+	char *endChar;
+	setlocale(LC_NUMERIC, "C");
+	uint32 intValue= strtoul(s.c_str(), &endChar, 10);
+
+	if(*endChar!='\0'){
+		throw megaglest_runtime_error("Error converting from string to uint, found: [" + s + "]");
 	}
 
 	return intValue;
@@ -56,7 +69,7 @@ float strToFloat(const string &s){
 	float floatValue= static_cast<float>(strtod(s.c_str(), &endChar));
 
 	if(*endChar!='\0'){
-		throw runtime_error("Error converting from string to float, found: [" + s + "]");
+		throw megaglest_runtime_error("Error converting from string to float, found: [" + s + "]");
 	}
 
 	return floatValue;
@@ -87,6 +100,18 @@ bool strToInt(const string &s, int *i) {
 	return true;
 }
 
+bool strToUInt(const string &s, uint32 *i) {
+	char *endChar;
+
+	setlocale(LC_NUMERIC, "C");
+	*i= strtoul(s.c_str(), &endChar, 10);
+
+	if(*endChar!='\0'){
+		return false;
+	}
+	return true;
+}
+
 bool strToFloat(const string &s, float *f) {
 	char *endChar;
 	setlocale(LC_NUMERIC, "C");
@@ -113,6 +138,12 @@ string intToStr(int64 i) {
 	return (str[0] != '\0' ? str : "");
 }
 
+string uIntToStr(uint32 i) {
+	char str[strSize]="";
+	snprintf(str, strSize-1, "%u", i);
+	return (str[0] != '\0' ? str : "");
+}
+
 string intToHex(int i){
 	char str[strSize]="";
 	snprintf(str, strSize-1, "%x", i);
@@ -136,7 +167,10 @@ string doubleToStr(double d,int precsion) {
 }
 
 bool IsNumeric(const char *p, bool  allowNegative) {
-	if(p != NULL && strcmp(p,"-") == 0) {
+	if(p == NULL) {
+		return false;
+	}
+	if(strcmp(p,"-") == 0) {
 		return false;
 	}
     int index = 0;
