@@ -19,6 +19,7 @@
 #include "util.h"
 #include "platform_common.h"
 #include "platform_util.h"
+#include "randomgen.h"
 
 #ifdef WIN32
 #include <shlwapi.h>
@@ -49,7 +50,7 @@ string Properties::gameVersion = "";
 void Properties::load(const string &path, bool clearCurrentProperties) {
 
 	char lineBuffer[maxLine]="";
-	string line, key, value;
+	string line, key, value, original_value;
 	size_t pos=0;
 	this->path= path;
 
@@ -68,11 +69,12 @@ void Properties::load(const string &path, bool clearCurrentProperties) {
 	
 	if(fileStream.is_open() == false){
 		if(SystemFlags::getSystemSettingType(SystemFlags::debugSystem).enabled) SystemFlags::OutputDebug(SystemFlags::debugSystem,"In [%s::%s Line: %d] path = [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str());
-		throw runtime_error("File NOT FOUND, can't open file: [" + path + "]");
+		throw megaglest_runtime_error("File NOT FOUND, can't open file: [" + path + "]");
 	}
 
 	if(clearCurrentProperties == true) {
 		propertyMap.clear();
+		propertyMapTmp.clear();
 	}
 
 	while(fileStream.eof() == false) {
@@ -86,7 +88,7 @@ void Properties::load(const string &path, bool clearCurrentProperties) {
 		if(lineBuffer[0] != '\0') {
 			// If the file is NOT in UTF-8 format convert each line
 			if(is_utf8_language == false && Font::forceLegacyFonts == false) {
-				char *utfStr = String::ConvertToUTF8(&lineBuffer[0]);
+				char *utfStr = ConvertToUTF8(&lineBuffer[0]);
 
 				//printf("\nBefore [%s] After [%s]\n",&lineBuffer[0],utfStr);
 
@@ -96,7 +98,7 @@ void Properties::load(const string &path, bool clearCurrentProperties) {
 				delete [] utfStr;
 			}
 			else if(is_utf8_language == true && Font::forceLegacyFonts == true) {
-				char *asciiStr = String::ConvertFromUTF8(&lineBuffer[0]);
+				char *asciiStr = ConvertFromUTF8(&lineBuffer[0]);
 
 				//printf("\nBefore [%s] After [%s]\n",&lineBuffer[0],utfStr);
 
@@ -108,7 +110,7 @@ void Properties::load(const string &path, bool clearCurrentProperties) {
 		}
 
 		//process line if it it not a comment
-		if(lineBuffer[0] != ';') {
+		if(lineBuffer[0] != ';' && lineBuffer[0] != '#') {
 			//wstring wstr = lineBuffer;
 			//line.assign(wstr.begin(),wstr.end());
 
@@ -124,25 +126,30 @@ void Properties::load(const string &path, bool clearCurrentProperties) {
 			if(pos != string::npos){
 				key= line.substr(0, pos);
 				value= line.substr(pos+1);
+				original_value = value;
 
 				if(applyTagsToValue(value) == true) {
-					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Property key [%s] now has value [%s]\n",key.c_str(),value.c_str());
+					if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Property key [%s] now has value [%s] original_value [%s]\n",key.c_str(),value.c_str(),original_value.c_str());
 				}
 
 				bool replaceExisting = false;
 				if(propertyMap.find(key) != propertyMap.end()) {
 					replaceExisting = true;
 				}
-				propertyMap[key] = value;
+				propertyMap[key] = original_value;
+				propertyMapTmp[key] = value;
 
 				if(replaceExisting == false) {
-					propertyVector.push_back(PropertyPair(key, value));
+					propertyVector.push_back(PropertyPair(key, original_value));
+					propertyVectorTmp.push_back(PropertyPair(key, value));
 				}
 				else {
 					for(unsigned int i = 0; i < propertyVector.size(); ++i) {
 						PropertyPair &currentPair = propertyVector[i];
 						if(currentPair.first == key) {
-							currentPair.second = value;
+							currentPair.second = original_value;
+
+							propertyVectorTmp[i].second = value;
 							break;
 						}
 					}
@@ -166,11 +173,10 @@ std::map<string,string> Properties::getTagReplacementValues(std::map<string,stri
 	// #1
 	// First add the standard tags
 	//
-	char *homeDir = NULL;
 #ifdef WIN32
-	homeDir = getenv("USERPROFILE");
+	char *homeDir = getenv("USERPROFILE");
 #else
-	homeDir = getenv("HOME");
+	char *homeDir = getenv("HOME");
 #endif
 
 	mapTagReplacementValues["~/"] = (homeDir != NULL ? homeDir : "");
@@ -203,8 +209,8 @@ std::map<string,string> Properties::getTagReplacementValues(std::map<string,stri
    }
 #endif
 
-	char *username = NULL;
-	username = getenv("USERNAME");
+	//char *username = NULL;
+   char *username = getenv("USERNAME");
 
 	mapTagReplacementValues["$USERNAME"] = (username != NULL ? username : "");
 	mapTagReplacementValues["%%USERNAME%%"] = (username != NULL ? username : "");
@@ -256,11 +262,10 @@ bool Properties::applyTagsToValue(string &value, std::map<string,string> *mapTag
 		}
 	}
 	else {
-	char *homeDir = NULL;
 #ifdef WIN32
-	homeDir = getenv("USERPROFILE");
+	char *homeDir = getenv("USERPROFILE");
 #else
-	homeDir = getenv("HOME");
+	char *homeDir = getenv("HOME");
 #endif
 
 	replaceAll(value, "~/", 			(homeDir != NULL ? homeDir : ""));
@@ -292,8 +297,8 @@ bool Properties::applyTagsToValue(string &value, std::map<string,string> *mapTag
    }
 #endif
 
-	char *username = NULL;
-	username = getenv("USERNAME");
+	//char *username = NULL;
+   char *username = getenv("USERNAME");
 	replaceAll(value, "$USERNAME", 		(username != NULL ? username : ""));
 	replaceAll(value, "%%USERNAME%%", 	(username != NULL ? username : ""));
 	replaceAll(value, "{USERNAME}", 	(username != NULL ? username : ""));
@@ -346,7 +351,19 @@ void Properties::save(const string &path){
 
 void Properties::clear(){
 	propertyMap.clear();
+	propertyMapTmp.clear();
 	propertyVector.clear();
+	propertyVectorTmp.clear();
+}
+
+int Properties::getPropertyCount() const {
+	return (int)propertyVectorTmp.size();
+}
+string Properties::getKey(int i) const {
+	return propertyVectorTmp[i].first;
+}
+string Properties::getString(int i) const {
+	return propertyVectorTmp[i].second;
 }
 
 bool Properties::getBool(const string &key, const char *defaultValueIfNotFound) const{
@@ -355,8 +372,10 @@ bool Properties::getBool(const string &key, const char *defaultValueIfNotFound)
 	}
 	catch(exception &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.what());
+		//throw megaglest_runtime_error("Error accessing value: " + key + " in: " + path+"\n[" + e.what() + "]");
 		throw runtime_error("Error accessing value: " + key + " in: " + path+"\n[" + e.what() + "]");
 	}
+	return false;
 }
 
 int Properties::getInt(const string &key,const char *defaultValueIfNotFound) const{
@@ -365,14 +384,16 @@ int Properties::getInt(const string &key,const char *defaultValueIfNotFound) con
 	}
 	catch(exception &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.what());
+		//throw megaglest_runtime_error("Error accessing value: " + key + " in: " + path + "\n[" + e.what() + "]");
 		throw runtime_error("Error accessing value: " + key + " in: " + path + "\n[" + e.what() + "]");
 	}
+	return 0;
 }
 
 int Properties::getInt(const string &key, int min, int max,const char *defaultValueIfNotFound) const{
 	int i= getInt(key,defaultValueIfNotFound);
 	if(i<min || i>max){
-		throw runtime_error("Value out of range: " + key + ", min: " + intToStr(min) + ", max: " + intToStr(max));
+		throw megaglest_runtime_error("Value out of range: " + key + ", min: " + intToStr(min) + ", max: " + intToStr(max));
 	}
 	return i;
 }
@@ -383,28 +404,30 @@ float Properties::getFloat(const string &key, const char *defaultValueIfNotFound
 	}
 	catch(exception &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.what());
+		//throw megaglest_runtime_error("Error accessing value: " + key + " in: " + path + "\n[" + e.what() + "]");
 		throw runtime_error("Error accessing value: " + key + " in: " + path + "\n[" + e.what() + "]");
 	}
+	return 0.0;
 }
 
 float Properties::getFloat(const string &key, float min, float max, const char *defaultValueIfNotFound) const{
 	float f= getFloat(key,defaultValueIfNotFound);
 	if(f<min || f>max){
-		throw runtime_error("Value out of range: " + key + ", min: " + floatToStr(min) + ", max: " + floatToStr(max));
+		throw megaglest_runtime_error("Value out of range: " + key + ", min: " + floatToStr(min) + ", max: " + floatToStr(max));
 	}
 	return f;
 }
 
 const string Properties::getString(const string &key, const char *defaultValueIfNotFound) const{
-	PropertyMap::const_iterator it;
-	it= propertyMap.find(key);
-	if(it==propertyMap.end()){
+	PropertyMap::const_iterator it = propertyMapTmp.find(key);
+	if(it == propertyMapTmp.end()) {
 	    if(defaultValueIfNotFound != NULL) {
 	        //printf("In [%s::%s - %d]defaultValueIfNotFound = [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,defaultValueIfNotFound);
 	        return string(defaultValueIfNotFound);
 	    }
 	    else {
-            throw runtime_error("Value not found in propertyMap: " + key + ", loaded from: " + path);
+            //throw megaglest_runtime_error("Value not found in propertyMap: " + key + ", loaded from: " + path);
+	    	throw runtime_error("Value not found in propertyMap: " + key + ", loaded from: " + path);
 	    }
 	}
 	else{
@@ -412,6 +435,32 @@ const string Properties::getString(const string &key, const char *defaultValueIf
 	}
 }
 
+const string Properties::getRandomKey(const bool realrandom) const{
+	PropertyMap::const_iterator it;
+	int max=getPropertyCount();
+	int randomIndex=-1;
+	if(realrandom == true){
+		Chrono seed(true);
+		srand((unsigned int)seed.getCurTicks());
+
+		randomIndex=rand()%max;
+	}
+	else{
+		RandomGen randgen;
+		randomIndex=randgen.randRange(0,max);
+	}
+	string s=getKey(randomIndex);
+	return (s != "" ? s : "nothing found");
+}
+
+bool Properties::hasString(const string &key) const {
+	PropertyMap::const_iterator it = propertyMapTmp.find(key);
+	if(it == propertyMapTmp.end()) {
+		return false;
+	}
+	return true;
+}
+
 void Properties::setInt(const string &key, int value){
 	setString(key, intToStr(value));
 }
@@ -427,28 +476,31 @@ void Properties::setFloat(const string &key, float value){
 void Properties::setString(const string &key, const string &value){
 	propertyMap.erase(key);
 	propertyMap.insert(PropertyPair(key, value));
+
+	propertyMapTmp.erase(key);
+	propertyMapTmp.insert(PropertyPair(key, value));
+
 }
 
 string Properties::toString(){
 	string rStr;
 
-	for(PropertyMap::iterator pi= propertyMap.begin(); pi!=propertyMap.end(); ++pi)
+	for(PropertyMap::iterator pi= propertyMapTmp.begin(); pi!=propertyMapTmp.end(); ++pi)
 		rStr+= pi->first + "=" + pi->second + "\n";
 
 	return rStr;
 }
 
-
-
-
 bool Properties::getBool(const char *key, const char *defaultValueIfNotFound) const{
 	try{
 		return strToBool(getString(key,defaultValueIfNotFound));
 	}
 	catch(exception &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.what());
+		//throw megaglest_runtime_error("Error accessing value: " + string(key) + " in: " + path+"\n[" + e.what() + "]");
 		throw runtime_error("Error accessing value: " + string(key) + " in: " + path+"\n[" + e.what() + "]");
 	}
+	return false;
 }
 
 int Properties::getInt(const char *key,const char *defaultValueIfNotFound) const{
@@ -457,8 +509,10 @@ int Properties::getInt(const char *key,const char *defaultValueIfNotFound) const
 	}
 	catch(exception &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.what());
+		//throw megaglest_runtime_error("Error accessing value: " + string(key) + " in: " + path + "\n[" + e.what() + "]");
 		throw runtime_error("Error accessing value: " + string(key) + " in: " + path + "\n[" + e.what() + "]");
 	}
+	return 0;
 }
 
 float Properties::getFloat(const char *key, const char *defaultValueIfNotFound) const{
@@ -467,20 +521,22 @@ float Properties::getFloat(const char *key, const char *defaultValueIfNotFound)
 	}
 	catch(exception &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.what());
+		//throw megaglest_runtime_error("Error accessing value: " + string(key) + " in: " + path + "\n[" + e.what() + "]");
 		throw runtime_error("Error accessing value: " + string(key) + " in: " + path + "\n[" + e.what() + "]");
 	}
+	return 0.0;
 }
 
 const string Properties::getString(const char *key, const char *defaultValueIfNotFound) const{
-	PropertyMap::const_iterator it;
-	it= propertyMap.find(key);
-	if(it==propertyMap.end()){
+	PropertyMap::const_iterator it = propertyMapTmp.find(key);
+	if(it == propertyMapTmp.end()) {
 	    if(defaultValueIfNotFound != NULL) {
 	        //printf("In [%s::%s - %d]defaultValueIfNotFound = [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,defaultValueIfNotFound);
 	        return string(defaultValueIfNotFound);
 	    }
 	    else {
-            throw runtime_error("Value not found in propertyMap: " + string(key) + ", loaded from: " + path);
+            //throw megaglest_runtime_error("Value not found in propertyMap: " + string(key) + ", loaded from: " + path);
+	    	throw runtime_error("Value not found in propertyMap: " + string(key) + ", loaded from: " + path);
 	    }
 	}
 	else{
