@@ -21,13 +21,14 @@
 #include <xercesc/util/PlatformUtils.hpp>
 #include <xercesc/framework/LocalFileFormatTarget.hpp>
 #include "util.h"
-#include "types.h"
+#include "data_types.h"
 #include "properties.h"
 #include "platform_common.h"
 #include "platform_util.h"
 #include "cache_manager.h"
-#include "leak_dumper.h"
 
+#include "rapidxml/rapidxml_print.hpp"
+#include "leak_dumper.h"
 
 XERCES_CPP_NAMESPACE_USE
 
@@ -50,7 +51,7 @@ public:
 			XMLString::transcode(domError.getMessage(), msgStr, strSize-1);
 			XMLString::transcode(domError.getLocation()->getURI(), fileStr, strSize-1);
 			Shared::Platform::uint64 lineNumber= domError.getLocation()->getLineNumber();
-			throw runtime_error("Error parsing XML, file: " + string(fileStr) + ", line: " + intToStr(lineNumber) + ": " + string(msgStr));
+			throw megaglest_runtime_error("Error parsing XML, file: " + string(fileStr) + ", line: " + intToStr(lineNumber) + ": " + string(msgStr));
 		}
 		return true;
 	}
@@ -61,6 +62,7 @@ public:
 // =====================================================
 
 bool XmlIo::initialized= false;
+bool XmlIoRapid::initialized= false;
 
 XmlIo::XmlIo() {
 	try{
@@ -71,7 +73,7 @@ XmlIo::XmlIo() {
 	}
 	catch(const XMLException &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error initializing XML system, msg %s\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.getMessage());
-		throw runtime_error("Error initializing XML system");
+		throw megaglest_runtime_error("Error initializing XML system");
 	}
 
 	try {
@@ -82,7 +84,7 @@ XmlIo::XmlIo() {
 	}
 	catch(const DOMException &ex) {
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Exception while creating XML parser, msg: %s\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.getMessage());
-		throw runtime_error("Exception while creating XML parser");
+		throw megaglest_runtime_error("Exception while creating XML parser");
 	}
 }
 
@@ -103,22 +105,52 @@ XmlIo::~XmlIo() {
 	cleanup();
 }
 
-XmlNode *XmlIo::load(const string &path, std::map<string,string> mapTagReplacementValues) {
+XmlNode *XmlIo::load(const string &path, std::map<string,string> mapTagReplacementValues,bool noValidation,bool skipStackCheck) {
+	//printf("Load file using Xerces engine [%s]\n",path.c_str());
+
+	try {
+		if(SystemFlags::VERBOSE_MODE_ENABLED) printf("XERCES_FULLVERSIONDOT [%s]\nnoValidation = %d\npath [%s]\n",XERCES_FULLVERSIONDOT,noValidation,path.c_str());
 
-	try{
 		ErrorHandler errorHandler;
 #if XERCES_VERSION_MAJOR < 3
  		DOMBuilder *parser= (static_cast<DOMImplementationLS*>(implementation))->createDOMBuilder(DOMImplementationLS::MODE_SYNCHRONOUS, 0);
  		parser->setErrorHandler(&errorHandler);
- 		parser->setFeature(XMLUni::fgXercesSchema, true);
- 		parser->setFeature(XMLUni::fgXercesSchemaFullChecking, true);
- 		parser->setFeature(XMLUni::fgDOMValidation, true);
+ 		if(noValidation == false) {
+			parser->setFeature(XMLUni::fgXercesSchema, true);
+			parser->setFeature(XMLUni::fgXercesSchemaFullChecking, true);
+			//parser->setFeature(XMLUni::fgDOMValidateIfSchema, true);
+			parser->setFeature(XMLUni::fgDOMValidation, true);
+ 		}
+ 		else {
+			//parser->setFeature(XMLUni::fgSAX2CoreValidation, false);
+			//parser->setFeature(XMLUni::fgXercesDynamic, true);
+
+			//parser->setFeature(XMLUni::fgXercesSchema, false);
+			//parser->setFeature(XMLUni::fgXercesSchemaFullChecking, false);
+			//parser->setFeature(XMLUni::fgDOMValidateIfSchema, true);
+			//parser->setFeature(XMLUni::fgDOMValidation, false);
+
+ 			parser->setFeature(XMLUni::fgXercesSchemaFullChecking, false);
+ 			parser->setFeature(XMLUni::fgXercesLoadExternalDTD, false);
+ 			parser->setFeature(XMLUni::fgXercesCacheGrammarFromParse, true);
+ 			parser->setFeature(XMLUni::fgXercesUseCachedGrammarInParse, true);
+ 		}
 #else
 		DOMLSParser *parser = (static_cast<DOMImplementationLS*>(implementation))->createLSParser(DOMImplementationLS::MODE_SYNCHRONOUS, 0);
  		DOMConfiguration  *config = parser->getDomConfig();
- 		config->setParameter(XMLUni::fgXercesSchema, true);
-		config->setParameter(XMLUni::fgXercesSchemaFullChecking, true);
-		config->setParameter(XMLUni::fgDOMValidate, true);
+ 		if(noValidation == false) {
+			config->setParameter(XMLUni::fgXercesSchema, true);
+			config->setParameter(XMLUni::fgXercesSchemaFullChecking, true);
+			//config->setParameter(XMLUni::fgDOMValidateIfSchema, true);
+			config->setParameter(XMLUni::fgDOMValidate, true);
+ 		}
+ 		else {
+			config->setParameter(XMLUni::fgXercesSchema, false);
+			config->setParameter(XMLUni::fgXercesSchemaFullChecking, false);
+			config->setParameter(XMLUni::fgXercesLoadExternalDTD, false);
+			config->setParameter(XMLUni::fgXercesCacheGrammarFromParse, true);
+			config->setParameter(XMLUni::fgXercesUseCachedGrammarInParse, true);
+ 		}
 #endif
 		XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *document= parser->parseURI(path.c_str());
 #ifdef WIN32
@@ -127,7 +159,7 @@ XmlNode *XmlIo::load(const string &path, std::map<string,string> mapTagReplaceme
 		}
 #endif
 		if(document == NULL) {
-			throw runtime_error("Can not parse URL: " + path);
+			throw megaglest_runtime_error("Can not parse URL: " + path);
 		}
 
 		XmlNode *rootNode= new XmlNode(document->getDocumentElement(),mapTagReplacementValues);
@@ -135,18 +167,34 @@ XmlNode *XmlIo::load(const string &path, std::map<string,string> mapTagReplaceme
 		return rootNode;
 	}
 	catch(const DOMException &ex) {
-		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Exception while loading: [%s], %s\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),XMLString::transcode(ex.msg));
-		throw runtime_error("Exception while loading: " + path + ": " + XMLString::transcode(ex.msg));
+		char szBuf[8096]="";
+		snprintf(szBuf,8096,"In [%s::%s Line: %d] Exception while loading: [%s], msg:\n%s",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),XMLString::transcode(ex.msg));
+		SystemFlags::OutputDebug(SystemFlags::debugError,"%s\n",szBuf);
+
+		throw megaglest_runtime_error(szBuf);
 	}
+	return NULL;
 }
 
 void XmlIo::save(const string &path, const XmlNode *node){
+	//printf("Saving file using Xerces engine [%s]\n",path.c_str());
+
 	try{
 		XMLCh str[strSize];
 		XMLString::transcode(node->getName().c_str(), str, strSize-1);
 
 		XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *document= implementation->createDocument(0, str, 0);
 		DOMElement *documentElement= document->getDocumentElement();
+		for(unsigned int i = 0; i < node->getAttributeCount() ; ++i){
+			XmlAttribute *attr = node->getAttribute(i);
+
+			XMLCh strName[strSize];
+			XMLString::transcode(attr->getName().c_str(), strName, strSize-1);
+			XMLCh strValue[strSize];
+			XMLString::transcode(attr->getValue("",false).c_str(), strValue, strSize-1);
+
+			documentElement->setAttribute(strName,strValue);
+		}
 
 		for(unsigned int i=0; i<node->getChildCount(); ++i){
 			documentElement->appendChild(node->getChild(i)->buildElement(document));
@@ -171,15 +219,179 @@ void XmlIo::save(const string &path, const XmlNode *node){
 	}
 	catch(const DOMException &e){
 		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Exception while saving: [%s], %s\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),XMLString::transcode(e.msg));
-		throw runtime_error("Exception while saving: " + path + ": " + XMLString::transcode(e.msg));
+		throw megaglest_runtime_error("Exception while saving: " + path + ": " + XMLString::transcode(e.msg));
 	}
 }
+
+// =====================================================
+//	class XmlIoRapid
+// =====================================================
+XmlIoRapid::XmlIoRapid() {
+	try{
+		//printf("XmlIo init\n");
+
+		XmlIoRapid::initialized= true;
+	}
+	catch(const exception &e){
+		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Error initializing XML system, msg %s\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,e.what());
+		throw megaglest_runtime_error("Error initializing XML system");
+	}
+
+	try {
+		doc = new xml_document<>();
+	}
+	catch(const DOMException &ex) {
+		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Exception while creating XML parser, msg: %s\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,ex.getMessage());
+		throw megaglest_runtime_error("Exception while creating XML parser");
+	}
+}
+
+XmlIoRapid &XmlIoRapid::getInstance() {
+	static XmlIoRapid io;
+	return io;
+}
+
+void XmlIoRapid::cleanup() {
+	if(XmlIoRapid::initialized == true) {
+		XmlIoRapid::initialized= false;
+		//printf("XmlIo cleanup\n");
+		delete doc;
+		doc = NULL;
+	}
+}
+
+XmlIoRapid::~XmlIoRapid() {
+	cleanup();
+}
+
+XmlNode *XmlIoRapid::load(const string &path, std::map<string,string> mapTagReplacementValues,bool noValidation,bool skipStackCheck) {
+	Chrono chrono;
+	chrono.start();
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Using RapidXml to load file [%s]\n",path.c_str());
+	//printf("Using RapidXml to load file [%s]\n",path.c_str());
+
+	XmlNode *rootNode = NULL;
+	try {
+#if defined(WIN32) && !defined(__MINGW32__)
+		FILE *fp = _wfopen(utf8_decode(path).c_str(), L"rb");
+		ifstream xmlFile(fp);
+#else
+		ifstream xmlFile(path.c_str(),ios::binary);
+#endif
+		if(xmlFile.is_open() == false) {
+			throw megaglest_runtime_error("Can not open file: [" + path + "]");
+		}
+
+		xmlFile.unsetf(ios::skipws);
+
+        // Determine stream size
+		xmlFile.seekg(0, ios::end);
+        streampos size = xmlFile.tellg();
+        xmlFile.seekg(0);
+
+        // Load data and add terminating 0
+        vector<char> buffer;
+        buffer.resize((unsigned int)size + 1);
+        xmlFile.read(&buffer.front(), static_cast<streamsize>(size));
+        buffer[(unsigned int)size] = 0;
+
+        // This is required because rapidxml seems to choke when we load lua
+        // scenarios that have lua + xml style comments
+        replaceAllBetweenTokens(buffer, "<!--","-->", "", true);
+        doc->parse<parse_no_data_nodes>(&buffer.front());
+
+		rootNode= new XmlNode(doc->first_node(),mapTagReplacementValues);
+
+#if defined(WIN32) && !defined(__MINGW32__)
+		if(fp) {
+			fclose(fp);
+		}
+#endif
+	}
+	catch(const exception &ex) {
+		char szBuf[8096]="";
+		snprintf(szBuf,8096,"In [%s::%s Line: %d] Exception while loading: [%s], msg:\n%s",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),ex.what());
+		SystemFlags::OutputDebug(SystemFlags::debugError,"%s\n",szBuf);
+
+		throw megaglest_runtime_error(szBuf);
+	}
+
+	//if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] took msecs: %ld for file [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chrono.getMillis(),path.c_str());
+	//printf("In [%s::%s Line: %d] took msecs: %ld for file [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,chrono.getMillis(),path.c_str());
+
+	return rootNode;
+}
+
+void XmlIoRapid::save(const string &path, const XmlNode *node){
+	try {
+		xml_document<> doc;
+
+		// xml declaration
+		xml_node<>* decl = doc.allocate_node(node_declaration);
+		decl->append_attribute(doc.allocate_attribute(doc.allocate_string("version"), doc.allocate_string("1.0")));
+		decl->append_attribute(doc.allocate_attribute(doc.allocate_string("encoding"), doc.allocate_string("utf-8")));
+		decl->append_attribute(doc.allocate_attribute(doc.allocate_string("standalone"), doc.allocate_string("no")));
+		doc.append_node(decl);
+
+		// root node
+		xml_node<>* root = doc.allocate_node(node_element, doc.allocate_string(node->getName().c_str()));
+		for(unsigned int i = 0; i < node->getAttributeCount() ; ++i){
+			XmlAttribute *attr = node->getAttribute(i);
+			root->append_attribute(doc.allocate_attribute(
+					doc.allocate_string(attr->getName().c_str()),
+					doc.allocate_string(attr->getValue("",false).c_str())));
+		}
+		doc.append_node(root);
+
+		// child nodes
+		for(unsigned int i = 0; i < node->getChildCount(); ++i) {
+			root->append_node(node->getChild(i)->buildElement(&doc));
+		}
+
+//		std::string xml_as_string;
+//		// watch for name collisions here, print() is a very common function name!
+//		print(std::back_inserter(xml_as_string), doc);
+//		// xml_as_string now contains the XML in string form, indented
+//		// (in all its angle bracket glory)
+//
+//		std::string xml_no_indent;
+//		// print_no_indenting is the only flag that print() knows about
+//		print(std::back_inserter(xml_no_indent), doc, print_no_indenting);
+//		// xml_no_indent now contains non-indented XML
+
+#if defined(WIN32) && !defined(__MINGW32__)
+		FILE *fp = _wfopen(utf8_decode(path).c_str(), L"wb");
+		ofstream xmlFile(fp);
+#else
+		ofstream xmlFile(path.c_str(),ios::binary);
+#endif
+		if(xmlFile.is_open() == false) {
+			throw megaglest_runtime_error("Can not open file: [" + path + "]");
+		}
+
+		//xmlFile << xml_no_indent;
+//		xmlFile << xml_as_string;
+		xmlFile << doc;
+
+#if defined(WIN32) && !defined(__MINGW32__)
+		if(fp) {
+			fclose(fp);
+		}
+#endif
+	}
+	catch(const exception &e){
+		SystemFlags::OutputDebug(SystemFlags::debugError,"In [%s::%s Line: %d] Exception while saving: [%s], %s\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),e.what());
+		throw megaglest_runtime_error("Exception while saving [" + path + "] msg: " + e.what());
+	}
+}
+
 // =====================================================
 //	class XmlTree
 // =====================================================
-
-XmlTree::XmlTree(){
+XmlTree::XmlTree(xml_engine_parser_type engine_type) {
 	rootNode= NULL;
+	this->engine_type = engine_type;
+	this->skipStackCheck = false;
 }
 
 void XmlTree::init(const string &name){
@@ -190,46 +402,61 @@ typedef std::vector<XmlTree*> LoadStack;
 //static LoadStack loadStack;
 static string loadStackCacheName = string(__FILE__) + string("_loadStackCacheName");
 
-void XmlTree::load(const string &path, std::map<string,string> mapTagReplacementValues) {
-	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] about to load [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str());
+void XmlTree::load(const string &path, std::map<string,string> mapTagReplacementValues, bool noValidation,bool skipStackCheck) {
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] about to load [%s] skipStackCheck = %d\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str(),skipStackCheck);
 
-	//printf("XmlTree::load p [%p]\n",this);
-	assert(!loadPath.size());
+	this->skipStackCheck = skipStackCheck;
+	if(this->skipStackCheck == false) {
+		//printf("XmlTree::load p [%p]\n",this);
+		assert(!loadPath.size());
 
-	LoadStack &loadStack = CacheManager::getCachedItem<LoadStack>(loadStackCacheName);
-	Mutex &mutex = CacheManager::getMutexForItem<LoadStack>(loadStackCacheName);
-	MutexSafeWrapper safeMutex(&mutex);
+		LoadStack &loadStack = CacheManager::getCachedItem<LoadStack>(loadStackCacheName);
+		Mutex &mutex = CacheManager::getMutexForItem<LoadStack>(loadStackCacheName);
+		MutexSafeWrapper safeMutex(&mutex);
 
-	for(LoadStack::iterator it= loadStack.begin(); it!= loadStack.end(); ++it){
-		if((*it)->loadPath == path){
-			throw runtime_error(path + " recursively included");
+		for(LoadStack::iterator it= loadStack.begin(); it!= loadStack.end(); ++it){
+			if((*it)->loadPath == path){
+				throw megaglest_runtime_error(path + " recursively included");
+			}
 		}
+		loadStack.push_back(this);
+		safeMutex.ReleaseLock();
 	}
-	loadStack.push_back(this);
-	safeMutex.ReleaseLock();
 
 	loadPath = path;
-	this->rootNode= XmlIo::getInstance().load(path, mapTagReplacementValues);
+	if(this->engine_type == XML_XERCES_ENGINE) {
+		this->rootNode= XmlIo::getInstance().load(path, mapTagReplacementValues, noValidation,this->skipStackCheck);
+	}
+	else {
+		this->rootNode= XmlIoRapid::getInstance().load(path, mapTagReplacementValues, noValidation,this->skipStackCheck);
+	}
 
 	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("In [%s::%s Line: %d] about to load [%s]\n",extractFileFromDirectoryPath(__FILE__).c_str(),__FUNCTION__,__LINE__,path.c_str());
 }
 
 void XmlTree::save(const string &path){
-	XmlIo::getInstance().save(path, rootNode);
+	if(this->engine_type == XML_XERCES_ENGINE) {
+		XmlIo::getInstance().save(path, rootNode);
+	}
+	else {
+		XmlIoRapid::getInstance().save(path, rootNode);
+	}
 }
 
 XmlTree::~XmlTree() {
 	//printf("XmlTree::~XmlTree p [%p]\n",this);
 
-	LoadStack &loadStack = CacheManager::getCachedItem<LoadStack>(loadStackCacheName);
-	Mutex &mutex = CacheManager::getMutexForItem<LoadStack>(loadStackCacheName);
-	MutexSafeWrapper safeMutex(&mutex);
+	if(this->skipStackCheck == false) {
+		LoadStack &loadStack = CacheManager::getCachedItem<LoadStack>(loadStackCacheName);
+		Mutex &mutex = CacheManager::getMutexForItem<LoadStack>(loadStackCacheName);
+		MutexSafeWrapper safeMutex(&mutex);
 
-	LoadStack::iterator it= find(loadStack.begin(),loadStack.end(),this);
-	if(it != loadStack.end()) {
-		loadStack.erase(it);
+		LoadStack::iterator it= find(loadStack.begin(),loadStack.end(),this);
+		if(it != loadStack.end()) {
+			loadStack.erase(it);
+		}
+		safeMutex.ReleaseLock();
 	}
-	safeMutex.ReleaseLock();
 
 	delete rootNode;
 }
@@ -240,7 +467,7 @@ XmlTree::~XmlTree() {
 
 XmlNode::XmlNode(DOMNode *node, std::map<string,string> mapTagReplacementValues): superNode(NULL) {
     if(node == NULL || node->getNodeName() == NULL) {
-        throw runtime_error("XML structure seems to be corrupt!");
+        throw megaglest_runtime_error("XML structure seems to be corrupt!");
     }
 
 	//get name
@@ -285,6 +512,44 @@ XmlNode::XmlNode(DOMNode *node, std::map<string,string> mapTagReplacementValues)
 	}
 }
 
+XmlNode::XmlNode(xml_node<> *node, std::map<string,string> mapTagReplacementValues) : superNode(NULL) {
+	if(node == NULL || node->name() == NULL) {
+        throw megaglest_runtime_error("XML structure seems to be corrupt!");
+    }
+
+	//get name
+	name = node->name();
+
+	//check document
+	if(node->type() == node_document) {
+		name="document";
+	}
+
+	if(SystemFlags::VERBOSE_MODE_ENABLED) printf("Found XML Node\nName [%s]\nValue [%s]\n",name.c_str(),node->value());
+
+	//check children
+	for(xml_node<> *currentNode = node->first_node();
+			currentNode; currentNode = currentNode->next_sibling()) {
+		if(currentNode != NULL && currentNode->type() == node_element) {
+			XmlNode *xmlNode= new XmlNode(currentNode, mapTagReplacementValues);
+			children.push_back(xmlNode);
+		}
+    }
+
+	//check attributes
+	for (xml_attribute<> *attr = node->first_attribute();
+			attr; attr = attr->next_attribute()) {
+		XmlAttribute *xmlAttribute= new XmlAttribute(attr, mapTagReplacementValues);
+		attributes.push_back(xmlAttribute);
+	}
+
+	//get value
+	if(node->type() == node_element && children.size() == 0) {
+		text = node->value();
+		Properties::applyTagsToValue(this->text);
+	}
+}
+
 XmlNode::XmlNode(const string &name): superNode(NULL) {
 	this->name= name;
 }
@@ -300,7 +565,7 @@ XmlNode::~XmlNode() {
 
 XmlAttribute *XmlNode::getAttribute(unsigned int i) const {
 	if(i >= attributes.size()) {
-		throw runtime_error(getName()+" node doesn't have "+intToStr(i)+" attributes");
+		throw megaglest_runtime_error(getName()+" node doesn't have "+intToStr(i)+" attributes");
 	}
 	return attributes[i];
 }
@@ -312,7 +577,7 @@ XmlAttribute *XmlNode::getAttribute(const string &name,bool mustExist) const {
 		}
 	}
 	if(mustExist == true) {
-		throw runtime_error("\"" + getName() + "\" node doesn't have a attribute named \"" + name + "\"");
+		throw megaglest_runtime_error("\"" + getName() + "\" node doesn't have a attribute named \"" + name + "\"");
 	}
 
 	return NULL;
@@ -332,7 +597,7 @@ bool XmlNode::hasAttribute(const string &name) const {
 XmlNode *XmlNode::getChild(unsigned int i) const {
 	assert(!superNode);
 	if(i >= children.size()) {
-		throw runtime_error("\"" + getName()+"\" node doesn't have "+intToStr(i+1)+" children");
+		throw megaglest_runtime_error("\"" + getName()+"\" node doesn't have "+intToStr(i+1)+" children");
 	}
 	return children[i];
 }
@@ -352,7 +617,7 @@ XmlNode *XmlNode::getChild(const string &childName, unsigned int i) const{
 	if(superNode && !hasChildNoSuper(childName))
 		return superNode->getChild(childName,i);
 	if(i>=children.size()){
-		throw runtime_error("\"" + name + "\" node doesn't have "+intToStr(i+1)+" children named \"" + childName + "\"\n\nTree: "+getTreeString());
+		throw megaglest_runtime_error("\"" + name + "\" node doesn't have "+intToStr(i+1)+" children named \"" + childName + "\"\n\nTree: "+getTreeString());
 	}
 
 	int count= 0;
@@ -365,7 +630,7 @@ XmlNode *XmlNode::getChild(const string &childName, unsigned int i) const{
 		}
 	}
 
-	throw runtime_error("Node \""+getName()+"\" doesn't have "+intToStr(i+1)+" children named  \""+childName+"\"\n\nTree: "+getTreeString());
+	throw megaglest_runtime_error("Node \""+getName()+"\" doesn't have "+intToStr(i+1)+" children named  \""+childName+"\"\n\nTree: "+getTreeString());
 }
 
 bool XmlNode::hasChildAtIndex(const string &childName, int i) const {
@@ -434,6 +699,23 @@ DOMElement *XmlNode::buildElement(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *do
 	return node;
 }
 
+xml_node<>* XmlNode::buildElement(xml_document<> *document) const {
+	xml_node<>* node = document->allocate_node(node_element, document->allocate_string(name.c_str()));
+
+	for(unsigned int i = 0; i < attributes.size(); ++i) {
+		node->append_attribute(
+				document->allocate_attribute(
+						document->allocate_string(attributes[i]->getName().c_str()),
+						document->allocate_string(attributes[i]->getValue().c_str())));
+	}
+
+	for(unsigned int i = 0; i < children.size(); ++i) {
+		node->append_node(children[i]->buildElement(document));
+	}
+
+	return node;
+}
+
 string XmlNode::getTreeString() const {
 	string str;
 
@@ -470,6 +752,21 @@ XmlAttribute::XmlAttribute(DOMNode *attribute, std::map<string,string> mapTagRep
 	name= str;
 }
 
+XmlAttribute::XmlAttribute(xml_attribute<> *attribute, std::map<string,string> mapTagReplacementValues) {
+	skipRestrictionCheck 			= false;
+	usesCommondata 					= false;
+	this->mapTagReplacementValues 	= mapTagReplacementValues;
+	//char str[strSize]				= "";
+
+	//XMLString::transcode(attribute->getNodeValue(), str, strSize-1);
+	value= attribute->value();
+	usesCommondata = ((value.find("$COMMONDATAPATH") != string::npos) || (value.find("%%COMMONDATAPATH%%") != string::npos));
+	skipRestrictionCheck = Properties::applyTagsToValue(this->value,&this->mapTagReplacementValues);
+
+	//XMLString::transcode(attribute->getNodeName(), str, strSize-1);
+	name= attribute->name();
+}
+
 XmlAttribute::XmlAttribute(const string &name, const string &value, std::map<string,string> mapTagReplacementValues) {
 	skipRestrictionCheck 			= false;
 	usesCommondata 					= false;
@@ -489,7 +786,7 @@ bool XmlAttribute::getBoolValue() const {
 		return false;
 	}
 	else {
-		throw runtime_error("Not a valid bool value (true or false): " +getName()+": "+ value);
+		throw megaglest_runtime_error("Not a valid bool value (true or false): " +getName()+": "+ value);
 	}
 }
 
@@ -497,10 +794,14 @@ int XmlAttribute::getIntValue() const {
 	return strToInt(value);
 }
 
+uint32 XmlAttribute::getUIntValue() const {
+	return strToUInt(value);
+}
+
 int XmlAttribute::getIntValue(int min, int max) const {
 	int i= strToInt(value);
 	if(i<min || i>max){
-		throw runtime_error("Xml Attribute int out of range: " + getName() + ": " + value);
+		throw megaglest_runtime_error("Xml Attribute int out of range: " + getName() + ": " + value);
 	}
 	return i;
 }
@@ -512,7 +813,7 @@ float XmlAttribute::getFloatValue() const{
 float XmlAttribute::getFloatValue(float min, float max) const{
 	float f= strToFloat(value);
 	if(f<min || f>max){
-		throw runtime_error("Xml attribute float out of range: " + getName() + ": " + value);
+		throw megaglest_runtime_error("Xml attribute float out of range: " + getName() + ": " + value);
 	}
 	return f;
 }
@@ -534,7 +835,7 @@ const string XmlAttribute::getRestrictedValue(string prefixValue, bool trimValue
 
 		for(unsigned int i= 0; i<value.size(); ++i){
 			if(allowedCharacters.find(value[i])==string::npos){
-				throw runtime_error(
+				throw megaglest_runtime_error(
 					string("The string \"" + value + "\" contains a character that is not allowed: \"") + value[i] +
 					"\"\nFor portability reasons the only allowed characters in this field are: " + allowedCharacters);
 			}
@@ -551,4 +852,8 @@ const string XmlAttribute::getRestrictedValue(string prefixValue, bool trimValue
 	return result;
 }
 
+void XmlAttribute::setValue(string val) {
+	value = val;
+}
+
 }}//end namespace
